@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "base/file_utilities.h"
 #include "base/file_functions.h"
 #include "base/string_utilities.h"
@@ -37,7 +35,9 @@
 #include "model/wb_model_diagram_form.h"
 #include "wb_overview_physical.h"
 #include "wb_overview_physical_schema.h"
-#include "objimpl/ui/mforms_ObjectReference_impl.h"
+#include "wb_catalog_tree_view.h"
+
+#include "objimpl/wrapper/mforms_ObjectReference_impl.h"
 #include "workbench/wb_model_file.h"
 #include "model/wb_overview_physical.h"
 #include "model/wb_user_datatypes.h"
@@ -45,6 +45,7 @@
 #include "wbcanvas/model_diagram_impl.h"
 #include "wbcanvas/workbench_physical_model_impl.h"
 #include "workbench/wb_model_file.h"
+#include "grtdb/db_helpers.h"
 #include "grtdb/db_object_helpers.h"
 #include "grt/clipboard.h"
 #include "grt/common.h"
@@ -63,7 +64,6 @@
 using namespace base;
 using namespace bec;
 using namespace wb;
-using namespace base;
 
 DEFAULT_LOG_DOMAIN("ModelContext");
 
@@ -103,8 +103,8 @@ WBContextModel::WBContextModel(WBContextUI *wbui)
   
   base::NotificationCenter::get()->add_observer(this, "GNMainFormChanged");
 
-  // setup auto-save for model
-  int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
+  // Setup auto-save for model, only full seconds.
+  int interval = (int)_wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
   if (interval > 0)
     _auto_save_timer = _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), interval);
   _auto_save_interval = interval;
@@ -180,14 +180,28 @@ mforms::TreeNodeView *WBContextModel::create_user_type_list()
   return type_list;
 }
 
+//--------------------------------------------------------------------------------------------------
 
-mforms::TreeNodeView *WBContextModel::create_history_tree()
+mforms::TreeNodeView* WBContextModel::create_history_tree()
 {
-  HistoryTree *history_tree= new HistoryTree(_wbui->get_wb()->get_grt_manager(), _wbui->get_wb()->get_grt()->get_undo_manager());
+  HistoryTree *history_tree = new HistoryTree(_wbui->get_wb()->get_grt_manager(), _wbui->get_wb()->get_grt()->get_undo_manager());
   history_tree->refresh();
   return history_tree;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+mforms::TreeNodeView* WBContextModel::create_catalog_tree()
+{
+  CatalogTreeBE *model = _wbui->get_wb()->get_component<WBComponentPhysical>()->get_catalog_tree_model();
+  CatalogTreeView *tree = new CatalogTreeView(model);
+
+  // No tree loading/refresh for now. This will happen when the diagram is ready
+  // and we know which object is already placed.
+  return tree;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 grt::GRT *WBContextModel::get_grt()
 {
@@ -207,7 +221,7 @@ void WBContextModel::option_changed(grt::internal::OwnedDict*dict, bool, const s
 bool WBContextModel::auto_save_document()
 {
   WBContext *wb= _wbui->get_wb();
-  int interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
+  ssize_t interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveModelInterval", 60);
   if (interval <= 0)
     return false;
   
@@ -238,7 +252,7 @@ bool WBContextModel::auto_save_document()
     if (_auto_save_timer)
         wb->get_grt_manager()->cancel_timer(_auto_save_timer);
     // schedule new interval
-    _auto_save_timer = wb->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), interval);
+    _auto_save_timer = wb->get_grt_manager()->run_every(boost::bind(&WBContextModel::auto_save_document, this), (double)interval);
     return false;
   }
   
@@ -489,9 +503,9 @@ void WBContextModel::release_image(const std::string &file)
   // _wbui->get_wb()->get_file()->release_image(file);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 /** 
- ****************************************************************************
  * @brief Delegate method for creating canvas views
  *
  * This is called by the GRT bridge when a new view object is created.
@@ -501,7 +515,7 @@ void WBContextModel::release_image(const std::string &file)
  * The free_canvas_view() method will be called once the view can be freed.
  * 
  * @param name the name of the canvas. Will be the object-id of the view.
- *****************************************************************************/
+ */
 mdc::CanvasView *WBContextModel::create_diagram(const model_DiagramRef &view)
 {
   return _wbui->get_wb()->execute_in_main_thread<mdc::CanvasView*>("create_diagram", boost::bind(&WBContextModel::create_diagram_main, this, view));
@@ -592,7 +606,7 @@ mdc::CanvasView *WBContextModel::create_diagram_main(const model_DiagramRef &dia
 
 
 
-void WBContextModel::activate_canvas_object(const model_ObjectRef &object, int flags)
+void WBContextModel::activate_canvas_object(const model_ObjectRef &object, ssize_t flags)
 {
   bool newwindow= flags & 1;
   
@@ -790,15 +804,14 @@ GrtObjectRef WBContextModel::duplicate_object(const db_DatabaseObjectRef &object
     copy_context.update_references();
     
     // post-processing
-    // - uniquefy foreign key names
-    int max_fk_len = workbench_physical_ModelRef::cast_from(dbtable->owner()->owner()->owner())->rdbms()->maximumIdentifierLength();
+    // - Make foreign key names unique.
+    ssize_t max_fk_len = workbench_physical_ModelRef::cast_from(dbtable->owner()->owner()->owner())->rdbms()->maximumIdentifierLength();
     grt::ListRef<db_ForeignKey> fks(dbtable->foreignKeys());
     std::set<std::string> used_fk_names = bec::SchemaHelper::get_foreign_key_names(db_SchemaRef::cast_from(dbtable->owner()));
     for (size_t c= fks.count(), i = 0; i < c; i++)
     {
       db_ForeignKeyRef fk(fks[i]);
-      
-      fk->name(bec::SchemaHelper::get_unique_foreign_key_name(used_fk_names, fk->name(), max_fk_len));
+      fk->name(bec::SchemaHelper::get_unique_foreign_key_name(used_fk_names, fk->name(), (int)max_fk_len));
     }
     
     if (grt::find_named_object_in_list(db_SchemaRef::cast_from(dbtable->owner())->tables(), dbtable->name()).is_valid())
@@ -1251,6 +1264,25 @@ void WBContextModel::selection_changed()
 }
 
 
+GrtVersionRef WBContextModel::get_target_version()
+{
+  if (get_active_model(true).is_valid())
+  {
+    db_CatalogRef catalog(workbench_physical_ModelRef::cast_from(get_active_model(true))->catalog());
+    if (catalog->version().is_valid())
+      return catalog->version();
+    else
+    {
+      std::string target_version = _wbui->get_wb()->get_grt_manager()->get_app_option_string("DefaultTargetMySQLVersion");
+      if (target_version.empty())
+        target_version = "5.5";
+
+      return bec::parse_version(_wbui->get_wb()->get_grt_manager()->get_grt(), target_version);
+    }
+  }
+  return GrtVersionRef();
+}
+
 
 #ifndef BasicExport____
 
