@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -77,7 +77,10 @@ namespace wb
     mforms::TreeNodeView *create_user_type_list();
     void show_user_type_editor(workbench_physical_ModelRef model);
 
+    GrtVersionRef get_target_version();
+
     mforms::TreeNodeView *create_history_tree();
+    mforms::TreeNodeView *create_catalog_tree();
 
     model_DiagramRef get_active_model_diagram(bool main_form);
     model_ModelRef get_active_model(bool main_form);
@@ -98,7 +101,7 @@ namespace wb
     void realize();
     void unrealize();
     
-    void activate_canvas_object(const model_ObjectRef &object, int flags);
+    void activate_canvas_object(const model_ObjectRef &object, ssize_t flags);
     
     void update_page_settings();
     
