@@ -17,9 +17,6 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "wb_model_diagram_form.h"
 #include "workbench/wb_context.h"
 #include "wb_component.h"
@@ -513,6 +510,18 @@ void ModelDiagramForm::attach_canvas_view(mdc::CanvasView *cview)
 }
 
 
+void ModelDiagramForm::close()
+{
+  set_closed(true);
+  _mini_view->set_active_view(NULL, model_DiagramRef());
+
+  delete _mini_view;
+  _mini_view = 0;
+
+  _model_diagram->get_data()->unrealize();
+}
+
+
 void ModelDiagramForm::set_closed(bool flag)
 {
   if (_model_diagram.is_valid())
@@ -885,6 +894,12 @@ void ModelDiagramForm::handle_mouse_move(int x, int y, mdc::EventState state)
 }
 
 
+void ModelDiagramForm::handle_mouse_leave(int x, int y, mdc::EventState state)
+{
+  _view->handle_mouse_leave(x, y, state);
+}
+
+
 bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state)
 {
   if (press)
@@ -1099,7 +1114,7 @@ bool ModelDiagramForm::can_paste()
   {
     if (!(*iter).is_valid())
     {
-      g_message("copy buffer has null value");
+      log_warning("copy buffer has null value");
       return false;
     }
     bool result= false;
@@ -1158,7 +1173,7 @@ void ModelDiagramForm::cut()
   um->begin_undo_group();
   copy();
   
-  int count= get_copiable_selection().count();
+  int count = (int)get_copiable_selection().count();
   remove_selection();
   um->end_undo_group();
   um->set_action_description(strfmt(_("Cut %s"), edit_target_name.c_str()));
