@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,11 +25,11 @@ namespace mforms {
   class ToolBarItem;
 }
 
-#include "wb_sql_editor_form.h"
+#include "base/notifications.h"
+#include "mforms/tabview.h"
 
-#include <mforms/tabview.h>
-
-class SnippetList;
+class SnippetListView;
+class GrtThreadedTask;
 
 class MYSQLWBBACKEND_PUBLIC_FUNC QuerySidePalette : public mforms::TabView, base::Observer
 {
@@ -40,14 +40,17 @@ private:
   mforms::HyperText* _help_text;
   bec::GRTManager *_grtm;
   bec::GRTManager::Timer* _help_timer;
+  GrtThreadedTask::Ref _help_task; // For running help construction on a background thread.
   
   mforms::ScrollPanel* _snippet_box;
   mforms::ToolBar* _snippet_toolbar;
-  SnippetList* _snippet_list;
+  SnippetListView* _snippet_list;
 
-  bool _no_help;        // True if currently no help topic is visible.
-  bool _enable_help;    // True if help lookup should happen.
-  bool _switching_help; // Recursion stopper when switching to new topic and setting drop down.
+  bool _no_help;           // True if currently no help topic is visible.
+  bool _automatic_help;    // Automatically show help after moving the caret.
+  bool _switching_help;    // Recursion stopper when switching to new topic and setting drop down.
+  bool _pending_snippets_refresh;
+  std::string _last_topic;
 
   // Contains patterns for syntactic elements and their associated colors. Initialized on the fly
   // but must be cleared when any of the stored colors changes.
@@ -58,16 +61,22 @@ private:
   mforms::ToolBarItem *_back_item;
   mforms::ToolBarItem *_forward_item;
   mforms::ToolBarItem *_quick_jump_item;
+  mforms::ToolBarItem *_manual_help_item;
   std::vector<std::string> _topic_history;
-  int _current_topic;
+  int _current_topic_index;
   std::map<std::string, std::pair<std::string, std::string> > _topic_cache; // Plain text and html text under a specific topic.
 
   void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
-  void tab_changed();
 
-  void show_help_topic(std::string topic);
-  bool update_context_help(Sql_editor *editor);
-  bool update_help_history(std::string topic);
+  void show_help_text_for_topic(const std::string &topic);
+  grt::StringRef get_help_text_threaded(grt::GRT *);
+  void update_help_ui();
+  void show_help_hint_or_update();
+
+  bool find_context_help(MySQLEditor *editor);
+  grt::StringRef get_help_topic_threaded(grt::GRT *, const std::string &query, std::pair<ssize_t, ssize_t> caret);
+  void process_help_topic(const std::string &topic);
+  void update_help_history(const std::string &topic);
 
   void click_link(const std::string &link);
   mforms::ToolBar* prepare_snippet_toolbar();
@@ -75,14 +84,16 @@ private:
   void snippet_toolbar_item_activated(mforms::ToolBarItem *item);
   void help_toolbar_item_activated(mforms::ToolBarItem *item);
 
-  void check_format_structures(Sql_editor *editor);
+  void check_format_structures(MySQLEditor *editor);
   std::string format_help_as_html(const std::string &text);
-  
+
   void snippet_selection_changed();
 public:
   QuerySidePalette(const SqlEditorForm::Ref &owner);
   ~QuerySidePalette();
   void cancel_timer();
   void close_popover();
+
+  void refresh_snippets();
 };
 
