@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,8 +48,8 @@ private:
   wb::LiveSchemaTree::NewSchemaContentArrivedSlot schema_content_arrived_slot;
 public:
   EditorFormTester():
-      _expect_update_node_children(false),
       _expect_schema_content_arrived(false),
+      _expect_update_node_children(false),
       _mock_validate_schema_content(false),
       _mock_propagate_schema_content(false),
       _mock_propagate_update_node_children(false)
@@ -87,7 +87,7 @@ public:
   {
     _form->get_live_tree()->fetch_schema_contents(schema_name, schema_content_arrived_slot);
 
-    sleep(1);
+    g_usleep(1000000); // 1 second
 
     _form->grt_manager()->perform_idle_tasks();
   }
@@ -108,11 +108,6 @@ public:
     _form->get_live_tree()->_schema_tree->expand_toggled(schema_node, true);
   }
 
-  bool is_editable_select(const std::string &sql, std::string& schema_name, std::string& table_name, std::string& error)
-  {
-    return _form->get_live_tree()->is_editable_select(sql, schema_name, table_name, error);
-  }
-
   void exec_sql(std::string &sql)
   {
     _form->exec_sql_returning_results(sql, false);
@@ -696,100 +691,6 @@ TEST_FUNCTION(6)
   ensure_equals("TF006CHK005 : Unexpected foreign key delete rule", pchild_data->referenced_table, "language");
 }
 
-// Testing : SqlEditorForm::is_editable_select
-TEST_FUNCTION(7)
-{
-  form_tester.load_schema_list();
-  form_tester.load_schema_data("wb_sql_editor_form_test");
-
-  std::string error;
-  std::string schema_name;
-  std::string table_name;
-  bool ret;
-
-  /* Failure case: using functions as columns */
-  ensure("TF007CHK001 Unexpected editable select success", !form_tester.is_editable_select("select count(*) from wb_sql_editor_form_test.film", schema_name, table_name, error));
-  ensure_equals("TF007CHK002 Unexpected error on editable select", error, "The statement is not editable.");
-
-  /* Success case : using a non loaded schema, the required table will be loaded right away */
-  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.dummy_table", schema_name, table_name, error);
-  ensure_equals("TF007CHK004 Unexpected error on editable select", error, "");
-  ensure("TF007CHK003 Unexpected editable select failure", ret);
-
-  /* Success case : loading all columns from a table having primary key */
-  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
-  ensure_equals("TF007CHK006 Unexpected error on editable select", error, "");
-  ensure("TF007CHK005 Unexpected editable select failure", ret);
-
-  /* Success case : loading columns include primary key */
-  ret = form_tester.is_editable_select("select film_id, title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error);
-  ensure_equals("TF007CHK008 Unexpected error on editable select", error, "");
-  ensure("TF007CHK007 Unexpected editable select failure", ret);
-
-  /* Failure case : loading columns not include primary key */
-  ensure("TF007CHK009 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.film_text", schema_name, table_name, error));
-  ensure_equals("TF007CHK010 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table film_text.");
-
-
-  /* Success case : loading all columns from a table having complex primary key */
-  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
-  ensure_equals("TF007CHK012 Unexpected error on editable select", error, "");
-  ensure("TF007CHK011 Unexpected editable select failure", ret);
-
-  /* Success case : loading columns include primary key but not all */
-  ensure("TF007CHK013 Unexpected editable select success", !form_tester.is_editable_select("select dummy_first_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error));
-  ensure_equals("TF007CHK014 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table complex_pk_table.");
-
-
-  /* Success case : loading columns include all the primary key columns */
-  ret = form_tester.is_editable_select("select dummy_first_id, dummy_second_id, title, description from wb_sql_editor_form_test.complex_pk_table", schema_name, table_name, error);
-  ensure_equals("TF007CHK016 Unexpected error on editable select", error, "");
-  ensure("TF007CHK015 Unexpected editable select failure", ret);
-
-
-  /* failure case : loading all columns from a table not having primary key */
-  ensure("TF007CHK017 Unexpected editable select sucess", !form_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table", schema_name, table_name, error));
-  ensure_equals("TF007CHK018 Unexpected error on editable select", error, "Table no_pk_table has no primary keys or unique non nullable columns defined. Only tables with primary keys or unique non nullable columns can be edited.");
-
-
-  /* Success case : loading all columns from a table having primary key and unique non nullable column */
-  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
-  ensure_equals("TF007CHK021 Unexpected error on editable select", error, "");
-  ensure("TF007CHK020 Unexpected editable select failure", ret);
-
-  /* Success case : loading columns include primary key */
-  ret = form_tester.is_editable_select("select dummy_pk, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
-  ensure_equals("TF007CHK023 Unexpected error on editable select", error, "");
-  ensure("TF007CHK022 Unexpected editable select failure", ret);
-
-  /* Success case : loading columns include unique non nullable column but not PK */
-  ret = form_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error);
-  ensure_equals("TF007CHK025 Unexpected error on editable select", error, "");
-  ensure("TF007CHK024 Unexpected editable select failure", ret);
-
-  /* Failure case : loading columns not include primary key nor unique non nullable column */
-  ensure("TF007CHK026 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.pk_table_unique_not_null", schema_name, table_name, error));
-  ensure_equals("TF007CHK027 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table pk_table_unique_not_null.");
-  
-  /* Success case : loading all columns from a table NOT having primary key BUT unique non nullable column */
-  ret = form_tester.is_editable_select("select * from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
-  ensure_equals("TF007CHK031 Unexpected error on editable select", error, "");
-  ensure("TF007CHK030 Unexpected editable select failure", ret);
-
-  /* Success case : loading columns include unique non nullable column */
-  ret = form_tester.is_editable_select("select dummy_id, title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error);
-  ensure_equals("TF007CHK033 Unexpected error on editable select", error, "");
-  ensure("TF007CHK032 Unexpected editable select failure", ret);
-
-  /* Failure case : loading columns not unique non nullable column */
-  ensure("TF007CHK034 Unexpected editable select sucess", !form_tester.is_editable_select("select title, description from wb_sql_editor_form_test.no_pk_table_unique_not_null", schema_name, table_name, error));
-  ensure_equals("TF007CHK035 Unexpected error on editable select", error, "The select statement is missing primary key or unique non nullable columns on table no_pk_table_unique_not_null.");
-  
-  /* Failure case : allowed to do scalar operations over results (bug #64435) */
-  ensure("TF007CHK040 Unexpected editable select sucess", !form_tester.is_editable_select("select ifnull(max(language_id),0)+1 from language", schema_name, table_name, error));
-  ensure_equals("TF007CHK041 Unexpected error on editable select", error, "The statement is not editable.");
-  
-}
 
 TEST_FUNCTION(100)
 {
