@@ -17,9 +17,7 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
+#include "python_context.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
 #include "base/sqlstring.h"
@@ -30,6 +28,7 @@
 
 #include "grtpp.h"
 
+
 #include "grts/structs.h"
 #include "grts/structs.app.h"
 
@@ -41,6 +40,7 @@
 
 #include "sqlide/wb_context_sqlide.h"
 #include "sqlide/wb_sql_editor_form.h"
+#include "sqlide/wb_sql_editor_panel.h"
 #include "sqlide/wb_sql_editor_snippets.h"
 #include "sqlide/wb_sql_editor_help.h"
 #include "sqlide/wb_sql_editor_result_panel.h"
@@ -51,6 +51,7 @@
 #include "objimpl/db.query/db_query_Resultset.h"
 #include "objimpl/db.query/db_query_EditableResultset.h"
 #include "objimpl/db.query/db_query_QueryBuffer.h"
+#include "objimpl/wrapper/grt_PyObject_impl.h"
 #include "grts/structs.db.query.h"
 
 #include "grtdb/db_helpers.h"
@@ -72,7 +73,7 @@ static std::map<std::string,std::string> auto_save_sessions;
 
 class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_query_Editor::ImplData, public base::trackable
 {
-  void sql_editor_list_changed(Sql_editor::Ref editor, bool added)
+  void sql_editor_list_changed(MySQLEditor::Ref editor, bool added)
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
@@ -90,83 +91,22 @@ class MYSQLWBBACKEND_PUBLIC_FUNC db_query_EditorConcreteImplData : public db_que
     }
   }
 
-  void recordset_list_changed(int editor_index, Recordset::Ref rset, bool added)
-  {
-    boost::shared_ptr<SqlEditorForm> ref(_editor);
-    if (ref)
-    {
-      db_query_QueryEditorRef editor;
-      
-      boost::shared_ptr<Sql_editor> sqleditor(ref->sql_editor(editor_index));
-      if (sqleditor)
-      {
-        editor = db_query_QueryEditorRef::cast_from(sqleditor->grtobj());
-      }
-      else
-      {
-        log_warning("recordset_list_changed received for invalid editor index (%i). Internal state of db_query_QueryEditor might have become inconsistent\n",
-                    editor_index);
-        return;
-      }
-
-      if (added)
-      {
-        if (!rset->is_readonly())
-          editor->resultsets().insert(grtwrap_editablerecordset(editor, rset));
-        else
-          editor->resultsets().insert(grtwrap_recordset(editor, rset));
-      }
-      else
-      {
-        for (size_t c= editor->resultsets().count(), i= 0; i < c; i++)
-        {
-          db_query_ResultsetRef e(editor->resultsets()[i]);
-          
-          if (dynamic_cast<WBRecordsetResultset*>(e->get_data())->recordset == rset)
-          {
-            editor->resultsets().remove(i);
-            e->reset_references();
-            break;
-          }
-        }
-      }
-    }
-  }
-  
-  
-  virtual void refresh_editor(Sql_editor::Ref editor)
-  {
-    boost::shared_ptr<SqlEditorForm> ref(_editor);
-    if (ref)
-    {
-      int oindex = ref->active_sql_editor_index();
-      
-      for (size_t c= _self->queryEditors().count(), i= 0; i < c; i++)
-      {
-        db_query_QueryBufferRef e(_self->queryEditors()[i]);
-        if (e == editor->grtobj())
-        {
-          ref->active_sql_editor_index(i);
-          break;
-        }
-      }
-      ref->active_sql_editor_index(oindex);
-    }
-  }
-  
 public:
   db_query_EditorConcreteImplData(boost::shared_ptr<SqlEditorForm> editor,
                                   const db_query_EditorRef &self)
   : _self(dynamic_cast<db_query_Editor*>(self.valueptr())), _editor(editor)
   {
-    for (size_t i= 0; i < (size_t) editor->sql_editor_count(); i++)
+    for (int c = editor->sql_editor_count(), i = 0; i < c; i++)
     {
-      db_query_QueryEditorRef qb(db_query_QueryEditorRef::cast_from(editor->sql_editor(i)->grtobj()));
-      qb->owner(self);
-      _self->queryEditors().insert(qb);
+      SqlEditorPanel *panel = editor->sql_editor_panel(i);
+      if (panel)
+      {
+        db_query_QueryEditorRef qb(panel->grtobj());
+        qb->owner(self);
+        _self->queryEditors().insert(qb);
+      }
     }
     
-    editor->recordset_list_changed.connect(boost::bind(&db_query_EditorConcreteImplData::recordset_list_changed, this, _1, _2, _3));
     editor->sql_editor_list_changed.connect(boost::bind(&db_query_EditorConcreteImplData::sql_editor_list_changed, this, _1, _2));
   }
   
@@ -195,7 +135,7 @@ public:
     {
       _editor->new_sql_script_file();
       
-      return db_query_QueryEditorRef::cast_from(_editor->active_sql_editor()->grtobj());
+      return _editor->active_sql_editor_panel()->grtobj();
     }
     return db_query_QueryEditorRef();
   }
@@ -228,7 +168,7 @@ public:
       }
       catch (sql::SQLException &exc)
       {
-        log_error("Execption executing SQL code from GRT interface: %s\n", exc.what());
+        log_error("Exception executing SQL code from GRT interface: %s\n", exc.what());
       }
     }
     return result;
@@ -238,7 +178,7 @@ public:
   {
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)      
-      ref->exec_sql_retaining_editor_contents(sql, Sql_editor::Ref(), true);
+      ref->exec_sql_retaining_editor_contents(sql, NULL, true);
 
     return grt::IntegerRef(0);
   }
@@ -276,7 +216,7 @@ public:
     if (ref)
     {
       if (background)
-        ref->exec_sql_retaining_editor_contents(sql, Sql_editor::Ref(), false);
+        ref->exec_sql_retaining_editor_contents(sql, NULL, false);
       else
         ref->exec_main_sql(sql, log);
     }
@@ -328,9 +268,9 @@ public:
     boost::shared_ptr<SqlEditorForm> ref(_editor);
     if (ref)
     {
-      Sql_editor::Ref editor(ref->active_sql_editor());
-      if (editor)
-        return db_query_QueryEditorRef::cast_from(editor->grtobj());
+      SqlEditorPanel *panel = ref->active_sql_editor_panel();
+      if (panel)
+        return panel->grtobj();
     }
     return db_query_QueryEditorRef();
   }
@@ -378,6 +318,34 @@ public:
   {
     _editor.reset();
   }
+
+
+  grt_PyObjectRef createCPyConnection()
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+
+    WillEnterPython lock;
+    grt::PythonContext *py = grt::PythonContext::get();
+    py->run_buffer("import mysql.connector");
+    PyObject *ctor = py->get_global("mysql.connector.Connect");
+    if (!ctor)
+      throw std::logic_error("Could not get handle to Connector method");
+
+    grt::AutoPyObject kwarg(PyDict_New());
+
+    grt::DictRef params(ref->connection_descriptor()->parameterValues());
+
+    PyDict_SetItemString(kwarg, "host", grt::AutoPyObject(PyString_FromString(params.get_string("hostName").c_str())));
+    PyDict_SetItemString(kwarg, "port", grt::AutoPyObject(PyLong_FromSize_t(params.get_int("port"))));
+    PyDict_SetItemString(kwarg, "user", grt::AutoPyObject(PyString_FromString(params.get_string("userName").c_str())));
+    PyDict_SetItemString(kwarg, "password", grt::AutoPyObject(PyString_FromString(ref->dbc_auth_data()->password())));
+
+    grt::AutoPyObject connection(PyObject_Call(ctor, grt::AutoPyObject(PyTuple_New(0)), kwarg));
+    if (!connection)
+      throw grt::python_error("error opening connection");
+
+    return pyobject_to_grt(_self->get_grt(), connection);
+  }
     
 protected:  
   db_query_Editor *_self;
@@ -394,6 +362,17 @@ void WBContextSQLIDE::call_in_editor(void (SqlEditorForm::*method)())
     (form->*method)();
 }
 
+void WBContextSQLIDE::call_in_editor_panel(void (SqlEditorPanel::*method)())
+{
+  SqlEditorForm *form= get_active_sql_editor();
+  if (form)
+  {
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
+      (panel->*method)();
+  }
+}
+
 
 void WBContextSQLIDE::call_in_editor_str(void (SqlEditorForm::*method)(const std::string &arg), const std::string &arg)
 {
@@ -415,16 +394,28 @@ static void call_export(wb::WBContextSQLIDE *sqlide)
   SqlEditorForm *form= sqlide->get_active_sql_editor();
   if (form)
   {
-    int editor = form->active_sql_editor_index();
-    if (editor >= 0)
-      form->active_result_panel(editor)->show_export_recordset();
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel && panel->active_result_panel())
+      panel->active_result_panel()->show_export_recordset();
   }
 }
 
+
+inline bool has_active_resultset(wb::WBContextSQLIDE *sqlide)
+{
+  SqlEditorForm *form = sqlide->get_active_sql_editor();
+  if (form)
+  {
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
+      return panel->active_result_panel() != NULL;
+  }
+  return false;
+}
+
 static bool validate_export(wb::WBContextSQLIDE *sqlide)
 {
-  SqlEditorForm *form = sqlide->get_active_sql_editor();
-  return form && form->active_sql_editor_index() >= 0 && form->active_result_panel(form->active_sql_editor_index());
+  return has_active_resultset(sqlide);
 }
 
 static void call_run_file(wb::WBContextSQLIDE *sqlide)
@@ -434,15 +425,29 @@ static void call_run_file(wb::WBContextSQLIDE *sqlide)
     sqlide->run_file(path);
 }
 
+static void call_save_file(wb::WBContextSQLIDE *sqlide)
+{
+  SqlEditorForm *editor = sqlide->get_active_sql_editor();
+  if (editor)
+  {
+    SqlEditorPanel *panel = editor->active_sql_editor_panel();
+    if (panel)
+    {
+      panel->save();
+    }
+  }
+}
+
+
 static void call_save_file_as(wb::WBContextSQLIDE *sqlide)
 {
   SqlEditorForm *editor = sqlide->get_active_sql_editor();
   if (editor)
   {
-    int i = editor->active_sql_editor_index();
-    if (i >= 0)
+    SqlEditorPanel *panel = editor->active_sql_editor_panel();
+    if (panel)
     {
-      editor->save_sql_script_file("", i);
+      panel->save_as("");
     }
   }
 }
@@ -453,19 +458,19 @@ static void call_revert(wb::WBContextSQLIDE *sqlide)
   SqlEditorForm *editor = sqlide->get_active_sql_editor();
   if (editor)
   {
-    int i = editor->active_sql_editor_index();
-    if (i >= 0)
+    SqlEditorPanel *panel = editor->active_sql_editor_panel();
+    if (panel)
     {
-      if (editor->sql_editor(i)->get_editor_control()->is_dirty())
+      if (panel->is_dirty())
       {
         int rc = mforms::Utilities::show_message(_("Revert to Saved"),
                             base::strfmt(_("Do you want to revert to the most recently saved version of '%s'?\nAny changes since them will be lost."),
-                                         editor->sql_editor_path(i).c_str()),
-                                                _("Revert"), _("Cancel"), "");
+                                         panel->filename().c_str()),
+                                                 _("Revert"), _("Cancel"), "");
         if (rc != mforms::ResultOk)
           return;
         
-        editor->revert_sql_script_file();
+        panel->revert_to_saved();
       }
     }
   }
@@ -476,26 +481,14 @@ static bool validate_revert(wb::WBContextSQLIDE *sqlide)
   SqlEditorForm *editor = sqlide->get_active_sql_editor();
   if (editor)
   {
-    int i = editor->active_sql_editor_index();
-    if (i >= 0 && i < editor->sql_editor_count() && !editor->sql_editor_path(i).empty())
-      return !editor->sql_editor_is_scratch(i);
+    SqlEditorPanel *panel = editor->active_sql_editor_panel();
+    if (panel)
+      return !panel->is_scratch() && !panel->filename().empty();
   }
   return false;
 }
 
 
-static void call_save_snippet(wb::WBContextSQLIDE *sqlide)
-{
-  SqlEditorForm *form = sqlide->get_active_sql_editor();
-  if (form)
-  {
-    std::string text = form->active_sql_editor()->selected_text();
-    if (form->save_snippet(text))
-      sqlide->get_grt_manager()->replace_status_text("SQL saved to snippets list.");
-  }
-}
-
-
 static void call_continue_on_error(wb::WBContextSQLIDE *sqlide)
 {
   SqlEditorForm *form = sqlide->get_active_sql_editor();
@@ -542,15 +535,12 @@ static void call_save_edits(wb::WBContextSQLIDE *sqlide)
   SqlEditorForm *form = sqlide->get_active_sql_editor();
   if (form)
   {
-    int i = form->active_sql_editor_index();
-    if (i >= 0)
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
     {
-      Recordset::Ref rs= form->active_recordset(i);
-      if (rs)
-      {
-        form->do_partial_ui_refresh(SqlEditorForm::SaveRecordsetChanges);
-        rs->apply_changes();
-      }
+      SqlEditorResult *result = panel->active_result_panel();
+      if (result)
+        result->apply_changes();
     }
   }
 }
@@ -560,15 +550,12 @@ static void call_discard_edits(wb::WBContextSQLIDE *sqlide)
   SqlEditorForm *form = sqlide->get_active_sql_editor();
   if (form)
   {
-    int i = form->active_sql_editor_index();
-    if (i >= 0)
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
     {
-      Recordset::Ref rs= form->active_recordset(i);
-      if (rs)
-      {
-        form->do_partial_ui_refresh(SqlEditorForm::DiscardRecordsetChanges);
-        rs->rollback();
-      }
+      SqlEditorResult *result = panel->active_result_panel();
+      if (result)
+        result->discard_changes();
     }
   }
 }
@@ -578,13 +565,17 @@ static void call_discard_edits(wb::WBContextSQLIDE *sqlide)
 static bool validate_save_edits(wb::WBContextSQLIDE *sqlide)
 {
   SqlEditorForm *form = sqlide->get_active_sql_editor();
-  if (!form)
-    return false;
-  int i = form->active_sql_editor_index();
-  if (i < 0)
-    return false;
-  Recordset::Ref rs= form->active_recordset(i);
-  return (rs && rs->has_pending_changes());
+  if (form)
+  {
+    SqlEditorPanel *panel = form->active_sql_editor_panel();
+    if (panel)
+    {
+      SqlEditorResult *result = panel->active_result_panel();
+      if (result)
+        return result->has_pending_changes();
+    }
+  }
+  return false;
 }
 
 
@@ -684,7 +675,7 @@ void WBContextSQLIDE::option_changed(grt::internal::OwnedDict*dict, bool, const
 bool WBContextSQLIDE::auto_save_workspaces()
 {
   WBContext *wb= _wbui->get_wb();
-  int interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
+  ssize_t interval= wb->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
   if (interval <= 0 || !_auto_save_active)
     return false;
 
@@ -708,7 +699,7 @@ bool WBContextSQLIDE::auto_save_workspaces()
   if (interval != _auto_save_interval)
   {
     // schedule new interval
-    wb->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), interval);
+    wb->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), (double)interval);
     return false;
   }
   
@@ -796,7 +787,6 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.commit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::commit));
   cmdui->add_builtin_command("query.rollback", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::rollback));
   cmdui->add_builtin_command("query.autocommit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_autocommit));
-  cmdui->add_builtin_command("query.gatherFieldInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_collect_field_info));
   cmdui->add_builtin_command("query.gatherPSInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_collect_ps_statement_events));
 
   cmdui->add_builtin_command("query.new_schema", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_schema"));
@@ -805,15 +795,14 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.new_view", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_view"));
   cmdui->add_builtin_command("query.new_routine", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_routine"));
   cmdui->add_builtin_command("query.new_function", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_function"));
-  
-  cmdui->add_builtin_command("query.savesnippet", boost::bind(call_save_snippet, this));
+
   cmdui->add_builtin_command("query.new_connection", boost::bind(call_new_connection, this));
   
-  cmdui->add_builtin_command("query.newQuery", boost::bind(&WBContextSQLIDE::call_in_editor_bool, this, &SqlEditorForm::new_sql_scratch_area, false));
+  cmdui->add_builtin_command("query.newQuery", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_scratch_area));
   //cmdui->add_builtin_command("query.newFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::new_sql_script_file));
   cmdui->add_builtin_command("query.newFile", boost::bind(new_script_tab, this));
   cmdui->add_builtin_command("query.openFile", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, (void(SqlEditorForm::*)(const std::string&))&SqlEditorForm::open_file, ""));
-  cmdui->add_builtin_command("query.saveFile", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::save_file));
+  cmdui->add_builtin_command("query.saveFile", boost::bind(call_save_file, this));
   cmdui->add_builtin_command("query.saveFileAs", boost::bind(call_save_file_as, this));
   cmdui->add_builtin_command("query.revert", boost::bind(call_revert, this), boost::bind(validate_revert, this));
 
@@ -827,12 +816,11 @@ void WBContextSQLIDE::init()
 
   cmdui->add_builtin_command("query.stopOnError", boost::bind(call_continue_on_error, this));
   
-  cmdui->add_builtin_command("query.explain", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_sql));
   cmdui->add_builtin_command("query.explain_current_statement",
                                                boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_current_statement));
 
-  cmdui->add_builtin_command("query.jump_to_placeholder", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::jump_to_placeholder));
-  cmdui->add_builtin_command("list-members", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::list_members),
+  cmdui->add_builtin_command("query.jump_to_placeholder", boost::bind(&WBContextSQLIDE::call_in_editor_panel, this, &SqlEditorPanel::jump_to_placeholder));
+  cmdui->add_builtin_command("list-members", boost::bind(&WBContextSQLIDE::call_in_editor_panel, this, &SqlEditorPanel::list_members),
                              boost::bind(validate_list_members, this));
 }
 
@@ -910,6 +898,11 @@ static void *connect_editor(SqlEditorForm::Ref editor, boost::shared_ptr<sql::Tu
     log_error("Got an authentication error during connection: %s\n", exc.what());
     return new std::string(exc.what());
   }
+  catch (grt::user_cancelled &)
+  {
+    log_info("User cancelled connection\n");
+    return new std::string(":CANCELLED");
+  }
   catch (std::exception &exc)
   {
     if (tunnel.get())
@@ -967,11 +960,15 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
     {
       grt::BaseListRef args(conn->get_grt(), grt::AnyType);
       args.ginsert(conn);
-      int result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
-      if (result)
+      ssize_t result = *grt::IntegerRef::cast_from(conn->get_grt()->call_module_function("WbAdmin", "handleExpiredPassword", args));
+      if (result != 0)
         return create_connected_editor(conn);
       throw grt::user_cancelled("password reset cancelled by user");
     }
+    else if (tmp == ":CANCELLED")
+    {
+      throw grt::user_cancelled("Cancelled");
+    }
     
     throw std::runtime_error(tmp);
   }
@@ -1005,9 +1002,9 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
   if (!_auto_save_active)
   {
     _auto_save_active= true;
-    int interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
+    ssize_t interval = _wbui->get_wb()->get_root()->options()->options().get_int("workbench:AutoSaveSQLEditorInterval", 60);
     if (interval > 0)
-      _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), interval);
+      _wbui->get_wb()->get_grt_manager()->run_every(boost::bind(&WBContextSQLIDE::auto_save_workspaces, this), (double)interval);
     _auto_save_interval = interval;
 
     if (!_option_change_signal_connected)
@@ -1049,7 +1046,9 @@ void WBContextSQLIDE::open_document(const std::string &path)
 {
   SqlEditorForm *editor= get_active_sql_editor();
   if (editor)
+  {
     editor->open_file(path);
+  }
   else
     mforms::Utilities::show_error(_("Open SQL Script"),
                                   _("Please select a connected SQL Editor tab to open a script file."),
@@ -1129,14 +1128,14 @@ void WBContextSQLIDE::update_plugin_arguments_pool(bec::ArgumentPool &args)
       db_query_QueryEditorRef qeditor(editor->activeQueryEditor());
       if (qeditor.is_valid())
       {
-        db_query_ResultsetRef rset(qeditor->activeResultset());
+        db_query_ResultPanelRef rpanel(qeditor->activeResultPanel());
       
         args.add_entries_for_object("activeSQLEditor", editor);
         args.add_entries_for_object("activeQueryBuffer", qeditor);
         args.add_entries_for_object("activeQueryEditor", qeditor);
         args.add_entries_for_object("", qeditor);
-        if (rset.is_valid())
-          args.add_entries_for_object("activeResultset", rset, "db.query.Resultset");
+        if (rpanel.is_valid() && rpanel->resultset().is_valid())
+          args.add_entries_for_object("activeResultset", rpanel->resultset(), "db.query.Resultset");
       }
       else
         args.add_entries_for_object("activeSQLEditor", editor);
