@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,22 +17,33 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include <string>
+
+#include "base/ui_form.h"
+#include "base/string_utilities.h"
+#include "base/file_utilities.h"
+#include "base/log.h"
+#include "base/notifications.h"
+
+#include "grtpp.h"
+
+#include "grts/structs.h"
+#include "grts/structs.app.h"
+
+#include "grt/editor_base.h"
 
 #include "wb_context_ui.h"
+#include "wb_context.h"
 #include "model/wb_component.h"
 #include "model/wb_component_physical.h"
-
+#include "model/wb_context_model.h"
 #include "model/wb_model_diagram_form.h"
+#include "sqlide/wb_context_sqlide.h"
 
 #include "wb_overview.h"
 #include "model/wb_overview_physical.h"
-#include "workbench/wb_history_tree.h"
-#include "wb_find_dialog.h"
-#include "model/wb_user_datatypes.h"
 #include "model/wb_diagram_options.h"
 
-#include "grt/editor_base.h"
 #include "grt/clipboard.h"
 #include "grtui/gui_plugin_base.h"
 
@@ -40,13 +51,11 @@
 #include "output_view.h"
 #include "webbrowser_view.h"
 
-#include "mforms/utilities.h"
-#include "mforms/app.h"
-#include "mforms/menubar.h"
+#include "mforms/appview.h"
 
-#include "base/string_utilities.h"
-#include "base/log.h"
-#include "base/notifications.h"
+#include "home_screen.h"
+
+#include "wb_command_ui.h"
 
 #include "plugin_install_window.h"
 
@@ -56,25 +65,18 @@ using namespace base;
 
 DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
 
+//--------------------------------------------------------------------------------------------------
+
 WBContextUI::WBContextUI(bool verbose)
-  : _wb(new WBContext(this, verbose)), _usertypes_tree(_wb), _command_ui(_wb)
+  : _wb(new WBContext(this, verbose)), _command_ui(new CommandUI(_wb))
 {
   NodeId n; // workaround a bug causing crash when output tab is shown with GLib-ERROR **: The thread system is not yet initialized.
 
   base::NotificationCenter::get()->add_observer(this, "GNAppClosing");
   
-  _catalog_tree= 0;
-
-  _history_tree= 0;
-
   _shell_window= 0;
   
-  _find_dialog= new FindDialogBE(_wb);
-  _find_dialog->set_activate_slot(boost::bind(&WBContextUI::activate_find_result, this, _1, _2, _3));
-
-  _output_view= new OutputView(_wb);
-  scoped_connect(_output_view->get_be()->signal_show(),boost::bind(&WBContextUI::show_output, this));
-    
+  _output_view = NULL;
   _active_form= 0;
   _active_main_form= 0;
   
@@ -83,8 +85,9 @@ WBContextUI::WBContextUI(bool verbose)
   
   _last_unsaved_changes_state= false;
   _quitting= false;
+  _processing_action_open_connection = false;
   
-  _home_screen= NULL;
+  _home_screen = NULL;
 
   // to notify that the save status of the doc has changed
   //_wb->get_grt()->get_undo_manager()->signal_changed().connect(boost::bind(&WBContextUI::history_changed, this));
@@ -92,8 +95,12 @@ WBContextUI::WBContextUI(bool verbose)
   
   // stuff to do when the active form is switched in the UI (through set_active_form)
   _form_change_signal.connect(boost::bind(&WBContextUI::form_changed, this));
+
+  _output_view = new OutputView(_wb);
+  scoped_connect(_output_view->get_be()->signal_show(),boost::bind(&WBContextUI::show_output, this));
 }
 
+//--------------------------------------------------------------------------------------------------
 
 WBContextUI::~WBContextUI()
 {
@@ -106,36 +113,50 @@ WBContextUI::~WBContextUI()
   _addon_download_window = 0;
   delete _plugin_install_window;
   _plugin_install_window = 0;
-  delete _find_dialog;
-  _find_dialog = 0;
-  delete _history_tree;
-  _history_tree = 0;
   if (_home_screen != NULL)
   {
     _home_screen->release();
     _home_screen = NULL;
   }
+
+  delete _output_view;
+  _output_view = 0;
+
   delete _shell_window;
-  _shell_window = 0;
+  _shell_window = NULL;
 
   delete _wb;
-  _wb = 0;
+  _wb = NULL;
+
+  delete _command_ui;
+  _command_ui = NULL;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void WBContextUI::handle_notification(const std::string &name, void *sender, base::NotificationInfo &info)
 {
   if (name == "GNAppClosing")
-  {
     home_screen_closing();
-  }
 }
 
 //--------------------------------------------------------------------------------------------------
 
 bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
 {
-  _initializing = true;
+  // Log set folders.
+  log_info("Initializing workbench context UI with these values:\n"
+    "\tbase dir: %s\n\tplugin path: %s\n\tstruct path: %s\n\tmodule path: %s\n\t"
+    "library path: %s\n\tuser data dir: %s\n\topen at start: %s\n\topen type: %s\n\trun at startup: %s\n\t"
+    "run type: %s\n\tForce SW rendering: %s\n\tForce OpenGL: %s\n\tquit when done: %s\n",
+    options->basedir.c_str(), options->plugin_search_path.c_str(), options->struct_search_path.c_str(),
+    options->module_search_path.c_str(), options->library_search_path.c_str(),
+    options->user_data_dir.c_str(), options->open_at_startup.c_str(), options->open_at_startup_type.c_str(),
+    options->run_at_startup.c_str(), options->run_language.c_str(),
+    options->force_sw_rendering ? "Yes" : "No",
+    options->force_opengl_rendering ? "Yes" : "No",
+    options->quit_when_done ? "Yes" : "No");
+
   bool flag = false;
   try
   {
@@ -145,25 +166,23 @@ bool WBContextUI::init(WBFrontendCallbacks *callbacks, WBOptions *options)
     //_shell_window = new GRTShellWindow(_wb->get_grt_manager());
     flag= _wb->init_(callbacks, options);
 
-    // has to be called after WBContext is initialized
-    add_backend_builtin_commands();
+    if (!options->testing)
+    {
+      // has to be called after WBContext is initialized
+      add_backend_builtin_commands();
 
-    // look for auto-save files (must be done at startup to not confuse with autosaves created by ourself)
-    WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
-    WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
+      // look for auto-save files (must be done at startup to not confuse with autosaves created by ourself)
+      WBContextModel::detect_auto_save_files(_wb->get_auto_save_dir());
+      WBContextSQLIDE::detect_auto_save_files(_wb->get_auto_save_dir());
+    }
 
-    // Finally show initial home screen.
-    show_home_screen();
-    _initializing = false;
   }
   catch (const std::exception& e)
   {
-    _initializing = false;
     log_error("WBContextUI::init, exception '%s'\n", e.what()); // log_error logs to stderr too in debug mode.
   }
   catch (...)
   {
-    _initializing = false;
     log_error("Some exception has happened. It was caught at WBContextUI::init.\n");
   }
 
@@ -182,8 +201,12 @@ GRTShellWindow* WBContextUI::get_shell_window()
 //--------------------------------------------------------------------------------------------------
 
 void WBContextUI::init_finish(WBOptions *options)
-{  
+{
+  g_assert(_wb->get_root().is_valid());
+  show_home_screen();
   _wb->init_finish_(options);
+
+  NotificationCenter::get()->send("GNAppStarted", NULL);
 }
 
 
@@ -200,22 +223,18 @@ bool WBContextUI::request_quit()
   if (!_wb->get_grt_manager()->in_main_thread())
     g_warning("request_quit() called in worker thread");
 
-  if (_wb->has_unsaved_changes())
   {
-    int answer= mforms::Utilities::show_message(_("Quit Application"),
-      _("Do you want to save pending changes to the document?\n\n"
-      "If you don't save your changes, they will be lost."),
-      _("Save"), _("Cancel"), _("Don't Save"));
-    if (answer == mforms::ResultOk)
-    {
-      if (!_wb->save_changes())
-        return false;
-    }
-    else if (answer == mforms::ResultCancel)
-      return false;
+    NotificationInfo info;
+    info["cancel"] = "0";
     
-    _wb->do_close_document(true);
+    NotificationCenter::get()->send("GNAppShouldClose", NULL, info);
+    
+    if (info["cancel"] != "0")
+      return false;
   }
+ 
+  if (!_wb->can_close_document())
+    return false;
 
   if (_wb->get_sqlide_context() && !_wb->get_sqlide_context()->request_quit())
     return false;
@@ -229,7 +248,8 @@ bool WBContextUI::request_quit()
 
 void WBContextUI::perform_quit()
 {
-  _quitting= true;
+  _quitting = true;
+  _wb->do_close_document(true);
   _wb->quit_application();
 }
 
@@ -249,19 +269,20 @@ void WBContextUI::reset()
 
   get_physical_overview()->send_refresh_children(bec::NodeId());
   
-  _wb->get_component<WBComponentPhysical>()->update_catalog_tree();
-
-  _usertypes_tree.set_catalog(_wb->get_document()->physicalModels()[0]->catalog());
-  _usertypes_tree.set_rdbms(_wb->get_document()->physicalModels()[0]->rdbms());
+  _wb->get_component<WBComponentPhysical>()->update_catalog_tree_model();
 }
 
 
 void WBContextUI::history_changed()
-{  
+{
+  if (!_wb->_file) //check if model is still opened, if not, leave
+    return;
+
   if (_wb->has_unsaved_changes() != _last_unsaved_changes_state)
     _wb->request_refresh(RefreshDocument, "", (NativeHandle)0);
 
-  get_command_ui()->revalidate_edit_menu_items();
+  _wb->get_grt_manager()->run_once_when_idle(boost::bind(&CommandUI::revalidate_edit_menu_items, get_command_ui()));
+
   _last_unsaved_changes_state= _wb->has_unsaved_changes();
 }
 
@@ -281,7 +302,7 @@ void WBContextUI::overview_selection_changed()
   if (get_active_main_form() == get_physical_overview())
   {
     _wb->request_refresh(RefreshSelection, "", (NativeHandle)get_physical_overview()->get_frontend_data());
-    (*_command_ui.signal_validate_edit_menu_items())();
+    get_command_ui()->revalidate_edit_menu_items();
   }
 }
 
@@ -289,27 +310,28 @@ void WBContextUI::overview_selection_changed()
 void WBContextUI::load_app_options(bool update)
 {
   if (!update)
-    _command_ui.load_data();
+    _command_ui->load_data();
 }
 
 
-std::string WBContextUI::get_diagram_path(const model_DiagramRef &diagram)
-{
-  std::string view_id= diagram.id();
-
-  // look in the physicalModels list for this diagram
-  for (size_t c= _wb->get_document()->physicalModels().count(), i= 0; i < c; i++)
-  {
-    workbench_physical_ModelRef model= _wb->get_document()->physicalModels()[i];
-    
-    for (size_t d= model->diagrams().count(), j= 0; j < d; j++)
-    {
-      if (model->diagrams().get(j).id() == view_id)
-        return strfmt("/wb/doc/physicalModels/%i/diagrams/%i", (int) i, (int) j);
-    }
-  }
-  return "";
-}
+// deprecated
+//std::string WBContextUI::get_diagram_path(const model_DiagramRef &diagram)
+//{
+//  std::string view_id= diagram.id();
+//
+//  // look in the physicalModels list for this diagram
+//  for (size_t c= _wb->get_document()->physicalModels().count(), i= 0; i < c; i++)
+//  {
+//    workbench_physical_ModelRef model= _wb->get_document()->physicalModels()[i];
+//    
+//    for (size_t d= model->diagrams().count(), j= 0; j < d; j++)
+//    {
+//      if (model->diagrams().get(j).id() == view_id)
+//        return strfmt("/wb/doc/physicalModels/%i/diagrams/%i", (int) i, (int) j);
+//    }
+//  }
+//  return "";
+//}
 
 
 static void add_script_file(WBContextUI *wbui)
@@ -344,71 +366,50 @@ static void add_note_file(WBContextUI *wbui)
   */
 void WBContextUI::add_backend_builtin_commands()
 {
-  _command_ui.add_builtin_command("overview.home", 
-                                  boost::bind(&WBContextUI::show_home_screen, this));
-  _command_ui.add_builtin_command("show_output_form", 
+  _command_ui->add_builtin_command("show_about",
+                                   boost::bind(&WBContextUI::show_about, this));
+  _command_ui->add_builtin_command("overview.home",
+                                     boost::bind(&WBContextUI::show_home_screen, this));
+  _command_ui->add_builtin_command("show_output_form", 
                                   boost::bind(&WBContextUI::show_output, this));
 
-  _command_ui.add_builtin_command("find_next", 
-    boost::bind(&FindDialogBE::find_next, _find_dialog));
-  _command_ui.add_builtin_command("find_previous", 
-    boost::bind(&FindDialogBE::find_previous, _find_dialog));
-
-  _command_ui.add_builtin_command("add_script_file", 
+  _command_ui->add_builtin_command("add_script_file", 
     boost::bind(add_script_file, this));
-  _command_ui.add_builtin_command("add_note_file", 
+  _command_ui->add_builtin_command("add_note_file", 
     boost::bind(add_note_file, this));
-
+/*
   // handle some url opening stuff ourselves through mforms
-  _command_ui.add_builtin_command("community_blog",
+  _command_ui->add_builtin_command("community_blog",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/", true));
-  _command_ui.add_builtin_command("community_faq",
+  _command_ui->add_builtin_command("community_faq",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=7", true));
-  _command_ui.add_builtin_command("community_learn",
+  _command_ui->add_builtin_command("community_learn",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=10", true));
-  _command_ui.add_builtin_command("community_discuss",
+  _command_ui->add_builtin_command("community_discuss",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=16", true));
-  _command_ui.add_builtin_command("community_contribute",
+  _command_ui->add_builtin_command("community_contribute",
     boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/?page_id=17", true));
-  _command_ui.add_builtin_command("web_mysql_home",
+ */
+  _command_ui->add_builtin_command("web_mysql_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/", true));
-  _command_ui.add_builtin_command("web_home",
+  _command_ui->add_builtin_command("web_home",
     boost::bind(&WBContextUI::show_web_page, this, "http://mysql.com/products/tools/workbench/", true));
-  _command_ui.add_builtin_command("list_bugs",
-    boost::bind(&WBContextUI::show_web_page, this, "http://bugs.mysql.com/saved/WB%20bugs", true));
-  _command_ui.add_builtin_command("plugin_site",
-    boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/addons/", true));
-  _command_ui.add_builtin_command("check_update",
-    boost::bind(&WBContextUI::show_web_page, this, 
-               base::strfmt("http://wb.mysql.com/workbench/version-check.php?version=%i.%i.%i&revision=%i", 
-                            APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER), false));
-  _command_ui.add_builtin_command("help_index", boost::bind(&WBContextUI::show_help_index, this));
-  _command_ui.add_builtin_command("locate_log_file", boost::bind(&WBContextUI::locate_log_file, this));
-  _command_ui.add_builtin_command("show_log_file", boost::bind(&WBContextUI::show_log_file, this));
+  _command_ui->add_builtin_command("list_bugs",
+    boost::bind(&WBContextUI::show_web_page, this, "http://bugs.mysql.com/saved/WBBugs", true));
+ // _command_ui->add_builtin_command("plugin_site",
+ //   boost::bind(&WBContextUI::show_web_page, this, "http://wb.mysql.com/addons/", true));
+//  _command_ui->add_builtin_command("check_update",
+//    boost::bind(&WBContextUI::show_web_page, this, 
+//               base::strfmt("http://wb.mysql.com/workbench/version-check.php?version=%i.%i.%i&revision=%i", 
+//                            APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER, APP_REVISION_NUMBER), false));
+  _command_ui->add_builtin_command("help_index", boost::bind(&WBContextUI::show_help_index, this));
+  _command_ui->add_builtin_command("locate_log_file", boost::bind(&WBContextUI::locate_log_file, this));
+  _command_ui->add_builtin_command("show_log_file", boost::bind(&WBContextUI::show_log_file, this));
 }
 
 #ifndef ___specialforms
 
-//-----------------------------------------------------------------------------------
-// special forms
-
-
-bec::ValueTreeBE *WBContextUI::get_catalog_tree()
-{
-  if (!_catalog_tree)
-  {
-    WBComponentPhysical *compo= _wb->get_component<wb::WBComponentPhysical>();
-    _catalog_tree= compo->get_catalog_tree();
-  }
-  return _catalog_tree;
-}
-
-
-UserDatatypeListBE *WBContextUI::get_usertypes_tree()
-{
-  return &_usertypes_tree;
-}
-
+//--------------------------------------------------------------------------------------------------
 
 PhysicalOverviewBE *WBContextUI::get_physical_overview()
 {
@@ -451,7 +452,7 @@ void WBContextUI::show_help_index()
   GUILock lock(_wb, _("Starting Doc Lib"), _("The MySQL Doc Library is opening currently, "
     "which should be finished in a moment .\n\nPlease stand by..."));
 
-  _wb->execute_plugin("wb.doclib.open");
+  _wb->execute_plugin("wb.docs.open");
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -462,6 +463,7 @@ void WBContextUI::locate_log_file()
     mforms::Utilities::open_url(base::Logger::log_dir());
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void WBContextUI::show_log_file()
 {
@@ -497,32 +499,15 @@ void WBContextUI::activate_figure(const grt::ValueRef &value)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void WBContextUI::form_changed()
 {  
   _wb->request_refresh(RefreshZoom, "", (NativeHandle)0);
   
-  if (_active_form != NULL && _active_form == _home_screen)
-  {
-    refresh_home_models();
-    refresh_home_instances(0);
-    refresh_home_connections();
-  }
-
   bec::UIForm *form = get_active_main_form();
   if (form && form->get_menubar())
-    form->get_menubar()->validate();
-}
-
-HistoryTreeBE *WBContextUI::get_history_tree()
-{
-  if (!_history_tree)
-  {
-    _history_tree= new HistoryTreeBE(_wb, _wb->get_grt()->get_undo_manager());
-    
-    scoped_connect(_wb->get_grt()->get_undo_manager()->signal_changed(),boost::bind(&HistoryTreeBE::tree_changed, _history_tree, -1, bec::NodeId()));
-  }
-  return _history_tree;
+    get_command_ui()->revalidate_menu_bar(form->get_menubar());
 }
 
 
@@ -603,52 +588,6 @@ bec::ValueInspectorBE *WBContextUI::create_inspector_for_selection(std::vector<s
   return 0;
 }
 
-
-static std::list<GrtObjectRef> get_selected_objects_from_form(WBContext *wb, bec::UIForm *form)
-{
-  std::list<GrtObjectRef> objects;
-
-  if (dynamic_cast<ModelDiagramForm*>(form))
-  {
-    grt::ListRef<model_Object> selection(dynamic_cast<ModelDiagramForm*>(form)->get_selection());
-
-    if (selection.is_valid())
-    {
-      for (size_t c= selection.count(), i= 0; i < c; i++)
-      {
-        model_ObjectRef object(selection[i]);
-        WBComponent *comp= wb->get_component_handling(object);
-        GrtObjectRef dbobject;
-
-        if (comp) 
-          dbobject= comp->get_object_for_figure(object);
-
-        if (dbobject.is_valid())
-          objects.push_back(dbobject);
-        else 
-          objects.push_back(object);
-      }
-    }
-  }
-  else if (dynamic_cast<OverviewBE*>(form))
-  {
-    grt::ListRef<GrtObject> selection(dynamic_cast<OverviewBE*>(form)->get_selection());
-
-    if (selection.is_valid())
-    {
-      for (size_t c= selection.count(), i= 0; i < c; i++)
-      {
-        GrtObjectRef object(selection[i]);
-
-        if (object.is_valid())
-          objects.push_back(object);
-      }
-    }
-  }
-  return objects;
-}
-
-
 std::string WBContextUI::get_description_for_selection(grt::ListRef<GrtObject> &activeObjList, std::vector<std::string> &items)
 {
   std::string res;
@@ -810,34 +749,6 @@ std::string WBContextUI::get_active_diagram_info()
   return "";
 }
 
-
-void WBContextUI::activate_find_result(const GrtObjectRef &location, const GrtObjectRef &object, const std::string &text)
-{
-  _wb->show_status_text(strfmt("Match: %s (%s)", 
-    text.c_str(), object.get_metaclass()->get_attribute("caption").c_str()));
-
-  if (location.is_valid())
-  {
-    model_DiagramRef view;
-
-    if (location.is_instance<model_Diagram>())
-      view= model_DiagramRef::cast_from(location);
-    else
-      view= _wb->get_parent_for_object<model_Diagram>(location);
-
-    if (view.is_valid())
-    {
-      ModelDiagramForm *form= get_wb()->get_model_context()->get_diagram_form_for_diagram_id(view.id());
-
-      if (form)
-      {
-        _wb->switched_view(form->get_view());
-        form->focus_and_make_visible(model_FigureRef::cast_from(location), true);
-      }
-    }
-  }
-}
-
 #endif // ___specialforms
 
 
@@ -862,13 +773,15 @@ void WBContextUI::set_doc_properties(const std::string &caption, const std::stri
 {
   app_DocumentInfoRef info= _wb->get_document()->info();
 
+  grt::AutoUndo undo(_wb->get_grt());
   info->caption(caption);
   info->version(version);
   info->author(author);
   info->project(project);
   info->dateCreated(date_created);
   info->dateChanged(date_changed);
-  info->description(description);  
+  info->description(description);
+  undo.end("Change document properties");
 }
 
 
@@ -981,7 +894,7 @@ bool WBContextUI::get_wb_options_value(const std::string &model, const std::stri
   if (!model.empty())
   {
     grt::DictRef model_options(get_model_options(model));
-    int use_global= model_options.get_int("useglobal", 1);
+    bool use_global = model_options.get_int("useglobal", 1) != 0;
 
     if (key == "useglobal")
     {
@@ -1132,7 +1045,10 @@ void WBContextUI::set_active_form(bec::UIForm *form)
     {
       _active_main_form= form;
 
-      NotificationCenter::get()->send("GNMainFormChanged", 0);
+      base::NotificationInfo info;
+      info["form"] = form ? form->form_id() : "";
+      info["context"] = get_active_context(true);
+      NotificationCenter::get()->send("GNMainFormChanged", 0, info);
     }
   }
 
@@ -1165,12 +1081,7 @@ std::string WBContextUI::get_document_name()
   if (_wb->get_filename().empty())
     return "Untitled";
   else
-  {
-    gchar *bn= g_path_get_basename(_wb->get_filename().c_str());
-    std::string title(bn);
-    g_free(bn);
-    return title;
-  }
+    return base::basename(_wb->get_filename());
 }
 
 
@@ -1206,3 +1117,38 @@ bool WBContextUI::start_plugin_install(const std::string &path)
     _plugin_install_window = new PluginInstallWindow(this);
   return _plugin_install_window->install_plugin(path);
 }
+
+//--------------------------------------------------------------------------------------------------
+
+static struct RegisterWBContextUINotifDocs
+{
+  RegisterWBContextUINotifDocs()
+  {
+    base::NotificationCenter::get()->register_notification("GNAppStarted",
+                                                           "application",
+                                                           "Sent when Workbench starts up and finishes with various initialization routines.",
+                                                           "",
+                                                           "");
+    
+    base::NotificationCenter::get()->register_notification("GNAppShouldClose",
+                                                           "application",
+                                                           "Sent when the user requests Workbench to close. Close can be cancelled by setting the 'cancel' field in the info dict to 1.",
+                                                           "",
+                                                           "cancel - set to 1 if exit should be cancelled");
+
+    base::NotificationCenter::get()->register_notification("GNMainFormChanged",
+                                                           "application",
+                                                           "Sent when the main tab from the application is switched.",
+                                                           "",
+                                                           "form - the ID of the newly active form\n"
+                                                           "context - the context name of the newly active form");
+
+    base::NotificationCenter::get()->register_notification("GNApplicationActivated",
+                                                           "application",
+                                                           "Sent when the application was activated.",
+                                                           "",
+                                                           "");
+  }
+} initdocs;
+
+
