@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <algorithm>
 
 #include <zip.h>
@@ -28,19 +26,22 @@
 #include <set>
 #include <stdexcept>
 #include <errno.h>
-#include <glib/gstdio.h>
 
 #include "grtpp.h"
+
+#include "base/log.h"
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
 #include "base/file_functions.h"
+#include "base/util_functions.h"
+
 #include "mforms/utilities.h"
 
 #include "grt/common.h"
 #include "grt/grt_manager.h"
 
 #include "grts/structs.workbench.h"
-
+#include <glib/gstdio.h>
 
 #define DOCUMENT_FORMAT "MySQL Workbench Model"
 // version history:
@@ -63,7 +64,7 @@
 #define DB_DIR "@db"
 #define DB_FILE "data.db"
 
-#define ZIP_FILE_COMMENT DOCUMENT_FORMAT" archive "ZIP_FILE_FORMAT
+#define ZIP_FILE_COMMENT DOCUMENT_FORMAT " archive " ZIP_FILE_FORMAT
 
 
 
@@ -76,23 +77,12 @@
  * and if so, the recovery function will kick in, using the autosave XML file.
  */
 
+DEFAULT_LOG_DOMAIN("model")
 
 using namespace bec;
 using namespace wb;
 using namespace base;
 
-/*
-static std::string fmt_ziperror(zip *z, const char *msg)
-{
-  int ziperr, syserr;
-  char buffer[1000];
-
-  zip_error_get(z, &ziperr, &syserr);
-  zip_error_to_str(buffer, sizeof(buffer), ziperr, syserr);
-
-  return strfmt("%s: %s", msg, buffer);
-}*/
-
 const std::string ModelFile::lock_filename("lock");
 
 void ModelFile::copy_file(const std::string &srcfile, const std::string &destfile)
@@ -219,16 +209,6 @@ void ModelFile::copy_file_to(const std::string &file, const std::string &dest)
   copy_file(get_path_for(file), dest);
 }
 
-time_t get_file_timestamp(const std::string &file)
-{
-  struct stat stbuf;
-
-  if (g_stat(file.c_str(), &stbuf) < 0)
-    return 0;
-  
-  return stbuf.st_mtime;
-}
-
 void ModelFile::open(const std::string &path, GRTManager *grtm)
 {
   bool file_is_zip;
@@ -324,10 +304,12 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
       return;
     }
 
-    time_t file_ts = get_file_timestamp(path);
-    time_t autosave_ts = get_file_timestamp(bec::make_path(auto_save_dir, MAIN_DOCUMENT_NAME));
+    time_t file_ts;
+    base::file_mtime(path, file_ts);
+    time_t autosave_ts;
+    base::file_mtime(bec::make_path(auto_save_dir, MAIN_DOCUMENT_NAME), autosave_ts);
     if (autosave_ts == 0)
-      autosave_ts = get_file_timestamp(auto_save_dir);
+      base::file_mtime(auto_save_dir, autosave_ts);
     
     // document dir already exists, ask if it should be recovered or deleted
     g_warning("Temporary document folder found: %s (ts=%lu, saved=%lu)", auto_save_dir.c_str(),
@@ -337,10 +319,10 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
                                         base::strfmt(_("The document %s was not properly closed in a previous session on %s.\n"
                                                        "The file you're about to open was last saved %s.\n"
                                                        "Would you like to use the recovered model? Continuing without recovering will remove the auto-saved data."),
-                                                     path.c_str(), bec::fmttime(autosave_ts, DATETIME_FMT).c_str(), bec::fmttime(file_ts, DATETIME_FMT).c_str()),
+                                                     path.c_str(), base::fmttime(autosave_ts, DATETIME_FMT).c_str(), base::fmttime(file_ts, DATETIME_FMT).c_str()),
                                         _("Recover"), _("Continue"), "") == mforms::ResultOk)
     {
-      g_message("Recovering %s...", path.c_str());
+      log_info("Recovering %s...", path.c_str());
       recover= true;
       _content_dir = auto_save_dir;
 
@@ -372,7 +354,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
     }
     else // Cancel recovery
     {
-      g_message("Cleaning up leftover auto-save directory %s", auto_save_dir.c_str());
+      log_info("Cleaning up leftover auto-save directory %s", auto_save_dir.c_str());
       rmdir_recursively(auto_save_dir.c_str());
     }
   }
@@ -549,7 +531,11 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
     throw std::runtime_error(strfmt(_("Cannot open document file: %s"), msg.c_str()));
   }
 
-  int count= zip_get_num_files(z);
+#ifdef zip_int64_t
+  zip_int64_t count = zip_get_num_entries(z, 0);
+#else
+  int count = zip_get_num_files(z);
+#endif
   for (int i= 0; i < count; i++)
   {
     zip_file *file= zip_fopen_index(z, i, 0);
@@ -603,10 +589,10 @@ std::list<std::string> ModelFile::unpack_zip(const std::string &zipfile, const s
     unpacked_files.push_back(outpath);
 
     char buffer[4098];
-    int c;
-    while ((c= zip_fread(file, buffer, sizeof(buffer))) > 0)
+    ssize_t c;
+    while ((c = (size_t)zip_fread(file, buffer, sizeof(buffer))) > 0)
     {
-      if (fwrite(buffer, 1, c, outfile) < (size_t)c)
+      if ((ssize_t)fwrite(buffer, 1, c, outfile) < c)
       {
         int err= ferror(outfile);
         fclose(outfile);
@@ -665,7 +651,6 @@ static void zip_dir_contents(zip *z, const std::string &destdir, const std::stri
         {
           try
           {
-            //not needed zip_add_dir(z, partial.c_str());
             zip_dir_contents(z, destdir.empty() ? entry : destdir+G_DIR_SEPARATOR+entry, tmp);
           }
           catch (...)
@@ -680,12 +665,21 @@ static void zip_dir_contents(zip *z, const std::string &destdir, const std::stri
         if (!add_directories)
         {
           zip_source *src= zip_source_file(z, tmp.c_str(), 0, 0);
+#ifdef zip_file_add
+          if (!src || zip_file_add(z, tmp.c_str(), src, ZIP_FL_OVERWRITE | ZIP_FL_ENC_UTF_8) < 0)
+          {
+            zip_source_free(src);
+            g_dir_close(dir);
+            throw std::runtime_error(zip_strerror(z));
+          }
+#else
           if (!src || zip_add(z, tmp.c_str(), src) < 0)
           {
             zip_source_free(src);
             g_dir_close(dir);
             throw std::runtime_error(zip_strerror(z));
           }
+#endif
         }
       }
     }
@@ -732,7 +726,11 @@ void ModelFile::pack_zip(const std::string &zipfile, const std::string &destdir,
     zip_comment += comment;
   }
 
+#if defined(zip_uint16_t) || defined(_WIN32)
+  zip_set_archive_comment(z, zip_comment.c_str(), (zip_uint16_t)zip_comment.size());
+#else
   zip_set_archive_comment(z, zip_comment.c_str(), zip_comment.size());
+#endif
 
   try
   {
@@ -1021,7 +1019,7 @@ void ModelFile::set_file_contents(const std::string &path, const char *data, siz
   std::string fpath= get_path_for(path);
 
   GError *error= NULL;
-  g_file_set_contents(fpath.c_str(), data, size, &error);
+  g_file_set_contents(fpath.c_str(), data, (gssize)size, &error);
   if (error != NULL)
     throw std::runtime_error(std::string("Error while setting file contents: ") + error->message);
 }
