@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,11 +17,9 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
 #include "grt/common.h"
 #include "grt/grt_manager.h"
 #include "wb_model_file.h"
-#include <libxml/xpath.h>
 #include "upgrade_helper.h"
 #include "grtdb/db_object_helpers.h"
 #include "grts/structs.db.mysql.h"
@@ -51,7 +49,7 @@ static void fixup_layer_tree(XMLTraverser &xml, xmlNodePtr layerNode, double xof
 
   //if (!sublayers) return;
 
-  for (size_t i= 0; (sublayer = xml.get_object_child_by_index(sublayer_list, i))!=0; i++)
+  for (size_t i = 0; (sublayer = xml.get_object_child_by_index(sublayer_list, (int)i)) != 0; i++)
   {
     double top= xml.get_object_double_value(sublayer, "top");
     double left= xml.get_object_double_value(sublayer, "left");
@@ -236,7 +234,7 @@ bool ModelFile::attempt_xml_document_upgrade(xmlDocPtr xmldoc, const std::string
 
           node= xml.get_object_child(node, "views");
 
-          for (size_t v= 0; (view= xml.get_object_child_by_index(node, v)) != 0; v++)
+          for (size_t v = 0; (view = xml.get_object_child_by_index(node, (int)v)) != 0; v++)
           {
             xmlNodePtr rootLayer= xml.get_object_child(view, "rootLayer");
 
@@ -670,36 +668,14 @@ workbench_DocumentRef ModelFile::attempt_document_upgrade(const workbench_Docume
           ListRef<db_Table> tables= schema->tables();
           for (size_t c= tables.count(), i= 0; i < c; i++)
           {
-              db_TableRef table= tables[i];
+              db_TableRef table = tables[i];
+              grt::ListRef<db_Trigger> triggers = table->triggers();
 
-              std::string triggers_sql;
-
-              triggers_sql.append(strfmt("DELIMITER %s\n\n", non_std_sql_delimiter.c_str())).append(
-                  "USE `").append(table->owner()->name()).append("`").
-                  append(non_std_sql_delimiter.c_str()).append("\n\n");
-
-              grt::ListRef<db_Trigger> triggers= table->triggers();
-              size_t triggers_count= triggers.count();
-              typedef std::map<int, db_TriggerRef> OrderedTriggers;
-              typedef std::list<db_TriggerRef> UnorderedTriggers;
-              OrderedTriggers ordered_triggers;
-              UnorderedTriggers unordered_triggers;
-
-              for (size_t i= 0; i < triggers_count; ++i)
+              for (size_t i = 0; i < triggers.count(); ++i)
               {
-                  db_TriggerRef trigger= triggers.get(i);
-                  int sequenceNumber= trigger->sequenceNumber();
-                  if (ordered_triggers.find(sequenceNumber) == ordered_triggers.end())
-                      ordered_triggers[sequenceNumber]= trigger;
-                  else
-                      unordered_triggers.push_back(trigger);
+                db_TriggerRef trigger = triggers.get(i);
+                sql_parser->parse_trigger(trigger, trigger->sqlDefinition());
               }
-
-              for (OrderedTriggers::iterator i= ordered_triggers.begin(), i_end= ordered_triggers.end(); i != i_end; ++i)
-                  sql_parser->parse_trigger(i->second, i->second->sqlDefinition());
-
-              for (UnorderedTriggers::iterator i= unordered_triggers.begin(), i_end= unordered_triggers.end(); i != i_end; ++i)
-                  sql_parser->parse_trigger(*i, (*i)->sqlDefinition());
           }
 
           ListRef<db_View> views = schema->views();
@@ -810,7 +786,7 @@ static void check_figure_layers(workbench_physical_DiagramRef view, std::list<st
   }
 
   // now go through all figures and add figures that are not in any layer to the root
-  for (int i= view->figures().count()-1; i>=0; --i)
+  for (ssize_t i= view->figures().count() - 1; i >= 0; --i)
   {
     model_FigureRef figure(view->figures()[i]);
 
@@ -1128,12 +1104,9 @@ static void check_schema_objects(db_SchemaRef schema, std::list<std::string> &lo
 void ModelFile::check_and_fix_inconsistencies(xmlDocPtr xmldoc, const std::string &version)
 {
   std::vector<std::string> ver= base::split(version, ".");
-  int major, minor, revision;
-  
-  major= atoi(ver[0].c_str());
-  minor= atoi(ver[1].c_str());
-  revision= atoi(ver[2].c_str());
   
+  int major= atoi(ver[0].c_str());
+    
   XMLTraverser traverser(xmldoc);
   
   if (major == 1)
