@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <sstream>
 
 #include "base/config_file.h"
@@ -40,7 +38,6 @@
 
 #include "user_defined_type_editor.h"
 #include "preferences_form.h"
-#include "bug_report_form.h"
 #include "document_properties_form.h"
 #include "grt_shell_window.h"
 #include "plugin_manager_window.h"
@@ -54,7 +51,6 @@
 using namespace wb;
 using namespace grt;
 using namespace base;
-using namespace base;
 
 DEFAULT_LOG_DOMAIN(DOMAIN_WB_MODULE)
 
@@ -92,29 +88,34 @@ std::string WorkbenchImpl::getSystemInfo(bool indent)
 {
   #ifdef _WIN32
     #define PLATFORM_NAME "Windows"
+      #ifdef _WIN64
+        #define ARCHITECTURE "64 bit"
+      #else
+        #define ARCHITECTURE "32 bit"
+      #endif
   #elif defined(__APPLE__)
     #define PLATFORM_NAME "Mac OS X"
+    #define ARCHITECTURE "32 bit"
   #else
     #define PLATFORM_NAME "Linux/Unix"
+    #define ARCHITECTURE "64 bit"
   #endif
 
 
   app_InfoRef info(app_InfoRef::cast_from(_wb->get_grt()->get("/wb/info")));
 
   const char* tab = indent ? "\t" : "";
-  std::string result = strfmt("%s%s %s (%s) for "PLATFORM_NAME" version %i.%i.%i %s revision %i build %i\n",
+  std::string result = strfmt("%s%s %s (%s) for "PLATFORM_NAME" version %i.%i.%i %s revision %i build %i (%s)\n",
     tab, info->name().c_str(), APP_EDITION_NAME, APP_LICENSE_TYPE, APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER,
-                              APP_RELEASE_TYPE, APP_REVISION_NUMBER, APP_BUILD_NUMBER);
+                              APP_RELEASE_TYPE, APP_REVISION_NUMBER, APP_BUILD_NUMBER, ARCHITECTURE);
   result += strfmt("%sConfiguration Directory: %s\n", tab, _wb->get_grt_manager()->get_user_datadir().c_str());
   result += strfmt("%sData Directory: %s\n", tab, _wb->get_grt_manager()->get_basedir().c_str());
 
   int cver= cairo_version();
   result += strfmt("%sCairo Version: %i.%i.%i\n", tab, (cver / 10000) % 100, (cver / 100) % 100, cver % 100);
 
-  gchar* tmp;
   result += strfmt("%sOS: %s\n", tab, get_local_os_name().c_str());
-  result += strfmt("%sCPU: %s\n", tab, tmp = get_local_hardware_info());
-  g_free(tmp);
+  result += strfmt("%sCPU: %s\n", tab, get_local_hardware_info().c_str());
 
 #ifdef _WIN32
   result += getFullVideoAdapterInfo(indent);
@@ -125,10 +126,10 @@ std::string WorkbenchImpl::getSystemInfo(bool indent)
     TCHAR* buffer = new TCHAR[locale_buffer_size];
     GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLANGUAGE, buffer, locale_buffer_size);
 
-    CW2A converted_buffer(buffer, CP_UTF8);
+    std::string converted = base::wstring_to_string(buffer);
     delete [] buffer;
 
-    result += strfmt("%sCurrent user language: %s\n", tab, converted_buffer.m_psz);
+    result += strfmt("%sCurrent user language: %s\n", tab, converted.c_str());
   }
   else
   {
@@ -157,6 +158,43 @@ std::string WorkbenchImpl::getSystemInfo(bool indent)
   return result;
 }
 
+
+std::map<std::string, std::string> WorkbenchImpl::getSystemInfoMap()
+{
+  std::map<std::string, std::string> result;
+  int cver= cairo_version();
+  
+  result["edition"] = APP_EDITION_NAME;
+  result["license"] = APP_LICENSE_TYPE;
+  result["version"] = strfmt("%u.%u.%u", APP_MAJOR_NUMBER, APP_MINOR_NUMBER, APP_RELEASE_NUMBER);
+  result["configuration directory"] = _wb->get_grt_manager()->get_user_datadir();
+  result["data directory"] = _wb->get_grt_manager()->get_basedir();
+  result["cairo version"] = strfmt("%u.%u.%u", (cver / 10000) % 100, (cver / 100) % 100, cver % 100);
+  result["os"] = get_local_os_name();
+  result["cpu"] = get_local_hardware_info();
+  
+#ifdef _WIN32
+  result["platform"] = "Windows";
+#elif defined(__APPLE__)
+  result["platform"] = "Mac OS X";
+#else
+  result["platform"] = "Linux/Unix";
+  {
+    int rc;
+    char *stdo;
+    if (g_spawn_command_line_sync("lsb_release -d", &stdo, NULL, &rc, NULL) && stdo)
+    {
+      char *d = strchr(stdo, ':');
+      if (d)
+        result["distribution"] = base::trim(g_strchug(d+1));
+      g_free(stdo);
+    }
+  }
+#endif  
+
+  return result;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 #define def_plugin(group, aName, type, aCaption, descr)\
@@ -367,7 +405,7 @@ ListRef<app_Plugin> WorkbenchImpl::getPluginInfo()
 
   def_model_plugin("model", "newDiagram", "Add New Diagram", "Add a New Diagram to the Model");
 
-  def_file_plugin("tools", "runScriptFile", NORMAL_PLUGIN_TYPE, "Run Script File", "Select a Script File to Execute", "Open Script and Execute", "open", "py,lua");
+  def_file_plugin("tools", "runScriptFile", STANDALONE_GUI_PLUGIN_TYPE, "Run Script File", "Select a Script File to Execute", "Open Script and Execute", "open", "py,lua");
   def_file_plugin("tools", "installModuleFile", STANDALONE_GUI_PLUGIN_TYPE, "Install Plugin", "Select a Module or Plugin File to Install", "Select Module to Install", "open", "py,lua,mwbplugin,mwbpluginz");
 
   def_form_model_plugin("form", "showUserTypeEditor", "User Types Editor", "Open User Defined Types Editor");
@@ -430,7 +468,7 @@ int WorkbenchImpl::newDocumentFromDB()
   args.ginsert(_wb->get_document()->physicalModels()[0]->catalog());
   grt::IntegerRef resultRef = grt::IntegerRef::cast_from(module->call_function("runDbImportWizard", args));
 
-  return *resultRef;
+  return (int)*resultRef;
 }
 
 
@@ -647,6 +685,8 @@ int WorkbenchImpl::editSelectedFigure(const model_DiagramRef &view)
 int WorkbenchImpl::editSelectedFigureInNewWindow(const model_DiagramRef &view)
 {
   ModelDiagramForm *form= dynamic_cast<ModelDiagramForm*>(_wb->get_active_form());
+  if (!form)
+    form = dynamic_cast<ModelDiagramForm*>(_wb->get_active_main_form());
   if (form)
   {
     ListRef<model_Object> list(form->get_selection());
@@ -789,7 +829,7 @@ int WorkbenchImpl::toggleGridAlign(const model_DiagramRef &view)
   {
     form->get_view()->set_grid_snapping(!form->get_view()->get_grid_snapping());
 
-    view->options().gset("AlignToGrid", form->get_view()->get_grid_snapping()?1:0);
+    _wb->get_grt_manager()->set_app_option("AlignToGrid", grt::IntegerRef(form->get_view()->get_grid_snapping()?1:0));
   }
   return 0;
 }
@@ -1354,30 +1394,32 @@ int WorkbenchImpl::showOptions()
 
 int WorkbenchImpl::reportBug(const std::string error_info)
 {
-  grt::Module *module;
+  unsigned short os_id = 1;
+  std::map<std::string, std::string> sys_info = getSystemInfoMap();
 
-  module = _wb->get_ui()->get_wb()->get_grt()->get_module("WbBugReport");
-
-  if (module == NULL)
-    throw std::logic_error("Internal error: can't find Workbench Bug Report module.");
-
-  // Calls the plugin
-  grt::ValueRef resultRef = module->call_function("test_connection",grt::BaseListRef());
-
-  // Gets the information returned by the plugin
-  std::string result = resultRef.repr();
-
-  std::vector<std::string> result_info = base::split(result, "|");
-
-  if (result_info[0] != "success")
-    mforms::Utilities::show_error("Connection Failed", result_info[1], "OK");
-  else
+  std::string os_details = sys_info["os"];
+  
+  if (sys_info["platform"] == "Linux/Unix")
   {
-    BugReportForm report(_wb->get_ui(), getSystemInfo(false), error_info);
-    report.show();
+    os_id = 5;
+    os_details = sys_info["distribution"];
   }
+  else if (sys_info["platform"] == "Mac OS X") os_id = 6;
+  else if (sys_info["platform"] == "Windows")  os_id = 7;   
 
-  return 0;
+  std::ostringstream stream;
+  stream << "http://bugs.mysql.com/report.php" 
+         << "?" << "in[status]="         << "Open"
+         << "&" << "in[php_version]="    << sys_info["version"]
+         << "&" << "in[os]="             << os_id
+         << "&" << "in[os_details]="     << os_details
+         << "&" << "in[tags]="           << "WBBugReporter"
+         << "&" << "in[really]="         << "0"
+         << "&" << "in[ldesc]="          << "----"
+         << "[For better reports, please attach the log file after submitting. You can find it in " << base::Logger::log_filename() << "]";
+         
+  mforms::Utilities::open_url(stream.str());
+  return 0;  
 }
 
 int WorkbenchImpl::showConnectionManager()
@@ -1689,8 +1731,8 @@ static const char *DEFAULT_RDBMS_ID= "com.mysql.rdbms.mysql";
  */
 db_mgmt_ConnectionRef WorkbenchImpl::create_connection(const std::string& host, const std::string& user,
                                                        const std::string socket_or_pipe_name,
-                                                       bool can_use_networking,
-                                                       bool can_use_socket_or_pipe,
+                                                       int can_use_networking,
+                                                       int can_use_socket_or_pipe,
                                                        int port,
                                                        const std::string& name)
 {
@@ -1724,8 +1766,8 @@ db_mgmt_ConnectionRef WorkbenchImpl::create_connection(const std::string& host,
   connection->driver(driver);
   connection->name(name);
 
-  connection->parameterValues().gset("hostName", "localhost");
-  connection->parameterValues().gset("userName", "root");
+  connection->parameterValues().gset("hostName", host);
+  connection->parameterValues().gset("userName", user);
   connection->parameterValues().gset("port", port);
 
   connection->hostIdentifier(bec::get_host_identifier_for_connection(connection));
@@ -1746,23 +1788,69 @@ grt::DictListRef WorkbenchImpl::getLocalServerList()
   log_debug("Reading locally installed MySQL servers\n");
 
   grt::DictListRef entries;
+#ifdef _WIN32
   try
   {
-#ifdef _WIN32
     int session = wmiOpenSession("", "", "");
     entries = wmiQuery(session,
       "select * from Win32_Service where (Name like \"%mysql%\" or DisplayName like \"%mysql%\")");
     wmiCloseSession(session);
-#else
-    // TODO: other OSes go here.
-#endif
+  }
+  catch (std::exception const& e)
+  {
+    // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+    log_error("Unable to locate installed MySQL Servers : %s.\n", e.what());
   }
   catch (...)
   {
-    // If for some reason (e.g. insufficient rights) the retrieval fails then we return an empty list.
+    log_error("Unable to locate installed MySQL Servers.\n");
   }
+  
+  
+#else
+    entries = grt::DictListRef(_wb->get_grt());
+  
+    char *stdo = NULL;
+    char *ster = NULL;
+    int rc=0;
+    GError *err = NULL;
+    if (g_spawn_command_line_sync("/bin/bash -c \"ps -ec | grep \\\"mysqld\\b\\\" | awk '{ print $1 }' | xargs ps -fp | awk 'NR > 1 {for(i=1;i<=7;i++)$i=\\\"\\\"; print $0}'\"", &stdo, &ster, &rc, &err) && stdo)
+    {
+      std::string processes(stdo);
+      
+      std::vector<std::string> servers = base::split(processes, "\n");
+      std::vector<std::string>::iterator index, end = servers.end();
 
-  log_debug("Found %ld installed MySQL servers\n", entries.is_valid() ? entries.count() : -1);
+      for(index = servers.begin(); index !=end; index++)
+      {
+        DictRef server(_wb->get_grt());
+        std::string command = *index;
+        if (command.length())
+        {
+          server.set("PathName", StringRef(command));
+          entries.insert(server);
+        }
+      }
+    }
+  
+    if (stdo)
+      g_free(stdo);
+  
+    if (err)
+    {
+      log_error("Error looking for installed servers, error %d : %s\n", err->code, err->message);
+      g_error_free(err);
+    }
+    
+    if (ster)
+    {
+      log_error("stderr from process list %s\n", ster);
+      g_free(ster);
+    }
+    
+#endif
+
+  log_debug("Found %li installed MySQL servers\n", entries.is_valid() ? (long)entries.count() : -1);
 
   return entries;
 }
@@ -1797,17 +1885,17 @@ int WorkbenchImpl::createConnectionsFromLocalServers()
       if (!key_file.has_section("mysqld"))
         continue;
 
-      bool can_use_networking = true;
-      bool can_use_socket_or_pipe = false;
+      bool can_use_networking = 1;
+      bool can_use_socket_or_pipe = 0;
       std::string socket_or_pipe_name;
 
       int port = key_file.get_int("port", "mysqld");
       if (port == INT_MIN)
         port = 3306;
       if (key_file.has_key("skip-networking", "mysqld"))
-        can_use_networking = false;
+        can_use_networking = 0;
       if (key_file.has_key("enable-named-pipe", "mysqld"))
-        can_use_socket_or_pipe = true;
+        can_use_socket_or_pipe = 1;
 
       if (can_use_socket_or_pipe)
       {
@@ -1844,11 +1932,34 @@ int WorkbenchImpl::createInstancesFromLocalServers()
     {
       grt::DictRef entry(servers[i]);
 
-      std::string service_display_name = entry.get_string("DisplayName", "invalid");
+      std::string display_name = entry.get_string("DisplayName", "invalid");
       std::string service_name = entry.get_string("Name", "invalid");
       std::string path = entry.get_string("PathName", "invalid");
-      std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
 
+
+      // Takes the parameters from the command line
+      std::string config_file = base::extract_option_from_command_line("--defaults-file", path);
+      std::string socket_or_pipe_name = base::extract_option_from_command_line("--socket", path);
+      std::string str_port = base::extract_option_from_command_line("--port", path);
+    
+      ssize_t port = INT_MIN;
+      if (str_port.length())
+        port = atoi(str_port.c_str());
+      
+      bool can_use_networking = true;
+      if (path.find("--skip-networking") != std::string::npos)
+        can_use_networking = false;
+      
+      bool can_use_socket_or_pipe = false;
+      if (path.find("--enable-named-pipe") != std::string::npos)
+        can_use_socket_or_pipe = true;
+      
+      // Creates the server instance
+      db_mgmt_ServerInstanceRef instance(_wb->get_grt());
+      
+
+      // If the configuration file is part of the command call
+      // Will take the parameters from there if were not available on the command call
       if (g_file_test(config_file.c_str(), G_FILE_TEST_EXISTS))
       {
         ConfigurationFile key_file(config_file, base::AutoCreateSections | base::AutoCreateKeys);
@@ -1857,99 +1968,114 @@ int WorkbenchImpl::createInstancesFromLocalServers()
         if (!key_file.has_section("mysqld"))
           continue;
 
-        bool can_use_networking = true;
-        bool can_use_socket_or_pipe = false;
-        std::string socket_or_pipe_name;
-
-        int port = key_file.get_int("port", "mysqld");
+        // Gets the port if not already retrieved from the command line
         if (port == INT_MIN)
-          port = 3306;
-        if (key_file.has_key("skip-networking", "mysqld"))
+          port = key_file.get_int("port", "mysqld");
+        
+        if (can_use_networking && key_file.has_key("skip-networking", "mysqld"))
           can_use_networking = false;
-        if (key_file.has_key("enable-named-pipe", "mysqld"))
+        
+        if (!can_use_socket_or_pipe && key_file.has_key("enable-named-pipe", "mysqld"))
           can_use_socket_or_pipe = true;
 
-        if (can_use_socket_or_pipe)
-        {
+        if (socket_or_pipe_name.size() == 0)
           socket_or_pipe_name = key_file.get_value("socket", "mysqld");
-          if (socket_or_pipe_name.size() == 0)
-            socket_or_pipe_name = "MySQL";
-        }
 
-        db_mgmt_ServerInstanceRef instance(_wb->get_grt());
+        // When a valid config path is identified it will be locked on the
+        // connection editor
+        instance->serverInfo().gset("sys.config.path.lock", 1);
+      }
 
-        instance->owner(_wb->get_root()->rdbmsMgmt());
+      // If the port was not found, uses the default port
+      if (port == INT_MIN)
+        port = 3306;
+			
+      // If the socket was not found uses a default value
+      if (socket_or_pipe_name.size() == 0)
+        socket_or_pipe_name = "MySQL";
+      
+      instance->owner(_wb->get_root()->rdbmsMgmt());
 
+      #ifdef _WIN32
         instance->serverInfo().gset("sys.system", "Windows");
         instance->serverInfo().gset("windowsAdmin", 1);
         instance->loginInfo().gset("wmi.userName", ""); // Only used for remote connections.
         instance->loginInfo().gset("wmi.hostName", ""); // Only used for remote connections.
-
-        instance->name("Local " + service_display_name);
-
-        instance->serverInfo().gset("sys.preset", "Custom");
-
-        instance->serverInfo().gset("sys.config.path", config_file);
-        instance->serverInfo().gset("sys.config.section", "mysqld");
         instance->serverInfo().gset("sys.mysqld.service_name", service_name);
+      #elif defined(__APPLE__)
+        instance->serverInfo().gset("sys.system", "MacOS X");
+      #else
+        instance->serverInfo().gset("sys.system", "Linux");
+      #endif
+      instance->serverInfo().gset("setupPending", 1);
+      
+      // If the display name is invalid will create one using the port
+      if (display_name=="invalid")
+        display_name = base::to_string(port);
 
-        // Finally look up a connection or create one we can use for this instance.
-        db_mgmt_ConnectionRef connection;
-        grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
-        for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
-          iterator = connections.begin(); iterator != end; ++iterator)
+      instance->name("Local " + display_name);
+
+      instance->serverInfo().gset("sys.preset", "Custom");
+
+      instance->serverInfo().gset("sys.config.path", config_file);
+      instance->serverInfo().gset("sys.config.section", "mysqld");
+
+      // Finally look up a connection or create one we can use for this instance.
+      db_mgmt_ConnectionRef connection;
+      grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+      for (grt::ListRef<db_mgmt_Connection>::const_iterator end = connections.end(),
+        iterator = connections.begin(); iterator != end; ++iterator)
+      {
+        grt::DictRef parameters((*iterator)->parameterValues());
+
+        // Must be a localhost connection.
+        std::string parameter = parameters.get_string("hostName");
+        if (!parameter.empty() && parameter != "localhost" && parameter != "127.0.0.1")
+          continue;
+
+        // For now we only consider connections with a root user. We might later add support
+        // for any user.
+        parameter = parameters.get_string("userName");
+        if (parameter != "root")
+          continue;
+
+        if (can_use_networking)
         {
-          grt::DictRef parameters((*iterator)->parameterValues());
-
-          // Must be a localhost connection.
-          std::string parameter = parameters.get_string("hostName");
-          if (!parameter.empty() && parameter != "localhost" && parameter != "127.0.0.1")
+          ssize_t other_port = parameters.get_int("port");
+          if (other_port != port)
             continue;
 
-          // For now we only consider connections with a root user. We might later add support
-          // for any user.
-          parameter = parameters.get_string("userName");
-          if (parameter != "root")
+          // Only native tcp/ip network connections for now.
+          if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native")
             continue;
-
-          if (can_use_networking)
+        }
+        else
+          if (can_use_socket_or_pipe)
           {
-            int other_port = parameters.get_int("port");
-            if (other_port != port)
+            parameter = parameters.get_string("socket");
+            if (parameter.size() == 0)
+              parameter = "MySQL"; // Default pipe/socket name.
+
+            if (parameter != socket_or_pipe_name)
               continue;
 
-            // Only native tcp/ip network connections for now.
-            if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native")
+            // Only native socket/pipe connections for now.
+            if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native_socket")
               continue;
           }
-          else
-            if (can_use_socket_or_pipe)
-            {
-              parameter = parameters.get_string("socket");
-              if (parameter.size() == 0)
-                parameter = "MySQL"; // Default pipe/socket name.
 
-              if (parameter != socket_or_pipe_name)
-                continue;
-
-              // Only native socket/pipe connections for now.
-              if ((*iterator)->driver()->id() != "com.mysql.rdbms.mysql.driver.native_socket")
-                continue;
-            }
-
-            // All parameters are ok. This is a connection we can use.
-            connection = *iterator;
-            break;
-        }
-
-        // If we did not find a connection for this instance then create a new one.
-        if (!connection.is_valid())
-          connection = create_connection("localhost", "root", socket_or_pipe_name, can_use_networking,
-          can_use_socket_or_pipe, port, _("Local instance ") + service_display_name);
-
-        instance->connection(connection);
-        instances.insert(instance);
+          // All parameters are ok. This is a connection we can use.
+          connection = *iterator;
+          break;
       }
+
+      // If we did not find a connection for this instance then create a new one.
+      if (!connection.is_valid())
+        connection = create_connection("localhost", "root", socket_or_pipe_name, can_use_networking,
+        can_use_socket_or_pipe, (int)port, _("Local instance ") + display_name);
+
+      instance->connection(connection);
+      instances.insert(instance);
     }
   }
   catch (std::exception &exc)
@@ -2102,14 +2228,95 @@ int WorkbenchImpl::initializeOtherRDBMS()
       {
         (*m)->call_function("initializeDBMSInfo", args);
       }
-      catch (std::exception)
+      catch (std::exception &)
       {
         failed = true;
       }
     }
   }
   if (failed)
-    g_message("Support for one or more RDBMS sources have failed. View log file for more details.");
+    log_warning("Support for one or more RDBMS sources have failed.");
+
+  _wb->load_other_connections();
+  
   return 1;
 }
+
+/**
+* Removes a connection from the stored connections list along with all associated data
+* (including its server instance entry).
+*/
+int WorkbenchImpl::deleteConnection(const db_mgmt_ConnectionRef &connection)
+{
+  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+  grt::ListRef<db_mgmt_ServerInstance> instances = _wb->get_root()->rdbmsMgmt()->storedInstances();
+
+  // Remove all associated server instances.
+  for (ssize_t i = instances.count() - 1; i >= 0; --i)
+  {
+    db_mgmt_ServerInstanceRef instance(instances[i]);
+    if (instance->connection() == connection)
+      instances->remove(i);
+  }
+
+  // Remove password associated with this connection (if stored in keychain/vault). Check first
+  // this service/username combination isn't used anymore by other connections.
+  bool credentials_still_used = false;
+  grt::DictRef parameter_values = connection->parameterValues();
+  std::string host = connection->hostIdentifier();
+  std::string user = parameter_values.get_string("userName");
+  for (grt::ListRef<db_mgmt_Connection>::const_iterator i = connections.begin();
+    i != connections.end(); ++i)
+  {
+    if (*i != connection)
+    {
+      grt::DictRef current_parameters = (*i)->parameterValues();
+      if (host == *(*i)->hostIdentifier() && user == current_parameters.get_string("userName"))
+      {
+        credentials_still_used = true;
+        break;
+      }
+    }
+  }
+  if (!credentials_still_used)
+  {
+    try
+    {
+      mforms::Utilities::forget_password(host, user);
+    }
+    catch (std::exception &exc)
+    {
+      log_warning("Could not clear password: %s\n", exc.what());
+    }    
+  }  
+
+  connections->remove(connection);
+
+  return 0;
+}
+
+int WorkbenchImpl::deleteConnectionGroup(const std::string& group)
+{
+  size_t group_length = group.length();
+
+  std::vector<db_mgmt_ConnectionRef> candidates;
+  grt::ListRef<db_mgmt_Connection> connections(_wb->get_root()->rdbmsMgmt()->storedConns());
+
+  ssize_t index = connections.count() - 1;
+  while (index >= 0)
+  {
+    std::string name = connections[index]->name();
+
+    if (name.compare(0, group_length, group) == 0)
+      candidates.push_back(connections[index]);
+
+    index--;
+  }
+
+  for (std::vector<db_mgmt_ConnectionRef>::const_iterator iterator = candidates.begin();
+    iterator != candidates.end(); ++iterator)
+    deleteConnection(*iterator);
+
+  return 0;
+}
 //--------------------------------------------------------------------------------------------------
