@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include <stdafx.h>
-
 #include "wb_tunnel.h"
 #include "wb_context.h"
 
@@ -34,8 +32,6 @@
 #include <sys/wait.h>
 #endif
 
-#include "python_context.h"
-
 #include <sstream>
 #include "boost/scoped_array.hpp"
 
@@ -52,6 +48,25 @@ public:
   tunnel_auth_error(const std::string &err) : std::runtime_error(err) {}
 };
 
+class tunnel_auth_retry : public std::runtime_error
+{
+public:
+  tunnel_auth_retry(const std::string &err) : std::runtime_error(err) {}
+};
+
+class tunnel_auth_cancelled : public std::runtime_error
+{
+public:
+  tunnel_auth_cancelled(const std::string &err) : std::runtime_error(err) {}
+};
+
+class tunnel_auth_key_error : public std::runtime_error
+{
+public:
+  tunnel_auth_key_error(const std::string &err) : std::runtime_error(err) {}
+};
+
+
 class SSHTunnel : public sql::TunnelConnection
 {
   TunnelManager *_tm;
@@ -123,7 +138,7 @@ void TunnelManager::start()
 {
   std::string progpath = bec::make_path(_wb->get_grt_manager()->get_basedir(), "sshtunnel.py");
 
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   grt::PythonContext *py = grt::PythonContext::get();
   if (py->run_file(progpath, false) < 0)
   {
@@ -136,7 +151,7 @@ void TunnelManager::start()
 
 int TunnelManager::lookup_tunnel(const char *server, const char *username, const char *target)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
 
   // Note: without the (char*) cast gcc will complain about passing a const char* to a char*.
   //       Ideally the function signature should be changed to take a const char*.
@@ -160,7 +175,7 @@ int TunnelManager::lookup_tunnel(const char *server, const char *username, const
 
 void TunnelManager::shutdown()
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   if (_tunnel)
   {
     PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "shutdown", NULL);
@@ -177,7 +192,7 @@ void TunnelManager::shutdown()
 int TunnelManager::open_tunnel(const char *server, const char *username, const char *password, 
                                const char *keyfile, const char *target)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "open_tunnel", (char*) "sssss",
                                       server, username, password, keyfile, target);
   if (!ret)
@@ -215,7 +230,7 @@ int TunnelManager::open_tunnel(const char *server, const char *username, const c
 
 void TunnelManager::wait_tunnel(int port)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
 
   log_debug("Waiting on tunnel to connect...\n");
 
@@ -243,8 +258,34 @@ void TunnelManager::wait_tunnel(int port)
     throw tunnel_auth_error("Authentication error. Please check that your username and password are correct and try again."
             "\nDetails (Original exception message):\n" + str);
 
-  if ( g_str_has_prefix(str.c_str(), "Authentication error") )
+  if (g_str_has_prefix(str.c_str(), "Server key has been stored"))
+  {
+    log_info("TunnelManager.wait_connection server key stored, retrying: %s\n", str.c_str());
+    throw tunnel_auth_retry("Retry due to fingerprint missing, user accept new fingerprint");
+  }
+
+  if (g_str_has_prefix(str.c_str(), "Host key for server "))
+  {
+    log_info("TunnelManager.wait_connection host key does not match, abandoning connection: %s\n", str.c_str());
+    throw tunnel_auth_key_error(str);
+  }
+
+  if (g_str_has_prefix(str.c_str(), "User cancelled"))
+  {
+    log_info("TunnelManager.wait_connection cancelled by the user: %s\n", str.c_str());
+    throw tunnel_auth_cancelled("Tunnel connection cancelled by the user");
+  }
+  if (g_str_has_prefix(str.c_str(), "IO Error"))
+  {
+    log_error("TunnelManager.wait_connection got IOError: %s\n", str.c_str());
+    throw tunnel_auth_key_error(str);
+  }
+
+  if (g_str_has_prefix(str.c_str(), "Authentication error"))
+  {
+    log_info("TunnelManager.wait_connection authentication error: %s\n", str.c_str());
     throw tunnel_auth_error(str);
+  }
 
   throw std::runtime_error("Error connecting SSH tunnel: "+str);
 }
@@ -254,7 +295,7 @@ bool TunnelManager::get_message_for(int port, std::string &type, std::string &me
 {
   std::list<std::pair<std::string, std::string> > messages;
 
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
 
   PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "get_message", (char*) "i", port);
   if (!ret)
@@ -279,12 +320,10 @@ bool TunnelManager::get_message_for(int port, std::string &type, std::string &me
   PyObject *obj = PyTuple_GetItem(ret, 0);
   if (obj && PyString_Check(obj))
     type = PyString_AsString(obj);
-  Py_XDECREF(obj);
 
   obj = PyTuple_GetItem(ret, 1);
   if (obj && PyString_Check(obj))
     message = PyString_AsString(obj);
-  Py_XDECREF(obj);
 
   Py_XDECREF(ret);
 
@@ -294,7 +333,7 @@ bool TunnelManager::get_message_for(int port, std::string &type, std::string &me
 
 void TunnelManager::close_tunnel(int port)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "close", (char*) "i", port);
   if (!ret)
   {
@@ -304,6 +343,18 @@ void TunnelManager::close_tunnel(int port)
   Py_XDECREF(ret);
 }
 
+void TunnelManager::set_keepalive(int port, int keepalive)
+{
+  WillEnterPython lock;
+  PyObject *ret = PyObject_CallMethod(_tunnel, (char*) "set_keepalive", (char*) "ii", port, keepalive);
+  if (!ret)
+  {
+    PyErr_Print();
+    return;
+  }
+  Py_XDECREF(ret);
+}
+
 
 boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_ConnectionRef connectionProperties)
 {
@@ -323,9 +374,9 @@ boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_Co
     std::string password = parameter_values.get_string("sshPassword");
     std::string keyfile = base::expand_tilde(parameter_values.get_string("sshKeyFile"));
     std::string target = parameter_values.get_string("hostName");
-    int target_port = parameter_values.get_int("port", 3306);
+    size_t target_port = parameter_values.get_int("port", 3306);
 
-    target = strfmt("%s:%i", target.c_str(), target_port);
+    target += ":" + base::to_string(target_port);
 
     // before anything, check if a tunnel already exists for this server/user/target tuple
     _wb->get_grt_manager()->replace_status_text("Looking for existing SSH tunnel to "+server+"...");
@@ -349,11 +400,23 @@ boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_Co
       {
         // interactively ask user for password
         service = strfmt("ssh@%s", server.c_str());
-        if (!mforms::Utilities::find_or_ask_for_password(_("Open SSH Tunnel"),
-                                                        service,
-                                                        username,
-                                                        reset_password,
-                                                        password))
+        
+        bool result = false;
+        try
+        {
+          result = mforms::Utilities::credentials_for_service(_("Open SSH Tunnel"),
+                                                              service, username,
+                                                              reset_password, password);
+        }
+        catch(std::exception &exc)
+        {
+          log_warning("Exception caught on credentials_for_service: %s", exc.what());
+          mforms::Utilities::show_error("Clear Password", 
+                                        base::strfmt("Could not clear password: %s", exc.what()),
+                                        "OK");
+        }
+        
+        if (!result)
           // we need to throw an exception to signal that tunnel could not be opened (and not that it was not needed)
           throw grt::user_cancelled("SSH password input cancelled by user");
       }
@@ -394,6 +457,7 @@ boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_Co
         if (tunnel)
         {
           tunnel->connect(connectionProperties);
+          set_keepalive(tunnel_port, _wb->get_grt_manager()->get_app_option_int("sshkeepalive", 0));
           log_info("SSH tunnel connect executed OK\n");
         }
       }
@@ -404,13 +468,37 @@ boost::shared_ptr<sql::TunnelConnection> TunnelManager::create_tunnel(db_mgmt_Co
         if (mforms::Utilities::show_error("Could not connect the SSH Tunnel", exc.what(), _("Retry"), _("Cancel")) == mforms::ResultOk)
         {
           reset_password= true;
-          mforms::Utilities::forget_password(service, username);
+          try
+          {
+            mforms::Utilities::forget_password(service, username);
+          }
+          catch (std::exception &exc)
+          {
+            log_warning("Could not clear password: %s\n", exc.what());
+          }    
+          
           password = "";
           goto retry;
         }
         else
           throw grt::user_cancelled("Tunnel connection cancelled");
       }
+      catch (tunnel_auth_retry &exc)
+      {
+        log_warning("Opening SSH tunnel: %s\n", exc.what());
+        goto retry;
+      }
+      catch (tunnel_auth_cancelled &exc)
+      {
+        log_debug("Tunnel auth cancelled: %s\n", exc.what());
+        throw grt::user_cancelled(exc.what());
+      }
+      catch (tunnel_auth_key_error &exc)
+      {
+        mforms::Utilities::show_error("Tunnel Connection Error", exc.what(), _("OK"));
+        log_debug("Tunnel auth key error: %s\n", exc.what());
+        throw grt::user_cancelled(exc.what());
+      }
       catch (std::exception &exc)
       {
         log_error("Exception while opening SSH tunnel: %s\n", exc.what());
