@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,18 +17,19 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "editor_base.h"
 #include "mforms/code_editor.h"
 
 using namespace bec;
 
+//--------------------------------------------------------------------------------------------------
+
 UndoObjectChangeGroup::UndoObjectChangeGroup(const std::string &object_id, const std::string &member)
   : _object_id(object_id), _member(member)
 {
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool UndoObjectChangeGroup::matches_group(UndoGroup *group) const
 {
@@ -39,29 +40,32 @@ bool UndoObjectChangeGroup::matches_group(UndoGroup *group) const
   return other->_object_id == _object_id && _member == other->_member;
 }
 
-
-//--------------------------------------------------------------------------------
-
+//----------------- BaseEditor ---------------------------------------------------------------------
 
 BaseEditor::BaseEditor(GRTManager *grtm, const grt::Ref<GrtObject> &object)
-: _grtm(grtm), _ignore_object_changes_for_ui_refresh(false), _object(object)
+: _grtm(grtm), _ignore_object_changes_for_ui_refresh(0), _object(object)
 {
+  _ignored_object_fields_for_ui_refresh.insert("oldName");
+
   _ignored_object_changes_for_ui_refresh = 0;
   if (object.is_valid())
     add_listeners(object);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 BaseEditor::~BaseEditor()
 {
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string BaseEditor::get_form_context_name() const
 {
   return "editor"; //WB_CONTEXT_EDITOR;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::add_listeners(const grt::Ref<GrtObject> &object)
 {
@@ -69,6 +73,7 @@ void BaseEditor::add_listeners(const grt::Ref<GrtObject> &object)
   scoped_connect(object->signal_list_changed(),boost::bind(&BaseEditor::on_object_changed, this));
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::object_member_changed(const std::string &member, const grt::ValueRef &ovalue)
 {
@@ -76,16 +81,26 @@ void BaseEditor::object_member_changed(const std::string &member, const grt::Val
     on_object_changed();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::freeze_refresh_on_object_change()
 {
   _ignore_object_changes_for_ui_refresh++;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool BaseEditor::is_refresh_frozen()
+{
+  return _ignore_object_changes_for_ui_refresh > 0;
+}
+
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::thaw_refresh_on_object_change(bool discard_pending)
 {
-  _ignore_object_changes_for_ui_refresh--;
+  if (_ignore_object_changes_for_ui_refresh > 0)
+    _ignore_object_changes_for_ui_refresh--;
   if (_ignore_object_changes_for_ui_refresh == 0)
   {
     if (_ignored_object_changes_for_ui_refresh > 0 && !discard_pending)
@@ -126,6 +141,8 @@ bool BaseEditor::can_close()
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void BaseEditor::on_object_changed()
 {
   if (_ignore_object_changes_for_ui_refresh == 0)
@@ -142,12 +159,14 @@ void BaseEditor::on_object_changed()
     _ignored_object_changes_for_ui_refresh++;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::undo_applied()
 {
   _ui_refresh_conn = _grtm->run_once_when_idle(boost::bind(&RefreshUI::do_ui_refresh, this));
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void BaseEditor::run_from_grt(const boost::function<void()> &slot)
 {
@@ -160,7 +179,7 @@ void BaseEditor::run_from_grt(const boost::function<void()> &slot)
 
 bool BaseEditor::is_editor_dirty()
 {
-  Sql_editor::Ref editor = get_sql_editor();
+  MySQLEditor::Ref editor = get_sql_editor();
   if (editor)
   {
     mforms::CodeEditor* code_editor = editor->get_editor_control();
