@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,13 @@
 #include <grtpp_util.h>
 #include <grtpp_shell.h>
 #include "common.h"
-
+#include "base/threading.h"
 #include "wbpublic_public_interface.h"
 
 #include <boost/shared_ptr.hpp>
 
+
+
 namespace bec {
 
   class WBPUBLICBACKEND_PUBLIC_FUNC GRTDispatcher;
@@ -39,52 +41,43 @@ namespace bec {
 
   class DispatcherCallbackBase
   {
-    GMutex *_mutex;
-    GCond *_cond;
-    int _refcount;
+    base::Mutex _mutex;
+    base::Cond _cond;
+    volatile base::refcount_t _refcount;
 
   public:
     DispatcherCallbackBase()
       : _refcount(1)
-      {
-        _mutex= g_mutex_new();
-        _cond= g_cond_new();
-      }
+      {}
     
     DispatcherCallbackBase *retain()
     {
-      _refcount++;
+      base::atomic_int_inc(&_refcount);
       return this;
     }
     
     void release()
     {
-      _refcount--;
-      if (_refcount <= 0)
+      if (base::atomic_int_dec_and_test_if_zero(&_refcount))
         delete this;
     }
     
     virtual ~DispatcherCallbackBase()
     {
       signal();
-      g_mutex_free(_mutex);
-      _mutex = 0;
-      g_cond_free(_cond);
-      _cond = 0;
     }
     
     virtual void execute()= 0;
     
     void wait()
     {
-      g_mutex_lock(_mutex);
-      g_cond_wait(_cond, _mutex);
-      g_mutex_unlock(_mutex);
+      base::MutexLock lock(_mutex);
+      _cond.wait(_mutex);
     }
     
     void signal()
     {
-      g_cond_signal(_cond);
+      _cond.signal();
     }
   };
   
@@ -193,7 +186,7 @@ namespace bec {
 
   private:
     std::string _name;
-    int _refcount;
+    volatile base::refcount_t _refcount;
     bool _cancelled;
     bool _finished;
     bool _messages_to_main_thread;
@@ -271,7 +264,6 @@ namespace bec {
     grt::ShellCommand _result;
   };
 
-
   //----------------------------------------------------------------------
 
   class WBPUBLICBACKEND_PUBLIC_FUNC GRTDispatcher 
@@ -283,10 +275,11 @@ namespace bec {
     GAsyncQueue *_task_queue;
     FlushAndWaitCallback _flush_main_thread_and_wait;
     
-    bool _busy;
+    volatile base::refcount_t _busy;
+    
     bool _threading_disabled;
-    bool _worker_running;
-    bool _worker_shutdown;
+    base::Semaphore _w_runing;
+    volatile bool _shutdown_callback;
     bool _is_main_dispatcher;
     bool _shut_down;
     
