@@ -17,12 +17,8 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include <grtpp_module_python.h>
 #include <grtpp_module_cpp.h>
-#include <grtpp_module_lua.h>
 
 #include <python_context.h>
 
@@ -30,7 +26,7 @@
 #include "base/threading.h"
 #include "glib/gstdio.h"
 #include "base/log.h"
-#include "objimpl/ui/grt_PyObject_impl.h"
+#include "objimpl/wrapper/grt_PyObject_impl.h"
 
 #include "base/notifications.h"
 #include "base/file_functions.h"
@@ -98,8 +94,6 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   _plugin_manager= _grt->get_native_module<PluginManagerImpl>();
 
   _messages_list= new MessageListStorage(this);
-
-  PythonContext::set_run_once_when_idle(boost::bind(&GRTManager::run_once_when_idle_, this, _1));
 }
 
 bool GRTManager::try_soft_lock_globals_tree()
@@ -370,7 +364,7 @@ void GRTManager::initialize(bool init_python, const std::string &loader_module_p
 
 bool GRTManager::initialize_shell(const std::string &shell_type)
 {
-  if (!_shell->setup(shell_type.empty() ? grt::LanguageLua : shell_type))
+  if (!_shell->setup(shell_type.empty() ? grt::LanguagePython : shell_type))
   {
     g_warning("Could not initialize GRT shell of type '%s'", shell_type.c_str());
     return false;
@@ -779,16 +773,6 @@ void GRTManager::rescan_modules()
 
 bool GRTManager::init_loaders(const std::string &loader_module_path, bool init_python)
 {
-  try
-  {
-    _grt->add_module_loader(new LuaModuleLoader(_grt));
-    if (_verbose) _shell->write_line(_("Lua loader initialized."));
-  }
-  catch (std::exception &exc)
-  {
-    _shell->write_line(strfmt("Error initializing Lua loader: %s", exc.what()));
-  }
-  
   if (init_python)
   {
     try
@@ -898,7 +882,7 @@ long GRTManager::get_app_option_int(const std::string &name, long default_)
 {
   grt::ValueRef value(get_app_option(name));
   if (value.is_valid() && grt::IntegerRef::can_wrap(value))
-    return *grt::IntegerRef::cast_from(value);
+    return (long)*grt::IntegerRef::cast_from(value);
   return default_;
 }
 
