@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,7 @@
 #include <grtpp.h>
 
 #include "base/trackable.h"
+#include "base/threading.h"
 #include "common.h"
 #include "grt_dispatcher.h"
 #include "grt_shell.h"
@@ -34,10 +35,6 @@
 #include "plugin_manager.h"
 #include <boost/signals2/connection.hpp>
 
-#ifdef _MSC_VER
-#pragma make_public(::bec::GRTManager)
-#endif
-
 namespace bec {
 
   class Clipboard;
@@ -157,7 +154,7 @@ namespace bec {
     void set_app_option(const std::string &name, const grt::ValueRef &value);
     
     boost::signals2::connection run_once_when_idle(const boost::function<void ()> &func);
-    void run_once_when_idle(base::trackable *owner, const boost::function<void ()> &func);
+    boost::signals2::connection run_once_when_idle(base::trackable *owner, const boost::function<void ()> &func);
 
     void block_idle_tasks();
     void unblock_idle_tasks();
@@ -202,8 +199,8 @@ namespace bec {
     grt::GRT *_grt;
     bool _has_unsaved_changes;
     GRTDispatcher::Ref _dispatcher;
-    GMutex *_idle_mutex;
-    GMutex *_timer_mutex;
+    base::Mutex _idle_mutex;
+    base::Mutex _timer_mutex;
 
 
   public:
@@ -212,7 +209,7 @@ namespace bec {
   protected:
     typedef std::map<GRTDispatcher::Ref, void*> DispMap;
     DispMap _disp_map;
-    GMutex *_disp_map_mutex;
+    base::Mutex _disp_map_mutex;
 
     PluginManager *_plugin_manager;
 
