@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grt_value_tree.h"
 
 #include <grtpp_util.h>
@@ -39,12 +37,14 @@ using namespace bec;
  * 
  */
 
+//--------------------------------------------------------------------------------------------------
 
 inline bool is_null_node(const NodeId &id)
 {
   return id.depth() == 0;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 static bool count_simple_members(const MetaClass::Member *member, int *c)
 {
@@ -53,6 +53,7 @@ static bool count_simple_members(const MetaClass::Member *member, int *c)
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 static int count_container_nodes(const ValueRef &value)
 {
@@ -101,7 +102,7 @@ static int count_container_nodes(const ValueRef &value)
   return count;
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 ValueTreeBE::ValueTreeBE(GRT *grt)
   : _grt(grt)
@@ -110,18 +111,21 @@ ValueTreeBE::ValueTreeBE(GRT *grt)
   _is_global_path= false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 ValueTreeBE::~ValueTreeBE()
 {
   _root.reset_children();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::show_captions(bool flag)
 {
   _show_captions= true;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::set_displayed_global_value(const std::string &path, bool show_root)
 {
@@ -150,6 +154,7 @@ void ValueTreeBE::set_displayed_global_value(const std::string &path, bool show_
   refresh();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::set_displayed_value(const ValueRef &value, const std::string &name)
 {
@@ -176,12 +181,14 @@ void ValueTreeBE::set_displayed_value(const ValueRef &value, const std::string &
   refresh();
 }
 
+//--------------------------------------------------------------------------------------------------
 
-void ValueTreeBE::set_node_filter(const boost::function<bool (NodeId, std::string, ValueRef, std::string&, IconId&)> &filter)
+void ValueTreeBE::set_node_filter(const boost::function<bool(NodeId, std::string, ValueRef, std::string&, IconId&)> &filter)
 {
   _node_filter= filter;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 NodeId ValueTreeBE::get_root() const
 {
@@ -191,33 +198,38 @@ NodeId ValueTreeBE::get_root() const
   return NodeId(0);
 }
 
+//--------------------------------------------------------------------------------------------------
 
-int ValueTreeBE::count_children(const NodeId &node_id)
+size_t ValueTreeBE::count_children(const NodeId &node_id)
 {
-  Node *n;
-
   if (is_null_node(node_id))
     return 1; // root node
 
-  if (!(n= get_node_for_id(node_id)))
+  Node *node = get_node_for_id(node_id);
+
+  if (node == NULL)
     return 0;
 
-  return n->subnodes.size();
+  return node->subnodes.size();
 }
 
+//--------------------------------------------------------------------------------------------------
 
-Type ValueTreeBE::get_field_type(const NodeId &node_id, int column)
+Type ValueTreeBE::get_field_type(const NodeId &node_id, ColumnId column)
 {
   switch (column)
   {
-  case Name: return StringType;
-  case Type: return IntegerType;
+  case Name:
+    return StringType;
+  case Type:
+    return IntegerType;
   }
   return AnyType;
 }
 
+//--------------------------------------------------------------------------------------------------
 
-NodeId ValueTreeBE::get_child(const NodeId &parent_id, int index)
+NodeId ValueTreeBE::get_child(const NodeId &parent_id, size_t index)
 {
   Node *node;
 
@@ -229,16 +241,17 @@ NodeId ValueTreeBE::get_child(const NodeId &parent_id, int index)
     throw std::out_of_range("Invalid parent node");
   }
 
-  if (index < (int)node->subnodes.size())
+  if (index < node->subnodes.size())
     return NodeId(parent_id).append(index);
 
   throw std::out_of_range("Attempt to access invalid child");
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ValueTreeBE::is_expandable(const NodeId &node_id)
 {
-  Node *node= get_node_for_id(node_id);
+  Node *node = get_node_for_id(node_id);
   if (!node && is_null_node(node_id))
     return true;
   else if (!node)
@@ -247,7 +260,19 @@ bool ValueTreeBE::is_expandable(const NodeId &node_id)
   return node->expandable;
 }
 
-  
+//--------------------------------------------------------------------------------------------------
+
+bool ValueTreeBE::is_expanded(const NodeId &node_id)
+{
+  Node *node = get_node_for_id(node_id);
+  if (node == NULL)
+    return false;
+
+  return node->expanded;
+}
+
+//--------------------------------------------------------------------------------------------------
+
 bool ValueTreeBE::get_row(const NodeId &node_id,
                           std::string &name,
                           std::string &type)
@@ -259,11 +284,11 @@ bool ValueTreeBE::get_row(const NodeId &node_id,
 
   name= info->name;
   type= info->type;
-//  icon= info->icon;
 
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 ValueRef ValueTreeBE::get_node_value(const NodeId &node_id)
 {
@@ -279,8 +304,9 @@ ValueRef ValueTreeBE::get_node_value(const NodeId &node_id)
   return get_value_by_path(_root_value, path);
 }
 
+//--------------------------------------------------------------------------------------------------
 
-bool ValueTreeBE::get_field_grt(const NodeId &node, int column, ValueRef &value)
+bool ValueTreeBE::get_field_grt(const NodeId &node, ColumnId column, ValueRef &value)
 {
   Node *info= get_node_for_id(node);
   
@@ -300,9 +326,9 @@ bool ValueTreeBE::get_field_grt(const NodeId &node, int column, ValueRef &value)
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
-
-IconId ValueTreeBE::get_field_icon(const NodeId &node_id, int column, IconSize size)
+IconId ValueTreeBE::get_field_icon(const NodeId &node_id, ColumnId column, IconSize size)
 {
   if ((ValueTreeColumns)column == Name)
   {
@@ -318,8 +344,9 @@ IconId ValueTreeBE::get_field_icon(const NodeId &node_id, int column, IconSize s
   return 0;
 }
 
+//--------------------------------------------------------------------------------------------------
 
-bool ValueTreeBE::get_field(const NodeId &node_id, int column, std::string &value)
+bool ValueTreeBE::get_field(const NodeId &node_id, ColumnId column, std::string &value)
 {
   Node *info;
   
@@ -338,12 +365,13 @@ bool ValueTreeBE::get_field(const NodeId &node_id, int column, std::string &valu
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::fill_node_info(const ValueRef &value,
                                  Node *info)
 {
-  info->type= type_to_str(value.type());
-  info->expandable= count_container_nodes(value) > 0;
+  info->type = type_to_str(value.type());
+  info->expandable = count_container_nodes(value) > 0;
 
   switch (value.type())
   {
@@ -425,30 +453,27 @@ void ValueTreeBE::fill_node_info(const ValueRef &value,
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
                               const std::string &path,
                               const BaseListRef &value)
 {
-  int c= value.count();
-  char buffer[30];
-
   node->reset_children();
-  for (int i= 0; i < c; i++)
+  for (size_t i= 0; i < value->count(); i++)
   {
     ValueRef v= value.get(i);
     std::string label;
     IconId icon= 0;
 
-    sprintf(buffer, "%i", i);
-
+    std::string local_path = base::to_string(i);
     if (v.is_valid() && !is_simple_type(v.type()) &&
-        (!_node_filter || _node_filter(node_id, buffer, v, label, icon)))
+      (!_node_filter || _node_filter(node_id, local_path, v, label, icon)))
     {
       Node *child= new Node();
 
       fill_node_info(v, child);
-      child->path= buffer;
+      child->path = local_path;
       child->name= label.empty() ? child->path : label;
       child->small_icon= icon != 0 ? icon : child->small_icon;
       child->large_icon= icon != 0 ? icon : child->large_icon;
@@ -465,8 +490,9 @@ void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
-void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node, 
+void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
                               const std::string &path,
                               const DictRef &value)
 {
@@ -501,6 +527,7 @@ void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ValueTreeBE::rescan_member(const MetaClass::Member *mem, const NodeId &node_id, Node *node, const ObjectRef &value)
 {
@@ -525,6 +552,8 @@ bool ValueTreeBE::rescan_member(const MetaClass::Member *mem, const NodeId &node
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
+
 void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
                               const std::string &path,
                               const ObjectRef &value)
@@ -535,6 +564,7 @@ void ValueTreeBE::rescan_node(const NodeId &node_id, Node *node,
   meta->foreach_member(boost::bind(&ValueTreeBE::rescan_member, this, _1, node_id, node, value));
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ValueTreeBE::expand_node(const NodeId &node_id)
 {
@@ -546,6 +576,8 @@ bool ValueTreeBE::expand_node(const NodeId &node_id)
   if (!(node= get_node_for_id(node_id)))
     return false;
 
+  node->expanded = true;
+
   ValueRef value(get_node_value(node_id));
   if (!value.is_valid())
     return false;
@@ -567,6 +599,7 @@ bool ValueTreeBE::expand_node(const NodeId &node_id)
   return true;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::collapse_node(const NodeId &node_id)
 {
@@ -575,15 +608,18 @@ void ValueTreeBE::collapse_node(const NodeId &node_id)
   if (!(node= get_node_for_id(node_id)))
     return;
 
+  node->expanded = false;
   node->reset_children();
 }
 
+//--------------------------------------------------------------------------------------------------
 
-void ValueTreeBE::set_activate_callback(const boost::function<void (ValueRef)> &activate_callback)
+void ValueTreeBE::set_activate_callback(const boost::function<void(ValueRef)> &activate_callback)
 {
   _activate_callback= activate_callback;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ValueTreeBE::activate_node(const NodeId &node_id)
 {
@@ -596,6 +632,7 @@ bool ValueTreeBE::activate_node(const NodeId &node_id)
   return false;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 ValueTreeBE::Node* ValueTreeBE::get_node_for_id(const NodeId &id)
 {
@@ -603,9 +640,9 @@ ValueTreeBE::Node* ValueTreeBE::get_node_for_id(const NodeId &id)
     return 0;
 
   Node *node= &_root;
-  for (int c= get_node_depth(id), i= 1; i < c; i++)
+  for (size_t c = get_node_depth(id), i = 1; i < c; i++)
   {
-    if ((int)node->subnodes.size() <= id[i])
+    if (node->subnodes.size() <= id[i])
       return 0;
 
     node= node->subnodes[id[i]];
@@ -613,6 +650,7 @@ ValueTreeBE::Node* ValueTreeBE::get_node_for_id(const NodeId &id)
   return node;
 }
 
+//--------------------------------------------------------------------------------------------------
 
 std::string ValueTreeBE::get_path_for_node(const NodeId &id, bool full)
 {
@@ -626,9 +664,9 @@ std::string ValueTreeBE::get_path_for_node(const NodeId &id, bool full)
     path= node->path;
   else
     path= "";
-  for (int i= 1; i < get_node_depth(id); i++)
+  for (size_t i= 1; i < get_node_depth(id); i++)
   {
-    if ((int)node->subnodes.size() <= id[i])
+    if (node->subnodes.size() <= id[i])
       return "";
 
     node= node->subnodes[id[i]];
@@ -649,7 +687,7 @@ std::string ValueTreeBE::get_path_for_node(const NodeId &id, bool full)
   return path;
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::get_expanded_nodes(std::vector<NodeId> &expanded, const NodeId &path, const Node *node)
 {
@@ -660,9 +698,9 @@ void ValueTreeBE::get_expanded_nodes(std::vector<NodeId> &expanded, const NodeId
   for (std::vector<Node*>::const_iterator iter= node->subnodes.begin();
        iter != node->subnodes.end(); ++iter, ++i)
   {
-    if ((*iter)->subnodes.size() > 0)
+    if ((*iter)->expanded)
     {
-      id[id.depth()-1]= i;
+      id[id.depth()-1] = i;
       expanded.push_back(path);
     }
   }
@@ -678,6 +716,7 @@ void ValueTreeBE::get_expanded_nodes(std::vector<NodeId> &expanded, const NodeId
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void ValueTreeBE::refresh()
 {
@@ -702,10 +741,14 @@ void ValueTreeBE::refresh()
 
   // collapse root node
   _root.reset_children();
-  _root.expandable= count_container_nodes(_root_value) > 0;
+  _root.expandable = count_container_nodes(_root_value) > 0;
   
   // re-expand nodes
   for (std::vector<NodeId>::const_iterator i= expanded_nodes.begin();
        i != expanded_nodes.end(); ++i)
     expand_node(*i);
+
+  tree_changed();
 }
+
+//--------------------------------------------------------------------------------------------------
