@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,20 +17,20 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include "base/log.h"
+#include "base/string_utilities.h"
 
 #include "plugin_manager.h"
 #include "editor_base.h"
 #include "grt_manager.h"
-#include "base/string_utilities.h"
 
 #include "interfaces/plugin.h"
 
+DEFAULT_LOG_DOMAIN("plugins")
+
 using namespace bec;
 using namespace grt;
 
-
-
 static std::string get_args_hash(const grt::BaseListRef &list)
 {
   std::string hash;
@@ -147,7 +147,7 @@ void PluginManagerImpl::set_plugin_enabled(const app_PluginRef &plugin, bool fla
     disabled_list.insert(plugin->name());
     // remove the plugin from all groups
     grt::ListRef<app_PluginGroup> groups(get_plugin_groups());
-    for (unsigned int c= groups.count(), i= 0; i < c; i++)
+    for (size_t c = groups.count(), i = 0; i < c; i++)
       groups[i]->plugins().remove_value(plugin);
   }
 }
@@ -199,7 +199,11 @@ void PluginManagerImpl::rescan_plugins()
 
   // clear plugin list
   while (plugin_list.count() > 0)
+  {
+    // this should only be done on shutdown, the actual plugin objects are still used after this
+    // plugin_list[0]->reset_references();
     plugin_list.remove(0);
+  }
   
   // clear group contents
   {
@@ -207,7 +211,7 @@ void PluginManagerImpl::rescan_plugins()
     app_PluginGroupRef group;
   
     groups= get_plugin_groups();
-    for (unsigned int c= groups.count(), i= 0; i < c; i++)
+    for (size_t c = groups.count(), i = 0; i < c; i++)
     {
       group= groups[i];
       while (group->plugins().count() > 0)
@@ -287,8 +291,7 @@ void PluginManagerImpl::rescan_plugins()
   }
 }
 
-
-
+//--------------------------------------------------------------------------------------------------
 
 app_PluginGroupRef PluginManagerImpl::get_group(const std::string &group_name)
 {
@@ -297,7 +300,7 @@ app_PluginGroupRef PluginManagerImpl::get_group(const std::string &group_name)
 
   groups= get_plugin_groups();
 
-  for (unsigned int c= groups.count(), i= 0; i < c; i++)
+  for (size_t c = groups.count(), i = 0; i < c; i++)
   {
     if (groups[i]->name() == group_name)
     {
@@ -357,7 +360,7 @@ void PluginManagerImpl::register_plugins(grt::ListRef<app_Plugin> plugins)
 {
   grt::ListRef<app_Plugin> list= get_plugin_list();
   
-  for (unsigned int c= plugins.count(), i= 0; i < c; i++)
+  for (size_t c= plugins.count(), i= 0; i < c; i++)
     list.insert(plugins[i]);
 }
 
@@ -547,7 +550,7 @@ app_PluginRef PluginManagerImpl::select_plugin_for_input(const std::string &grou
 {
   ListRef<app_Plugin> plugins= get_plugin_list(group);
   app_PluginRef best_match;
-  int rating= -1;
+  ssize_t rating= -1;
 
   for (size_t c= plugins.count(), i= 0; i < c; i++)
   {
@@ -1235,8 +1238,8 @@ grt::BaseListRef ArgumentPool::build_argument_list(const app_PluginRef &plugin)
     grt::ValueRef argument= find_match(pdef, searched_key);
     if (!argument.is_valid())
     {
-      g_message("Cannot satisfy plugin input for %s: %s", plugin->name().c_str(), searched_key.c_str());
-      g_message("Missing input: %s", pdef.repr().c_str());
+      log_warning("Cannot satisfy plugin input for %s: %s", plugin->name().c_str(), searched_key.c_str());
+      log_warning("Missing input: %s", pdef.repr().c_str());
       
       throw grt::grt_runtime_error("Cannot execute "+*plugin->name(),
                                    "Plugin requires unavailable argument value.");
