@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,8 +25,6 @@
  * backend classes. 
  */
 #include <algorithm>
-//#include <list>
-//#include <vector>
 
 #include <grtpp.h>
 #include "grt/icon_manager.h"
@@ -42,6 +40,10 @@
 #include <set>
 #include <algorithm>
 
+namespace mforms {
+  class MenuBase;
+}
+
 namespace bec
 {
   /** A tree node index.
@@ -79,17 +81,14 @@ namespace bec
   
       Pool()
           : _pool(4)
-          , _sync(g_mutex_new())
       {}
     
       ~Pool()
       {
         {
-          base::GMutexLock lock(_sync);
+          base::MutexLock lock(_sync);
           std::for_each(_pool.begin(), _pool.end(), Pool::delete_object);
         }
-        
-        g_mutex_free(_sync);
       }
     
       T* get()
@@ -97,7 +96,7 @@ namespace bec
         T* item = 0;
         try
         {
-          base::GMutexLock lock(_sync);
+          base::MutexLock lock(_sync);
           if ( _pool.size() > 0 )
           { 
             item = _pool.back();
@@ -119,13 +118,13 @@ namespace bec
     
       void put(T* item)
       {
-        base::GMutexLock lock(_sync);
+        base::MutexLock lock(_sync);
         _pool.push_back(item);
       }
   
     private:
       std::vector<T*>  _pool;
-      GMutex          *_sync; //! Protect against concurrent access within threads
+      base::Mutex          _sync; //! Protect against concurrent access within threads
   };
 
   /**
@@ -140,7 +139,7 @@ namespace bec
   {
     typedef std::string*                uid;   //!< To map short-living NodeId path to a persistent value
                                                //!< This is needed for Gtk::TreeModel iterators
-    typedef std::vector<int>          Index; 
+    typedef std::vector<size_t>        Index; 
     static Pool<Index>                 *_pool; //!< Pool of allocated std::vectors (Index)
     Index                              *index; //!< Path itself
 
@@ -149,73 +148,12 @@ namespace bec
       return _pool ? _pool : (_pool = new Pool<Index>);
     }
 
-    // NodeId will get index from the pool
-    NodeId()
-        : index(0)
-    {
-      index = pool()->get();
-    }
 
-    NodeId(const NodeId &copy)
-        : index(0)
-    {
-      index = pool()->get();
-      if ( copy.index )
-        *index = *copy.index;
-    }
-
-    NodeId(const int i)
-        : index(0)
-    {
-      index = pool()->get();
-      index->push_back(i);
-    }
-
-    NodeId(const std::string &str)
-        : index(0)
-    {
-      index = pool()->get();
-      try
-      {
-        const char* chr = str.c_str();
-        const int size = (int)str.length();
-        std::string num;
-        num.reserve(size);
-        
-        for ( int i = 0; i < size; i++ )
-        {
-          if ( isdigit(chr[i]) )
-            num.push_back(chr[i]);
-          else if ( '.' == chr[i] || ':' == chr[i] )
-          {
-            if ( !num.empty() )
-            {
-              index->push_back(atoi(num.c_str()));
-              num.clear();
-            }
-          }
-          else
-            throw std::runtime_error("Wrong format of NodeId");
-        }
-        
-        if ( !num.empty() )
-          index->push_back(atoi(num.c_str()));
-      }
-      catch (...)
-      {
-        index->clear();
-        pool()->put(index);
-        index = 0;
-        throw;
-      }
-    }
-
-    ~NodeId() 
-    {
-      index->clear();
-      pool()->put(index);
-      index = 0;
-    }
+    NodeId();
+    NodeId(const NodeId &copy);
+    NodeId(size_t i);
+    NodeId(const std::string &str);
+    ~NodeId();
 
     inline NodeId &operator = (const NodeId &node)
     {
@@ -224,153 +162,45 @@ namespace bec
       return *this;
     }
 
-    bool operator < (const NodeId &r) const
-    {
-      bool ret = true;
-      
-      if (index && r.index)
-      {
-        // Shorter nodeids must go before longer. For example in a list ["0.1", "0.1.1"]
-        // longer nodeid is a subnode of the "0.1", so in case of deletion subnode deleted first
-        // (That's true only when traversing list from the end)
-        if (index->size() < r.index->size())
-          ret = true;
-        else if (index->size() > r.index->size())
-          ret = false;
-        else
-        {
-          // It is assumed that this nodeid is less than @r. Walk index vectors. If current value
-          // from this->index is less than or equal to the corresponding value from r.index the pair is skipped
-          // as it complies with assumption that this node is less than @r.
-          // Once current value becomes greater than @r's the assumption about current node's 
-          // less than @r becomes false, therefore this node is greater than @r.
-          const int size = index->size();
-          for (int i = 0; i < size; ++i)
-          {
-            if ((*index)[i] > (*r.index)[i])
-            {
-              ret = false;
-              break;
-            }
-          }
-        }
-      }
-      
-      return ret;
-    }
+    bool operator < (const NodeId &r) const;
 
     inline bool operator == (const NodeId &node) const
     {
       return equals(node);
     }
 
-    bool equals(const NodeId &node) const
+    bool equals(const NodeId &node) const;
+
+    inline size_t depth() const
     {
-      // TODO: Check if we need to compare content of the index and node.index vectors
-      return index && node.index && *node.index == *index;
+      return index->size();
     }
 
-    int depth() const
-    {
-      return (int)index->size();
-    }
+    size_t& operator[] (size_t i) const;
 
-    inline int& operator[] (unsigned int i) const
-    {
-      if ( i < index->size() )
-        return const_cast<int&>((*index)[i]);
-      throw std::range_error("invalid index");
-    }
-
-    inline int end() const
-    {
-      if ( index->size() > 0 )
-        return (*index)[index->size() - 1];
-      throw std::logic_error("invalid node id. NodeId::back applied to an empty NodeId instance.");
-    }
-    
-    inline int back() const
+    size_t end() const;    
+    inline size_t back() const
     {
       return end();
     }
     
-    // Set leaf to the previous index, e.g. for node with path "1.3.2" it will become "1.3.1".
-    inline bool previous() const
-    {
-      bool ret = false;
-      if ( index->size() > 0 )
-      {
-        --((*index)[index->size() - 1]);
-        ret = true;
-      }
-      return ret;
-    }
-    
-    // Set leaf to the next index, e.g. for node with path "1.3.2" it will become "1.3.3".
-    inline bool next() const
-    {
-      bool ret = false;
-      if ( index->size() > 0 )
-      {
-        ++((*index)[index->size() - 1]);
-        ret = true;
-      }
-      return ret;
-    }
+    bool previous() const;
+    bool next() const;
 
     inline bool is_valid() const
     {
       return index->size() != 0;
     }
     
-    inline NodeId parent() const
-    {
-      if (depth() < 2)
-        return NodeId();
-      NodeId copy(*this);
-      copy.index->pop_back();
-      return copy;
-    }
+    NodeId parent() const;
+    std::string repr(const char separator = '.') const;
 
-    inline std::string repr(const char separator = '.') const
-    {
-      std::string r = "";
-      const int depth = (int)index->size();
-      for (int i= 0; i < depth; i++)
-      {
-        char buf[30];
-        g_snprintf(buf, sizeof(buf), "%i", (*index)[i]);
-        if (i > 0)
-          r= r + separator + buf;
-        else
-          r= buf;
-      }
-      return r;
-    }
-
-    inline NodeId &append(const int i)
-    {
-      if (i < 0)
-        throw std::invalid_argument("negative node index is invalid");
-
-      index->push_back(i);
-
-      return *this;
-    }
-
-    inline NodeId &prepend(const int i)
-    {
-      if (i < 0)
-        throw std::invalid_argument("negative node index is invalid");
-
-      index->insert(index->begin(), i);
-
-      return *this;
-    }
+    NodeId &append(size_t i);
+    NodeId &prepend(size_t i);
   };
 
   //----------------------------------------------------------------------------
-  //----------------------------------------------------------------------------
+
   /**
     \class NodeIds
     \brief Mapper of short-living NodeId to a persistent item
@@ -398,7 +228,7 @@ namespace bec
       void flush();
 
       //! Maps path with type of std::string from NodeId. This function is used for
-      //! convience. See map_node_id(const NodeId&)
+      //! convenience. See map_node_id(const NodeId&)
       NodeId::uid map_node_id(const std::string& path_from_nodeid);
       NodeId::uid map_node_id(const NodeId& nid)
       {
@@ -444,20 +274,21 @@ namespace bec
   }
   
   /** Base list model class.
-   *
-   * @ingroup begrt
    */
   class WBPUBLICBACKEND_PUBLIC_FUNC ListModel : public base::trackable
   {
    private:
-    NodeIds               _nodeid_map;
-    boost::signals2::signal<void (bec::NodeId, int)>    _tree_changed_signal;
+    NodeIds _nodeid_map;
+    boost::signals2::signal<void (bec::NodeId, int)> _tree_changed_signal;
     
    public:
-    virtual ~ListModel() {};
+     typedef size_t ColumnId;
+     typedef size_t RowId;
+     
+     virtual ~ListModel() {};
 
-    virtual int count()= 0;
-    virtual NodeId get_node(int index);
+    virtual size_t count() = 0;
+    virtual NodeId get_node(size_t index);
     virtual bool has_next(const NodeId &node);
     virtual NodeId get_next(const NodeId &node);
 
@@ -483,34 +314,34 @@ namespace bec
     {
       return _nodeid_map.map_node_id(nodeuid);
     }
-//    virtual void set_data(NodeId node, void *data)= 0;
-//    virtual void *get_data(NodeId node)= 0;
+    virtual bool get_field(const NodeId &node, ColumnId column, std::string &value);
+    virtual bool get_field(const NodeId &node, ColumnId column, ssize_t &value);
+    virtual bool get_field(const NodeId &node, ColumnId column, bool &value);
+    virtual bool get_field(const NodeId &node, ColumnId column, double &value);
 
-    virtual bool get_field(const NodeId &node, int column, std::string &value);
-    virtual bool get_field(const NodeId &node, int column, int &value);
-    virtual bool get_field(const NodeId &node, int column, long long &value);
-    virtual bool get_field(const NodeId &node, int column, bool &value);
-    virtual bool get_field(const NodeId &node, int column, double &value);
-
-    virtual bool get_field_repr(const NodeId &node, int column, std::string &value) { return get_field(node, column, value); }
+    virtual bool get_field_repr(const NodeId &node, ColumnId column, std::string &value) { return get_field(node, column, value); }
 
     // representation of the field as a GRT value
-    virtual grt::ValueRef get_grt_value(const NodeId &node, int column);
+    virtual grt::ValueRef get_grt_value(const NodeId &node, ColumnId column);
 
-    virtual std::string get_field_description(const NodeId &node, int column);
-    virtual IconId get_field_icon(const NodeId &node, int column, IconSize size);
+    virtual std::string get_field_description(const NodeId &node, ColumnId column);
+    virtual IconId get_field_icon(const NodeId &node, ColumnId column, IconSize size);
 
-    virtual void refresh()= 0;
+    virtual void refresh() = 0;
     virtual void refresh_node(const NodeId &node) {}
 
     virtual void reset() {} //!
 
-    virtual void reorder(const NodeId &node, int index) { throw std::logic_error("not implemented"); }
+    virtual void reorder(const NodeId &node, size_t index) { throw std::logic_error("not implemented"); }
     void reorder_up(const NodeId &node);
     void reorder_down(const NodeId &node);
 
     virtual bool activate_node(const NodeId &node) { throw std::logic_error("not implemented"); return false; }
 
+    // Parent can be NULL if the root node is meant.
+    virtual void update_menu_items_for_nodes(mforms::MenuBase *parent, const std::vector<NodeId> &nodes) { };
+
+    // Deprecated. Use update_menu_items_for_nodes for new code. MenuItemList and related code will go.
     virtual MenuItemList get_popup_items_for_nodes(const std::vector<NodeId> &nodes) { return MenuItemList(); }
     //! Returns true if item was processed by BE, false - BE is unable to process command and FE should do it
     virtual bool activate_popup_item_for_nodes(const std::string &name, const std::vector<NodeId> &nodes) { throw std::logic_error("not implemented"); }
@@ -519,15 +350,14 @@ namespace bec
     virtual bool delete_node(const NodeId &node) { throw std::logic_error("not implemented"); }
 
     // for editable lists only
-    virtual grt::Type get_field_type(const NodeId &node, int column);
+    virtual grt::Type get_field_type(const NodeId &node, ColumnId column);
 
 
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
-    virtual bool set_field(const NodeId &node, int column, int value);
-    virtual bool set_field(const NodeId &node, int column, long long value);
-    virtual bool set_field(const NodeId &node, int column, double value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, ssize_t value);
+    virtual bool set_field(const NodeId &node, ColumnId column, double value);
 
-    virtual bool set_convert_field(const NodeId &node, int column, const std::string &value);
+    virtual bool set_convert_field(const NodeId &node, ColumnId column, const std::string &value);
 
     //! By default we do not allow to edit items.
     //! This is a recently added method. It will replace occasionally used is_renameable
@@ -541,29 +371,25 @@ namespace bec
     virtual void dump(int show_field);
   protected:
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
 
     grt::ValueRef parse_value(grt::Type type, const std::string &value);
   };
 
-
-
   /** Base tree model class.
-   *
-   * @ingroup begrt
    */
   class WBPUBLICBACKEND_PUBLIC_FUNC TreeModel : public ListModel
   {
   public:
-    virtual int count();
-    virtual NodeId get_node(int index);
+    virtual size_t count();
+    virtual NodeId get_node(size_t index);
 
     virtual NodeId get_root() const;
-    virtual int get_node_depth(const NodeId &node);
+    virtual size_t get_node_depth(const NodeId &node);
     inline NodeId get_parent(const NodeId &node) const { return node.parent(); }
 
-    virtual int count_children(const NodeId &parent)= 0;
-    virtual NodeId get_child(const NodeId &parent, int index) { return NodeId(parent).append(index); }
+    virtual size_t count_children(const NodeId &parent) = 0;
+    virtual NodeId get_child(const NodeId &parent, size_t index) { return NodeId(parent).append(index); }
     virtual bool has_next(const NodeId &node);
     virtual NodeId get_next(const NodeId &node);
 
@@ -574,14 +400,9 @@ namespace bec
 
     void save_expand_info(const std::string &path);
 
-
     virtual void dump(int show_field);
-  protected:
-
   };
 
-
-
   class WBPUBLICBACKEND_PUBLIC_FUNC GridModel : public ListModel
   {
   public:
@@ -597,17 +418,16 @@ namespace bec
       BlobType
     };
 
-    virtual int get_column_count() const= 0;
-    virtual std::string get_column_caption(int column)= 0;
-    virtual ColumnType get_column_type(int column)= 0;
+    virtual size_t get_column_count() const = 0;
+    virtual std::string get_column_caption(ColumnId column)= 0;
+    virtual ColumnType get_column_type(ColumnId column)= 0;
     virtual bool is_readonly() const { return false; } //!
     virtual std::string readonly_reason() const { return std::string(); } //!
-    virtual bool is_field_null(const bec::NodeId &node, int column) { return false; } //!
-    virtual bool set_field_null(const bec::NodeId &node, int column) { return set_convert_field(node, column, ""); } //!
-    virtual void set_edited_field(int row_index, int col_index) { }
+    virtual bool is_field_null(const bec::NodeId &node, ColumnId column) { return false; } //!
+    virtual bool set_field_null(const bec::NodeId &node, ColumnId column) { return set_convert_field(node, column, ""); } //!
+    virtual void set_edited_field(RowId row_index, ColumnId col_index) { }
 
   public:
-    typedef size_t ColumnId;
     typedef std::list<std::pair<ColumnId, int> > SortColumns;
     virtual void sort_by(ColumnId column, int direction, bool retaining) {}
     virtual SortColumns sort_columns() const { return SortColumns(); }
@@ -618,12 +438,4 @@ namespace bec
 
 };
 
-#ifdef _MSC_VER
-#pragma make_public(::bec::ListModel)
-#pragma make_public(::bec::TreeModel)
-#pragma make_public(::bec::NodeId)
-#pragma make_public(::bec::GridModel)
-#endif
-
-
-#endif /* _TREE_MODEL_H_ */
+#endif // _TREE_MODEL_H_
