@@ -1,5 +1,5 @@
-/* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+/*
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,24 +17,26 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #ifndef _WIN32
 #include <pcre.h>
 #include <stdio.h>
 #endif
 
-//XXX should remove this mysql specific code from here --alfredo
-#include <mysql.h> // needed for enum_field_types
+#include "base/string_utilities.h"
+#include "base/util_functions.h"
+#include "base/log.h"
+
 #include <boost/scoped_array.hpp>
 
 #include "db_object_helpers.h"
-#include <grtpp_undo_manager.h>
-#include <grtpp_util.h>
+#include "grtpp_undo_manager.h"
+#include "grtpp_util.h"
 #include "grt/common.h"
 #include "grt/parse_utils.h"
 #include "grts/structs.workbench.physical.h"
-#include "base/string_utilities.h"
+#include "grtdb/db_helpers.h"
+
+DEFAULT_LOG_DOMAIN("dbhelpers");
 
 #ifndef _WIN32
 #include <string.h>
@@ -56,7 +58,7 @@ public:
   operator T* () { return _ptr; }
 };
 
-
+// Important: this functions works only for model objects, not live objects!
 db_mgmt_RdbmsRef get_rdbms_for_db_object(const ::grt::ValueRef &object)
 {
   GrtObjectRef parent= GrtObjectRef::cast_from(object);
@@ -75,7 +77,7 @@ db_SimpleDatatypeRef CatalogHelper::get_datatype(grt::ListRef<db_SimpleDatatype>
 {
   for (size_t c= types.count(), i= 0; i < c; i++)
   {
-    if (g_strcasecmp(types[i]->name().c_str(), name.c_str())==0)
+    if (base::string_compare(types[i]->name(), name, false)==0)
       return types[i];
     //for (size_t d= types[i]->synonyms().count(), j= 0; j < d; j++)
     //{
@@ -87,6 +89,57 @@ db_SimpleDatatypeRef CatalogHelper::get_datatype(grt::ListRef<db_SimpleDatatype>
   return db_SimpleDatatypeRef();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+bool CatalogHelper::is_type_valid_for_version(const db_SimpleDatatypeRef &type, const GrtVersionRef &target_version)
+{
+  std::string validity = type->validity();
+  GrtVersionRef valid_version;
+  if (!validity.empty())
+  {
+    bool match = false;
+    switch (validity[0])
+    {
+      case '<':
+        if (validity[1] == '=')
+        {
+          valid_version = parse_version(type.get_grt(), validity.substr(2));
+          if (version_equal(target_version, valid_version) || version_greater(valid_version, target_version))
+            match = true;
+        }
+        else
+        {
+          valid_version = parse_version(type.get_grt(), validity.substr(1));
+          if (version_greater(valid_version, target_version))
+            match = true;
+        }
+        break;
+      case '>':
+        if (validity[1] == '=')
+        {
+          valid_version = parse_version(type.get_grt(), validity.substr(2));
+          if (version_equal(target_version, valid_version) || version_greater(target_version, valid_version))
+            match = true;
+        }
+        else
+        {
+          valid_version = parse_version(type.get_grt(), validity.substr(1));
+          if (version_greater(target_version, valid_version))
+            match = true;
+        }
+        break;
+      case '=':
+        valid_version = parse_version(type.get_grt(), validity.substr(1));
+        if (version_equal(target_version, valid_version))
+          match = true;
+        break;
+    }
+    return match;
+  }
+  return true;
+}
+
+
 
 std::string CatalogHelper::dbobject_to_dragdata(const db_DatabaseObjectRef &object)
 {
@@ -259,20 +312,23 @@ db_TableRef TableHelper::create_associative_table(const db_SchemaRef &schema,
   // when the 1st FK is created, the PK and FK will have the same columns, so the index will be reused
   // when the 2nd FK is created, the PK will have both columns so it can't be used by the 1st FK, so we have
   // to create a new one for it
-  db_IndexRef index = create_index_for_fk(schema->get_grt(), first_fk);
+  db_IndexRef index = create_index_for_fk(schema->get_grt(), first_fk, rdbms->maximumIdentifierLength());
   first_fk->index(index);
   atable->indices().insert(index);
+
+  atable->createDate(grt::StringRef(base::fmttime(0, DATETIME_FMT)));
+  atable->lastChangeDate(atable->createDate());
   
   undo.end("Create Associative Table");
 
   return atable;
 }
 
-
-static std::string format_ident_with_table(const std::string &fmt, const db_TableRef &table)
-{
-  return replace_variable(fmt, "%table%", table->name().c_str());
-}
+//
+//static std::string format_ident_with_table(const std::string &fmt, const db_TableRef &table)
+//{
+//  return replace_variable(fmt, "%table%", table->name().c_str());
+//}
 
 static std::string format_ident_with_stable_dtable(const std::string &fmt, const db_TableRef &stable, const db_TableRef &dtable)
 {
@@ -286,10 +342,13 @@ static std::string format_ident_with_column(const std::string &fmt, const db_Col
 }
 
 
-db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk)
+db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk, const size_t max_len)
 {
   std::string index_name(fk->name().c_str());
-  //index_name.append("_index");
+  if (index_name.length() > (max_len-5))
+      index_name.resize(max_len-5);
+  index_name.append("_idx");
+  index_name = grt::get_name_suggestion_for_list_object(fk->owner()->indices(),index_name,false);
 
   // add the corresponding index
   db_IndexRef index= grt->create_object<db_Index>(db_TableRef::cast_from(fk->owner()).get_metaclass()->get_member_type("indices").content.object_class.c_str());
@@ -318,36 +377,123 @@ db_IndexRef TableHelper::create_index_for_fk(grt::GRT *grt, const db_ForeignKeyR
 }
 
 
-db_IndexRef TableHelper::find_index_usable_by_fk(const db_ForeignKeyRef &fk)
+void TableHelper::reorder_foreign_key_for_index(const db_ForeignKeyRef &fk, const db_IndexRef &index)
+{
+  // make the order of the columns in the FK match that of the index
+  size_t column_count = fk->columns().count();
+
+  if (fk->columns().count() != fk->referencedColumns().count())
+  {
+    log_error("Internal consistency error: number of items in fk->columns and fk->referencedColumns() for %s.%s.%s do not match\n",
+              fk->owner()->owner()->name().c_str(), fk->owner()->name().c_str(), fk->name().c_str());
+    return;
+  }
+  if (column_count > index->columns().count())
+  {
+    log_error("Internal consistency error: number of items in index for FK is less than columns in FK %s.%s.%s\n",
+             fk->owner()->owner()->name().c_str(), fk->owner()->name().c_str(), fk->name().c_str());
+    return;
+  }
+
+  for (size_t j= 0; j < column_count; j++)
+  {
+    if (index->columns()[j]->referencedColumn() != fk->columns()[j])
+    {
+      // if the column in the index is not the expected one, reorder the FK
+
+      // find the position of the column in the FK
+      for (size_t k = j+1; k < column_count; k++)
+      {
+        if (index->columns()[j]->referencedColumn() == fk->columns()[k])
+        {
+          // reorder the columns in the FK
+          fk->columns().reorder(k, j);
+          fk->referencedColumns().reorder(k, j);
+          break;
+        }
+      }
+      break;
+    }
+  }
+}
+
+
+db_IndexRef TableHelper::find_index_usable_by_fk(const db_ForeignKeyRef &fk, const db_IndexRef &other_than, bool allow_any_order)
 {
   size_t column_count = fk->columns().count();
   db_TableRef table(db_TableRef::cast_from(fk->owner()));
+
+  if (column_count == 0)
+    return db_IndexRef();
   
   for (size_t c= table->indices().count(), i= 0; i < c; i++)
   {
     db_IndexRef index(table->indices()[i]);
-    
-    if (index->columns().count() == column_count)
+
+    if (index == other_than)
+      continue;
+
+    // smaller indexes than the FK won't work
+    size_t index_column_count = index->columns().count();
+    if (index_column_count >= column_count)
     {
-      bool all_columns_match= true;
-      for (size_t j= 0; j < column_count; j++)
+      bool index_usable;
+      if (allow_any_order)
       {
-        bool ok= false;
-        for (size_t k= 0; k < column_count; k++)
+        index_usable = true;
+        for (size_t j= 0; j < column_count; j++)
         {
-          if (index->columns()[k]->referencedColumn() == fk->columns()[j])
+          bool ok= false;
+          // check whether this FK column is in the index
+          for (size_t k= 0; k < index_column_count; k++)
           {
-            ok= true;
+            if (index->columns()[k]->referencedColumn() == fk->columns()[j])
+            {
+              ok= true;
+              break;
+            }
+          }
+          if (!ok)
+          {
+            index_usable= false;
             break;
           }
         }
-        if (!ok)
+
+        if (index_usable)
         {
-          all_columns_match= false;
-          break;
+          size_t c = 0;
+          // now check whether only columns from the prefix of the index are in use in the FK..
+          // example: FK (a, b) and INDEX (a, b, c) is OK
+          //          FK (b, c) and INDEX (a, b, c) is not ok
+          for (size_t j= 0; j < column_count && c < column_count; j++)
+          {
+            if (fk->columns().get_index(index->columns()[j]->referencedColumn()) != grt::BaseListRef::npos)
+              c++; // jth column is in the FK, so it's OK so far
+            else
+            {
+              index_usable = false;
+              break; // jth column of the index is not in the FK, so this index can't be used
+            }
+          }
         }
       }
-      if (all_columns_match)
+      else
+      {
+        index_usable = false;
+        // make sure that the FK form prefix of the columns in the index
+        for (size_t j= 0; j < column_count; j++)
+        {
+          if (index->columns()[j]->referencedColumn() != fk->columns()[j])
+          {
+            index_usable = false;
+            break;
+          }
+          else
+            index_usable = true; // there's at least one match
+        }
+      }
+      if (index_usable)
         return index;
     }
   }
@@ -415,42 +561,87 @@ db_ForeignKeyRef TableHelper::create_empty_foreign_key(grt::GRT *grt, const db_T
 
   table->foreignKeys().insert(fk);
 
+  /* don't always create an index for the FK, it should only be created if there are no other indexes that match it
+   the check for indexes should be done every time the FK columns are edited
   db_IndexRef index(create_index_for_fk(grt, fk));
   fk->index(index);
   table->indices().insert(index);
-
+  */
   undo.end(_("Create Foreign Key"));
 
   return fk;
 }
 
 
-void TableHelper::update_foreign_key_index(grt::GRT *grt, const db_ForeignKeyRef &fk)
+void TableHelper::update_foreign_key_index(const db_ForeignKeyRef &fk)
 {
   //! todo: add undo group
 
+  db_TableRef table(fk->owner());
   db_IndexRef index(fk->index());
-  
-  if (!index.is_valid())
-  {
-    g_warning("ForeignKey %s has no index", fk->name().c_str());
-    return;
-  }
 
-  // recreate index columns
-  while (index->columns().count() > 0)
-    index->columns().remove(0);
-
-  ListRef<db_Column> fk_columns(fk->columns());
-  for (size_t n= 0, count= fk_columns.count(); n < count; ++n)
+  if (index.is_valid())
   {
-    db_ColumnRef column(fk_columns.get(n));
-    db_IndexColumnRef index_column(grt->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
-    index_column->owner(index);
+    db_IndexRef other;
+    if ((other = find_index_usable_by_fk(fk, index, true)).is_valid())
+    {
+      // the index from this FK is redundant, so remove it
+      fk->index(db_IndexRef());
+      table->indices().remove_value(index);
+      reorder_foreign_key_for_index(fk, other);
+      return;
+    }
+
+    // make sure that the columns in the index match the ones in the FK
+
+    // remove columns that are gone from the FK
+    for (ssize_t i = index->columns().count() - 1; i >= 0; --i)
+    {
+      if (fk->columns().get_index(index->columns()[i]) == grt::BaseListRef::npos)
+        index->columns().remove(i);
+    }
+
+    // recreate index columns
+    while (index->columns().count() > 0)
+      index->columns().remove(0);
+
+    ListRef<db_Column> fk_columns(fk->columns());
+    for (size_t n= 0, count= fk_columns.count(); n < count; ++n)
+    {
+      db_ColumnRef column(fk_columns.get(n));
+      db_IndexColumnRef index_column(fk->get_grt()->create_object<db_IndexColumn>(index.get_metaclass()->get_member_type("columns").content.object_class));
+      index_column->owner(index);
       //"name", column->name().valueptr(),
-    index_column->referencedColumn(column);
-    index->columns().insert(index_column);
+      index_column->referencedColumn(column);
+      index->columns().insert(index_column);
+    }
+
+    if (index->columns().count() == 0)
+    {
+      // the index is empty, remove it
+      fk->index(db_IndexRef());
+      table->indices().remove_value(index);
+      return;
+    }
   }
+  else
+    create_index_for_fk_if_needed(fk);
+}
+
+
+bool TableHelper::create_index_for_fk_if_needed(db_ForeignKeyRef fk)
+{
+  db_IndexRef index = find_index_usable_by_fk(fk, db_IndexRef(), true);
+  if (index.is_valid())
+    reorder_foreign_key_for_index(fk, index);
+  else if (fk->columns().count() > 0)
+  {
+    index = create_index_for_fk(fk.get_grt(), fk);
+    fk->index(index);
+    fk->owner()->indices().insert(index);
+    return true;
+  }
+  return false;
 }
 
 
@@ -459,15 +650,11 @@ bool TableHelper::create_missing_indexes_for_foreign_keys(const db_TableRef &tab
   bool created= false;
   GRTLIST_FOREACH(db_ForeignKey, table->foreignKeys(), fk)
   {
-    db_IndexRef index;
     // check if an index already exists for the FK columns and if not, create one
-    if (!(*fk)->index().is_valid() && !find_index_usable_by_fk(*fk).is_valid())
-    {
-      index = create_index_for_fk(table->get_grt(), *fk);
-      (*fk)->index(index);
-      table->indices().insert(index);
-      created= true;
-    }
+    if (!(*fk)->index().is_valid())
+      created = created || create_index_for_fk_if_needed(*fk);
+    else
+      reorder_foreign_key_for_index(*fk, (*fk)->index());
   }
   return created;
 }
@@ -507,7 +694,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   std::string column_name_format;
   std::string scolumn_name;
   std::string dcolumn_name;
-  int max_identifier_length= rdbms->maximumIdentifierLength();
+  size_t max_identifier_length= rdbms->maximumIdentifierLength();
 
   // check if there is a PK
   if (!pk.is_valid() || pk->columns().count() == 0)
@@ -572,7 +759,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   
   new_fk->name(SchemaHelper::get_unique_foreign_key_name(db_SchemaRef::cast_from(table->owner()), 
                                                          name_format, 
-                                                         max_identifier_length));
+                                                         (int)max_identifier_length));
   new_fk->oldName(new_fk->name());
   
   new_fk->owner(table); // set the owner last
@@ -581,7 +768,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   // check if an index already exists for the FK columns and if not, create one
   if (!find_index_usable_by_fk(new_fk).is_valid())
   {
-    index = create_index_for_fk(grt, new_fk);
+    index = create_index_for_fk(grt, new_fk, rdbms->maximumIdentifierLength());
     new_fk->index(index);
     // index is inserted later down
   }
@@ -615,7 +802,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   std::string name_format;
   std::string scolumn_name;
   std::string dcolumn_name;
-  int max_identifier_length= rdbms->maximumIdentifierLength();
+  size_t max_identifier_length= rdbms->maximumIdentifierLength();
   bool ref_mandatory;
 
   grt::AutoUndo undo(grt, table->is_global());
@@ -661,7 +848,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   
   new_fk->name(SchemaHelper::get_unique_foreign_key_name(db_SchemaRef::cast_from(table->owner()), 
                                                          name_format,
-                                                         max_identifier_length));
+                                                         (int)max_identifier_length));
   new_fk->oldName(new_fk->name());
   
   new_fk->referencedMandatory(ref_mandatory?1:0);
@@ -671,7 +858,7 @@ db_ForeignKeyRef TableHelper::create_foreign_key_to_table(const db_TableRef &tab
   // check if an index already exists for the FK columns and if not, create one
   if (!find_index_usable_by_fk(new_fk).is_valid())
   {
-    db_IndexRef index(create_index_for_fk(grt, new_fk));
+    db_IndexRef index(create_index_for_fk(grt, new_fk,rdbms->maximumIdentifierLength()));
     new_fk->index(index);
     
     // add the FK to the source table
@@ -754,41 +941,81 @@ db_mysql_StorageEngineRef TableHelper::get_engine_by_name(grt::GRT *grt, const s
   return db_mysql_StorageEngineRef();
 }
 
-inline size_t calc_db_comment_index(const std::string &comment,const size_t &db_comment_len)
+/**
+ * Returns a pointer to the position of the first empty line (NOT a line break!) or the first character after the
+ * maximum count, whichever comes first. If no line break was found and the string is shorter
+ * than the maximum length the result points to the terminating 0.
+ */
+static void split_comment(const std::string &comment, size_t db_comment_len,
+                          std::string *comment_ret, std::string *leftover_ret)
 {
   size_t res;
-  gchar* endl_ptr = g_utf8_strrchr(comment.c_str(),0,'\n');
-  if (endl_ptr)
-    res = g_utf8_pointer_to_offset(comment.c_str(),endl_ptr);
+  // XXX: check for Unicode line breaks! especially asian languages don't use the ANSI new line.
+  const gchar* pointer_to_linebreak = NULL;
+
+  { // find 1st occurrence of 2 consecutive newlines
+    std::string::size_type pos = comment.find("\n\n");
+    if (pos == std::string::npos)
+      pos = comment.find("\r\n\r\n");
+
+    if (pos != std::string::npos)
+      pointer_to_linebreak = comment.c_str() + pos;
+  }
+
+  // We need the number of characters which the string part includes, so convert to a char count.
+  if (pointer_to_linebreak != NULL)
+    res = g_utf8_pointer_to_offset(comment.c_str(), pointer_to_linebreak);
   else
-    res = g_utf8_strlen(comment.c_str(),-1);
+    res = comment.size(); // it's wrong to use g_utf8_strlen here because the comment length must be measured in bytes, not unichars
 
+  // don't break in the middle of a utf8 sequence
   if (res > db_comment_len)
-    res = db_comment_len;
-  return res;
-  /*
-  size_t db_comment_index = comment.find("\n");
-  if (db_comment_index == std::string::npos)
-    db_comment_index = comment.size();
-  if (db_comment_index > db_comment_len)
-    db_comment_index = db_comment_len;
-  return db_comment_index;*/
+  {
+    if (g_utf8_get_char_validated(comment.c_str() + db_comment_len, (gssize)(res - db_comment_len)) == (gunichar)-1)
+      res = g_utf8_pointer_to_offset(comment.c_str(), g_utf8_find_prev_char(comment.c_str(), comment.c_str() + db_comment_len));
+    else
+      res = db_comment_len;
+  }
+  if (comment_ret)
+    *comment_ret = comment.substr(0, res);
+  if (leftover_ret)
+  {
+    if (pointer_to_linebreak != NULL)
+      *leftover_ret = comment.substr(res+1);
+    else
+      *leftover_ret = comment.substr(res);
+  }
+}
+
+std::string TableHelper::get_sync_comment(const std::string &comment, const size_t max_len)
+{
+  std::string ret;
+  if (comment.size() > max_len)
+    split_comment(comment, max_len, &ret, NULL);
+  else
+    ret = comment;
+  return ret;
 };
 
-std::string TableHelper::get_sync_comment(const std::string &comment)
-{
-  //TODO add comment size depending on server version
-  boost::scoped_array<char> buf(new char [128]);
-  g_utf8_strncpy(buf.get(),comment.c_str(),calc_db_comment_index(comment,60));
-  return std::string(buf.get());
-};
 
-std::string TableHelper::get_document_comment(const std::string &comment)
+std::string TableHelper::normalize_table_name_list(const std::string &schema, const std::string &table_name_list)
 {
-  //TODO add comment size depending on server version
-  boost::scoped_array<char> buf(new char [128]);
-  return std::string(g_utf8_offset_to_pointer(comment.c_str(),calc_db_comment_index(comment,60)));
-};
+  std::vector<std::string> names = base::split(table_name_list, ",");
+
+  for (std::vector<std::string>::iterator it = names.begin(); it != names.end(); ++it)
+  {
+    std::vector<std::string> tokens(base::split_qualified_identifier(base::trim(*it)));
+    if (tokens.size() == 1)
+      tokens.insert(tokens.begin(), schema);
+
+    for (std::vector<std::string>::iterator t = tokens.begin(); t != tokens.end(); ++t)
+      *t = base::quote_identifier(base::unquote_identifier(*t), '`');
+
+    *it = base::join(tokens, ".");
+  }
+
+  return base::join(names, ",");
+}
 
 void ColumnHelper::copy_column(const db_ColumnRef &from, db_ColumnRef &to)
 {
@@ -838,7 +1065,7 @@ ColumnTypeCompareResult ColumnHelper::compare_column_types(const db_ColumnRef &f
 
 void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &value)
 {
-  column->defaultValueIsNull(g_strcasecmp(value.c_str(), "NULL") ? 0 : 1);
+  column->defaultValueIsNull(base::string_compare(value, "NULL", false) ? 0 : 1);
   column->defaultValue(value.c_str());
 
   // If a default value of NULL was set then the column can no longer be a not-null column.
@@ -849,47 +1076,47 @@ void ColumnHelper::set_default_value(db_ColumnRef column, const std::string &val
 }
 
 //------------------------------------------------------------------------------------
-
-
-//XXX the following code must be moved to db.mysql module
-
-static enum_field_types get_mysql_type_from_string(const char *type_name)
-{
-  if(g_strcasecmp(type_name, "INT") == 0)
-    return MYSQL_TYPE_LONG;
-  if(g_strcasecmp(type_name, "TINYINT") == 0)
-    return MYSQL_TYPE_TINY;
-  if(g_strcasecmp(type_name, "SMALLINT") == 0)
-    return MYSQL_TYPE_SHORT;
-  if(g_strcasecmp(type_name, "BIGINT") == 0)
-    return MYSQL_TYPE_LONGLONG;
-  if(g_strcasecmp(type_name, "FLOAT") == 0)
-    return MYSQL_TYPE_FLOAT;
-  if(g_strcasecmp(type_name, "DOUBLE") == 0)
-    return MYSQL_TYPE_DOUBLE;
-  if(g_strcasecmp(type_name, "TIME") == 0)
-    return MYSQL_TYPE_TIME;
-  if(g_strcasecmp(type_name, "DATE") == 0)
-    return MYSQL_TYPE_DATE;
-  if(g_strcasecmp(type_name, "DATETIME") == 0)
-    return MYSQL_TYPE_DATETIME;
-  if(g_strcasecmp(type_name, "TIMESTAMP") == 0)
-    return MYSQL_TYPE_TIMESTAMP;
-  if(g_strcasecmp(type_name, "TEXT") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "CHAR") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "VARCHAR") == 0)
-    return MYSQL_TYPE_STRING;
-  if(g_strcasecmp(type_name, "BLOB") == 0)
-    return MYSQL_TYPE_BLOB;
-  if(g_strcasecmp(type_name, "BINARY") == 0)
-    return MYSQL_TYPE_BLOB;
-  if(g_strcasecmp(type_name, "VARBINARY") == 0)
-    return MYSQL_TYPE_BLOB;
-
-  return MYSQL_TYPE_NULL;
-}
+//
+//
+////XXX the following code must be moved to db.mysql module
+//
+//static enum_field_types get_mysql_type_from_string(const char *type_name)
+//{
+//  if(g_strcasecmp(type_name, "INT") == 0)
+//    return MYSQL_TYPE_LONG;
+//  if(g_strcasecmp(type_name, "TINYINT") == 0)
+//    return MYSQL_TYPE_TINY;
+//  if(g_strcasecmp(type_name, "SMALLINT") == 0)
+//    return MYSQL_TYPE_SHORT;
+//  if(g_strcasecmp(type_name, "BIGINT") == 0)
+//    return MYSQL_TYPE_LONGLONG;
+//  if(g_strcasecmp(type_name, "FLOAT") == 0)
+//    return MYSQL_TYPE_FLOAT;
+//  if(g_strcasecmp(type_name, "DOUBLE") == 0)
+//    return MYSQL_TYPE_DOUBLE;
+//  if(g_strcasecmp(type_name, "TIME") == 0)
+//    return MYSQL_TYPE_TIME;
+//  if(g_strcasecmp(type_name, "DATE") == 0)
+//    return MYSQL_TYPE_DATE;
+//  if(g_strcasecmp(type_name, "DATETIME") == 0)
+//    return MYSQL_TYPE_DATETIME;
+//  if(g_strcasecmp(type_name, "TIMESTAMP") == 0)
+//    return MYSQL_TYPE_TIMESTAMP;
+//  if(g_strcasecmp(type_name, "TEXT") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "CHAR") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "VARCHAR") == 0)
+//    return MYSQL_TYPE_STRING;
+//  if(g_strcasecmp(type_name, "BLOB") == 0)
+//    return MYSQL_TYPE_BLOB;
+//  if(g_strcasecmp(type_name, "BINARY") == 0)
+//    return MYSQL_TYPE_BLOB;
+//  if(g_strcasecmp(type_name, "VARBINARY") == 0)
+//    return MYSQL_TYPE_BLOB;
+//
+//  return MYSQL_TYPE_NULL;
+//}
 
 void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
 {
@@ -955,8 +1182,7 @@ void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
     || (strcmp(column->simpleType()->name().c_str(), "DATE") == 0)
     || (strcmp(column->simpleType()->name().c_str(), "TIME") == 0)
     || (strcmp(column->simpleType()->name().c_str(), "TIMESTAMP") == 0)
-    || (strcmp(column->simpleType()->name().c_str(), "YEAR") == 0)
-    || (strcmp(column->simpleType()->name().c_str(), "BIT") == 0))
+    || (strcmp(column->simpleType()->name().c_str(), "YEAR") == 0))
   {
     column->length(grt::IntegerRef(-1));
     if (default_default_value)
@@ -967,8 +1193,8 @@ void CatalogHelper::apply_defaults(db_mysql_ColumnRef column)
 
 void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_engine)
 {
-  cat->defaultCharacterSetName("latin1");
-  cat->defaultCollationName("latin1_swedish_ci");
+  cat->defaultCharacterSetName("utf8");
+  cat->defaultCollationName("utf8_general_ci");
 
   for(size_t i= 0, schemata_count= cat->schemata().count(); i < schemata_count; i++)
   {
@@ -976,7 +1202,6 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
 
     if(strlen(schema->defaultCharacterSetName().c_str()) == 0)
       schema->defaultCharacterSetName(cat->defaultCharacterSetName());
-
     if(strlen(schema->defaultCollationName().c_str()) == 0)
       schema->defaultCollationName(get_cs_def_collation(schema->defaultCharacterSetName().c_str()));
 
@@ -987,8 +1212,16 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
       if(strlen(table->defaultCharacterSetName().c_str()) == 0)
         table->defaultCharacterSetName(schema->defaultCharacterSetName());
 
-      if(strlen(table->defaultCollationName().c_str()) == 0)
-        table->defaultCollationName(schema->defaultCollationName());
+      if(table->defaultCharacterSetName() == schema->defaultCharacterSetName())
+      {
+        if(strlen(table->defaultCollationName().c_str()) == 0)
+          table->defaultCollationName(schema->defaultCollationName());
+      }
+      else
+      {
+        if(strlen(table->defaultCollationName().c_str()) == 0)
+          table->defaultCollationName(get_cs_def_collation(table->defaultCharacterSetName()));
+      }
       
       if(strlen(table->tableEngine().c_str()) == 0)
         table->tableEngine(default_engine.empty()?"InnoDB":default_engine);
@@ -1013,3 +1246,342 @@ void CatalogHelper::apply_defaults(db_mysql_CatalogRef cat, std::string default_
     }
   }
 }
+
+static int pcre_compile_exec(const char *pattern, const char *str, int *patres, int patresnum)
+{
+  const char *errptr;
+  int erroffs;
+  int c;
+  pcre *patre= pcre_compile(pattern, 0, &errptr, &erroffs, NULL);
+  if (!patre)
+    throw std::logic_error("error compiling regex "+std::string(errptr));
+
+  c= pcre_exec(patre, NULL, str, (int)strlen(str), 0, 0, patres, patresnum);
+  pcre_free(patre);
+
+  return c;
+}
+
+static db_SimpleDatatypeRef findType(const grt::ListRef<db_SimpleDatatype> &types, const GrtVersionRef &target_version, const std::string &name)
+{
+  for (size_t c= types.count(), i= 0; i < c; i++)
+  {
+    if (base::string_compare(types[i]->name(), name, false)==0)
+    {
+      if (!target_version.is_valid() || CatalogHelper::is_type_valid_for_version(types[i], target_version))
+        return types[i];
+    }
+  }
+  return db_SimpleDatatypeRef();
+}
+
+static bool parseTypeDefinition(const std::string &type,
+                                const GrtVersionRef &targetVersion,
+                                const grt::ListRef<db_SimpleDatatype> &typeList,
+                                db_SimpleDatatypeRef &simpleType,
+                                int &precision,
+                                int &scale,
+                                int &length,
+                                std::string &explicitParams)
+{
+  auto_array_ptr<char> buffer(new char[type.size()+1]);
+  int patres[21];
+  int c;
+    
+  /* split the datatype name from the parameters
+   * (\w+)\s*(\(.*\))?
+   *
+   * 1.. TYPE
+   * 2.. params (optional)
+   */
+  
+  c= pcre_compile_exec("^(\\w+)\\s*(\\(.*\\))?\\s*$", type.c_str(), patres, sizeof(patres)/sizeof(int));
+  
+  if (c > 0 &&
+      pcre_copy_substring(type.c_str(), patres, c, 1, buffer, static_cast<int>(type.size()+1)) > 0)
+  {
+    // find the simple datatype with this type_name
+    simpleType= findType(typeList, targetVersion, std::string(buffer));
+    
+    if (simpleType.is_valid())
+    {
+      std::string params;
+      std::string param1;
+      std::string param2;
+      
+      // split the parameter into its components
+      
+      if (pcre_copy_substring(type.c_str(), patres, c, 2, buffer, static_cast<int>(type.size()+1)) > 0)
+        params= buffer;
+      
+      switch (*simpleType->parameterFormatType())
+      {
+        case 0: // no params
+          if (!params.empty())
+            return false;
+          break;
+          
+        case 1: // (n)
+          c= pcre_compile_exec("^\\((\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+          }
+          else
+            return false;
+          break;
+          
+        case 2: // [(n)]
+          c= pcre_compile_exec("^(\\((\\d+)\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+          }
+          break;
+          
+        case 3: // (m, n)
+          c= pcre_compile_exec("^\\((\\d+),\\s*(\\d+)\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            
+            if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+            else
+              return false;
+          }
+          else
+            return false;
+          break;
+          
+        case 4: // (m[,n])
+          c= pcre_compile_exec("^\\((\\d+)(,\\s*(\\d+))?\\)$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 3, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+          }
+          else
+            return false;
+          break;
+          
+        case 5: // [(m,n)]
+          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+            else
+              return false;
+          }
+          break;
+          
+        case 6: // [(m[,n])]
+          c= pcre_compile_exec("^(\\((\\d+)(,\\s*(\\d+))?\\))?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          if (pcre_copy_substring(params.c_str(), patres, c, 2, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            param1= buffer;
+            if (pcre_copy_substring(params.c_str(), patres, c, 4, buffer, static_cast<int>(params.size()+1)) > 0)
+              param2= buffer;
+          }
+          break;
+          
+        case 10: // ('a','b','c')
+          if (params.empty())
+            return false;
+                
+          c= pcre_compile_exec("^\\((.*?)\\)?$", params.c_str(), patres, sizeof(patres)/sizeof(int));
+          if (c < 0)
+            return false;
+          // For now we just keep the parameter list as a whole. If necessary this can be split later for other uses.
+          param1= buffer;
+          // validate format
+          if (pcre_copy_substring(params.c_str(), patres, c, 1, buffer, static_cast<int>(params.size()+1)) > 0)
+          {
+            std::list<std::string> tokens;
+            if (!bec::tokenize_string_list(std::string(buffer), '\'', true, tokens))
+              return false;
+          }
+          else
+            return false;
+          break;
+      }
+      
+      // get precision
+      if (*simpleType->numericPrecision() != bec::EMPTY_TYPE_PRECISION)
+      {
+        if (!param1.empty())
+          precision= atoi(param1.c_str());
+        
+        // get scale
+        if (*simpleType->numericScale() != bec::EMPTY_TYPE_SCALE)
+        {
+          if (!param2.empty())
+            scale= atoi(param2.c_str());
+        }
+      }
+      // get length
+      else if (simpleType->characterMaximumLength() != bec::EMPTY_TYPE_MAXIMUM_LENGTH 
+               || simpleType->characterOctetLength()   != bec::EMPTY_TYPE_OCTET_LENGTH)
+      {
+        if (!param1.empty())
+          length= atoi(param1.c_str());
+      }
+      else if (*simpleType->parameterFormatType() == 10)
+        explicitParams= param1;
+    }
+    else
+      return false; // unknown type
+  }
+  else
+    return false; // bad format
+  
+  return true;
+}
+
+bool bec::parseType(const std::string &type,
+    const GrtVersionRef &targetVersion,
+    const grt::ListRef<db_SimpleDatatype> &typeList,
+    const grt::ListRef<db_UserDatatype>& user_types,
+    const grt::ListRef<db_SimpleDatatype>& default_type_list,
+    db_SimpleDatatypeRef &simpleType,
+    db_UserDatatypeRef& userType,
+    int &precision,
+    int &scale,
+    int &length,
+    std::string &datatypeExplicitParams)
+{
+    if (user_types.is_valid())
+    {
+        std::string::size_type argp= type.find('(');
+        std::string typeName= type;
+
+        if (argp != std::string::npos)
+            typeName= type.substr(0, argp);
+
+        // 1st check if this is a user defined type
+        for (size_t c= user_types.count(), i= 0; i < c; i++)
+        {
+            db_UserDatatypeRef utype(user_types[i]);
+
+            if (base::string_compare(utype->name(), typeName, false) == 0)
+            {
+                userType= utype;
+                break;
+            }
+        }
+    }
+
+    if (userType.is_valid())
+    {
+        // if the type spec has an argument, we replace the arguments from the type definition
+        // with the one provided by the user
+        std::string finalType= userType->sqlDefinition();
+        std::string::size_type tp;
+        bool overriden_args= false;
+
+        if ((tp= type.find('(')) != std::string::npos) // has user specified args
+        {
+            std::string::size_type p= finalType.find('('); 
+            if (p != std::string::npos) // strip the original args
+                finalType= finalType.substr(0, p);
+
+            // extract the user spcified args and append to the specification
+            finalType.append(type.substr(tp));
+
+            overriden_args= true;
+        }
+
+        // parse usertype definition    
+        if (!parseTypeDefinition(finalType, targetVersion, typeList.is_valid() ? typeList : default_type_list, simpleType,
+            precision, scale, length, datatypeExplicitParams))
+            return false;
+
+        simpleType = db_SimpleDatatypeRef();
+        if (!overriden_args)
+        {
+            precision = bec::EMPTY_COLUMN_PRECISION;
+            scale = bec::EMPTY_COLUMN_SCALE;
+            length = bec::EMPTY_COLUMN_LENGTH;
+            datatypeExplicitParams = "";
+        }
+    }
+    else
+    {
+        if (!parseTypeDefinition(type, targetVersion, typeList.is_valid() ? typeList : default_type_list, simpleType,
+            precision, scale, length, datatypeExplicitParams))
+            return false;
+
+        userType = db_UserDatatypeRef();
+    }
+    return true;
+}
+
+
+
+std::string bec::get_default_collation_for_charset(const db_SchemaRef &schema, const std::string &character_set)
+{
+  if (schema->owner().is_valid())
+  {
+    db_CatalogRef catalog(db_CatalogRef::cast_from(schema->owner()));
+    db_CharacterSetRef charset = find_named_object_in_list(catalog->characterSets(), character_set);
+    if (charset.is_valid())
+    {
+      return charset->defaultCollation();
+    }
+  }
+  else
+    log_warning("While checking diff, catalog ref was found to be invalid\n");
+
+
+  return "";
+}
+
+std::string bec::get_default_collation_for_charset(const db_TableRef &table, const std::string &character_set)
+{
+  if (table->owner().is_valid())
+    return bec::get_default_collation_for_charset(db_SchemaRef::cast_from(table->owner()), character_set);
+  else
+    log_warning("While checking diff, table ref was found to be invalid\n");
+  return "";
+}
+
+
+std::string bec::TableHelper::generate_comment_text(const std::string& comment_text, size_t comment_lenght)
+{
+  if (comment_text.size() > comment_lenght)
+  {
+    std::string comment, leftover;
+
+    split_comment(comment_text, comment_lenght, &comment, &leftover);
+
+    if (!comment.empty())
+      comment = "'"+base::escape_sql_string(comment)+"'";
+    if (!leftover.empty())
+    {
+      base::replace(leftover, "*/", "*\\/");
+      comment.append(" /* comment truncated */ /*")
+      .append(leftover)
+      .append("*/");
+    }
+    return comment;
+  }
+  else if (!comment_text.empty())
+    return "'"+base::escape_sql_string(comment_text)+"'";
+
+  return "";
+}
