@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,14 +17,14 @@
  * 02110-1301  USA
  */
 
-#ifndef _DB_OBJECT_HELPERS_H_
-#define _DB_OBJECT_HELPERS_H_
+#pragma once
 
 #include <grtpp.h>
 #include <grts/structs.db.h>
 #include <grts/structs.db.mgmt.h>
 #include <grts/structs.db.mysql.h>
 #include <set>
+#include "base/string_utilities.h"
 
 #include "wbpublic_public_interface.h"
 
@@ -57,9 +57,7 @@ namespace bec {
   {
     static void apply_defaults(db_mysql_CatalogRef catalog, std::string default_engine);
     static db_SimpleDatatypeRef get_datatype(grt::ListRef<db_SimpleDatatype> types, const std::string &name);
-    static GrtVersionRef parse_version(grt::GRT *grt, const std::string &version);
-    static bool version_equal(GrtVersionRef a, GrtVersionRef b);
-    static bool version_greater(GrtVersionRef a, GrtVersionRef b);
+
     static bool is_type_valid_for_version(const db_SimpleDatatypeRef &type, const GrtVersionRef &target_version);
 
     static std::string dbobject_list_to_dragdata(const std::list<db_DatabaseObjectRef> &object);
@@ -101,8 +99,10 @@ namespace bec {
                                                      const grt::DictRef &global_options,
                                                      const grt::DictRef &options);
 
+    static bool create_index_for_fk_if_needed(db_ForeignKeyRef fk);
     static db_IndexRef create_index_for_fk(grt::GRT *grt, const db_ForeignKeyRef &fk, const size_t max_len = 64);
-    static db_IndexRef find_index_usable_by_fk(const db_ForeignKeyRef &fk, const db_IndexRef &other_than = db_IndexRef());
+    static db_IndexRef find_index_usable_by_fk(const db_ForeignKeyRef &fk, const db_IndexRef &other_than = db_IndexRef(), bool allow_any_order=false);
+    static void reorder_foreign_key_for_index(const db_ForeignKeyRef &fk, const db_IndexRef &index);
     static std::string generate_foreign_key_name();
     static db_ForeignKeyRef create_empty_foreign_key(grt::GRT *grt, const db_TableRef &table, const std::string &name);
     static void update_foreign_key_index(const db_ForeignKeyRef &fk);
@@ -122,8 +122,12 @@ namespace bec {
 
     static db_TableRef clone_table(const db_TableRef &table);
     static db_mysql_StorageEngineRef get_engine_by_name(grt::GRT *grt, const std::string &name);
-    static std::string get_sync_comment(const std::string &comment, const int max_len = 60);
-    static std::string get_document_comment(const std::string &comment, const int max_len = 60);
+    static std::string get_sync_comment(const std::string &comment, const size_t max_len = 60);
+
+    // add schema name and backticks to table names if needed
+    static std::string normalize_table_name_list(const std::string &schema, const std::string &table_name_list);
+
+    static std::string generate_comment_text(const std::string& comment_text, size_t comment_lenght);
   };
 
   enum ColumnTypeCompareResult { COLUMNS_TYPES_EQUAL          = 0, 
@@ -156,8 +160,17 @@ namespace bec {
     void operator() (db_mysql_ColumnRef column) 
     {
       db_UserDatatypeRef udt= column->userType();
-      if(udt.is_valid())
+      if (udt.is_valid())
+      {
         column->setParseType(column->formattedType(), catalog->simpleDatatypes());
+        column->flags().remove_all();
+        std::vector<std::string> flags(base::split(*udt->flags(), ","));
+        for (std::vector<std::string>::const_iterator i = flags.begin(); i != flags.end(); ++i)
+        {
+          if (column->flags().get_index(*i) == grt::BaseListRef::npos)
+            column->flags().insert(*i);
+        }
+      }
     }
   };
 
@@ -199,7 +212,7 @@ namespace bec {
     Schema_action sa(cat, rdbms);
     ct::for_each<ct::Schemata>(cat, sa);
   }
-  bool parseType(const std::string &type,
+  bool WBPUBLICBACKEND_PUBLIC_FUNC parseType(const std::string &type,
     const GrtVersionRef &target_version,
     const grt::ListRef<db_SimpleDatatype> &typeList,
     const grt::ListRef<db_UserDatatype>& user_types,
@@ -211,6 +224,11 @@ namespace bec {
     int &length,
     std::string &datatypeExplicitParams);
 
+  inline bool is_int_datatype(const std::string &type)
+  {
+    return type == "BIGINT" || type == "MEDIUMINT" || type == "SMALLINT" || type == "TINYINT" || type == "INT";
+  }
+  
+  std::string WBPUBLICBACKEND_PUBLIC_FUNC get_default_collation_for_charset(const db_SchemaRef &schema, const std::string &character_set);
+  std::string WBPUBLICBACKEND_PUBLIC_FUNC get_default_collation_for_charset(const db_TableRef &table, const std::string &character_set);
 };
-
-#endif /* _DB_OBJECT_HELPERS_H_ */
