@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,25 +16,26 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _EDITOR_DBOBJECT_H_
-#define _EDITOR_DBOBJECT_H_
+
+#pragma once
+
+#include "grtpp_notifications.h"
 
 #include "wbpublic_public_interface.h"
 #include "grt/editor_base.h"
 
-#include "grtsqlparser/invalid_sql_parser.h"
+#include "grtsqlparser/mysql_parser_services.h"
 
 namespace bec {
 
-  class WBPUBLICBACKEND_PUBLIC_FUNC DBObjectEditorBE : public BaseEditor
+  class WBPUBLICBACKEND_PUBLIC_FUNC DBObjectEditorBE : public BaseEditor, public grt::GRTObserver
   {
   public:
+    virtual ~DBObjectEditorBE();
+
     virtual bool should_close_on_delete_of(const std::string &oid);
 
-    virtual GrtObjectRef get_object() { return get_dbobject(); }
-    virtual db_DatabaseObjectRef get_dbobject()= 0;
-    virtual db_mgmt_RdbmsRef get_rdbms() { return _rdbms; }
-    GrtVersionRef get_rdbms_target_version();
+    virtual db_DatabaseObjectRef get_dbobject() { return db_DatabaseObjectRef::cast_from(get_object()); };
 
     virtual std::string get_name();
     virtual void set_name(const std::string &name);
@@ -43,17 +44,19 @@ namespace bec {
     virtual void set_comment(const std::string &descr);
 
     virtual std::string get_sql();
+    virtual void set_sql(const std::string &sql);
+
     virtual bool is_sql_commented();
     virtual void set_sql_commented(bool flag);
 
-    virtual Sql_editor::Ref get_sql_editor();
+    bool has_editor();
+    virtual MySQLEditor::Ref get_sql_editor();
     virtual void reset_editor_undo_stack();
 
     db_SchemaRef get_schema();
     virtual std::string get_schema_name();
 
     db_CatalogRef get_catalog();
-    
     db_SchemaRef get_schema_with_name(const std::string &schema_name);
 
     virtual std::vector<std::string> get_all_table_names();
@@ -68,67 +71,36 @@ namespace bec {
     std::string format_charset_collation(const std::string &charset, const std::string &collation);
 
     void update_change_date();
-    void sql_mode(const std::string &value);
+    void send_refresh();
+    void set_sql_mode(const std::string &value);
 
-  protected:
-    db_mgmt_RdbmsRef _rdbms;
-    Sql_editor::Ref _sql_editor;
-
-    DBObjectEditorBE(GRTManager *grtm, const db_DatabaseObjectRef &object, const db_mgmt_RdbmsRef &rdbms);
-
-    virtual std::string get_object_type();
-
-  public:
-    void check_sql();
-
-  public:
-    typedef std::vector<std::string> Log_messages;
-    typedef boost::function<void (const Log_messages&)> Sql_parser_log_cb;
-    typedef Sql_parser_base::Parse_error_cb Sql_parser_err_cb;
-
-    void set_sql_parser_log_cb(const Sql_parser_log_cb &cb);
-    void set_sql_parser_err_cb(const Sql_parser_err_cb &cb);
-    void set_sql(const std::string &sql, bool sync, const db_DatabaseObjectRef &template_obj, const std::string &comment= "");
-
-  protected:
-    typedef boost::function<grt::ValueRef (grt::GRT*, grt::StringRef)> Sql_parser_task_cb;
-    void set_sql_parser_task_cb(const Sql_parser_task_cb &cb);
-    Invalid_sql_parser::Ref _sql_parser;
-    std::string _non_std_sql_delimiter;
-
-  private:
-    void sql_parser_task_finished_cb(grt::ValueRef value);
-    void sql_parser_msg_cb(const grt::Message &msg);
-
-    Sql_parser_task_cb _sql_parser_task_cb;
-    Log_messages _sql_parser_log;
-    Sql_parser_log_cb _sql_parser_log_cb;
-    Sql_parser_err_cb _sql_parser_err_cb;
-
-  public:
     virtual bool is_editing_live_object();
     virtual void apply_changes_to_live_object();
     virtual void refresh_live_object();
-    bool is_server_version_at_least(int major, int minor);
+    virtual bool can_close();
 
     boost::function<bool (DBObjectEditorBE*, bool)> on_apply_changes_to_live_object;
     boost::function<void (DBObjectEditorBE*)> on_refresh_live_object;
     boost::function<void (DBObjectEditorBE*)> on_create_live_table_stubs;
     boost::function<bool (DBObjectEditorBE*, std::string&, std::string&)> on_expand_live_table_stub;
 
-  public:
-    virtual bool can_close();
+  protected:
+    parser::ParserContext::Ref _parser_context;
+    parser::ParserContext::Ref _autocompletion_context; // Temporary.
+    parser::MySQLParserServices::Ref _parser_services;
+
+    DBObjectEditorBE(GRTManager *grtm, const db_DatabaseObjectRef &object);
 
   private:
+    MySQLEditor::Ref _sql_editor;
+    db_CatalogRef _catalog;
+
     boost::signals2::scoped_connection _val_notify_conn;
     void notify_from_validation(const grt::Validator::Tag& tag, const grt::ObjectRef&, const std::string&, const int level);//level is grt::MessageType
     // Real-time validation part
     grt::MessageType    _last_validation_check_status;
     std::string         _last_validation_message;
     
-    bool custom_string_compare(const std::string &first, const std::string &second);
+    virtual void handle_grt_notification(const std::string &name, grt::ObjectRef sender, grt::DictRef info);
   };
 };
-
-
-#endif /* _EDITOR_DBOBJECT_H_ */
