@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _EDITOR_TABLE_H_
-#define _EDITOR_TABLE_H_
+#pragma once
 
 #include "grtdb/editor_dbobject.h"
 #include "grt/tree_model.h"
@@ -39,7 +38,13 @@ typedef boost::shared_ptr<Recordset> RecordsetRef;
 class Recordset_table_inserts_storage;
 typedef boost::shared_ptr<Recordset_table_inserts_storage> RecordsetTableInsertsStorageRef;
 
-namespace mforms { class Form; }
+namespace mforms {
+  class Form;
+  class View;
+  class Box;
+  class ContextMenu;
+  class RecordGrid;
+}
 
 namespace bec {
 
@@ -56,22 +61,22 @@ namespace bec {
   public:
     enum ColumnListColumns {
       Name,
-        Type,
-        IsPK,
-        IsNotNull,
-        IsUnique,
-        IsBinary,
-        IsUnsigned,
-        IsZerofill,
-        Flags,
-        Default,
-        CharsetCollation,
-        Charset,
-        Collation,
-        Comment,
-        HasCharset,
-      
-        LastColumn
+      Type,
+      IsPK,
+      IsNotNull,
+      IsUnique,
+      IsBinary,
+      IsUnsigned,
+      IsZerofill,
+      Flags,
+      Default,
+      CharsetCollation,
+      Charset,
+      Collation,
+      Comment,
+      HasCharset,
+
+      LastColumn
     };
 
     TableColumnsListBE(TableEditorBE *owner);
@@ -95,20 +100,26 @@ namespace bec {
     virtual IconId get_field_icon(const NodeId &node, int column, IconSize size);
 
     virtual void refresh();
-    virtual int count();
-    int real_count();
+    virtual size_t count();
+    size_t real_count();
 
     bool set_column_type(const NodeId &node, const GrtObjectRef &type);
 
     bool set_column_type_from_string(db_ColumnRef &column, const std::string &type);
     
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
-    virtual bool set_field(const NodeId &node, int column, int value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, ssize_t value);
 
-    virtual void reorder(const NodeId &node, int nindex);
-    void reorder_many(const std::vector<int> &rows, int nindex);
+    /**
+     * This is needed so we can reset placeholder info when then user cancelled the edit operation.
+     * Used in gtk frontend.
+     */
+    void reset_placeholder();
 
-    std::vector<std::string> get_datatype_flags(const ::bec::NodeId &node, bool all=false);
+    virtual void reorder(const NodeId &node, size_t nindex);
+    void reorder_many(const std::vector<size_t> &rows, size_t nindex);
+
+    std::vector<std::string> get_datatype_flags(const ::bec::NodeId &node, bool all = false);
     bool set_column_flag(const ::bec::NodeId &node, const std::string& flag_name, int is_set);
     int get_column_flag(const ::bec::NodeId &node, const std::string& flag_name);
 
@@ -128,16 +139,15 @@ namespace bec {
     
   protected:
     TableEditorBE *_owner;
-    int _editing_placeholder_row;
+    size_t _editing_placeholder_row;
     
     void update_primary_index_order();
     
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
   };
 
 
-
   class WBPUBLICBACKEND_PUBLIC_FUNC IndexColumnsListBE : public ListModel
   {
   public:
@@ -151,29 +161,28 @@ namespace bec {
     IndexColumnsListBE(IndexListBE *owner);
 
     virtual void refresh();
-    virtual int count();
+    virtual size_t count();
 
     void set_column_enabled(const NodeId &node, bool flag);
     bool get_column_enabled(const NodeId &node);
 
-    virtual bool set_field(const NodeId &node, int column, int value);
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, ssize_t value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
 
-    int get_max_order_index();
+    size_t get_max_order_index();
 
   protected:
     IndexListBE *_owner;
 
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
 
     db_IndexColumnRef get_index_column(const db_ColumnRef &column);
-    int get_index_column_index(const db_ColumnRef &column);
-    void set_index_column_order(const db_IndexColumnRef &column, int order);
+    size_t get_index_column_index(const db_ColumnRef &column);
+    void set_index_column_order(const db_IndexColumnRef &column, size_t order);
   };
 
 
-
   class WBPUBLICBACKEND_PUBLIC_FUNC IndexListBE : public ListModel
   {
     friend class IndexColumnsListBE;
@@ -190,15 +199,14 @@ namespace bec {
     IndexListBE(TableEditorBE *owner);
 
     virtual void refresh();
-    virtual int count();
-    int real_count();
+    virtual size_t count();
+    size_t real_count();
 
     // for editable lists only
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
 
     IndexColumnsListBE *get_columns() { return &_column_list; }
 
-
     db_IndexRef get_selected_index();
     void select_index(const NodeId &node);
 
@@ -215,7 +223,7 @@ namespace bec {
 
   protected:
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
 
     NodeId add_column(const db_ColumnRef &column, const db_IndexRef &index= db_IndexRef());
     void remove_column(const NodeId &node);
@@ -227,7 +235,6 @@ namespace bec {
   };
 
 
-
   class WBPUBLICBACKEND_PUBLIC_FUNC FKConstraintColumnsListBE : public ListModel
   {
   public:
@@ -240,29 +247,29 @@ namespace bec {
     FKConstraintColumnsListBE(FKConstraintListBE *owner);
 
     virtual void refresh();
-    virtual int count();
+    virtual size_t count();
 
-    std::vector<std::string> get_ref_columns_list(const NodeId &node, bool filtered=true);
+    std::vector<std::string> get_ref_columns_list(const NodeId &node, bool filtered = true);
 
     // for editable lists only
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
-    virtual bool set_field(const NodeId &node, int column, int value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, ssize_t value);
 
     bool set_column_is_fk(const NodeId &node, bool flag);
-    int get_fk_column_index(const NodeId &node);
+    ssize_t get_fk_column_index(const NodeId &node);
     bool get_column_is_fk(const NodeId &node);
 
     FKConstraintListBE *get_owner() { return _owner; }
 
   protected:
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
     
     bool set_fk_column_pair(const db_ColumnRef &column, const db_ColumnRef &refcolumn); 
     
     // temporary list of referenced columns for each FK column
     // if id is in the map, then it's enabled, if column is nil, it's unset
-    // only valid entries will be commited to actual table
+    // only valid entries will be committed to actual table
     std::map<std::string, db_ColumnRef> _referenced_columns; 
     
     FKConstraintListBE *_owner;
@@ -277,12 +284,12 @@ namespace bec {
   public:
     enum FKConstraintListColumns {
       Name,
-        OnDelete,
-        OnUpdate,
-        RefTable,
-        Comment,
-        Index,
-        ModelOnly
+      OnDelete,
+      OnUpdate,
+      RefTable,
+      Comment,
+      Index,
+      ModelOnly
     };
     FKConstraintListBE(TableEditorBE *owner);
 
@@ -292,12 +299,12 @@ namespace bec {
     virtual void remove_column(const NodeId &node);
 
     virtual void refresh();
-    virtual int count();
-    int real_count();
+    virtual size_t count();
+    size_t real_count();
 
     // for editable lists only
-    virtual bool set_field(const NodeId &node, int column, const std::string &value);
-    virtual bool set_field(const NodeId &node, int column, int value);
+    virtual bool set_field(const NodeId &node, ColumnId column, const std::string &value);
+    virtual bool set_field(const NodeId &node, ColumnId column, ssize_t value);
     
     void select_fk(const NodeId &node);
     db_ForeignKeyRef get_selected_fk();
@@ -313,16 +320,15 @@ namespace bec {
     virtual bool activate_popup_item_for_nodes(const std::string &name, const std::vector<NodeId>& nodes);
   protected:
     // for internal use only
-    virtual bool get_field_grt(const NodeId &node, int column, grt::ValueRef &value);
+    virtual bool get_field_grt(const NodeId &node, ColumnId column, grt::ValueRef &value);
     
   protected:
     FKConstraintColumnsListBE _column_list;
     TableEditorBE *_owner;
     NodeId _selected_fk;
-    int    _editing_placeholder_row;
+    size_t  _editing_placeholder_row;
   };
   
-  
   class WBPUBLICBACKEND_PUBLIC_FUNC TableEditorBE : public DBObjectEditorBE
   {
   public:
@@ -333,15 +339,20 @@ namespace bec {
       RefreshColumnList,
       RefreshColumnCollation
     };
-    virtual std::string get_title();
 
-    virtual db_DatabaseObjectRef get_dbobject() { return get_table(); }
-    virtual db_TableRef get_table()= 0;
+    TableEditorBE(GRTManager *grtm, const db_TableRef &table);
+
+    virtual std::string get_title();
+    virtual bool can_close();
+
+    db_TableRef get_table() { return db_TableRef::cast_from(get_object()); };
 
     virtual TableColumnsListBE *get_columns()= 0;
     virtual IndexListBE *get_indexes()= 0;
     FKConstraintListBE *get_fks() { return &_fk_list; }
+
     RecordsetRef get_inserts_model();
+    mforms::View *get_inserts_panel();
 
     virtual void set_name(const std::string &name);
 
@@ -353,7 +364,7 @@ namespace bec {
     virtual NodeId add_column(const std::string &name);
     virtual void remove_column(const NodeId &column);
     void rename_column(const db_ColumnRef &column, const std::string &name);
-    NodeId duplicate_column(const db_ColumnRef &col, int insert_after=-1);
+    NodeId duplicate_column(const db_ColumnRef &col, ssize_t insert_after = -1);
 
     db_ColumnRef get_column_with_name(const std::string &name);
 
@@ -365,7 +376,7 @@ namespace bec {
 
     // index editing
     virtual NodeId add_index(const std::string &name);
-    virtual bool remove_index(const NodeId &index);
+    virtual bool remove_index(const NodeId &index, bool delete_even_if_foreign);
 
     virtual NodeId add_index_with_columns(const std::vector<NodeId> &columns);
 
@@ -375,38 +386,31 @@ namespace bec {
 
     virtual std::vector<std::string> get_index_types()= 0;
 
-    virtual void set_triggers_sql(const std::string &sql, bool sync);
-    grt::ValueRef parse_triggers_sql(grt::GRT*, grt::StringRef sql);
-
     void show_export_wizard(mforms::Form *owner);
     void show_import_wizard();
     
-    virtual Sql_editor::Ref get_sql_editor();
+    virtual MySQLEditor::Ref get_sql_editor();
 
     virtual db_TableRef create_stub_table(const std::string &schema, const std::string &table) = 0;
 
-    protected:
-    db_TableRef _table;
-    FKConstraintListBE _fk_list;
 
-    TableEditorBE(GRTManager *grtm,
-                  const db_TableRef &table,
-                  const db_mgmt_RdbmsRef &rdbms);
+    void column_count_changed();
+  protected:
+    FKConstraintListBE _fk_list;
 
     void undo_called(grt::UndoAction *action, grt::UndoAction *expected);
 
   private:
+    mforms::Box *_inserts_panel;
+    mforms::RecordGrid *_inserts_grid;
     RecordsetRef _inserts_model;
     RecordsetTableInsertsStorageRef _inserts_storage;
+
+    void inserts_column_resized(int);
+    void restore_inserts_columns();
+
+    void update_selection_for_menu_extra(mforms::ContextMenu *menu, const std::vector<int> &rows, int column);
+    void open_field_editor(int row, int column);
   };
 
 };
-
-
-#ifdef _WIN32
-#pragma make_public(::bec::TableColumnsListBE)
-#pragma make_public(::bec::IndexListBE)
-#pragma make_public(::bec::TableEditorBE)
-#endif
-
-#endif /* _EDITOR_TABLE_H_ */
