@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,17 +16,19 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
 
 #include "grtdb_connection_editor.h"
 
 #include "cppdbc.h"
 #include <grt/common.h>
 #include "base/string_utilities.h"
+#include "base/log.h"
 
 #include "mforms/uistyle.h"
 #include "mforms/utilities.h"
 
+DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
+
 //------------------------------------------------------------------------------
 /*
 grtui::DbConnectionEditor::DbConnectionEditor(const db_mgmt_ManagementRef &mgmt, const grt::ListRef<db_mgmt_Rdbms> &allowed_rdbms)
@@ -145,7 +147,7 @@ db_mgmt_ConnectionRef grtui::DbConnectionEditor::run(const db_mgmt_ConnectionRef
   if ((index = _connection_list.get_index(connection)) != grt::BaseListRef::npos)
   {
     reset_stored_conn_list();
-    _stored_connection_list.select_node(_stored_connection_list.node_at_row(index));
+    _stored_connection_list.select_node(_stored_connection_list.node_at_row((int)index));
     change_active_stored_conn();
   }
   else
@@ -194,12 +196,16 @@ void grtui::DbConnectionEditor::reset_stored_conn_list()
   grt::ListRef<db_mgmt_Connection>::const_iterator last = list.end();
   for (; iter != last; ++iter)
   {
-    added_row = _stored_connection_list.root_node()->add_child();
-    if (added_row)
+    if (_panel.is_connectable_driver_type((*iter)->driver()))
     {
-      added_row->set_string(0, (*iter)->name());
-      if ((*iter)->name() == selected_name)
-        selected_node= added_row;
+      added_row = _stored_connection_list.root_node()->add_child();
+      if (added_row)
+      {
+        added_row->set_string(0, (*iter)->name());
+        added_row->set_tag((*iter)->id());
+        if ((*iter)->name() == selected_name)
+          selected_node= added_row;
+      }
     }
   }
 
@@ -212,7 +218,7 @@ void grtui::DbConnectionEditor::reset_stored_conn_list()
 void grtui::DbConnectionEditor::add_stored_conn(bool copy)
 {
   grt::ListRef<db_mgmt_Connection> list(_connection_list);
-  const int length = (std::string("New connection 1")).length() - 1;
+  size_t length = std::string("New connection 1").length() - 1;
   int max_conn_nr = 0;
   for (size_t i = 0; i < list.count(); ++i)
   {
@@ -233,6 +239,11 @@ void grtui::DbConnectionEditor::add_stored_conn(bool copy)
   new_connection->owner(_panel.get_be()->get_db_mgmt());
   new_connection->name(buf);
   new_connection->driver(_panel.selected_driver());
+  if (find_named_object_in_list(new_connection->driver()->parameters(), "useSSL").is_valid())
+  {
+    // prefer SSL if possible by default
+    new_connection->parameterValues().set("useSSL", grt::IntegerRef(1));
+  }
   list.insert(new_connection);
   
   if (copy)
@@ -240,14 +251,14 @@ void grtui::DbConnectionEditor::add_stored_conn(bool copy)
   else
     _panel.set_connection(new_connection);
   reset_stored_conn_list();
-  _stored_connection_list.select_node(_stored_connection_list.node_at_row(list.count()-1));
+  _stored_connection_list.select_node(_stored_connection_list.node_at_row((int)list.count() - 1));
   change_active_stored_conn();
 }
 
 //------------------------------------------------------------------------------
 void grtui::DbConnectionEditor::del_stored_conn()
 {
-  const int idx = _stored_connection_list.get_selected_row();
+  int idx = _stored_connection_list.get_selected_row();
   if (idx >= 0 && idx < (int)_connection_list.count())
   {
     grt::ListRef<db_mgmt_Connection> conns(_connection_list);
@@ -292,13 +303,32 @@ void grtui::DbConnectionEditor::del_stored_conn()
       }
     }
     if (!credentials_still_used)
-      mforms::Utilities::forget_password(host, user);
+    {
+      try
+      {
+        mforms::Utilities::forget_password(host, user);
+      }
+      catch (std::exception &exc)
+      {
+        log_warning("Exception caught when clearning the password: %s", exc.what());
+        mforms::Utilities::show_error("Clear Password", 
+                                      base::strfmt("Could not clear password: %s", exc.what()),
+                                      "OK");
+      }    
+    }
     
     if (idx < (int)conns.count())
+    {
       conns.remove(idx);
+
+      // Set the current connection to the one before the found index.
+      if (idx > 0)
+        --idx;
+      if (idx < (int)conns->count())
+        _panel.get_be()->set_connection_and_update(conns[idx]);
+    }
     
     reset_stored_conn_list();
-    change_active_stored_conn();
   }
 }
 
