@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grtdb_object_filter.h"
 #include "grt/grt_string_list_model.h"
 #include "text_input_dialog.h"
@@ -27,18 +25,20 @@
 using namespace grtui;
 using namespace base;
 
+//--------------------------------------------------------------------------------------------------
+
 static void refill_list(mforms::ListBox &list, bec::GrtStringListModel *model)
 {
   list.clear();
   for (size_t c= model->count(), i= 0; i < c; i++)
   {
     std::string item;
-    model->get_field(i, 0, item);
+    model->get_field((int)i, 0, item);
     list.add_item(item);
   }
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 DBObjectFilterFrame::DBObjectFilterFrame(bec::GRTManager *grtm)
 : mforms::Panel(mforms::BorderedPanel), _filter_be(grtm), _enabled_flag(NULL), _box(false),
@@ -142,7 +142,7 @@ DBObjectFilterFrame::DBObjectFilterFrame(bec::GRTManager *grtm)
   _detailed_table.add(&_filter_help_label, 0, 3, 8, 9, mforms::HFillFlag);
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::set_object_class(const std::string &oclass, const std::string &caption_format)
 {
@@ -162,7 +162,7 @@ void DBObjectFilterFrame::set_object_class(const std::string &oclass, const std:
   }
 }
 
-
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::set_models(bec::GrtStringListModel *model, bec::GrtStringListModel *excl_model, bool *enabled_flag)
 {
@@ -176,26 +176,35 @@ void DBObjectFilterFrame::set_models(bec::GrtStringListModel *model, bec::GrtStr
   else
     set_active(true);
 
-  refresh();
+  refresh(-1, -1);
 }
 
+//--------------------------------------------------------------------------------------------------
 
-void DBObjectFilterFrame::refresh()
+/**
+ * Reloads the models and the list boxes. Selects the given indices if > -1.
+ */
+void DBObjectFilterFrame::refresh(ssize_t object_list_selection, ssize_t mask_list_selection)
 {
   _model->refresh();
   _exclude_model->refresh();
   
   refill_list(_object_list, _model);
+  if (object_list_selection > -1 && object_list_selection < (ssize_t)_model->count())
+    _object_list.set_selected(object_list_selection);
+
   refill_list(_mask_list, _exclude_model);
+  if (mask_list_selection > -1 && mask_list_selection < (ssize_t)_exclude_model->count())
+    _mask_list.set_selected(mask_list_selection);
 
-  _summary_label.set_text(strfmt(_("%i Total Objects, %i Selected"), 
-                                 _model->total_items_count(), _model->active_items_count()));
+  std::stringstream out;
+  out << _model->total_items_count() << " Total Objects, " << _model->active_items_count() << " Selected";
+  _summary_label.set_text(out.str());
 
-  //_box.set_enabled(get_active());
-  
   update_button_enabled();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::update_button_enabled()
 {
@@ -203,6 +212,7 @@ void DBObjectFilterFrame::update_button_enabled()
   _del1_button.set_enabled(_mask_list.get_selected_index() >= 0);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::toggle_enabled()
 {
@@ -211,18 +221,21 @@ void DBObjectFilterFrame::toggle_enabled()
 //  _box.set_enabled(get_active());
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::set_active(bool flag)
 {
   _check.set_active(flag);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool DBObjectFilterFrame::get_active()
 {
   return _check.get_active();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::toggle_detailed()
 {
@@ -239,6 +252,7 @@ void DBObjectFilterFrame::toggle_detailed()
   relayout();
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::add_mask()
 {  
@@ -252,49 +266,60 @@ void DBObjectFilterFrame::add_mask()
     _exclude_model->add_item(dlg.get_value(), -1);
     _model->invalidate();
     
-    refresh();
+    refresh(-1, -1);
   }
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::add_clicked(bool all)
 {
   _filter_combo.set_selected(-1);
 
-  std::vector<int> indices;
-
+  std::vector<size_t> indices;
+  ssize_t new_selection = -1;
   if (all)
   {
-    for (int c= _model->count(), i= 0; i < c; i++)
+    for (size_t i = 0; i < _model->count(); ++i)
       indices.push_back(i);
   }
   else
-    indices= _object_list.get_selected_indices();
+  {
+    indices = _object_list.get_selected_indices();
+    new_selection = indices[0] - 1;
+    if (new_selection < 0)
+      new_selection = 0;
+  }
 
   _model->copy_items_to_val_masks_list(indices);
-  _model->invalidate();
-
-  refresh();
+  _model->invalidate(); // Weird work flow here. Need to mark the model as invalidate or it will refuse to refresh.
+  refresh(new_selection, -1);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 void DBObjectFilterFrame::del_clicked(bool all)
 {
   _filter_combo.set_selected(-1);
 
-  std::vector<int> indices;
-
+  std::vector<size_t> indices;
+  ssize_t new_selection = -1;
   if (all)
   {
-    for (int c= _exclude_model->count(), i= 0; i < c; i++)
+    for (size_t c = _exclude_model->count(), i = 0; i < c; i++)
       indices.push_back(i);
   }
   else
-    indices= _mask_list.get_selected_indices();
+  {
+    indices = _mask_list.get_selected_indices();
+    new_selection = indices[0] - 1;
+    if (new_selection < 0)
+      new_selection = 0;
+  }
 
   _exclude_model->remove_items(indices);
   _model->invalidate();
-
-  refresh();
+  refresh(-1, new_selection);
 }
 
+//--------------------------------------------------------------------------------------------------
