@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,16 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#include "stdafx.h"
+
+#include "base/log.h"
 
 #include "wizard_progress_page.h"
 #include "grt/common.h"
 #include "grt/grt_manager.h"
 
-using namespace grtui;
+DEFAULT_LOG_DOMAIN("wizard")
 
+using namespace grtui;
 
 void WizardProgressPage::TaskRow::set_state(WizardProgressPage::TaskState state)
 {
@@ -76,7 +78,7 @@ void WizardProgressPage::TaskRow::set_state(WizardProgressPage::TaskState state)
 
   std::string path= bec::IconManager::get_instance()->get_icon_path(file);
   if (path.empty())
-    g_message("Could not find icon %s", file.c_str());
+    log_warning("Could not find icon %s", file.c_str());
   icon.set_image(path);
 }
 
@@ -121,7 +123,8 @@ WizardProgressPage::WizardProgressPage(WizardForm *form, const std::string &id,
   _task_table.set_row_spacing(8);
   _task_table.set_column_count(2);
 
-  add(&_status_text, false, false);
+  _status_text.set_wrap_text(true);
+  add(&_status_text, false, true);
 
   if (has_progressbar)
   {
@@ -190,9 +193,9 @@ WizardProgressPage::TaskRow *WizardProgressPage::add_task(bool async,
 
   row->label.set_text(caption);
 
-  _task_table.set_row_count(_tasks.size()+1);
-  _task_table.add(&row->icon, 0, 1, _tasks.size(), _tasks.size()+1, 0);
-  _task_table.add(&row->label, 1, 2, _tasks.size(), _tasks.size()+1, mforms::HFillFlag);
+  _task_table.set_row_count((int)_tasks.size() + 1);
+  _task_table.add(&row->icon, 0, 1, (int)_tasks.size(), (int)_tasks.size() + 1, 0);
+  _task_table.add(&row->label, 1, 2, (int)_tasks.size(), (int)_tasks.size() + 1, mforms::HFillFlag);
 
   row->execute= execute;
   row->status_text= status_text;
