@@ -17,15 +17,13 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "recordset_cdbc_storage.h"
 #include "recordset_be.h"
 #include "sqlide_generics_private.h"
 #include "sqlide_generics.h"
 #include "grtsqlparser/sql_facade.h"
 #include "base/string_utilities.h"
+#include "base/sqlstring.h"
 #include <sqlite/query.hpp>
 #include <boost/cstdint.hpp>
 #include <boost/foreach.hpp>
@@ -122,6 +120,146 @@ private:
 };
 
 
+size_t Recordset_cdbc_storage::determine_pkey_columns(Recordset::Column_names &column_names, Recordset::Column_types &column_types, Recordset::Column_types &real_column_types)
+{
+  // a connection other than the user connection must be used for fetching metadata, otherwise we change the state of the connection
+  sql::Dbc_connection_handler::ConnectionRef aux_dbms_conn_ref= this->aux_dbms_conn_ref();
+
+  sql::DatabaseMetaData *conn_meta(aux_dbms_conn_ref->getMetaData());
+  try
+  {
+    // XXX this can be slow because of the I_S queries, depending on the server
+    std::auto_ptr<sql::ResultSet> rs(conn_meta->getBestRowIdentifier("", _schema_name, _table_name, 0, 0));
+    size_t rowid_col_count= rs->rowsCount();
+    if (rowid_col_count > 0)
+    {
+      _pkey_columns.reserve(rowid_col_count);
+      while (rs->next())
+      {
+        Recordset::Column_names::const_iterator i=
+        std::find(column_names.begin(), column_names.end(), rs->getString("COLUMN_NAME"));
+        if (i != column_names.end())
+        {
+          ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
+          column_names.push_back(column_names[col]);
+          column_types.push_back(column_types[col]);
+          real_column_types.push_back(real_column_types[col]);
+          _pkey_columns.push_back(col); // copy original value of pk field(s)
+        }
+        else
+          rowid_col_count--;
+      }
+
+      if (rowid_col_count != rs->rowsCount())
+      {
+        _readonly = true;
+        _readonly_reason = "To edit table data, the SELECT statement must include the primary key column(s).";
+      }
+    }
+    else
+    {
+      _readonly = true;
+      _readonly_reason = "The table has no unique row identifier (primary key or a NOT NULL unique index)";
+    }
+    return rowid_col_count;
+  }
+  catch (sql::SQLException &exc)
+  {
+    _readonly = true;
+    _readonly_reason = base::strfmt("Could not determine a unique row identifier (%s)", exc.what());
+  }
+  return 0;
+}
+
+
+size_t Recordset_cdbc_storage::determine_pkey_columns_alt(Recordset::Column_names &column_names, Recordset::Column_types &column_types, Recordset::Column_types &real_column_types)
+{
+  // a connection other than the user connection must be used for fetching metadata, otherwise we change the state of the connection
+  sql::Dbc_connection_handler::ConnectionRef aux_dbms_conn_ref= this->aux_dbms_conn_ref();
+
+  std::auto_ptr<sql::Statement> stmt(aux_dbms_conn_ref->createStatement());
+  std::string q = base::sqlstring("SHOW INDEX FROM !.!", 0) << _schema_name << _table_name;
+  try
+  {
+    std::auto_ptr<sql::ResultSet> rs(stmt->executeQuery(q));
+    std::list<std::string> primary_columns;
+    std::list<std::string> unique_notnull_columns, columns;
+
+    bool found_not_null = false;
+    std::string prev_key;
+    while (rs->next())
+    {
+      std::string column = rs->getString("Column_name");
+      std::string null = rs->getString("Null");
+      std::string key = rs->getString("Key_name");
+
+      if (key == "PRIMARY")
+      {
+        primary_columns.push_back(column);
+      }
+      else
+      {
+        // at least one of the columns must be NOT NULL in a UNIQUE key
+        if (prev_key != key)
+        {
+          if (found_not_null)
+            unique_notnull_columns = columns;
+          found_not_null = false;
+          columns.clear();
+        }
+        columns.push_back(column);
+        if (null == "")
+          found_not_null = true;
+      }
+    }
+
+    if (!primary_columns.empty())
+      columns = primary_columns;
+    else
+      columns = unique_notnull_columns;
+
+    if (!columns.empty())
+    {
+      size_t rowid_col_count= columns.size();
+      // now check whether the query contains all the columns in the keys
+      for (std::list<std::string>::const_iterator column = columns.begin(); column != columns.end(); ++column)
+      {
+        Recordset::Column_names::const_iterator i = std::find(column_names.begin(), column_names.end(), *column);
+
+        if (i != column_names.end())
+        {
+          ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
+          column_names.push_back(column_names[col]);
+          column_types.push_back(column_types[col]);
+          real_column_types.push_back(real_column_types[col]);
+          _pkey_columns.push_back(col); // copy original value of pk field(s)
+        }
+        else
+          rowid_col_count--;
+      }
+
+      if (rowid_col_count != columns.size())
+      {
+        _readonly = true;
+        _readonly_reason = "To edit table data, the SELECT statement must include the primary key column(s).";
+      }
+      return rowid_col_count;
+    }
+  }
+  catch (sql::SQLException &exc)
+  {
+    _readonly = true;
+    _readonly_reason = base::strfmt("Could not determine a unique row identifier (%s)", exc.what());
+    return 0;
+  }
+
+  _readonly = true;
+  _readonly_reason = "The table has no unique row identifier (primary key or a NOT NULL unique index)";
+
+  return 0;
+}
+
+
 void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connection *data_swap_db)
 {
   sql::Dbc_connection_handler::ConnectionRef dbms_conn_ref= this->dbms_conn_ref();
@@ -214,7 +352,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
         known_types["VARCHAR"]= std::string();
         known_types["CHAR"]= std::string();
 
-        known_types["GEOMETRY"]= sqlite::unknown_t();
+        known_types["GEOMETRY"]= sqlite::blob_ref_t();
 
         known_types["UNKNOWN"]= sqlite::unknown_t();
 
@@ -256,7 +394,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
   if (_gather_field_info)
   {
-    for (ColumnId i = 0; i < editable_col_count; ++i)
+    for (unsigned int i = 0; i < (unsigned int)editable_col_count; ++i)
     {
       FieldInfo info;
       info.catalog = rs_meta->getCatalogName(i+1);
@@ -264,7 +402,9 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
       info.table = rs_meta->getTableName(i+1);
       info.field = rs_meta->getColumnLabel(i+1);
       info.type = rs_meta->getColumnTypeName(i+1);
-//      ss << "Collation:" << "\n";
+#if defined(__APPLE__) || defined(_WIN32) || defined(MYSQLCPPCONN_VERSION_1_1_4)
+      info.charset = rs_meta->getColumnCharset(i+1);
+#endif
       info.display_size = rs_meta->getColumnDisplaySize(i+1);
       info.precision = rs_meta->getPrecision(i+1);
       info.scale = rs_meta->getScale(i+1);
@@ -278,7 +418,7 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
   column_types.reserve(editable_col_count);
   // some column types might be defined in derived class. don't redefine types for those columns.
-  for (ColumnId n= column_types.size(); editable_col_count > n; ++n)
+  for (unsigned int n = (unsigned int)column_types.size(); (unsigned int)editable_col_count > n; ++n)
   {
     std::string type_name= rs_meta->getColumnTypeName(n+1);
     type_name = base::toupper(type_name);
@@ -292,46 +432,15 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
   // column names
   column_names.reserve(editable_col_count);
   // some column names might be defined in derived class. don't redefine names for those columns.
-  for (ColumnId n= column_names.size(); editable_col_count > n; ++n)
+  for (unsigned int n = (unsigned int)column_names.size(); (unsigned int)editable_col_count > n; ++n)
     column_names.push_back(rs_meta->getColumnLabel(n+1));
 
   // determine pkey or unique identifier columns
   ColumnId rowid_col_count= 0;
   if (!_table_name.empty()) // we need PK info only for editable statements and table_name member is filled only for those
   {
-    // a connection other than the user connection must be used for fetching metadata, otherwise we change the state of the connection
-    sql::Dbc_connection_handler::ConnectionRef aux_dbms_conn_ref= this->aux_dbms_conn_ref();
-
-    sql::DatabaseMetaData *conn_meta(aux_dbms_conn_ref->getMetaData());
-    std::auto_ptr<sql::ResultSet> rs(conn_meta->getBestRowIdentifier("", _schema_name, _table_name, 0, 0));
-    rowid_col_count= rs->rowsCount();
-    if (rowid_col_count > 0)
-    {
-      _pkey_columns.reserve(rowid_col_count);
-      column_names.reserve(editable_col_count + rowid_col_count);
-      column_types.reserve(editable_col_count + rowid_col_count);
-      real_column_types.reserve(editable_col_count + rowid_col_count);
-      while (rs->next())
-      {
-        Recordset::Column_names::const_iterator i=
-          std::find(column_names.begin(), column_names.end(), rs->getString("COLUMN_NAME"));
-        if (i != column_names.end())
-        {
-          ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
-          column_names.push_back(column_names[col]);
-          column_types.push_back(column_types[col]);
-          real_column_types.push_back(real_column_types[col]);
-          _pkey_columns.push_back(col); // copy original value of pk field(s)
-        }
-        else
-          rowid_col_count--;
-      }
-    }
-    else
-    {
-      _readonly = true;
-      _readonly_reason = "The table has no unique row identifier (primary key or a NOT NULL unique index)";
-    }
+    // some day, I_S will be fast enough and we can switch to the non-alt version of the method for that server and above
+    rowid_col_count = determine_pkey_columns_alt(column_names, column_types, real_column_types);
   }
 
   // columns values of that must be null to signify that actual value to be fetched on-demand (e.g. when open blob editor)
@@ -348,17 +457,16 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
     create_data_swap_tables(data_swap_db, column_names, column_types);
 
-    ColumnId col_count= editable_col_count + rowid_col_count;
     FetchVar fetch_var(rs.get());
-    Var_vector row_values(col_count);
+    Var_vector row_values(editable_col_count + rowid_col_count);
 
     std::list<boost::shared_ptr<sqlite::command> > insert_commands= prepare_data_swap_record_add_statement(data_swap_db, column_names);
-
+    // XXX this will fetch all records before displaying them, which will result in a huge unnecessary lag in the UI
     while (rs->next())
     {
       for (ColumnId n= 0; editable_col_count > n; ++n)
       {
-        if (rs->isNull(n+1) || null_value_columns[n])
+        if (rs->isNull((int)n + 1) || null_value_columns[n])
         {
           row_values[n]= sqlite::null_t();
         }
@@ -451,7 +559,8 @@ public:
   result_type operator()(const std::string &v) { return result_type(new std::stringstream(v)); }
   template<typename V> result_type operator()(const V &t) { return result_type(new std::stringstream()); }
 };
-void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script)
+
+void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script, bool skip_transaction)
 {
   sql::Dbc_connection_handler::ConnectionRef dbms_conn= dbms_conn_ref();
 
@@ -505,14 +614,16 @@ void Recordset_cdbc_storage::run_sql_script(const Sql_script &sql_script)
   }
   if (err_count)
   {
-    dbms_conn->rollback();
+    if (!skip_transaction)
+      dbms_conn->rollback();
     msg= strfmt("%i error(s) saving changes to table %s", err_count, full_table_name().c_str());
     on_sql_script_run_statistics((processed_statement_count - err_count), err_count);
     throw std::runtime_error(msg.c_str());
   }
   else
   {
-    dbms_conn->commit();
+    if (!skip_transaction)
+      dbms_conn->commit();
     on_sql_script_run_statistics((processed_statement_count - err_count), err_count);
   }
 }
