@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,6 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
 #include "recordset_text_storage.h"
 #include "recordset_be.h"
 #include "base/string_utilities.h"
@@ -66,20 +63,21 @@ static void process_templates(const std::list<std::string> &files)
       Recordset_text_storage::TemplateInfo info;
       info.path = base::strip_extension(*f)+".tpl";
       info.name = base::strip_extension(base::basename(*f));
-      info.extension = cf.get_string("extension");
-      info.description = cf.get_string("description");
+      info.extension = cf.get_value("extension");
+      info.description = cf.get_value("description");
       info.pre_quote_strings = cf.get_bool("pre_quote_strings");
-      info.include_column_types = cf.get_string("include_column_types");
-      info.null_syntax = cf.get_string("null_syntax");
-      info.row_separator = cf.get_string("row_separator");
+      info.quote = cf.get_value("quote");
+      info.include_column_types = cf.get_value("include_column_types");
+      info.null_syntax = cf.get_value("null_syntax");
+      info.row_separator = cf.get_value("row_separator");
       if (info.include_column_types != "xls")
         info.include_column_types = "";
-      std::string args = cf.get_string("arguments");
+      std::string args = cf.get_value("arguments");
       std::vector<std::string> arguments = base::split_token_list(args, ';');
       for (std::vector<std::string>::const_iterator arg = arguments.begin(); arg != arguments.end(); ++arg)
       {
         std::string name, symbol;
-        if (base::partition(*arg, ":", name, symbol))
+        if (!arg->empty() && base::partition(*arg, ":", name, symbol))
           info.arguments.push_back(std::make_pair(base::unquote_identifier(name), symbol));
       }
       _templates[info.name] = info;
@@ -164,11 +162,15 @@ ColumnId Recordset_text_storage::aux_column_count()
 }
 
 
-void Recordset_text_storage::do_apply_changes(const Recordset *recordset, sqlite::connection *data_swap_db)
+void Recordset_text_storage::do_apply_changes(const Recordset *recordset, sqlite::connection *data_swap_db, bool skip_commitmig )
 {
   throw std::runtime_error("Recordset_text_storage::apply_changes is not implemented");
 }
 
+static std::string escape_sql_string_(const std::string &s)
+{
+  return base::escape_sql_string(s, false);
+}
 
 void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::connection *data_swap_db)
 {
@@ -233,7 +235,9 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
   ColumnId visible_col_count= recordset->get_column_count();
   sqlide::QuoteVar qv;
   {
-    qv.escape_string= std::ptr_fun(sqlide::QuoteVar::escape_ansi_sql_string);
+    if (info.quote != "")
+      qv.quote = info.quote;
+    qv.escape_string= std::ptr_fun(escape_sql_string_);
     // swap db (sqlite) stores unknown values as quoted strings
     qv.store_unknown_as_string= true;
     qv.allow_func_escaping= false;
@@ -270,7 +274,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (out_col_type == "String")
       {
         TemplateDictionary* col_index_dict = dict->AddSectionDictionary("STRING_COLUMN");
-        col_index_dict->SetIntValue("STRING_COLUMN_INDEX", col+1);
+        col_index_dict->SetIntValue("STRING_COLUMN_INDEX", (long)col + 1);
       }
     }
   }
@@ -304,7 +308,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (Recordset::emit_partition_queries(data_swap_db, data_queries, data_results))
       {
         bool next_row_exists= true;
-        sqlite::Variant v;
+        sqlite::variant_t v;
         do
         {
           TemplateDictionary row_dict_base("/");
@@ -323,7 +327,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
               TemplateDictionary* field_dict;
               ColumnId partition_column= col - col_begin;
               bool is_null;
-              data_rs->get_variant(partition_column, v);
+              v = data_rs->get_variant((int)partition_column);
 
               is_null = sqlide::is_var_null(v); // for some reason, the apply_visitor stuff isnt handling NULL
 
@@ -366,7 +370,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
             std::string result_text;
             tpl->Expand(&result_text, &row_dict_base);
             
-            fprintf(outf, "%s", result_text.c_str());
+            fwrite(result_text.data(), 1, result_text.size(), outf);
           }                  
         }
         while (next_row_exists);
@@ -377,8 +381,8 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
     {
       std::string result_text;
       post_tpl->Expand(&result_text, dict.get());
-      
-      fprintf(outf, "%s", result_text.c_str());
+
+      fwrite(result_text.data(), 1, result_text.size(), outf);
     }
 
     fclose(outf);
@@ -394,7 +398,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       if (Recordset::emit_partition_queries(data_swap_db, data_queries, data_results))
       {
         bool next_row_exists= true;
-        sqlite::Variant v;
+        sqlite::variant_t v;
         do
         {
           TemplateDictionary* row_dict = dict->AddSectionDictionary("ROW");
@@ -405,7 +409,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
                  col_end= std::min<ColumnId>(visible_col_count, (partition + 1) * Recordset::DATA_SWAP_DB_TABLE_MAX_COL_COUNT); col < col_end; ++col)
             {
               ColumnId partition_column= col - col_begin;
-              data_rs->get_variant(partition_column, v);
+              v = data_rs->get_variant((int)partition_column);
               TemplateDictionary* field_dict = row_dict->AddSectionDictionary("FIELD");
               field_dict->SetValueWithoutCopy("FIELD_NAME", (*column_names)[col]);
               std::string field_value;
@@ -434,7 +438,7 @@ void Recordset_text_storage::do_serialize(const Recordset *recordset, sqlite::co
       GError *error = 0;
       
       // use g_file_set because it can handle utf8 filenames
-      if (!g_file_set_contents(_file_path.c_str(), result_text.data(), result_text.size(), &error))
+      if (!g_file_set_contents(_file_path.c_str(), result_text.data(), (gssize)result_text.size(), &error))
       {
         std::string message = error->message;
         g_free(error);
@@ -451,7 +455,7 @@ void Recordset_text_storage::do_unserialize(Recordset *recordset, sqlite::connec
 }
 
 
-void Recordset_text_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::Variant &blob_value)
+void Recordset_text_storage::do_fetch_blob_value(Recordset *recordset, sqlite::connection *data_swap_db, RowId rowid, ColumnId column, sqlite::variant_t &blob_value)
 {
 }
 
