@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,7 @@
  * 02110-1301  USA
  */
 
-
-#include "stdafx.h"
-
+#include "grtdb/db_helpers.h"
 #include "sql_script_run_wizard.h"
 
 #include "mforms/code_editor.h"
@@ -48,7 +46,7 @@ SqlScriptReviewPage::SqlScriptReviewPage(grtui::WizardForm *form, GrtVersionRef
   // Add them only if online DDL options were given (which only happens for meta data changes).
   // This is not supposed to be shown for pre 5.6 servers
   if (!algorithm.empty() && !lock.empty() &&
-      (version.is_valid() && (version->majorNumber() > 5 || (version->majorNumber() == 5 && version->minorNumber() >= 6))))
+      (version.is_valid() && bec::is_supported_mysql_version_at_least(version, 5, 6)))
   {
     mforms::Panel *frame = mforms::manage(new mforms::Panel(mforms::TitledBoxPanel));
     frame->set_title(_("Online DDL"));
@@ -123,6 +121,9 @@ SqlScriptReviewPage::SqlScriptReviewPage(grtui::WizardForm *form, GrtVersionRef
     case 6:
       _sql_editor->set_language(mforms::LanguageMySQL56);
       break;
+    case 7:
+      _sql_editor->set_language(mforms::LanguageMySQL57);
+      break;
     default: // Should not be called actually. All valid versions should be handled above.
       _sql_editor->set_language(mforms::LanguageMySQL);
     }
@@ -134,9 +135,7 @@ SqlScriptReviewPage::SqlScriptReviewPage(grtui::WizardForm *form, GrtVersionRef
 
 SqlScriptReviewPage::~SqlScriptReviewPage()
 {
-  // do not release managed() objects
-//  _algorithm_selector->release();
-//  _lock_selector->release();
+  // No need to release _algorithm_selector and _lock_selector. They are managed with release_on_add.
   _sql_editor->release();
 };
 
@@ -190,7 +189,7 @@ void SqlScriptReviewPage::option_changed()
 
 SqlScriptApplyPage::SqlScriptApplyPage(grtui::WizardForm *form)
 :
-grtui::WizardProgressPage(form, "apply"),
+grtui::WizardProgressPage(form, "apply", true),
 _err_count(0)
 {
   set_title(_("Applying SQL script to the database ..."));
@@ -201,7 +200,7 @@ _err_count(0)
     boost::bind(&SqlScriptApplyPage::execute_sql_script, this),
     _("Executing SQL Statements..."));
 
-  end_adding_tasks(true, _("SQL script was successfully applied to the database."));
+  end_adding_tasks(_("SQL script was successfully applied to the database."));
 
   set_status_text("");
 }
@@ -298,6 +297,7 @@ SqlScriptRunWizard::SqlScriptRunWizard(bec::GRTManager *grtm, GrtVersionRef vers
   std::string algorithm, std::string lock)
   : grtui::WizardForm(grtm)
 {
+  set_name("script_run_wizard");
   set_title(_("Apply SQL Script to Database"));
   review_page = new SqlScriptReviewPage(this, version, algorithm, lock);
   add_page(mforms::manage(review_page));
