@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,11 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-#ifndef _BASE_BRIDGE_H_
-#define _BASE_BRIDGE_H_
 
-#include <mdc.h>
-#include <grtpp.h>
+#pragma once
+
+#include "mdc.h"
+#include "grtpp.h"
 
 #include "grts/structs.model.h"
 
@@ -35,8 +35,6 @@ protected:
   bec::GRTManager *get_grtm() { return bec::GRTManager::get_instance_for(get_object()->get_grt()); }
   
   void run_later(const boost::function<void ()> &slot);
-  static void run_later(grt::GRT *grt, const boost::function<void ()> &slot);
-  
 
   virtual GrtObject *get_object()= 0;
 
@@ -45,5 +43,3 @@ protected:
 public:
   virtual void unrealize()= 0;
 };
-
-#endif /* _BASE_BRIDGE_H_ */
