@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
+#include <cmath>
 
 #include "grts/structs.db.mgmt.h"
 
@@ -33,8 +33,6 @@
 
 using namespace base;
 
-DEFAULT_LOG_DOMAIN("AdvancedSidebar");
-
 #include "advanced_sidebar.h"
 
 #include <math.h>
@@ -61,6 +59,13 @@ using namespace mforms;
 #endif
 #endif
 
+#if !defined(__APPLE__) && !defined(_WIN32)
+#define BACK_COLOR ""
+#else
+#define BACK_COLOR "#d9e2ef"
+#endif
+
+
 #define SECTION_ENTRY_HEIGHT 20  // Height of a single section entry.
 #define SECTION_ENTRY_SPACING 0  // Vertical distance between two section entries.
 #define SECTION_ENTRY_INDENT 13  // Horizontal distance from the left section border to the entry icon.
@@ -74,7 +79,7 @@ SidebarEntry::SidebarEntry(const string& name, const string& title, const string
   _name = name;
   _title = title;
   if (!icon.empty())
-    _icon = Utilities::load_icon(icon);
+    _icon = Utilities::load_icon(icon, true);
   else
     _icon = NULL;
   _type = type;
@@ -103,7 +108,7 @@ void SidebarEntry::set_icon(const std::string &icon)
 {
   if (_icon)
     cairo_surface_destroy(_icon);
-  _icon = Utilities::load_icon(icon);
+  _icon = Utilities::load_icon(icon, true);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -115,9 +120,10 @@ void SidebarEntry::set_enabled(bool flag)
 
 //--------------------------------------------------------------------------------------------------
 
-
-void SidebarEntry::paint(cairo_t* cr, Rect bounds, bool hot, bool active, const Color& selection_color)
+void SidebarEntry::paint(cairo_t *cr, base::Rect bounds, bool hot, bool active, const Color& selection_color)
 {
+  _bounds = bounds;
+
   // Fill background if the item is active.
   if (active)
   {
@@ -131,10 +137,11 @@ void SidebarEntry::paint(cairo_t* cr, Rect bounds, bool hot, bool active, const
   double offset;
   if (_icon != NULL)
   {
-    offset = (bounds.height() - cairo_image_surface_get_height(_icon)) / 2;
-    cairo_set_source_surface(cr, _icon, bounds.left(), bounds.top() + offset);
-    cairo_paint(cr);
-    cairo_rel_move_to(cr, cairo_image_surface_get_width(_icon) + SECTION_ENTRY_ICON_SPACING, 0);
+    int width, height;
+    mforms::Utilities::get_icon_size(_icon, width, height);
+    offset = (bounds.height() - height) / 2;
+    mforms::Utilities::paint_icon(cr, _icon, bounds.left(), bounds.top() + offset);
+    cairo_rel_move_to(cr, width + SECTION_ENTRY_ICON_SPACING, 0);
   }
 
   cairo_select_font_face(cr, SIDEBAR_FONT, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);
@@ -174,30 +181,32 @@ bool SidebarEntry::contains(double x, double y)
 
 //----------------- SidebarSection -----------------------------------------------------------------
 
-SidebarSection::Button::Button(const std::string &icon_name, const std::string &alt_icon_name)
+SidebarSection::Button::Button(const std::string &accessible_name, const std::string &icon_name, const std::string &alt_icon_name)
 : icon(NULL), alt_icon(NULL), hot(false), down(false), state(false)
 {
+  _icon_name = icon_name;
+  _alt_icon_name = alt_icon_name;
+
+  _name = accessible_name;
   move(0, 0);
 
   if (!icon_name.empty())
-    icon = Utilities::load_icon(icon_name);
+    icon = Utilities::load_icon(icon_name, true);
   if (!alt_icon_name.empty())
-    alt_icon = Utilities::load_icon(alt_icon_name);
+    alt_icon = Utilities::load_icon(alt_icon_name, true);
 
   width = 0;
   height = 0;
   if (icon)
   {
-    width = cairo_image_surface_get_width(icon);
-    height = cairo_image_surface_get_height(icon);
+    Utilities::get_icon_size(icon, width, height);
   }
   else if (alt_icon)
   {
-    width = cairo_image_surface_get_width(alt_icon);
-    height = cairo_image_surface_get_height(alt_icon);
+    Utilities::get_icon_size(alt_icon, width, height);
   }
-  bounds_width = width;
-  bounds_height = height;
+  bounds_width = width + 4;
+  bounds_height = height + 4;
 }
 
 
@@ -212,12 +221,22 @@ SidebarSection::Button::~Button()
 
 void SidebarSection::Button::draw(cairo_t *cr)
 {
+  if (Utilities::icon_needs_reload(icon))
+  {
+    if (icon)
+      cairo_surface_destroy(icon);
+    if (alt_icon)
+      cairo_surface_destroy(alt_icon);
+    icon = Utilities::load_icon(_icon_name, true);
+    alt_icon = Utilities::load_icon(_alt_icon_name, true);
+  }
+
   cairo_surface_t *image = (state && alt_icon != NULL) ? alt_icon : icon;
 
   if (image)
   {
-    double image_left = x + 0.5;
-    double image_top = y + 0.5;
+    double image_left = x;
+    double image_top = y;
     if (hot || down)
     {
       // Draw a bezel shape under the icon.
@@ -245,10 +264,9 @@ void SidebarSection::Button::draw(cairo_t *cr)
       cairo_stroke(cr);
     }
 
-    image_left += (bounds_width - width) / 2;
-    image_top += (bounds_height - height) / 2;
-    cairo_set_source_surface(cr, image, image_left + 0.5, image_top + 0.5);
-    cairo_paint(cr);
+    image_left += floor((bounds_width - width) / 2 + 1);
+    image_top += floor((bounds_height - height) / 2 + 1);
+    Utilities::paint_icon(cr, image, image_left, image_top);
   }
 }
 
@@ -294,29 +312,34 @@ SidebarSection::SidebarSection(SimpleSidebar* owner, const std::string& name, co
   if (flags & mforms::TaskSectionRefreshable)
   {
 #ifdef __APPLE__
-    _refresh_button = new Button("refresh_sidebar_mac.png", "busy_sidebar_mac.png");
+    _refresh_button = new Button("Refresh", "wb-sidebar-refresh.png", "busy_sidebar_mac.png");
 #else
-    _refresh_button = new Button("refresh_sidebar.png", "busy_sidebar.png");
+    _refresh_button = new Button("Refresh", "refresh_sidebar.png", "busy_sidebar.png");
 #endif
+    _enabled_buttons.push_back(_refresh_button);
   }
   else
   {
 #ifdef __APPLE__
-    _refresh_button = new Button("", "busy_sidebar_mac.png");
+    _refresh_button = new Button("Busy", "", "busy_sidebar_mac.png");
 #else
-    _refresh_button = new Button("", "busy_sidebar.png");
+    _refresh_button = new Button("Busy", "", "busy_sidebar.png");
 #endif
   }
 
   if (flags & mforms::TaskSectionToggleModeButton)
   {
-    _toggle_mode_button = new Button("wb-sidebar-collapse.png", "wb-sidebar-expand.png");
+    _toggle_mode_button = new Button("Toggle Mode", "wb-sidebar-collapse.png", "wb-sidebar-expand.png");
+    _enabled_buttons.push_back(_toggle_mode_button);
     if (flags & TaskSectionToggleModeButtonPreSelected)
       _toggle_mode_button->state = true;
   }
 
   if (flags & mforms::TaskSectionShowConfigButton)
-    _config_button = new Button("wb_perform_config.png", "");
+  {
+    _config_button = new Button("Configuration", "wb_perform_config.png", "");
+    _enabled_buttons.push_back(_config_button);
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -414,22 +437,22 @@ void SidebarSection::layout(cairo_t* cr)
     _config_button->bounds_height = SECTION_HEADER_HEIGHT + SECTION_TOP_SPACING;
   }
 
-  if (_refresh_button)
-  {
-    _refresh_button->move((int)bounds.size.width - _refresh_button->bounds_width - SECTION_SIDE_SPACING, SECTION_TOP_SPACING);
-    _refresh_button->bounds_width = SECTION_HEADER_HEIGHT;
-    _refresh_button->bounds_height = SECTION_HEADER_HEIGHT;
-  }
-
   if (_toggle_mode_button)
   {
-    if (_refresh_button)
-      _toggle_mode_button->move(_refresh_button->x -_toggle_mode_button->bounds_width - SECTION_ICON_SPACING, SECTION_TOP_SPACING);
-    else
-      _toggle_mode_button->move((int)bounds.size.width - _toggle_mode_button->bounds_width - SECTION_SIDE_SPACING, SECTION_TOP_SPACING);
+    _toggle_mode_button->move((int)bounds.size.width - _toggle_mode_button->bounds_width - SECTION_SIDE_SPACING, SECTION_TOP_SPACING);
     _toggle_mode_button->bounds_width = SECTION_HEADER_HEIGHT;
     _toggle_mode_button->bounds_height = SECTION_HEADER_HEIGHT;
   }
+
+  if (_refresh_button)
+  {
+    if (_toggle_mode_button)
+      _refresh_button->move(_toggle_mode_button->x -_refresh_button->bounds_width - SECTION_ICON_SPACING, SECTION_TOP_SPACING);
+    else
+      _refresh_button->move((int)bounds.size.width - _refresh_button->bounds_width - SECTION_SIDE_SPACING, SECTION_TOP_SPACING);
+    _refresh_button->bounds_width = SECTION_HEADER_HEIGHT;
+    _refresh_button->bounds_height = SECTION_HEADER_HEIGHT;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -459,7 +482,7 @@ int SidebarSection::find_entry(const std::string& name)
   for (size_t i = 0; i < _entries.size(); i++)
   {
     if (_entries[i]->name() == name)
-      return i;
+      return (int)i;
   }
 
   return -1;
@@ -479,6 +502,17 @@ void SidebarSection::toggle_expand()
 
 //--------------------------------------------------------------------------------------------------
 
+void SidebarSection::update_mode_button(bool active)
+{
+  if (_toggle_mode_button != NULL)
+  {
+    _toggle_mode_button->state = active;
+    set_needs_repaint();
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 int SidebarSection::add_entry(const std::string& name, const std::string& title, const std::string& icon, TaskEntryType type)
 {
   int result = find_entry(name);
@@ -489,7 +523,7 @@ int SidebarSection::add_entry(const std::string& name, const std::string& title,
   _entries.push_back(entry);
   set_layout_dirty(true);
 
-  return _entries.size() -1 ;
+  return (int)_entries.size() - 1;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -567,6 +601,44 @@ bool SidebarSection::select(const std::string& name)
   }
 }
 
+//------------------------------------------------------------------------------------------------
+int SidebarSection::get_acc_child_count()
+{ 
+  return (int)(_entries.size() + _enabled_buttons.size());
+}
+
+//------------------------------------------------------------------------------------------------
+Accessible* SidebarSection::get_acc_child(int index)
+{ 
+  mforms::Accessible* accessible = NULL;
+
+  if ((size_t)index < _enabled_buttons.size())
+    accessible = _enabled_buttons[index];
+  else
+    accessible = _entries[index - _enabled_buttons.size()];
+  
+  return accessible;
+}
+
+
+//------------------------------------------------------------------------------------------------
+mforms::Accessible* SidebarSection::hit_test(int x, int y)
+{ 
+  mforms::Accessible* accessible = NULL;
+
+  //if (_expand_text_active)
+  if (_config_button && _config_button->check_hit(x, y))
+    accessible = _config_button;
+  else if (_refresh_button && _refresh_button->check_hit(x, y))
+    accessible = _refresh_button;
+  else if (_toggle_mode_button && _toggle_mode_button->check_hit(x, y))
+    accessible = _toggle_mode_button;
+  else
+    accessible = entry_from_point(x, y);
+
+  return accessible;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void draw_header_text(cairo_t* cr, Rect &bounds, const std::string& text, bool active)
@@ -642,8 +714,11 @@ void SidebarSection::repaint(cairo_t *cr, int areax, int areay, int areaw, int a
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_leave()
+bool SidebarSection::mouse_leave()
 {
+  if (DrawBox::mouse_leave())
+    return true;
+
   if (_hot_entry != NULL || _expand_text_visible || _expand_text_active ||
       (_config_button && _config_button->hot) ||
       (_refresh_button && _refresh_button->hot) ||
@@ -671,13 +746,19 @@ void SidebarSection::mouse_leave()
       _toggle_mode_button->hot = false;
     }
     set_needs_repaint();
+    return true;
   }
+
+  return false;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_move(int x, int y)
+bool SidebarSection::mouse_move(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_move(button, x, y))
+    return true;
+
   bool need_refresh = false;
   
   if (y < SECTION_TOP_SPACING + SECTION_HEADER_HEIGHT)
@@ -745,10 +826,10 @@ void SidebarSection::mouse_move(int x, int y)
     SidebarEntry* entry = entry_from_point(x, y);
     if (entry != _hot_entry)
     {
-      if (_hot_entry != NULL || entry->type() == mforms::TaskEntryLink)
+      if (_hot_entry != NULL || (entry != NULL && (entry->type() == mforms::TaskEntryLink || entry->type() == mforms::TaskEntryAlwaysActiveLink)))
         need_refresh = true;
 
-      if (entry != NULL && entry->type() == mforms::TaskEntryLink)
+      if (entry != NULL && (entry->type() == mforms::TaskEntryLink || entry->type() == mforms::TaskEntryAlwaysActiveLink))
         _hot_entry = entry; // Only links can appear as hot entries.
       else
         _hot_entry = NULL;
@@ -757,53 +838,71 @@ void SidebarSection::mouse_move(int x, int y)
 
   if (need_refresh)
     set_needs_repaint();
+
+  return need_refresh;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_down(int button, int x, int y)
+bool SidebarSection::mouse_down(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_down(button, x, y))
+    return true;
+
+  bool result = false;
   if (button == MouseButtonLeft)
   {
     if (_config_button && _config_button->hot)
     {
       _config_button->down = true;
       set_needs_repaint();
+      result = true;
     }
     else if (_refresh_button && _refresh_button->hot)
     {
       _refresh_button->down = true;
       set_needs_repaint();
+      result = true;
     }
     else if (_toggle_mode_button && _toggle_mode_button->hot)
     {
       _toggle_mode_button->down = true;
       set_needs_repaint();
+      result = true;
     }
     else if (_expand_text_visible)
     {
       _expand_text_active = true;
       set_needs_repaint();
+      result = true;
     }
     else
     {
       SidebarEntry* entry = entry_from_point(x, y);
       if (entry && entry->enabled() && entry->type() == mforms::TaskEntrySelectableItem)
+      {
         set_selected(entry);
+        result = true;
+      }
     }
   }
+
+  return result;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_click(int button, int x, int y)
+bool SidebarSection::mouse_click(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_click(button, x, y))
+    return true;
+
+  bool handled = false;
+
   switch (button)
   {
-  case MouseButtonLeft:
+    case MouseButtonLeft:
     {
-      bool handled = false;
-
       if (_expand_text_active)
       {
         handled = true;
@@ -811,75 +910,93 @@ void SidebarSection::mouse_click(int button, int x, int y)
         _expand_text_active = false;
         set_needs_repaint();
       }
-      else if (_config_button && _config_button->down)
+      else if (_config_button != NULL && _config_button->down)
       {
         handled = true;
         (*_owner->on_section_command())("configure");
       }
-      else if (_refresh_button && _refresh_button->down && _refresh_button->icon)
+      else if (_refresh_button != NULL && _refresh_button->down && _refresh_button->icon)
       {
         handled = true;
-        if (dynamic_cast<AdvancedSidebar*>(_owner))
-          dynamic_cast<AdvancedSidebar*>(_owner)->tool_action_clicked("refresh");
+        AdvancedSidebar *aSidebar;
+        if ((aSidebar = dynamic_cast<AdvancedSidebar*>(_owner)) != NULL)
+          aSidebar->tool_action_clicked("refresh");
       }
-      else if (_toggle_mode_button && _toggle_mode_button->down)
+      else if (_toggle_mode_button != NULL && _toggle_mode_button->down)
       {
         handled = true;
-        if (dynamic_cast<AdvancedSidebar*>(_owner))
-        {
-          if (_toggle_mode_button->state)
-            dynamic_cast<AdvancedSidebar*>(_owner)->tool_action_clicked("toggle_view_expand");
-          else
-            dynamic_cast<AdvancedSidebar*>(_owner)->tool_action_clicked("toggle_view_collapse");
-          _toggle_mode_button->state = !_toggle_mode_button->state;
-          set_needs_repaint();
-        }
+        bool new_state = !_toggle_mode_button->state;
+        if (new_state)
+          (*_owner->on_section_command())("switch_mode_on");
+        else
+          (*_owner->on_section_command())("switch_mode_off");
+        _owner->update_mode_buttons(new_state);
+        _toggle_mode_button->down = false;
+        _toggle_mode_button->hot = false;
       }
 
       if (!handled)
       {
         SidebarEntry* entry = entry_from_point(x, y);
-        if (entry && entry->enabled() && ((entry == _hot_entry) || (entry == _selected_entry)))
+        if (entry && (entry->enabled() || entry->type() == mforms::TaskEntryAlwaysActiveLink) && ((entry == _hot_entry) || (entry == _selected_entry)))
           (*_owner->on_section_command())(entry->name());
       }
     }
-    break;
+      break;
 
-  case MouseButtonRight:
-    /*
-    if (_context_menu != NULL && _selected_link != NULL)
-      _context_menu->popup_at(this, x + 5, y + 5);
-    */
-    break;
+    case MouseButtonRight:
+      /*
+       if (_context_menu != NULL && _selected_link != NULL)
+       _context_menu->popup_at(this, x + 5, y + 5);
+       */
+      break;
+      
+    default:
+      break;
   }
+
+  return handled;
 }
 
 //--------------------------------------------------------------------------------------------------
 
-void SidebarSection::mouse_up(int button, int x, int y)
+bool SidebarSection::mouse_up(mforms::MouseButton button, int x, int y)
 {
+  if (DrawBox::mouse_up(button, x, y))
+    return true;
+
+  bool result = false;
+
   switch (button)
   {
-  case MouseButtonLeft:
-    if (_config_button && _config_button->down)
-    {
-      _config_button->down = false;
-      set_needs_repaint();
-    }
+    case MouseButtonLeft:
+      if (_config_button && _config_button->down)
+      {
+        _config_button->down = false;
+        result = true;
+      }
 
-    if (_refresh_button && _refresh_button->down)
-    {
-      _refresh_button->down = false;
-      set_needs_repaint();
-    }
+      if (_refresh_button && _refresh_button->down)
+      {
+        _refresh_button->down = false;
+        result = true;
+      }
 
       if (_toggle_mode_button && _toggle_mode_button->down)
-    {
-      _toggle_mode_button->down = false;
-      set_needs_repaint();
-    }
-    break;
+      {
+        _toggle_mode_button->down = false;
+        result = true;
+      }
+      break;
+
+    default:
+      break;
   }
+  
+  if (result)
+    set_needs_repaint();
+
+  return result;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -892,8 +1009,8 @@ void SidebarSection::get_layout_size(int* w, int* h)
     layout(_layout_context);
   }
 
-  *w= _layout_width;
-  *h= _layout_height;
+  *w = (int)_layout_width;
+  *h = (int)_layout_height;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -908,7 +1025,7 @@ void SidebarSection::clear_selection()
     set_needs_repaint();
 }
 
-//----------------- SimpleSidebar ----------------------------------------------------------------
+//----------------- SimpleSidebar ------------------------------------------------------------------
 
 // implicitly initialize the adv. sidebar
 bool SimpleSidebar::__init = init_factory_method();
@@ -926,13 +1043,14 @@ SimpleSidebar::SimpleSidebar()
 {
   switch (Color::get_active_scheme())
   {
+  case base::ColorSchemeHighContrast:
   case base::ColorSchemeStandardWin7:
   case base::ColorSchemeStandardWin8:
   case base::ColorSchemeStandardWin8Alternate:
     set_back_color(base::Color::get_application_color_as_string(AppColorPanelContentArea, false));
     break;
   default:
-    set_back_color("#d9e2ef");
+    set_back_color(BACK_COLOR);
     break;
   }
 }
@@ -963,7 +1081,7 @@ int SimpleSidebar::find_section(const std::string& name)
   for (size_t i = 0; i < _sections.size(); i++)
   {
     if (_sections[i]->name() == name)
-      return i;
+      return (int)i;
   }
 
   return -1;
@@ -982,7 +1100,7 @@ int SimpleSidebar::add_section(const std::string &name, const string& title, mfo
   _sections.push_back(box);
   add(box, false, true);
 
-  return _sections.size() - 1;
+  return (int)_sections.size() - 1;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1099,12 +1217,12 @@ void SimpleSidebar::set_collapse_states(const std::string& data)
       if (section < 0)
         continue;
 
-      state = atoi(ptr+1);
+      state = atoi(ptr + 1);
     }
     else
       continue;
 
-    if ((state!=0) == _sections[section]->expanded())
+    if ((state != 0) == _sections[section]->expanded())
       _sections[section]->toggle_expand();
   }
 }
@@ -1206,6 +1324,14 @@ void SimpleSidebar::clear_selection()
     _sections[i]->clear_selection();
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void SimpleSidebar::update_mode_buttons(bool active)
+{
+  for (size_t i = 0; i < _sections.size(); i++)
+    _sections[i]->update_mode_button(active);
+}
+
 //----------------- AdvancedSidebar ----------------------------------------------------------------
 
 // implicitly initialize the adv. sidebar
@@ -1221,8 +1347,8 @@ bool AdvancedSidebar::init_factory_method()
 //--------------------------------------------------------------------------------------------------
 
 AdvancedSidebar::AdvancedSidebar()
-: _new_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader),
-_filtered_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader),
+: _new_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader | TreeCanBeDragSource),
+_filtered_schema_tree(TreeNoColumns | TreeNoBorder | TreeSidebar | TreeNoHeader | TreeCanBeDragSource),
 _schema_search_box(true),
 _schema_search_text(mforms::SmallSearchEntry),
 _remote_search_enabled(false),
@@ -1281,10 +1407,14 @@ mforms::TaskSidebar* AdvancedSidebar::create_instance()
  */
 void AdvancedSidebar::setup_schema_tree()
 {
-  std::string background_color = "#d9e2ef";
-  std::string toolbar_background_color = "#d9e2ef";
+  std::string background_color; // Empty, to indicate default color.
+  std::string toolbar_background_color = BACK_COLOR;
   switch (Color::get_active_scheme())
   {
+  case base::ColorSchemeHighContrast: // Don't touch the tree back color. We use the one provided by the OS.
+    toolbar_background_color = base::Color::get_application_color_as_string(AppColorPanelToolbar, false);
+    break;
+
   case base::ColorSchemeStandardWin7:
   case base::ColorSchemeStandardWin8:
   case base::ColorSchemeStandardWin8Alternate:
@@ -1292,7 +1422,8 @@ void AdvancedSidebar::setup_schema_tree()
     toolbar_background_color = base::Color::get_application_color_as_string(AppColorPanelToolbar, false);
     break;
 
-    default: // Set already.
+    default:
+      background_color = BACK_COLOR;
       break;
   }
 
@@ -1332,6 +1463,11 @@ void AdvancedSidebar::setup_schema_tree()
 #endif
 
   _schema_search_text.set_placeholder_text("Filter objects");
+  _schema_search_text.set_tooltip("You can use wildcards to search for objects in multiple schemas.\n"
+      "* - a substitue for zero or more characters\n"
+      "? - a substitue for single character\n"
+      "Search is possible only through already loaded schemas."
+      );
   _schema_search_box.add(&_schema_search_text, true, true);
   scoped_connect(_schema_search_text.signal_changed(), boost::bind(&AdvancedSidebar::on_search_text_changed, this));
   scoped_connect(_remote_search.signal_clicked(), boost::bind(&AdvancedSidebar::on_remote_search_clicked, this));
@@ -1351,10 +1487,14 @@ void AdvancedSidebar::handle_notification(const std::string &name, void *sender,
 {
   if (name == "GNColorsChanged")
   {
-    std::string background_color = "#d9e2ef";
-    std::string toolbar_background_color = "#d9e2ef";
+    std::string background_color; // Empty to indicate default color.
+    std::string toolbar_background_color = BACK_COLOR;
     switch (Color::get_active_scheme())
     {
+    case base::ColorSchemeHighContrast:
+      toolbar_background_color = base::Color::get_application_color_as_string(AppColorPanelToolbar, false);
+      break;
+
     case base::ColorSchemeStandardWin7:
     case base::ColorSchemeStandardWin8:
     case base::ColorSchemeStandardWin8Alternate:
@@ -1362,7 +1502,8 @@ void AdvancedSidebar::handle_notification(const std::string &name, void *sender,
       toolbar_background_color = base::Color::get_application_color_as_string(AppColorPanelToolbar, false);
       break;
 
-      default: // Set already.
+      default:
+        background_color = BACK_COLOR;
         break;
     }
 
