@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,14 +17,15 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
+#include "grtdb/db_helpers.h"
 #include "base/string_utilities.h"
 #include "new_server_instance_wizard.h"
 #include "grt/grt_manager.h"
 #include "grtui/grtdb_connection_editor.h"
 
 #include "mforms/uistyle.h"
+#include "base/log.h"
+DEFAULT_LOG_DOMAIN(DOMAIN_WB_CONTEXT_UI)
 
 #define INTRO_TEXT \
   "This wizard will guide you through the creation of a Server Profile to manage a MySQL server. "\
@@ -91,9 +92,8 @@ TestDatabaseSettingsPage::TestDatabaseSettingsPage(WizardForm* host)
   set_heading(_("The database connection information is being tested. This might take a few "
                 "moments depending on your network connection."));
 
-  grtui::WizardProgressPage::TaskRow *task;
 
-  task = add_task(_("Open Database Connection"),
+  add_task(_("Open Database Connection"),
                   boost::bind(&TestDatabaseSettingsPage::open_connection, this),
                   _("Connecting to database server..."));
 
@@ -177,12 +177,13 @@ bool TestDatabaseSettingsPage::get_server_version()
   current_task()->label.set_text("Server Version: " + version);
   add_log_text(strfmt("MySQL server version is %s", version.c_str()));
 
-  int major, minor;
-  if (sscanf(version.c_str(), "%i.%i", &major, &minor) == 2 && major < 5)
+  
+  // check that we're connecting to a known and supported version of the server
+  if (!bec::is_supported_mysql_version(version))
   {
     current_task()->label.set_text("Get Server Version: Unsupported Server Version");
-    add_log_text(strfmt("MySQL %i.%i is older than 5.0 and is not supported by MySQL Workbench", major, minor));
-    throw std::runtime_error(strfmt("MySQL %i.%i is older than 5.0 and is not supported by Workbench", major, minor));
+    add_log_text(strfmt("Unknown/unsupported server version or connection protocol detected (%s).\nMySQL Workbench is developed and test for MySQL Server versions 5.1, 5.5, 5.6 and 5.7.\nA connection can be established but some MySQL Workbench features may not work properly.", version.c_str()));
+    throw std::runtime_error(strfmt("Unknown/unsupported server version or connection protocol detected (%s).\nMySQL Workbench provides connectivity to MySQL Server versions 5.1, 5.5, 5.6 and 5.7.\nA connection can be established but some MySQL Workbench features may not work properly.", version.c_str()));
   }
 
   return true;
@@ -834,7 +835,21 @@ void WindowsManagementPage::enter(bool advancing)
 
       std::string title = strfmt(_("Remote Windows Login (%s)"), host.c_str()) + "|";
       title += _("Please enter a Windows user login and password for the remote server with rights to WMI");
-      if (!Utilities::credentials_for_service(title, "wmi@" + host, user, false, password))
+      
+      bool result = false;
+      try
+      {
+        result = Utilities::credentials_for_service(title, "wmi@" + host, user, false, password);
+      }
+      catch(std::exception &exc)
+      {
+        log_warning("Exception caught when clearning the password: %s", exc.what());
+        mforms::Utilities::show_error("Clear Password", 
+                                      base::strfmt("Could not clear password: %s", exc.what()),
+                                      "OK");
+      }
+      
+      if (!result)
       {
         _progress_label.set_text(_("Need valid user credentials to connect to server."));
         wizard()->set_problem(_("Need valid user credentials to connect to server."));
@@ -929,7 +944,18 @@ void WindowsManagementPage::enter(bool advancing)
 
       // In case this error was caused by wrong credentials (we cannot be sure from the error message)
       // we remove the already stored password to make it possible to (re) enter the current one.
-      Utilities::forget_password("wmi@" + host, user);
+      try
+      {
+        Utilities::forget_password("wmi@" + host, user);
+      }
+      catch (std::exception &exc)
+      {
+        log_warning("Exception caught when clearning the password: %s", exc.what());
+        mforms::Utilities::show_error("Clear Password", 
+                                      base::strfmt("Could not clear password: %s", exc.what()),
+                                      "OK");
+      }
+      
       values().gset("wmi_user_name", "");
     }
   }
@@ -1179,12 +1205,10 @@ void ReviewPage::enter(bool advancing)
     {
       std::string start= serverInfo.get_string("sys.mysqld.start");
       std::string stop= serverInfo.get_string("sys.mysqld.stop");
-      std::string status= serverInfo.get_string("sys.mysqld.status");
       bool use_sudo= serverInfo.get_int("sys.usesudo") != 0;
       summary.append(_("Commands for MySQL Management\n"));
       summary.append(strfmt(_("    Start MySQL:  %s\n"), start.c_str()));
       summary.append(strfmt(_("    Stop MySQL:  %s\n"), stop.c_str()));
-      summary.append(strfmt(_("    Check status:  %s\n"), status.c_str()));
       if (os_title != "Windows")
         summary.append(strfmt(_("    Use sudo:  %s\n"), use_sudo ? _("Yes") : _("No")));
     }
@@ -1441,7 +1465,7 @@ CommandsPage::CommandsPage(WizardForm* host)
   set_spacing(20);
   set_padding(8);
   _description.set_text(_("The values on this page comprise rather low level commands, which are used "
-                          "to control the MySQL server instance, check its status and so on.\n\n"
+                          "to control the MySQL server instance (start or stop it) and others.\n\n"
                           "If you are unsure what these values mean leave them untouched. The defaults "
                           "are usually a good choice already (for single server machines)."));
   _description.set_wrap_text(true);
@@ -1449,7 +1473,7 @@ CommandsPage::CommandsPage(WizardForm* host)
   
   _content.set_column_count(2);
   _content.set_column_spacing(8);
-  _content.set_row_count(5);
+  _content.set_row_count(4);
   _content.set_row_spacing(5);
   
   _start_label.set_text(_("Command to start the MySQL server:"));
@@ -1461,15 +1485,10 @@ CommandsPage::CommandsPage(WizardForm* host)
   _stop_label.set_text_align(MiddleRight);
   _content.add(&_stop_label, 0, 1, 1, 2, HFillFlag);
   _content.add(&_stop_command, 1, 2, 1, 2, HFillFlag | HExpandFlag);
-  
-  _status_label.set_text(_("Status check command:"));
-  _status_label.set_text_align(MiddleRight);
-  _content.add(&_status_label, 0, 1, 2, 3, HFillFlag);
-  _content.add(&_status_command, 1, 2, 2, 3, HFillFlag | HExpandFlag);
-  
+    
   _use_sudo.set_text(_("Check this box if you want or need the above commands \n"
                        "to be executed with elevated Operating System Privileges."));
-  _content.add(&_use_sudo, 1, 2, 4, 5, HFillFlag);
+  _content.add(&_use_sudo, 1, 2, 3, 4, HFillFlag);
   
   add(&_content, false, true);
 }
@@ -1490,7 +1509,6 @@ void CommandsPage::enter(bool advancing)
     // Prefill values from defaults.
     _start_command.set_value(wizard()->get_server_info("sys.mysqld.start"));
     _stop_command.set_value(wizard()->get_server_info("sys.mysqld.stop"));
-    _status_command.set_value(wizard()->get_server_info("sys.mysqld.status"));
     _use_sudo.set_active(wizard()->get_server_info("sys.usesudo") != "0");
   }
 }
@@ -1501,7 +1519,6 @@ bool CommandsPage::advance()
 {
   values().gset("command_start", base::trim(_start_command.get_string_value()));
   values().gset("command_stop", base::trim(_stop_command.get_string_value()));
-  values().gset("command_status", base::trim(_status_command.get_string_value()));
   values().gset("use_sudo", _use_sudo.get_active());
 
   return true;
@@ -1655,12 +1672,10 @@ db_mgmt_ServerInstanceRef NewServerInstanceWizard::assemble_server_instance()
   {
     std::string start= values().get_string("command_start");
     std::string stop= values().get_string("command_stop");
-    std::string status= values().get_string("command_status");
     bool use_sudo= values().get_int("use_sudo") != 0;
   
     _instance->serverInfo().gset("sys.mysqld.start", start);
     _instance->serverInfo().gset("sys.mysqld.stop", stop);
-    _instance->serverInfo().gset("sys.mysqld.status", status);
     _instance->serverInfo().gset("sys.usesudo", use_sudo);
   }
 
