@@ -3,6 +3,7 @@
 #include <gtkmm/cellrenderercombo.h>
 #include <gtkmm/icontheme.h>
 #include "grt/common.h"
+#include "base/string_utilities.h"
 
 //#define DEBUG
 #undef DEBUG
@@ -314,6 +315,22 @@ ColumnsModel::append_double_column(const int bec_tm_idx, const std::string& name
 }
 
 //------------------------------------------------------------------------------
+void ColumnsModel::disable_edit_first_row(Gtk::CellRenderer* cell, const Gtk::TreeIter& iter)
+{
+  Gtk::CellRendererText *txt = (Gtk::CellRendererText *)cell;
+  if(txt)
+  {
+    Gtk::TreeModel::Path path = this->_treeview->get_model()->get_path(iter);
+    if (path[0] == 0)
+      txt->property_editable() = false;
+    else
+      txt->property_editable() = true;
+  }
+
+
+
+}
+
 const StringColumn&
 ColumnsModel::append_string_column(const int bec_tm_idx, const std::string& name, const Editable editable, const Iconic have_icon)
 {
@@ -341,8 +358,9 @@ ColumnsModel::append_string_column(const int bec_tm_idx, const std::string& name
   int nr_of_cols= _treeview->append_column(*column);
   _treeview->get_column(nr_of_cols-1)->set_resizable(true);
 
-  if (editable == EDITABLE)
+  if (editable == EDITABLE || editable == EDITABLE_WO_FIRST)
   {
+
     std::vector<Gtk::CellRenderer*> rends= column->get_cell_renderers();
 
     Gtk::CellRendererText *cell = (Gtk::CellRendererText*)rends[icon ? 1 : 0];
@@ -354,6 +372,9 @@ ColumnsModel::append_string_column(const int bec_tm_idx, const std::string& name
                                                 , sigc::ref(*col)
                                   )
                                  );
+
+    if (editable == EDITABLE_WO_FIRST)
+      column->set_cell_data_func(*cell, sigc::mem_fun(this, &ColumnsModel::disable_edit_first_row));
   }
 
   return *col;
@@ -946,7 +967,7 @@ void ListModelWrapper::get_value_vfunc(const iterator& iter, int column, Glib::V
       case G_TYPE_INT:
       case G_TYPE_UINT:
         {
-          int iv = 0;
+          ssize_t iv = 0;
           (*_tm)->get_field(node, column, iv);
           set_glib_int(value, iv);
           lmwdprint("LMW::get_value_vfunc: %s: node %s, col %i: value: %i\n", _name.c_str(), node.repr().c_str(), column, iv);
@@ -1023,13 +1044,13 @@ void ListModelWrapper::set_value_impl(const iterator& row, int column, const Gli
           Glib::Value<bool>   v;
           v.init(value.gobj());
           lmwdprint("LMW::set_value_impl:%s node %s, column %i, value %i\n", _name.c_str(), node.repr().c_str(), column, v.get());
-          (*_tm)->set_field(node, column, v.get());
+          (*_tm)->set_field(node, column, (ssize_t)v.get());
           break;
         }
       case G_TYPE_INT:
       case G_TYPE_UINT:
         {
-          Glib::Value<int>   v;
+          Glib::Value<ssize_t>   v;
           v.init(value.gobj());
           lmwdprint("LMW::set_value_impl: node %s, column %i, value %i\n", node.repr().c_str(), column, v.get());
           (*_tm)->set_field(node, column, v.get());
