@@ -96,7 +96,7 @@ class ListModelWrapper;
 Gtk::TreeModel::Path node2path(const ::bec::NodeId& node);
 
 //! Enums to define columns view and behaviour. These are used in various ColumnsModel::append_*_column
-enum Editable {RO = 0, EDITABLE = 1};
+enum Editable {RO = 0, EDITABLE = 1, EDITABLE_WO_FIRST = 2};
 //! Enums to define columns view and behaviour. These are used in various ColumnsModel::append_*_column
 enum Iconic {NO_ICON = 0, WITH_ICON = 1};
 enum ToggleAction {TOGGLE_BY_WRAPPER, TOGGLE_BY_GTKMM};
@@ -139,6 +139,7 @@ class ColumnsModel : public Gtk::TreeModel::ColumnRecord
     //!
     const IntColumn&        append_int_column(const int bec_tm_idx, const std::string& name, const Editable editable= RO);
     const DoubleColumn&  append_double_column(const int bec_tm_idx, const std::string& name, const Editable editable= RO);
+    void disable_edit_first_row(Gtk::CellRenderer* cell, const Gtk::TreeIter& iter);
     const StringColumn&  append_string_column(const int bec_tm_idx, const std::string& name, const Editable editable= RO, const Iconic have_icon= NO_ICON);
     const StringColumn&  append_markup_column(const int bec_tm_idx, const std::string& name, const Iconic have_icon= NO_ICON);
     const StringColumn& append_combo_column(const int bec_tm_idx, const std::string& name, Glib::RefPtr<Gtk::ListStore> list_w, const Editable editable= RO, bool popup_only=false);
