@@ -312,7 +312,8 @@ on_cell_data(Gtk::CellRenderer* cr, const Gtk::TreeModel::iterator &iter, Gtk::T
     editing_iter= tree->get_model()->get_iter(_editing_path);
 
   load_cell_data(_property_data, iter->get_value(*_model_data_column), (_editing && editing_iter == iter), _floating_point_visible_scale);
-  _property_pixbuf= iter->get_value(*_model_pixbuf_column);
+  if (_model_pixbuf_column)
+    _property_pixbuf= iter->get_value(*_model_pixbuf_column);
 }
 
 //------------------------------------------------------------------------------
@@ -393,7 +394,11 @@ template <typename Renderer, typename RendererValueType, typename ModelValueType
 void CustomRenderer<Renderer, RendererValueType, ModelValueType>::on_editing_canceled()
 {
   _editing= false;
-  set_edit_state(-1);
+  Gtk::TreePath path;
+  Gtk::TreeViewColumn *col;
+  _treeview->get_cursor(path, col);
+  if (!path.empty())
+    set_edit_state(path[0]);
  _data_renderer.on_editing_canceled();
 }
 
@@ -402,7 +407,11 @@ template <typename Renderer, typename RendererValueType, typename ModelValueType
 void CustomRenderer<Renderer, RendererValueType, ModelValueType>::on_editing_done(Gtk::CellEditable* editable)
 {
   _editing= false;
-  set_edit_state(-1);
+  Gtk::TreePath path;
+  Gtk::TreeViewColumn *col;
+  _treeview->get_cursor(path, col);
+  if (!path.empty())
+    set_edit_state(path[0]);
   _on_editing_done.disconnect();
 }
 
