@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,9 +17,10 @@
  * 02110-1301  USA
  */
 
-#include "tut_stdafx.h"
 #include "base/file_utilities.h"
 #include "base/string_utilities.h"
+#include "base/threading.h"
+#include "wb_helpers.h"
 
 using namespace base;
 
@@ -99,7 +100,8 @@ TEST_DATA_CONSTRUCTOR(file_utilities_test)
   remove(file_unicode_name);
 
   // Initialize the thread system
-  if (!g_thread_supported ()) g_thread_init (NULL);
+  if (!g_thread_supported ()) 
+    base::threading_init();
 }
 
 TEST_DATA_DESTRUCTOR(file_utilities_test)
@@ -2215,7 +2217,7 @@ TEST_FUNCTION(55)
       std::list<std::string> test_result = base::scan_for_files_matching(search_pattern);
 
       ensure("TEST 55.1: File matching returned an empty list",!test_result.empty());
-      ensure_equals("TEST 55.2: Invalid list size", test_result.size(), 4);
+      ensure_equals("TEST 55.2: Invalid list size", test_result.size(), 4U);
 
       while (!test_result.empty())
       {
@@ -2236,7 +2238,7 @@ TEST_FUNCTION(55)
       std::list<std::string> test_result = base::scan_for_files_matching(search_pattern,true);
 
       ensure("TEST 55.1: File matching returned an empty list",!test_result.empty());
-      ensure_equals("TEST 55.2: Invalid list size", test_result.size(), 5);
+      ensure_equals("TEST 55.2: Invalid list size", test_result.size(), 5U);
 
       while (!test_result.empty())
       {
@@ -2304,6 +2306,9 @@ TEST_FUNCTION(60)
  */
 TEST_FUNCTION(65)
 {
+  remove(TEST_FILE_NAME01);
+  remove(TEST_FILE_NAME02);
+
   try
   {
       // test cases for constructor, check(const std::string &path)
@@ -2338,7 +2343,7 @@ TEST_FUNCTION(65)
   }
   catch (std::exception &exc)
   {
-      throw grt::os_error(strfmt("Runtime error: %s", exc.what()));
+    throw grt::os_error(strfmt("Runtime error: %s", exc.what()));
   }
 
   // Clean leftover test files
@@ -2549,7 +2554,7 @@ TEST_FUNCTION(70)
     }
 
     // Kick off the child thread
-    _child_thread = g_thread_create(_child_thread_func, this, TRUE, NULL);
+    _child_thread = base::create_thread(_child_thread_func, this);
     // Wait for _main_thread to finish
     g_thread_join(_child_thread);
 }
