@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,8 @@
  */
 #include "../lf_mforms.h"
 #include "../lf_form.h"
+#include "../lf_menubar.h"
+#include "../lf_wizard.h"
 
 static GThread *_main_thread = 0;
 
@@ -35,10 +37,15 @@ void FormImpl::set_title(::mforms::Form *self, const std::string &title)
   FormImpl* form = self->get_data<FormImpl>();
   if (form)
   {
-    form->_window->set_title(title);
+    form->set_title(title);
   }
 }
 
+void FormImpl::set_title(const std::string &title)
+{
+  _window->set_title(title);
+}
+
 void FormImpl::accept_clicked(bool *status, const bool is_run)
 {
   *status= true;
@@ -207,6 +214,23 @@ void FormImpl::center(Form *self)
 }
 
 
+void FormImpl::set_menubar(mforms::Form *self, mforms::MenuBar *menu)
+{
+  FormImpl* form = self->get_data<FormImpl>();
+  Gtk::MenuBar *mbar = widget_for_menubar(menu);
+  if (form && mbar)
+  {
+    Gtk::Box *box = dynamic_cast<Gtk::Box*>(self->get_content()->get_data<ViewImpl>()->get_inner());
+    if (!box)
+      throw std::logic_error("set_menubar called on a window without a Box as toplevel content");
+    box->pack_start(*mbar, false, true);
+    box->reorder_child(*mbar, 0);
+
+    on_add_menubar_to_window(menu, form->_window);
+  }
+}
+
+
 FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, mforms::FormFlag form_flag)
   : ViewImpl(form), _in_modal_loop(0), _result(false)
 {
@@ -214,12 +238,25 @@ FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, mforms::FormFlag
 
   if (owner)
   {
-    FormImpl* impl = owner->get_data<FormImpl>();
-    if (impl)
+    if (dynamic_cast<mforms::Wizard*>(owner))
     {
-      Gtk::Window *w = impl->get_window();
-      if (w)
-        _window->set_transient_for(*w);
+      WizardImpl* impl = owner->get_data<WizardImpl>();
+      if (impl)
+      {
+        Gtk::Window *w = impl->get_window();
+        if (w)
+          _window->set_transient_for(*w);
+      }
+    }
+    else
+    {
+      FormImpl* impl = owner->get_data<FormImpl>();
+      if (impl)
+      {
+        Gtk::Window *w = impl->get_window();
+        if (w)
+          _window->set_transient_for(*w);
+      }
     }
   }
 
@@ -232,7 +269,21 @@ FormImpl::FormImpl(::mforms::Form *form, ::mforms::Form *owner, mforms::FormFlag
   if (form_flag & mforms::FormMinimizable)
     flags |= Gdk::DECOR_MINIMIZE;
 
-  _window->signal_realize().connect(sigc::bind(sigc::mem_fun(this, &FormImpl::realized), form, flags)); 
+  _window->set_events(Gdk::FOCUS_CHANGE_MASK);
+  _window->signal_realize().connect(sigc::bind(sigc::mem_fun(this, &FormImpl::realized), form, flags));
+
+  _window->signal_focus_in_event().connect(sigc::bind< ::mforms::Form *>(sigc::mem_fun(this, &FormImpl::on_focus_event), form));
+  _window->signal_focus_out_event().connect(sigc::bind< ::mforms::Form *>(sigc::mem_fun(this, &FormImpl::on_focus_event), form));
+}
+
+
+bool FormImpl::on_focus_event(GdkEventFocus* ev, ::mforms::Form *form)
+{
+  if (ev->in)
+    form->activated();
+  else
+    form->deactivated();
+  return false;
 }
 
 void FormImpl::realized(mforms::Form *owner, Gdk::WMDecoration flags)
@@ -248,6 +299,11 @@ void FormImpl::set_size(int width, int height)
   _window->set_size_request(width, height);
 }
 
+void FormImpl::set_name(const std::string &name)
+{
+  _window->set_role(name);
+}
+
 void FormImpl::init_main_form(Gtk::Window* main)
 {
   mforms::Form* the_main_form = mforms::Form::main_form();
@@ -273,6 +329,7 @@ void FormImpl::init()
   f->_form_impl.set_content= &FormImpl::set_content;
   f->_form_impl.flush_events= &FormImpl::flush_events;
   f->_form_impl.center      = &FormImpl::center;
+  f->_form_impl.set_menubar = &FormImpl::set_menubar;
 }
 
 };
