@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,7 @@
  * 02110-1301  USA
  */
  
-#include "config.h"
+//#include "config.h"
 #if defined(HAVE_GNOME_KEYRING) || defined(HAVE_OLD_GNOME_KEYRING)
 extern "C" {
   #include <gnome-keyring.h>
@@ -26,14 +26,19 @@ extern "C" {
   #define USE_KEYRING 1
 #endif
 
-#include <sys/mman.h>
 #include <glib/gstdio.h>
+#ifdef HAVE_PRCTL_H
+#include <sys/prctl.h>
+#endif
 
 // this is broken
 //#undef USE_KEYRING
 #include "../lf_mforms.h"
 
 #include "base/file_functions.h"
+#include "base/file_utilities.h"
+#include "base/log.h"
+#include "base/string_utilities.h"
 #include "../lf_utilities.h"
 #include "gtk_helpers.h"
 #include "mforms.h"
@@ -41,154 +46,40 @@ extern "C" {
 namespace mforms {
 namespace gtk {
 
-class PasswordCache
+/**
+  * Get the current active window for this application
+  */
+GtkWindow *get_current_window()
 {
-  char *storage;
-  size_t storage_len;
-  size_t storage_size;
+  GList *window_list = gtk_window_list_toplevels();
+  do
+  {
+    GtkWindow *wnd = (GtkWindow *)window_list->data;
+    
+    if (gtk_window_is_active(wnd))
+      return wnd;
+  } while((window_list = g_list_next(window_list)) != NULL);
 
-  PasswordCache()
-  {
-    storage_len = 0;
-    storage_size = 4*1024;
-    storage = (char*)malloc(storage_size);
-    if (!storage)
-      perror("malloc password cache");
-    else
-      if (mlock(storage, storage_size) < 0)
-      {
-        perror("mlock password cache");
-        free(storage);
-        storage = NULL;
-      }
-  }
-  static PasswordCache instance;
-public:
-  static PasswordCache *get()
-  {
-    return &instance;
-  }
+  return NULL;
+}
+
+/**
+ *  Try to set this dialog as transcient to the parent window. Setting this
+ *  to the main window could stick this dialog to the wrong window (leaving
+ *  a window in between them in the cases like popping a dialog in the
+ *  scripting shell.
+ *  To find the proper parent window, we're searching for the current active window
+ *  on this application.
+ */
+void set_dialog_transcient(Gtk::MessageDialog &dialog)
+{
+  GtkWindow *parent_window = get_current_window();
   
-  ~PasswordCache()
-  {
-    if (storage)
-    {
-      memset(storage, 0, storage_size);
-      if (munlock(storage, storage_size) < 0)
-        perror("munlock password cache");
-      free(storage);
-    }
-  }
-
-  void add_password(const std::string &service, const std::string &account, const char *password)
-  {
-    if (storage)
-    {
-      if (!password) password = "";
-      
-      {
-        const char *opassword = find_password(service, account);
-        if (opassword)
-        {
-          if (strcmp(password, opassword) == 0)
-            return;
-          remove_password(service, account);
-        }
-      }
-
-      size_t reclen = sizeof(size_t) + service.size() + 1 + account.size() + 1 + strlen(password) + 1;
-      
-      // increase buffer size if new record doesnt fit
-      while (storage_len + reclen > storage_size)
-      {
-        char *new_block;
-        size_t new_size = storage_size + 4*1024;
-
-        new_block = (char*)malloc(new_size);
-        if (new_block)
-        {
-          if (mlock(new_block, new_size) < 0)
-          {
-            perror("mlock password cache");
-            free(new_block);
-            throw std::runtime_error("Could not increase password cache size");
-          }
-          memcpy(new_block, storage, storage_len);
-          memset(storage, 0, storage_size);
-          if (munlock(storage, storage_size) < 0)
-            perror("munlock password cache");
-          free(storage);
-          storage = new_block;
-          storage_size = new_size;
-        }
-        else
-          throw std::runtime_error("Could not increase password cache size");
-      }
-
-      // store length of this record
-      *(size_t*)(storage + storage_len) = reclen;
-      storage_len += sizeof(reclen);
-      // store contents
-      memcpy(storage + storage_len, service.data(), service.size()+1);
-      storage_len += service.size()+1;
-      memcpy(storage + storage_len, account.data(), account.size()+1);
-      storage_len += account.size()+1;
-      memcpy(storage + storage_len, password, strlen(password)+1);
-      storage_len += strlen(password)+1;
-    }
-    else
-      throw std::runtime_error("Password storage is not available");
-  }
-  
-  size_t find_block(const std::string &service, const std::string &account)
-  {
-    size_t offset = 0;
-    while (offset < storage_len)
-    {
-      size_t recsize = *(size_t*)(storage+offset);
-      const char *recservice = storage+offset+sizeof(recsize);
-      const char *recaccount = storage+offset+sizeof(recsize)+strlen(recservice)+1;
-
-      if (strcmp(recservice, service.c_str()) == 0 &&
-          strcmp(recaccount, account.c_str()) == 0)
-        return offset;
-
-      offset += recsize;
-    }
-    return (size_t)-1;
-  }
-
-  void remove_password(const std::string &service, const std::string &account)
-  {
-    if (storage)
-    {
-      size_t offset = find_block(service, account);
-      if (offset != (size_t)-1)
-      {
-        size_t recsize = *(size_t*)(storage+offset);
-
-        memmove(storage+offset, storage+offset+recsize, storage_len-recsize);
-        storage_len -= recsize;
-      }
-    }
-  }
-
-  const char *find_password(const std::string &service, const std::string &account)
-  {
-    if (storage)
-    {
-      size_t offset = find_block(service, account);
-      if (offset != (size_t)-1)
-      {
-        const char *pwd = storage + offset + sizeof(size_t) + service.size()+1 + account.size()+1;
-        return pwd;
-      }
-    }
-    return 0;
-  }
-};
-PasswordCache PasswordCache::instance;
-
+  //  Check if a parent was found and only set transcient if it was. Passing
+  //  a NULL parent would remove the transcient flag.
+  if (get_current_window() != NULL)
+    gtk_window_set_transient_for(((Gtk::Window *)&dialog)->gobj(), parent_window);
+}
 //--------------------------------------------------------------------------------
   
 
@@ -205,10 +96,8 @@ int UtilitiesImpl::show_message(const std::string &title, const std::string &tex
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-  //unnecessary.. apparently newer window managers set this automatically for dialogs
-  //and setting this to main_window explicitly will break that and cause the dialog
-  //to be assigned to the wrong window when a non-main window is the actual owner
-  //dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -229,7 +118,8 @@ int UtilitiesImpl::show_error(const std::string &title, const std::string &text,
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-//  dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -249,7 +139,8 @@ int UtilitiesImpl::show_warning(const std::string &title, const std::string &tex
   if (!other.empty())
     dlg.add_button(other, mforms::ResultOther);
 
-  //dlg.set_transient_for(*get_mainwindow());
+  set_dialog_transcient(dlg);
+  
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -283,7 +174,9 @@ int UtilitiesImpl::show_message_with_checkbox(const std::string &title, const st
   if (!other.empty())
     dlg.add_action_widget(*Gtk::manage(new Gtk::Button(other)), mforms::ResultOther);
   dlg.show_all();
- // dlg.set_transient_for(*get_mainwindow());
+
+  set_dialog_transcient(dlg);
+
   int r= dlg.run();
   if (r == Gtk::RESPONSE_DELETE_EVENT)
     return mforms::ResultCancel;
@@ -302,13 +195,22 @@ std::string UtilitiesImpl::get_clipboard_text()
 
 void UtilitiesImpl::open_url(const std::string &url)
 {
+  char *quoted_url = g_uri_escape_string(url.c_str(), 
+                                         G_URI_RESERVED_CHARS_GENERIC_DELIMITERS 
+                                         G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS, 
+                                         FALSE);
+
   const gchar *argv[] = {
     "xdg-open",
-    url.c_str(),
+    quoted_url,
     NULL 
   };
   GError *error = NULL;
-  if (!g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+  gboolean result = g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
+  
+  free(quoted_url);
+  
+  if (!result)
   {
     char *err = g_strdup_printf("Error opening url with xdg-open: %s", error->message);
     g_error_free(error);
@@ -342,7 +244,11 @@ std::string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 #endif
     break;
   case mforms::ApplicationData:
-    path = g_get_user_data_dir();
+    path = g_get_home_dir();
+    break;
+  case mforms::ApplicationSettings:
+    path = g_get_home_dir();
+    path.append("/.mysql/workbench");
     break;
   case WinProgramFiles:
   case WinProgramFilesX86:
@@ -358,16 +264,54 @@ std::string UtilitiesImpl::get_special_folder(mforms::FolderType type)
 }
 
 //------------------------------------------------------------------------------
-inline bool run_slot(const boost::function<bool ()> slot)
+static std::map<TimeoutHandle, sigc::connection> timeouts;
+static TimeoutHandle last_timeout_handle = 0;
+static base::Mutex timeout_mutex;
+
+inline bool run_slot(const boost::function<bool ()> slot, TimeoutHandle handle)
 {
-  return slot();
+  if (!slot())
+  {
+    base::MutexLock lock(timeout_mutex);
+    std::map<TimeoutHandle, sigc::connection>::iterator it;
+    if ((it = timeouts.find(handle)) != timeouts.end())
+      timeouts.erase(it);
+    return false;
+  }
+  return true;
 }
 
 //------------------------------------------------------------------------------
-void UtilitiesImpl::add_timeout(float interval, const boost::function<bool ()> &slot)
+TimeoutHandle UtilitiesImpl::add_timeout(float interval, const boost::function<bool ()> &slot)
 {
-  if (slot)
-    Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(run_slot), slot), interval*1000);
+  try
+  {
+    if (slot)
+    {
+      base::MutexLock lock(timeout_mutex);
+      ++last_timeout_handle;
+      timeouts[last_timeout_handle] = Glib::signal_timeout().connect(sigc::bind(sigc::ptr_fun(run_slot), slot, last_timeout_handle), interval*1000);
+      return last_timeout_handle;
+    }
+  }
+  catch (std::exception &exc)
+  {
+    static const char* const default_log_domain = "Utilities";
+    log_exception("Utilities: exception in add timeout function", exc);
+  }
+  return 0;
+}
+
+
+void UtilitiesImpl::cancel_timeout(TimeoutHandle h)
+{
+  base::MutexLock lock(timeout_mutex);
+  std::map<TimeoutHandle, sigc::connection>::iterator it;
+  if ((it = timeouts.find(h)) != timeouts.end())
+  {
+    it->second.disconnect();
+    timeouts.erase(it);
+  }
 }
 
 
@@ -379,7 +323,6 @@ void UtilitiesImpl::store_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->add_password(service, account, password.c_str());
     return;
   }
 
@@ -411,11 +354,7 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    const char *pwd = PasswordCache::get()->find_password(service, account);
-    if (!pwd)
-      return false;
-    password = pwd;
-    return true;
+    return false;
   }
 
   bool ret = false;
@@ -462,7 +401,6 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->remove_password(service, account);
     return;
   }
 
@@ -476,8 +414,6 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
   wb_pwd_schema.attributes[1].type = GNOME_KEYRING_ATTRIBUTE_TYPE_STRING;
   // attributes list must be terminated by a pair {0,0}, since we zeroed the whole struct we're safe here
 
-  gchar *kpwd = 0;
-
   GnomeKeyringResult res = gnome_keyring_delete_password_sync(&wb_pwd_schema,
                                                             "service", service.c_str(),
                                                             "account", account.c_str(),
@@ -519,7 +455,6 @@ void UtilitiesImpl::store_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->add_password(service, account, password.c_str());
     return;
   }
 
@@ -584,11 +519,7 @@ bool UtilitiesImpl::find_password(const std::string &service, const std::string
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    const char *pwd = PasswordCache::get()->find_password(service, account);
-    if (!pwd)
-      return false;
-    password = pwd;
-    return true;
+    return false;
   }
 
   return find_password_and_id(service, account, password);
@@ -599,7 +530,6 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 {
   if (getenv("WB_NO_GNOME_KEYRING"))
   {
-    PasswordCache::get()->remove_password(service, account);
     return;
   }
 
@@ -617,21 +547,16 @@ void UtilitiesImpl::forget_password(const std::string &service, const std::strin
 
 void UtilitiesImpl::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
-  PasswordCache::get()->add_password(service, account, password.c_str());
 }
 
 bool UtilitiesImpl::find_password(const std::string &service, const std::string &account, std::string &password)
 {
-  const char *pwd = PasswordCache::get()->find_password(service, account);
-  if (!pwd)
-    return false;
-  password = pwd;
-  return true;
+  g_message("no gnome keyring support");
+  return false;
 }
 
 void UtilitiesImpl::forget_password(const std::string &service, const std::string &account)
 {
-  PasswordCache::get()->remove_password(service, account);
 }
 
 #endif // !USE_KEYRING
@@ -654,6 +579,7 @@ class TransparentMessage : public Gtk::Window
   public:
     TransparentMessage();
     void show_message(const std::string& title, const std::string& text, const sigc::slot<bool> &cancel_slot=sigc::slot<bool>());
+    void run();
     void stop();
     bool response;
     bool running_modal;
@@ -661,6 +587,7 @@ class TransparentMessage : public Gtk::Window
     Gtk::Button *cancel_button;
     sigc::slot<bool> cancel_slot;
     Glib::Mutex mutex;
+    bool _is_runing;
 
     virtual bool on_button_release_event(GdkEventButton* ev);
     void cancel_clicked();
@@ -668,7 +595,7 @@ class TransparentMessage : public Gtk::Window
 
 //------------------------------------------------------------------------------
 TransparentMessage::TransparentMessage()
-  : response(false)
+  : response(false), _is_runing(false)
 {
   Gtk::Window *owner = get_mainwindow();
   if (owner)
@@ -760,12 +687,14 @@ void TransparentMessage::show_message(const std::string& title, const std::strin
   window->set_opacity(0.85);
 #endif
   window->process_updates(true);
-
-  show_now();
-
-  window->freeze_updates(); // Added to stop idle window update requests storm in gnome-3
+  show_all();
 }
 
+void TransparentMessage::run()
+{
+  _is_runing = true;
+  Gtk::Main::run();
+}
 
 //------------------------------------------------------------------------------
 void TransparentMessage::stop()
@@ -775,7 +704,11 @@ void TransparentMessage::stop()
   {
     unrealize();
     running_modal = false;
-    Gtk::Main::quit();
+    if (_is_runing)
+    {
+      Gtk::Main::quit();
+      _is_runing = false;
+    }
   }
   response= true;
 }
@@ -790,8 +723,12 @@ void TransparentMessage::cancel_clicked()
     {
       unrealize();
       running_modal = false;
-      Gtk::Main::quit();
-      hide();
+      if (_is_runing)
+      {
+        Gtk::Main::quit();
+        hide();
+        _is_runing = false;
+      }
     }
     response = false;
   }
@@ -847,12 +784,50 @@ bool UtilitiesImpl::move_to_trash(const std::string &path)
   return true;
 }
 
+
+//------------------------------------------------------------------------------
+void UtilitiesImpl::reveal_file(const std::string &path)
+{
+  const gchar *argv[] = {
+    "xdg-open",
+    base::dirname(path).c_str(),
+    NULL 
+  };
+  GError *error = NULL;
+  if (!g_spawn_async(NULL, (gchar**)argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error))
+  {
+    char *err = g_strdup_printf("Error opening folder with xdg-open: %s", error->message);
+    g_error_free(error);
+    std::runtime_error exc(err);
+    g_free(err);
+    throw exc;
+  }
+
+}
+
+
+void UtilitiesImpl::set_thread_name(const std::string &name)
+{
+#ifdef HAVE_PRCTL_H 
+  if (!name.empty())
+    prctl(PR_SET_NAME, name.c_str(),0,0,0);
+#endif
+}
+
+void UtilitiesImpl::beep()
+{
+  get_mainwindow()->get_window()->beep();
+}
+
 //------------------------------------------------------------------------------
 
 
 bool UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const std::string &text,
                                                  const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_task)
 {
+  if (!start_task)
+    throw std::invalid_argument("start_task param cannot be empty");
+
   if (!tmc)
     tmc = new TransparentMessage();
   if (tmc)
@@ -861,7 +836,8 @@ bool UtilitiesImpl::run_cancelable_wait_message(const std::string &title, const
     tmc->running_modal = true;
 
     Glib::signal_idle().connect(sigc::bind_return(start_task, false));
-    Gtk::Main::run();
+
+    tmc->run();
     bool response = tmc->response;
     delete tmc;
     tmc = 0;
@@ -883,7 +859,107 @@ void UtilitiesImpl::stop_cancelable_wait_message()
   }
 }
 
+   
+Glib::RefPtr<Gdk::Pixbuf> UtilitiesImpl::get_cached_icon(const std::string &icon)
+{
+  std::map<std::string, Glib::RefPtr<Gdk::Pixbuf> > cache;
 
+  if (cache.find(icon) != cache.end())
+    return cache[icon];
+
+  if (icon == "folder")
+  {
+    Glib::RefPtr<Gdk::Pixbuf> pix = get_mainwindow()->render_icon(Gtk::Stock::DIRECTORY, Gtk::ICON_SIZE_MENU);
+    cache[icon] = pix;
+    return pix;
+  }
+  else
+  {
+    std::string path = mforms::App::get()->get_resource_path(icon);
+    if (!path.empty() && g_file_test(path.c_str(), G_FILE_TEST_IS_REGULAR))
+    {
+      cache[icon] = Gdk::Pixbuf::create_from_file(path);
+      return cache[icon];
+    }
+    else
+      g_warning("Can't find icon %s", icon.c_str());
+  }
+  return Glib::RefPtr<Gdk::Pixbuf>();
+}
+
+
+//------------------------------------------------------------------------------
+#include <pango/pangoft2.h>
+
+class FontMeasurement
+{
+public:
+  PangoLayout *_layout;
+  FontMeasurement(PangoLayout *pango) : _layout(pango)
+  {
+  }
+
+//  FontMeasurement(const FontMeasurement &other)
+//  {
+//    if (other._layout!=NULL)
+//      g_object_ref(other._layout);
+//  }
+//
+//  void operator=(const FontMeasurement &other)
+//  {
+//    if (other._layout!=NULL)
+//      g_object_ref(other._layout);
+//  }
+
+  ~FontMeasurement()
+  {
+    if (_layout != NULL)
+      g_object_unref (_layout);
+  }
+};
+
+std::map<std::string, FontMeasurement*> FontMeasurementDescriptors;
+
+double UtilitiesImpl::get_text_width(const std::string &text, const std::string &font_desc)
+{
+  std::string font;
+  float size = 0;
+  bool bold = false;
+  bool italic = false;
+  
+  if (!base::parse_font_description(font_desc, font, size, bold, italic))
+    return 0;
+
+  PangoLayout *layout = NULL;
+  std::map<std::string, FontMeasurement*>::iterator it = FontMeasurementDescriptors.find(font_desc);
+  if (it == FontMeasurementDescriptors.end())
+  {
+    PangoFontDescription *font_description = pango_font_description_new ();
+    FontMeasurement *font_measurement = new FontMeasurement(pango_layout_new(get_mainwindow()->get_pango_context()->gobj()));
+
+    pango_font_description_set_family (font_description, font.c_str());
+    pango_font_description_set_style (font_description, italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
+    pango_font_description_set_variant (font_description, PANGO_VARIANT_NORMAL);
+    pango_font_description_set_weight (font_description, bold ?  PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
+    pango_font_description_set_stretch (font_description, PANGO_STRETCH_NORMAL);
+    pango_font_description_set_size (font_description, size * PANGO_SCALE);
+    pango_layout_set_font_description (font_measurement->_layout, font_description);
+    pango_font_description_free(font_description);
+
+    FontMeasurementDescriptors[font_desc] = font_measurement;
+    layout = font_measurement->_layout;
+  }
+  else
+    layout = it->second->_layout;
+
+  pango_layout_set_text (layout, text.c_str(), -1);
+
+  int width = 0;
+  pango_layout_get_pixel_size (layout, &width, NULL);
+  
+  return (double)width;
+  
+}
 
 //------------------------------------------------------------------------------
 
@@ -904,13 +980,18 @@ void UtilitiesImpl::init()
   f->_utilities_impl.get_clipboard_text= &UtilitiesImpl::get_clipboard_text;
   f->_utilities_impl.open_url= &UtilitiesImpl::open_url;
   f->_utilities_impl.add_timeout= &UtilitiesImpl::add_timeout;
+  f->_utilities_impl.cancel_timeout= &UtilitiesImpl::cancel_timeout;
   f->_utilities_impl.get_special_folder= &UtilitiesImpl::get_special_folder;
   f->_utilities_impl.store_password= &UtilitiesImpl::store_password;
   f->_utilities_impl.find_password= &UtilitiesImpl::find_password;
   f->_utilities_impl.forget_password= &UtilitiesImpl::forget_password;
   f->_utilities_impl.move_to_trash= &UtilitiesImpl::move_to_trash;
+  f->_utilities_impl.reveal_file= &UtilitiesImpl::reveal_file;
   f->_utilities_impl.perform_from_main_thread = &MainThreadRequestQueue::perform;
+  f->_utilities_impl.set_thread_name = &UtilitiesImpl::set_thread_name;
+  f->_utilities_impl.beep = &UtilitiesImpl::beep;
 
+  f->_utilities_impl.get_text_width = &UtilitiesImpl::get_text_width;
   MainThreadRequestQueue::get(); // init from main thread
 }
 
@@ -978,5 +1059,7 @@ void *MainThreadRequestQueue::perform(const boost::function<void* ()> &slot, boo
 }
 
 
+   
+   
 };
 };
