@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _CODE_EDITOR_H_
-#define _CODE_EDITOR_H_
+#pragma once
 
 #ifndef SCI_NAMESPACE
 #define SCI_NAMESPACE
@@ -47,12 +46,13 @@ namespace mforms {
     LanguageMySQL51,
     LanguageMySQL55,
     LanguageMySQL56,
+    LanguageMySQL57,
+
     LanguageHtml,   // includes embedded xml, javascript, php, vb, python
-    LanguageLua,
     LanguagePython,
     LanguageCpp,    // Lexer for C++, C, Java, and JavaScript (which includes JSON).
 
-    LanguageMySQL = LanguageMySQL56, // Always the latest language.
+    LanguageMySQL = LanguageMySQL56, // Always the latest (released) language.
   };
 
   /**
@@ -60,15 +60,24 @@ namespace mforms {
    */
   enum LineMarkup {
     LineMarkupNone          = 0,      // No markup for the given line.
-    LineMarkupBreakpoint    = 1 << 0, // Line has a marker set for a break point.
-    LineMarkupBreakpointHit = 1 << 1, // Line has a marker set for a break point which is currently hit.
-    LineMarkupError         = 1 << 2, // Line's background is drawn in red to mark an execution error.
-    LineMarkupStatement     = 1 << 3, // Marks a line as having a statement starting on it.
+    LineMarkupStatement     = 1 << 0, // Marks a line as having a statement starting on it.
+    LineMarkupError         = 1 << 1, // Line's background is drawn in red to mark an execution error.
+    LineMarkupBreakpoint    = 1 << 2, // Line has a marker set for a break point.
+    LineMarkupBreakpointHit = 1 << 3, // Line has a marker set for a break point which is currently hit.
     LineMarkupCurrent       = 1 << 4, // Current execution line.
 
     LineMarkupAll           = 0xFF,   // All markup, useful for remove_markup.
   };
 
+  // A collection of markup, attached to the original line and going to be removed
+  // or moved to a new line.
+  typedef struct {
+    int original_line;
+    int new_line;
+    LineMarkup markup;
+  } LineMarkupChangeEntry;
+  typedef std::vector<LineMarkupChangeEntry> LineMarkupChangeset;
+
 #ifndef SWIG
   inline LineMarkup operator| (LineMarkup a, LineMarkup b)
   {
@@ -83,16 +92,17 @@ namespace mforms {
   };
 
   enum CodeEditorFeature {
-    FeatureNone              = 0,
-    FeatureWrapText          = 1 << 0, // Enables word wrapping.
-    FeatureGutter            = 1 << 1, // Show/Hide gutter.
-    FeatureReadOnly          = 1 << 2,
-    FeatureShowSpecial       = 1 << 3, // Show white spaces and line ends with special chars.
-    FeatureUsePopup          = 1 << 4, // Use built-in context menu.
-    FeatureConvertEolOnPaste = 1 << 5, // Convert line endings to the current value in the editor
-                                       // when pasting text.
+    FeatureNone               = 0,
+    FeatureWrapText           = 1 << 0, // Enables word wrapping.
+    FeatureGutter             = 1 << 1, // Show/Hide gutter.
+    FeatureReadOnly           = 1 << 2,
+    FeatureShowSpecial        = 1 << 3, // Show white spaces and line ends with special chars.
+    FeatureUsePopup           = 1 << 4, // Use built-in context menu.
+    FeatureConvertEolOnPaste  = 1 << 5, // Convert line endings to the current value in the editor
+                                        // when pasting text.
     FeatureScrollOnResize     = 1 << 6, // Scroll caret into view if it would be hidden by a resize action.
     FeatureFolding            = 1 << 7, // Enable code folding.
+    FeatureAutoIndent         = 1 << 8, // Auto indent the new line on pressing enter.
 
     FeatureAll               = 0xFFFF,
   };
@@ -178,9 +188,9 @@ public:
 #ifndef SWIG
   struct CodeEditorImplPtrs
   {
-    bool (__stdcall *create)(CodeEditor* self);
-    sptr_t (__stdcall *send_editor)(CodeEditor* self, unsigned int message, uptr_t wParam, sptr_t lParam);
-    void (__stdcall *set_status_text)(CodeEditor* self, const std::string &text);
+    bool (*create)(CodeEditor* self);
+    sptr_t (*send_editor)(CodeEditor* self, unsigned int message, uptr_t wParam, sptr_t lParam);
+    void (*set_status_text)(CodeEditor* self, const std::string &text);
   };
 #endif
 #endif
@@ -207,7 +217,7 @@ public:
     /** Appends the given number of chars to end of the document. Allows to add nulls too.
      *  The length is (as always) a byte count.
      */
-    void append_text(const char* text, int length);
+    void append_text(const char* text, size_t length);
 
     /** Replaces the selected text in the editor by the new text. If no text is selected then 
      *  the new text is inserted at the caret position. */
@@ -223,7 +233,7 @@ public:
     /** Returns the text in the given range (inclusive endpoints), regardless of the selection state.
      *  The range is automatically adjusted if it lies outside the available total text range.
      */
-    const std::string get_text_in_range(int start, int end);
+    const std::string get_text_in_range(size_t start, size_t end);
 
     /** Returns a direct pointer to the text in the editor control (no copying takes place) and its length
      *  in bytes. The text can contain embedded nulls and should therefore not be handled like
@@ -239,50 +249,56 @@ public:
      * NOTE: Scintilla uses bytes everywhere when a position or length is set or read. So be very
      *       careful when doing char maths (we use utf-8 with a variable code length per character).
      */
-    void set_selection(int start, int length);
+    void set_selection(size_t start, size_t length);
 
     /** Gets the current selection range. */
-    void get_selection(int &start, int &length);
+    void get_selection(size_t &start, size_t &length);
 
     /** Removes the current selection without moving the caret. */
     void clear_selection();
 
     /** Gets the byte range for the given line. Returns false if the line number is invalid */
-    bool get_range_of_line(int line, int &start, int &end);
+    bool get_range_of_line(ssize_t line, ssize_t &start, ssize_t &end);
     
     /** Sets the language for the syntax highlighter. */
     void set_language(SyntaxHighlighterLanguage language);
 
     /** Adds the given markup to a line if not yet there. Does not touch other markup. */
-    void show_markup(LineMarkup markup, int line);
+    void show_markup(LineMarkup markup, size_t line);
 
     /** Removes the given markup from that line, without affecting other markup (except for LineMarkupAll).
      *  If markup is LineMarkupAll then all markers are removed for the given line.
      *  If line is < 0 then all marker are removed from all lines.
      *  However, it is not possible to remove a specific marker from all lines.
      */
-    void remove_markup(LineMarkup markup, int line);
+    void remove_markup(LineMarkup markup, ssize_t line);
+
+    /**
+     * Determines if the given line contains the given markup.
+     * Returns true if at least one of the given markup types was found.
+     */
+    bool has_markup(LineMarkup markup, size_t line);
 
     /** Adds the given indicator styling to a range of characters. */
-    void show_indicator(RangeIndicator indicator, int start, int length);
+    void show_indicator(RangeIndicator indicator, size_t start, size_t length);
 
     /** Returns the indicator styling at the given position (if any). */
-    RangeIndicator indicator_at(int position);
+    RangeIndicator indicator_at(size_t position);
 
     /** Removes the given indicator styling from the given range. */
-    void remove_indicator(RangeIndicator indicator, int start, int length);
+    void remove_indicator(RangeIndicator indicator, size_t start, size_t length);
 
     /** Returns the number of lines currently in the editor. */
-    int line_count();
+    size_t line_count();
 
     /** The total length of the text in the editor in bytes. */
-    int text_length();
+    size_t text_length();
 
     /** Returns the character position of the given line. */
-    int position_from_line(int line_number);
+    size_t position_from_line(size_t line_number);
 
     /** Returns the line number from the given character position. */
-    int line_from_position(int position);
+    size_t line_from_position(size_t position);
 
     void set_font(const std::string &fontDescription); // e.g. "Trebuchet MS bold 9"
     
@@ -292,16 +308,20 @@ public:
     /** Toggles the given feature(s) to their opposite state. */
     void toggle_features(CodeEditorFeature features);
 
+    void set_read_only(bool flag);
+
+    void reset_undo_stack();
+
     /** Resets the editor's dirty state or queries it. */
     void reset_dirty();
     bool is_dirty();
 
     /** Retrieves or sets the position of the caret in the editor, specified as byte position. */
-    int get_caret_pos();
-    void set_caret_pos(int position);
+    size_t get_caret_pos();
+    void set_caret_pos(size_t position);
     
     /** Retrieves the line and column (both zero-based) for a given byte position. */
-    void get_line_column_pos(int position, int &line, int &column);
+    void get_line_column_pos(size_t position, size_t &line, size_t &column);
 
     /** Standard edit functions used from menus. */
     bool can_undo();
@@ -339,7 +359,7 @@ public:
 
     /** Searches for the given text according to the parameters and replaces it by the text.
      *  Returns the number of replaced text occurrences. */
-    int find_and_replace_text(const std::string& search_text, const std::string& new_text, 
+    size_t find_and_replace_text(const std::string& search_text, const std::string& new_text,
       FindFlags flags, bool do_all);
 
     /** Searches for the next placeholder char combination and selects it when found. The text is
@@ -356,8 +376,8 @@ public:
      *                 first with auto_completion_register_images. Use -1 as id when no image is needed.
      *                 The list should be sorted to make matching working properly.
      */
-    void auto_completion_show(int chars_entered, const std::vector<std::pair<int, std::string> >& entries);
-    void auto_completion_show(int chars_entered, const std::vector<std::string>& entries);
+    void auto_completion_show(size_t chars_entered, const std::vector<std::pair<int, std::string> >& entries);
+    void auto_completion_show(size_t chars_entered, const std::vector<std::string>& entries);
 
     /** Can be used to cancel auto completion while it is in progress (i.e. during handling an
      * auto completion event. Has no effect otherwise. */
@@ -404,7 +424,7 @@ public:
     void auto_completion_fillups(const std::string& fillups);
 
     /** Show the editor's calltip window close to the given position or hide it. */
-    void show_calltip(bool show, int position, const std::string& value);
+    void show_calltip(bool show, size_t position, const std::string& value);
 
     /** Sets the EOL mode used by the editor. If @convert is true all lines in the document are converted
      *  to use the new line ending.
@@ -417,7 +437,7 @@ public:
      when this object is deleted. */
     void set_context_menu(Menu *menu) { _context_menu = menu; };
     
-    /** Returns the context menu object attached to the treeview. */
+    /** Returns the context menu object attached to the editor. */
     Menu *get_context_menu() { return _context_menu; }
     
     /** Returns the host which is controlling this editor instance (if any). */
@@ -459,7 +479,7 @@ public:
      *    The position closest to the mouse pointer.
      *    x and y client coordinates where the mouse lingered.
      */
-    boost::signals2::signal<void (bool, int, int, int)>* signal_dwell() { return &_dwell_event; }
+    boost::signals2::signal<void (bool, size_t, int, int)>* signal_dwell() { return &_dwell_event; }
 
     /** Signal emitted when the user typed an ordinary text character (as opposed to a command character).
      *  It can be used e.g. to trigger auto completion.
@@ -468,7 +488,14 @@ public:
      */
     boost::signals2::signal<void (int)>* signal_char_added() { return &_char_added_event; }
 
-    /** Emited when editing ends (control loses focus)
+    /** Signal emitted when the Scintilla backend removes a set marker (e.g. on editing, pasting, manual marker setting).
+     *  Parameters are:
+     *    A vector of line + markup pairs.
+     *    A flag telling if those markers where deleted or only updated (moved).
+     */
+    boost::signals2::signal<void(const LineMarkupChangeset &changeset, bool deleted)>* signal_marker_changed() { return &_marker_changed_event; }
+
+    /** Signal emitted when the control loses input focus.
      */
     boost::signals2::signal<void ()>* signal_lost_focus() { return &_signal_lost_focus; }
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -490,19 +517,22 @@ public:
     std::map<int, void*> _images; // Registered RGBA images.
     void *_host;
     bool _scroll_on_resize;
+    bool _auto_indent;
 
     void setup_marker(int marker, const std::string& name);
+    void check_markers_removed(int position, int length);
+    void check_markers_moved(int position, int lines_added);
+
     void load_configuration(SyntaxHighlighterLanguage language);
 
     boost::signals2::signal<void (int, int, int, bool)> _change_event;
     boost::signals2::signal<void (int, int, mforms::ModifierKey)> _gutter_clicked_event;
     boost::signals2::signal<void (AutoCompletionEventType, int, const std::string&)> _auto_completion_event;
-    boost::signals2::signal<void (bool, int, int, int)> _dwell_event;
+    boost::signals2::signal<void (bool, size_t, int, int)> _dwell_event;
     boost::signals2::signal<void (int)> _char_added_event;
     boost::signals2::signal<void ()> _signal_lost_focus;
+    boost::signals2::signal<void(const LineMarkupChangeset &changeset, bool deleted)> _marker_changed_event;
 
     boost::function<void (CodeEditor*, bool)> _show_find_panel;
   };
 };
-
-#endif // _CODE_EDITOR_H_
