@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,7 @@
  * 02110-1301  USA
  */
 
-#ifndef _MFORMS_Utilities_H_
-#define _MFORMS_Utilities_H_
+#pragma once
 
 /**
  * Implementation some miscellaneous stuff needed in mforms.
@@ -29,6 +28,12 @@
 
 #include "cairo/cairo.h"
 
+#ifdef __GNUC__
+#define WARN_UNUSED_RETURN_VALUE __attribute__((warn_unused_result))
+#else
+#define WARN_UNUSED_RETURN_VALUE
+#endif
+
 namespace mforms {
   // Constants for special folders on a system.
   enum MFORMS_EXPORT FolderType {
@@ -82,54 +87,61 @@ namespace mforms {
     DialogQuery,
     DialogSuccess, // Like DialogMessage but with a special icon to signal a successful operation.
   };
+  
+  class Box;
+  class Button;
+
+  typedef int TimeoutHandle;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 #ifndef SWIG
   struct MFORMS_EXPORT UtilitiesImplPtrs
   {
-    int (__stdcall *show_message)(const std::string &title, const std::string &text,
-                                  const std::string &ok, const std::string &cancel,
-                                  const std::string &other);
-    int (__stdcall *show_error)(const std::string &title, const std::string &text,
-                                  const std::string &ok, const std::string &cancel,
-                                  const std::string &other);
-    int (__stdcall *show_warning)(const std::string &title, const std::string &text,
-                                  const std::string &ok, const std::string &cancel,
-                                  const std::string &other);
-    int (__stdcall *show_message_with_checkbox)(const std::string &title, const std::string &text,
-                                                const std::string &ok, const std::string &cancel,
-                                                const std::string &other,
-                                                const std::string &checkbox_text, // empty text = default "Don't show this message again" text
-                                                bool &remember_checked);
+    void (*beep) ();
+    int (*show_message)(const std::string &title, const std::string &text,
+                        const std::string &ok, const std::string &cancel,
+                        const std::string &other);
+    int (*show_error)(const std::string &title, const std::string &text,
+                      const std::string &ok, const std::string &cancel,
+                      const std::string &other);
+    int (*show_warning)(const std::string &title, const std::string &text,
+                        const std::string &ok, const std::string &cancel,
+                        const std::string &other);
+    int (*show_message_with_checkbox)(const std::string &title, const std::string &text,
+                                      const std::string &ok, const std::string &cancel,
+                                      const std::string &other,
+                                      const std::string &checkbox_text, // empty text = default "Don't show this message again" text
+                                      bool &remember_checked);
 
-    void (__stdcall *show_wait_message)(const std::string &title, const std::string &text);
-    bool (__stdcall *hide_wait_message)();
-    bool (__stdcall *run_cancelable_wait_message)(const std::string &title, const std::string &text,
+    void (*show_wait_message)(const std::string &title, const std::string &text);
+    bool (*hide_wait_message)();
+    bool (*run_cancelable_wait_message)(const std::string &title, const std::string &text,
                                                   const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_task);
-    void (__stdcall *stop_cancelable_wait_message)();
+    void (*stop_cancelable_wait_message)();
     
-    void (__stdcall *set_clipboard_text)(const std::string &text);
-    std::string (__stdcall *get_clipboard_text)();
-    std::string (__stdcall *get_special_folder)(mforms::FolderType type);
+    void (*set_clipboard_text)(const std::string &text);
+    std::string (*get_clipboard_text)();
+    std::string (*get_special_folder)(mforms::FolderType type);
     
-    void (__stdcall *open_url)(const std::string &url);
-    void (__stdcall *reveal_file)(const std::string &url); // TODO XXX Windows
-    bool (__stdcall *move_to_trash)(const std::string &path);
+    void (*open_url)(const std::string &url);
+    void (*reveal_file)(const std::string &url);
+    bool (*move_to_trash)(const std::string &path);
     
-    void (__stdcall *add_timeout)(float delay, const boost::function<bool ()> &callback);
-    
-    void (__stdcall *store_password)(const std::string &service, const std::string &account, const std::string &password);
-    bool (__stdcall *find_password)(const std::string &service, const std::string &account, std::string &password);
-    void (__stdcall *forget_password)(const std::string &service, const std::string &account);
+    TimeoutHandle (*add_timeout)(float delay, const boost::function<bool ()> &callback);
+    void (*cancel_timeout)(TimeoutHandle handle);
 
-    void* (__stdcall *perform_from_main_thread)(const boost::function<void* ()> &slot, bool wait_completion);
+    void (*store_password)(const std::string &service, const std::string &account, const std::string &password);
+    bool (*find_password)(const std::string &service, const std::string &account, std::string &password);
+    void (*forget_password)(const std::string &service, const std::string &account);
+
+    void* (*perform_from_main_thread)(const boost::function<void* ()> &slot, bool wait_completion);
+    void (*set_thread_name)(const std::string &name);
+
+    double (*get_text_width)(const std::string &text, const std::string &font);
   };
 #endif
 #endif
 
-  class Box;
-  class Button;
-
     /** Various Utility functions */
   class MFORMS_EXPORT Utilities
   {
@@ -142,38 +154,36 @@ namespace mforms {
 %ignore show_error(const std::string &title, const std::string &text, const std::string &ok);
 #endif
   public:
+    /** Plays the system's default error sound. */
+    static void beep(); // TODO: Mac, Linux
+
     /** Show a message dialog. Return value is from the DialogResult enum.
-     
-     In Python, all arguments are mandatory. */
+     * In Python, all arguments are mandatory. */
     static int show_message(const std::string &title, const std::string &text,
-                            const std::string &ok, const std::string &cancel="",
-                            const std::string &other="");
+                            const std::string &ok, const std::string &cancel = "",
+                            const std::string &other = "");
+
     /** Show an error dialog. Return value is from the DialogResult enum.
-     
-     In Python, all arguments are mandatory. */
+     * In Python, all arguments are mandatory. */
     static int show_error(const std::string &title, const std::string &text,
                             const std::string &ok, const std::string &cancel="",
                             const std::string &other="");
+
     /** Show a warning dialog. Return value is from the DialogResult enum.
-     
-     In Python, all arguments are mandatory. */
+     * In Python, all arguments are mandatory. */
     static int show_warning(const std::string &title, const std::string &text,
                             const std::string &ok, const std::string &cancel="",
                             const std::string &other="");
 
     /** Show a message dialog and save the answer, if the checkbox is enabled.
-     
-     In Python, all arguments are mandatory. */
+     * In Python, all arguments are mandatory. */
     static int show_message_and_remember(const std::string &title, const std::string &text,
                                          const std::string &ok, const std::string &cancel,
                                          const std::string &other,
                                          const std::string &answer_id, const std::string &checkbox_text);
     static void forget_message_answers();
     static void set_message_answers_storage_path(const std::string &path);
-  private:
-    static void save_message_answers();
 
-  public:
     static void show_wait_message(const std::string &title, const std::string &text);
     static bool hide_wait_message();
     
@@ -183,57 +193,52 @@ namespace mforms {
                                     void *&task_result);
 
     /** Asks the user to enter a string, which is returned to the caller.
-
-     @param title - the title of the input dialog
-     @param description - description of the value to enter.
-     @param default_value - an initial value for the edit control
-     @param ret_value - the string the user entered, can be empty or the default string too
-
-     @return true if user presses ok or false if its cancelled.
+     * @param title - the title of the input dialog
+     * @param description - description of the value to enter.
+     * @param default_value - an initial value for the edit control
+     * @param ret_value - the string the user entered, can be empty or the default string too
+     *
+     * @return true if user presses ok or false if its canceled.
      */
     static bool request_input(const std::string &title, const std::string &description,
       const std::string &default_value, std::string &ret_value);
 
     /** Prompts the user for a password and whether it should be stored.
-     
-     @param title - the title of the password dialog
-     @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
-     @param username - the username the password corresponds to, if empty the user will be able to enter it
-     @param ret_password - the password the user typed
-     @param ret_store - true if the user clicks in "Store Password" checkbox
-     
-     @return true if user presses ok or false if its cancelled.
-     
-     In Python, ret_password and ret_store are returned as a tuple. */
+     * @param title - the title of the password dialog
+     * @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
+     * @param username - the username the password corresponds to, if empty the user will be able to enter it
+     * @param ret_password - the password the user typed
+     * @param ret_store - true if the user clicks in "Store Password" checkbox
+     * 
+     * @return true if user presses ok or false if its canceled.
+     * In Python, ret_password and ret_store are returned as a tuple.
+     */
     static bool ask_for_password_check_store(const std::string &title, const std::string &service,
       std::string &username /*in/out*/, std::string &ret_password /*out*/, bool &ret_store /*out*/);
 
     /** Prompts the user for a password.
-     
-     @param title - the title of the password dialog
-     @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
-     @param username - the username the password corresponds to
-     @param ret_password - the password the user typed
-     
-     @return true if user presses ok or false if its cancelled.
-     
-     If you need the username to be editable by the user, use credentials_for_service()
-     
-     In Python, ret_password is returned by the function. */
+     * @param title - the title of the password dialog
+     * @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
+     * @param username - the username the password corresponds to
+     * @param ret_password - the password the user typed
+     * 
+     * @return true if user presses ok or false if its canceled.
+     * If you need the username to be editable by the user, use credentials_for_service()
+     * In Python, ret_password is returned by the function.
+     */
     static bool ask_for_password(const std::string &title, const std::string &service, const std::string &username,
                                  std::string &ret_password /*out*/);
 
     /** Tries to find a previously stored password and prompts the user if not found.
-     
-     @param title - the title of the password dialog
-     @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
-     @param username - the username the password corresponds to, if empty the user will be able to enter it
-     @param reset_password - whether the password should be reset without looking for a stored one
-     @param ret_password - the password the user typed
-
-     @return true if user presses ok or false if its cancelled.
-     
-     In Python, ret_password and ret_store are returned as a tuple. */
+     * @param title - the title of the password dialog
+     * @param service - the service the password refers to (ie sudo@hostname, Mysql@hostname etc)
+     * @param username - the username the password corresponds to, if empty the user will be able to enter it
+     * @param reset_password - whether the password should be reset without looking for a stored one
+     * @param ret_password - the password the user typed
+     *
+     * @return true if user presses ok or false if its canceled.
+     * In Python, ret_password and ret_store are returned as a tuple.
+     */
     static bool find_or_ask_for_password(const std::string &title, const std::string &service, const std::string &username,
                                          bool reset_password, std::string &ret_password /*out*/)
     {
@@ -250,31 +255,32 @@ namespace mforms {
 #endif
 
     /** Store the password for the given service and account.
-     
-     @li In Windows, an encrypted Vault file is used.
-     @li In Mac, the secure KeyChain is used.
-     @li In Linux, the gnome-keychain is used, unless it's not available (such as in KDE).
-     In that case, passwords will be forgotten when the application exits.
+     * @li In Windows, an encrypted Vault file is used.
+     * @li In Mac, the secure KeyChain is used.
+     * @li In Linux, the gnome-keychain is used, unless it's not available (such as in KDE).
+     * In that case, passwords will be forgotten when the application exits.
      */
     static void store_password(const std::string &service, const std::string &account, const std::string &password);
-    /** Locates the password for the given service and account. 
 
-     @return true if password was found else false.
+    /** Locates the password for the given service and account. 
+     * @return true if password was found else false.
      */
-    static bool find_password(const std::string &service, const std::string &account, std::string &password);
+    static bool find_password(const std::string &service, const std::string &account, std::string &ret_password);
+    
+    /** Locates the password for the given service and account in the in-memory cache only.
+     * @return true if password was found else false.
+     */
+    static bool find_cached_password(const std::string &service, const std::string &account, std::string &ret_password);
+
     /** Clears the stored password for the given service and account */
     static void forget_password(const std::string &service, const std::string &account);
     
-    
     /** Sets the given text to the system clipboard */
     static void set_clipboard_text(const std::string &text);
     /** Gets the text stored in the system clipboard */
     static std::string get_clipboard_text();
 
-    /** Gets the path of the requested special folder:     
-       Documents
-       Desktop
-     */
+    /** Gets the path of the requested special folder: documents, desktop etc. */
     static std::string get_special_folder(FolderType type);
 
     /** Opens the given URL in the default system browser. */
@@ -286,14 +292,17 @@ namespace mforms {
 
     /** Shows the given file path in the OS Finder/Explorer */
     static void reveal_file(const std::string &path);
+
 #ifndef SWIG
     /** Sets up a callback to be called after a given interval. 
      
      Interval is in seconds, with fractional values allowed.
      The callback must return true if it wants to be triggered again
      */
-    static void add_timeout(float interval, const boost::function<bool ()> &callback);
+    static TimeoutHandle add_timeout(float interval, const boost::function<bool ()> &callback) WARN_UNUSED_RETURN_VALUE;
 #endif
+    static void cancel_timeout(TimeoutHandle handle);
+
     /** Convenience function to add an OK and Cancel buttons in a box. 
      
      This function will reorder buttons according to the standard order in the platform
@@ -304,8 +313,17 @@ namespace mforms {
 #ifndef SWIG
     // Don't wrap this from mforms, because the typeinfo isnt getting shared across
     // modules... uncomment this if that's solved
-    static cairo_surface_t* load_icon(const std::string& name);
+    static cairo_surface_t* load_icon(const std::string& name, bool allow_hidpi=false);
+    static bool is_hidpi_icon(cairo_surface_t *s);
+    static bool icon_needs_reload(cairo_surface_t *s);
+
+    static void paint_icon(cairo_t *cr, cairo_surface_t *icon, double x, double y, float alpha = 1.0);
+    static void get_icon_size(cairo_surface_t *icon, int &w, int &h);
+
     static std::string shorten_string(cairo_t* cr, const std::string& text, double width);
+
+    //XXX TODO Linux, Windows
+    static double get_text_width(const std::string &text, const std::string &font);
 #endif
 
 #ifndef SWIG
@@ -322,7 +340,22 @@ namespace mforms {
 #endif // !SWIG
 
     static bool in_main_thread();
+    static void set_thread_name(const std::string &name);
+
+
+    // Should be called at the end of python thread, when there was some query involved.
+    static void driver_shutdown();
+
+#ifndef SWIG
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+    static void add_driver_shutdown_callback(const boost::function<void ()> &slot);
+#endif
+#endif
+
+  private:
+    static boost::function<void()> _driver_shutdown_cb;
+
+    static void save_message_answers();
   };
 
 };
-#endif // _MFORMS_Utilities_H_
