@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -40,7 +40,7 @@ namespace MySQL {
 };
 
 
-class TreeNodeImpl : public mforms::TreeNode
+class TreeNodeWrapper : public mforms::TreeNode
 {
   friend struct mforms::TreeNodeRef;
 
@@ -49,7 +49,7 @@ class TreeNodeImpl : public mforms::TreeNode
   std::vector<std::string> _values;
   std::vector<mforms::TreeNodeTextAttributes> _attributes;
   std::vector<std::string> _icons;
-  std::vector<TreeNodeImpl*> _children;
+  std::vector<TreeNodeWrapper*> _children;
   mforms::TreeNodeData *pdata;
   bool _expanded;
 
@@ -59,7 +59,7 @@ protected:
   bool is_root() const;
 
 public:
-  TreeNodeImpl();
+  TreeNodeWrapper();
   
   int node_index() const { return _index; }
 
@@ -68,6 +68,7 @@ public:
 
   virtual bool equals(const mforms::TreeNode &other);
   virtual bool is_valid() const;
+  virtual int level() const;
     
   virtual void set_icon_path(int column, const std::string &icon);
 
@@ -76,21 +77,28 @@ public:
   virtual void set_int(int column, int value);
   virtual void set_long(int column, boost::int64_t value);
   virtual void set_bool(int column, bool value);
+  virtual void set_float(int column, double value);
 
   virtual std::string get_string(int column) const;
   virtual int get_int(int column) const;
   virtual boost::int64_t get_long(int column) const;
   virtual bool get_bool(int column) const;
+  virtual double get_float(int column) const;
     
   virtual int count() const;
   virtual mforms::TreeNodeRef insert_child(int index);
+  virtual void insert_child(int index, const mforms::TreeNode &node);
+  virtual void move_child(mforms::TreeNodeRef child, int new_index);
   virtual void remove_from_parent();
   virtual mforms::TreeNodeRef get_child(int index) const;
+  virtual int get_child_index(mforms::TreeNodeRef child) const;
   virtual mforms::TreeNodeRef get_parent() const;
+  virtual mforms::TreeNodeRef previous_sibling() const;
+  virtual mforms::TreeNodeRef next_sibling() const;
   virtual void remove_children();
 
   virtual std::vector<mforms::TreeNodeRef> add_node_collection(const mforms::TreeNodeCollectionSkeleton &nodes, int position = -1);
-  void add_children_from_skeletons(std::vector<TreeNodeImpl*> &parents, const std::vector<mforms::TreeNodeSkeleton>& children);
+  void add_children_from_skeletons(std::vector<TreeNodeWrapper*> &parents, const std::vector<mforms::TreeNodeSkeleton>& children);
 
   virtual void expand();
   virtual void collapse();
