@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,18 +17,17 @@
  * 02110-1301  USA
  */
 
-
 /**
  * Utilities is a support class to trigger message boxes and the like in the front end.
  */
 
-#include "stdafx.h"
-
 #include "base/string_utilities.h"
 #include "base/file_functions.h"
 #include "base/log.h"
+#include "base/threading.h"
 
 #include "mforms/mforms.h"
+#include "mforms/password_cache.h"
 
 DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
 
@@ -40,6 +39,15 @@ GThread *_mforms_main_thread = NULL;
 static std::map<std::string, int> remembered_message_answers;
 static std::string remembered_message_answer_file;
 
+boost::function<void ()> mforms::Utilities::_driver_shutdown_cb;
+
+//--------------------------------------------------------------------------------------------------
+
+void Utilities::beep()
+{
+  ControlFactory::get_instance()->_utilities_impl.beep();
+}
+
 //--------------------------------------------------------------------------------------------------
 
 int Utilities::show_message(const std::string &title, const std::string &text,
@@ -151,43 +159,64 @@ bool Utilities::hide_wait_message()
 
 //--------------------------------------------------------------------------------------------------
 
-struct CancellableTaskData
+class CancellableTaskData
 {
+public:
   boost::function<void* ()> task;
-  boost::shared_ptr<bool> finished_flag;
+  bool finished;
   boost::shared_ptr<void*> result_ptr;
-  boost::shared_ptr<GMutex> mutex;
+
+  int ref_count;
+
+  base::Semaphore semaphore;
+  CancellableTaskData() : finished(false), ref_count(1), semaphore(0) {}
 };
 
-static void* cancellable_task_thread(CancellableTaskData *data)
+// To ensure the shared thread data is not freed too early regardless what finishes first
+// (the thread or run_cancelable_task()) we store this data here in a private structure, ref counted.
+static base::Mutex thread_data_mutex;
+static std::map<void *, CancellableTaskData *> thread_data;
+
+static void* cancellable_task_thread(void *)
 {
-  std::auto_ptr<CancellableTaskData> autodel(data);
+  CancellableTaskData *data = NULL;
 
-  void *ptr = NULL;
-  try
   {
-    ptr = data->task();
+    base::MutexLock lock(thread_data_mutex);
+    data = thread_data[g_thread_self()];
+    if (data != NULL)
+      data->ref_count++; // Increment ref count while the data is still protected.
   }
-  catch (std::exception &exc)
+
+  if (data != NULL) // Should never be NULL but...
   {
-    log_error("Cancellable task threw uncaught exception: %s", exc.what());
+    void *ptr = NULL;
+    try
+    {
+      ptr = data->task();
+    }
+    catch (std::exception &exc)
+    {
+      log_error("Cancellable task threw uncaught exception: %s", exc.what());
+    }
+
+    data->semaphore.wait(); // Wait for the main thread to signal it is ready.
+    *data->result_ptr = ptr;
+    data->finished = true;
+    ControlFactory::get_instance()->_utilities_impl.stop_cancelable_wait_message();
+
+    {
+      base::MutexLock lock(thread_data_mutex);
+      data->ref_count--;
+      if (data->ref_count == 0)
+      {
+        thread_data.erase(g_thread_self());
+        delete data;
+      }
+    }
   }
-  g_mutex_lock(data->mutex.get());
-  *data->result_ptr = ptr;
-  *data->finished_flag = true;
-  ControlFactory::get_instance()->_utilities_impl.stop_cancelable_wait_message();
-  g_mutex_unlock(data->mutex.get());
-  return 0;
-}
 
-static void real_mutex_unlock(boost::shared_ptr<GMutex> m)
-{
-  g_mutex_unlock(m.get());
-}
-
-static void mutex_free(GMutex *m)
-{
-  g_mutex_free(m);
+  return NULL;
 }
 
 bool Utilities::run_cancelable_task(const std::string &title, const std::string &text,
@@ -195,62 +224,79 @@ bool Utilities::run_cancelable_task(const std::string &title, const std::string
                                     const boost::function<bool ()> &cancel_task,
                                     void *&task_result)
 {
-  GThread *thread;
-  GError *error = NULL;
-  CancellableTaskData *data = new CancellableTaskData(); // this is freed by the thread
-  boost::shared_ptr<bool> finished(new bool(false));
   boost::shared_ptr<void*> result(new void*((void*)-1));
-  boost::shared_ptr<GMutex> lock(g_mutex_new(), mutex_free);
-  
-  data->mutex = lock;
-  data->task = task;
-  data->finished_flag = finished;
-  
-  // result_ptr is used by the task thread to pass back the result from the task callback
-  // we cannot directly pass a pointer to task_result, because if the task is canceled,
-  // this function will return and the caller may free the task_result object before the
-  // canceled task actually finishes executing, which would invalidate any ptr to task_result 
-  data->result_ptr = result;
-  
-  g_mutex_lock(lock.get());
-  // start a thread that will run the task
-  thread = g_thread_create((void*(*)(void*))cancellable_task_thread, data, 0, &error);
-  if (!thread)
+
+  CancellableTaskData *data = NULL;
+  GThread *thread = NULL;
+
+  // Start a thread that will run the task. Store thread data so it can be accessed by the thread
+  // safely.
   {
-    g_mutex_unlock(lock.get());
-    delete data;
-    std::string msg("Error creating thread: ");
-    msg.append(error->message);
-    g_error_free(error);
-    throw std::runtime_error(msg);
+    base::MutexLock lock(thread_data_mutex);
+    data = new CancellableTaskData(); // Ref count is 1.
+
+    GError *error = NULL;
+    thread = base::create_thread((void*(*)(void*))cancellable_task_thread, NULL, &error);
+    if (thread == NULL)
+    {
+      std::string msg("Error creating thread: ");
+      msg.append(error->message);
+      g_error_free(error);
+
+      delete data;
+      throw std::runtime_error(msg);
+    }
+
+    // result_ptr is used by the task thread to pass back the result from the task callback.
+    // We cannot directly pass a pointer to task_result, because if the task is canceled,
+    // this function will return and the caller may free the task_result object before the
+    // canceled task actually finishes executing, which would invalidate any ptr to task_result.
+    data->result_ptr = result;
+
+    // The thread will wait on the mutex as we still lock it, so it's safe to work on the data.
+    thread_data[thread] = data;
+    data->task = task;
   }
 
-  boost::function<void ()> signal_ready = boost::bind(&real_mutex_unlock, data->mutex);
+  // Callback for the frontend to signal the worker thread that it's ready.
+  boost::function<void ()> signal_ready = boost::bind(&base::Semaphore::post, &data->semaphore);
+
+  bool function_result = false;
 
 retry:
   if (ControlFactory::get_instance()->_utilities_impl.run_cancelable_wait_message(title, text, signal_ready, cancel_task))
   {
-    signal_ready = boost::function<void ()>();
 
-    // wait for the thread to finish clean up
-    
-    // sometimes, in the mac, there's a race and/or bug that causes the event loop
+    // Sometimes, in the mac, there's a race and/or bug that causes the event loop
     // from the wait panel dialog to be exited when a nested modal dialog is closed.
-    // clicking OK for the nested dialog, exits the wait panel, which would cause it
-    // to return prematurely.. just execute the modal panel again if that's the case
-    if (!*finished)
+    // Clicking OK for the nested dialog, exits the wait panel, which would cause it
+    // to return prematurely. Just execute the modal panel again if that's the case.
+    // Additional note: sometimes the wait panel must be closed on purpose, so this is necessary anyway.
+    if (!data->finished)
       goto retry;
 
-    // task completed
+    // Task completed. We can directly access the data here without lock as we still have
+    // the increased ref count here.
     task_result = *result;
-    return true;
+    function_result = true;
   }
   else
   {
+    // Task canceled by user.
     log_debug2("run_cancelable_wait_message returned false\n");
-    // task canceled by user
-    return false;
   }
+
+  {
+    base::MutexLock lock(thread_data_mutex);
+    data->ref_count--;
+    if (data->ref_count == 0)
+    {
+      thread_data.erase(thread);
+      delete data;
+    }
+  }
+
+  return function_result;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -296,7 +342,7 @@ bool Utilities::move_to_trash(const std::string &path)
     }
     else
     {
-      if (g_remove(path.c_str()) < 0)
+      if (!base::remove(path))
         return false;
     }
     return true;
@@ -311,9 +357,17 @@ void Utilities::reveal_file(const std::string &path)
 
 //--------------------------------------------------------------------------------------------------
 
-void Utilities::add_timeout(float interval, const boost::function<bool ()> &callback)
+TimeoutHandle Utilities::add_timeout(float interval, const boost::function<bool ()> &callback)
 {
-  ControlFactory::get_instance()->_utilities_impl.add_timeout(interval, callback);
+  return ControlFactory::get_instance()->_utilities_impl.add_timeout(interval, callback);
+}
+
+
+//--------------------------------------------------------------------------------------------------
+
+void Utilities::cancel_timeout(TimeoutHandle handle)
+{
+  ControlFactory::get_instance()->_utilities_impl.cancel_timeout(handle);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -366,16 +420,17 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
   content.add(&description_label, 1, 2, 0, 1, HFillFlag | VFillFlag);
   content.add(&edit, 2, 3, 0, 1, HFillFlag | VFillFlag);
 
-  button_box.set_spacing(10);
+  button_box.set_spacing(8);
   ok_button.set_text(_("OK"));
-  ok_button.set_size(75, -1);
+//  ok_button.set_size(75, -1);
   cancel_button.set_text(_("Cancel"));
-  cancel_button.set_size(75, -1);
+//  cancel_button.set_size(75, -1);
   Utilities::add_end_ok_cancel_buttons(&button_box, &ok_button, &cancel_button);
   content.add(&button_box, 1, 3, 1, 2, HFillFlag);
 
   input_form.set_content(&content);
   input_form.center();
+  edit.focus();
   bool result= input_form.run_modal(&ok_button, &cancel_button);
   if (result)
     ret_value = edit.get_string_value();
@@ -387,6 +442,10 @@ bool Utilities::request_input(const std::string &title, const std::string &descr
 
 void Utilities::store_password(const std::string &service, const std::string &account, const std::string &password)
 {
+  // in-memory cache
+  PasswordCache::get()->add_password(service, account, password.c_str());
+  
+  // OS storage
   log_debug("Storing password for '%s'@'%s'\n", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.store_password(service, account, password);
 }
@@ -397,13 +456,26 @@ bool Utilities::find_password(const std::string &service, const std::string &acc
 {
   const bool ret = ControlFactory::get_instance()->_utilities_impl.find_password(service, account, password);
   log_debug("Looking up password for '%s'@'%s' has %s\n", account.c_str(), service.c_str(), ret ? "succeeded" : "failed");
+    
+  if (ret)
+    PasswordCache::get()->add_password(service, account, password.c_str());
+
   return ret;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+bool Utilities::find_cached_password(const std::string &service, const std::string &account, std::string &password)
+{
+  return PasswordCache::get()->get_password(service, account, password);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void Utilities::forget_password(const std::string &service, const std::string &account)
 {
+  PasswordCache::get()->remove_password(service, account);
+  
   log_debug("Forgetting password for '%s'@'%s'\n", account.c_str(), service.c_str());
   ControlFactory::get_instance()->_utilities_impl.forget_password(service, account);
 }
@@ -508,11 +580,11 @@ static void *_ask_for_password_main(const std::string &title, const std::string
     content.add(&save_password_box, 2, 3, 4, 5, HExpandFlag | HFillFlag);
   }
 
-  button_box.set_spacing(10);
+  button_box.set_spacing(8);
   ok_button.set_text(_("OK"));
-  ok_button.set_size(75, -1);
+//  ok_button.set_size(75, -1);
   cancel_button.set_text(_("Cancel"));
-  cancel_button.set_size(75, -1);
+//  cancel_button.set_size(75, -1);
   Utilities::add_end_ok_cancel_buttons(&button_box, &ok_button, &cancel_button);
   if (prompt_storage)
     content.add(&button_box, 1, 3, 5, 6, HFillFlag);
@@ -521,6 +593,9 @@ static void *_ask_for_password_main(const std::string &title, const std::string
   
   password_form.set_content(&content);
   password_form.center();
+  
+  password_edit.focus();
+  
   bool result= password_form.run_modal(&ok_button, &cancel_button);
   if (result)
   {
@@ -529,6 +604,9 @@ static void *_ask_for_password_main(const std::string &title, const std::string
 
     if (user_edit != NULL)
       *username = user_edit->get_string_value();
+    
+    // always store in cache
+    PasswordCache::get()->add_password(service, *username, ret_password->c_str());
   }
   
   return (void*)result;  
@@ -609,9 +687,9 @@ bool Utilities::credentials_for_service(const std::string &title, const std::str
       }
       catch (std::exception &exc)
       {
+        log_warning("Could not store password vault: %s\n", exc.what());
         show_warning(title.empty() ? _("Error Storing Password") : title, 
                      std::string("There was an error storing the password:\n")+exc.what(), "OK");
-        log_warning("Could not store password vault: %s\n", exc.what());
       }
     }
     return true;
@@ -647,15 +725,29 @@ bool Utilities::in_modal_loop()
 
 //--------------------------------------------------------------------------------------------------
 
+static cairo_user_data_key_t hidpi_icon_key;
+
 /**
  * Helper function to simplify icon loading. Returns NULL if the icon could not be found or
  * something wrong happened while loading.
  */
-cairo_surface_t* Utilities::load_icon(const std::string& name)
+cairo_surface_t* Utilities::load_icon(const std::string& name, bool allow_hidpi)
 {
   if (name.empty())
     return NULL;
 
+  if (allow_hidpi && mforms::App::get()->backing_scale_factor() > 1.0)
+  {
+    std::string hidpi_name = base::strip_extension(name) + "@2x" + base::extension(name);
+    cairo_surface_t *tmp = load_icon(hidpi_name, false);
+    if (tmp)
+    {
+      // Mark the surface as being a hi-res variant of a standard icon.
+      cairo_surface_set_user_data(tmp, &hidpi_icon_key, (void*)1, NULL);
+      return tmp;
+    }
+  }
+
   std::string icon_path= App::get()->get_resource_path(name);
   cairo_surface_t* result= cairo_image_surface_create_from_png(icon_path.c_str());
   if (result && cairo_surface_status(result) != CAIRO_STATUS_SUCCESS)
@@ -669,6 +761,77 @@ cairo_surface_t* Utilities::load_icon(const std::string& name)
 
 //--------------------------------------------------------------------------------------------------
 
+bool Utilities::is_hidpi_icon(cairo_surface_t *s)
+{
+  return cairo_surface_get_user_data(s, &hidpi_icon_key) == (void*)1;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool Utilities::icon_needs_reload(cairo_surface_t *s)
+{
+  float scale = s && mforms::Utilities::is_hidpi_icon(s) ? 2 : 1;
+  return mforms::App::get()->backing_scale_factor() != scale;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Utilities::paint_icon(cairo_t *cr, cairo_surface_t *image, double x, double y, float alpha)
+{
+  float backing_scale_factor = mforms::App::get()->backing_scale_factor();
+
+  if (backing_scale_factor > 1 && mforms::Utilities::is_hidpi_icon(image))
+  {
+    cairo_save(cr);
+    cairo_scale(cr, 1 / backing_scale_factor, 1 / backing_scale_factor);
+    cairo_set_source_surface(cr, image, x * backing_scale_factor, y * backing_scale_factor);
+    if (alpha == 1.0)
+      cairo_paint(cr);
+    else
+      cairo_paint_with_alpha(cr, alpha);
+    cairo_restore(cr);
+  }
+  else if (backing_scale_factor == 1 && mforms::Utilities::is_hidpi_icon(image))
+  {
+    // special case where the icon is for hidpi but the screen is not
+    // this happens when the icon was cached while the window was
+    // in a hidpi screen but is then dragged to a stddpi screen
+    // ideally these cases would trigger a reload of the icon
+    cairo_save(cr);
+    cairo_scale(cr, 0.5, 0.5);
+    cairo_set_source_surface(cr, image, x*2, y*2);
+    if (alpha == 1.0)
+      cairo_paint(cr);
+    else
+      cairo_paint_with_alpha(cr, alpha);
+    cairo_restore(cr);
+    log_debug2("Icon is for hidpi screen but the screen is not.\n");
+  }
+  else
+  {
+    cairo_set_source_surface(cr, image, x, y);
+    if (alpha == 1.0)
+      cairo_paint(cr);
+    else
+      cairo_paint_with_alpha(cr, alpha);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void Utilities::get_icon_size(cairo_surface_t *icon, int &w, int &h)
+{
+  w = cairo_image_surface_get_width(icon);
+  h = cairo_image_surface_get_height(icon);
+  if (mforms::Utilities::is_hidpi_icon(icon))
+  {
+    w = (int)(w / 2);
+    h = (int)(h / 2);
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 /**
  * Shortens the string so that it fits into the given width. If there is already enough room then
  * the input is simply returned. Otherwise letters are removed (via binary search) and ellipses
@@ -677,8 +840,8 @@ cairo_surface_t* Utilities::load_icon(const std::string& name)
 std::string Utilities::shorten_string(cairo_t* cr, const std::string& text, double width)
 {
   int ellipsis_width= 0;
-  int length;
-  int l, h, n, w;
+  size_t length;
+  size_t l, h, n, w;
   cairo_text_extents_t extents;
   
   // If the text fits already, return the input.
@@ -710,9 +873,9 @@ std::string Utilities::shorten_string(cairo_t* cr, const std::string& text, doub
       // Skip to the nth position, which needs the following loop as we don't have direct
       // access to a char in an utf-8 buffer (one of the limitations of that transformation format).
       const gchar* tail= head;
-      for (int i= 0; i < n; i++)
+      for (size_t i= 0; i < n; i++)
         tail= g_utf8_next_char(tail);
-      gchar* part= g_strndup(head, tail - head);
+      gchar* part= g_strndup(head, (gsize)(tail - head));
       cairo_text_extents(cr, part, &extents);
       g_free(part);
       w= (int) ceil(extents.width) + ellipsis_width;
@@ -727,6 +890,15 @@ std::string Utilities::shorten_string(cairo_t* cr, const std::string& text, doub
   return "";
 }
 
+
+double Utilities::get_text_width(const std::string &text, const std::string &font)
+{
+  if (ControlFactory::get_instance()->_utilities_impl.get_text_width)
+    return ControlFactory::get_instance()->_utilities_impl.get_text_width(text, font);
+  else
+    return text.length() * 20; // temporary hardcoded until implemented in Windows and Linux
+}
+
 //--------------------------------------------------------------------------------------------------
 
 bool Utilities::in_main_thread()
@@ -734,3 +906,20 @@ bool Utilities::in_main_thread()
   return g_thread_self() == _mforms_main_thread;
 }
 
+
+void Utilities::set_thread_name(const std::string &name)
+{
+  if (ControlFactory::get_instance()->_utilities_impl.set_thread_name)
+    ControlFactory::get_instance()->_utilities_impl.set_thread_name(name);
+}
+
+void Utilities::driver_shutdown()
+{
+  if (Utilities::_driver_shutdown_cb)
+    Utilities::_driver_shutdown_cb();
+}
+
+void Utilities::add_driver_shutdown_callback(const boost::function<void ()> &slot)
+{
+  Utilities::_driver_shutdown_cb = slot;
+}
