@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,8 +32,7 @@
 #include <stdexcept>
 #include <boost/function.hpp>
 #include <libxml/xmlmemory.h>
- 		  	
-#include <glib.h>
+#include "base/threading.h"
 
 #include <string>
 
@@ -60,23 +59,7 @@
 #define THROW(...) throw (__VA_ARGS__)
 #endif
 
-#ifdef _WIN32
-#include <hash_set>
-typedef stdext::hash_set<std::string> string_hash_set;
-#else
-#include <ext/hash_set>
-
-struct string_hash : public std::unary_function<std::string, size_t> 
-{
-  size_t operator() (const std::string &v) const
-  {
-    return __gnu_cxx::hash<const char*>()(v.c_str());
-  }
-};
-
-typedef __gnu_cxx::hash_set<std::string, string_hash> string_hash_set;
-#endif
-
+#include <set>
 
 #define GRT_VERSION "4.1.0"
 
@@ -188,15 +171,9 @@ namespace grt {
 
 
 #include "grtpp_value.h"
-//this is too unstable and overcomplicated
-//#define USE_EXPRERIMENTAL_REFS
-#ifdef USE_EXPRERIMENTAL_REFS
-#include "grtpp_references.h"
-#endif
 
 namespace grt {
 
-#ifndef USE_EXPRERIMENTAL_REFS
   class GRT;
   
   namespace internal
@@ -561,7 +538,33 @@ namespace grt {
 
   typedef Ref<internal::Integer> IntegerRef;
 
-  /** Reference object class for integer GRT values.
+  // In Win32 ssize_t and int are the same, so we get a compiler error if we compile functions/c-tors with
+  // those types (redefinition error). Hence we need a check when to exclude them.
+  // A similar problem exists for uint64_t and size_t in Win64.
+#ifdef _WIN32
+  #define DEFINE_INT_FUNCTIONS
+
+  #ifdef _WIN64
+    #define DEFINE_SSIZE_T_FUNCTIONS
+  #else
+    #define DEFINE_UINT64_T_FUNCTIONS
+  #endif
+#else
+  #define DEFINE_SSIZE_T_FUNCTIONS
+
+  #ifdef __x86_64__
+  #define DEFINE_INT_FUNCTIONS
+  #endif
+
+  #ifdef __APPLE__
+    // Probably also depending on 32bit vs 64bit, but for now we compile only 32bit on Mac.
+    #define DEFINE_UINT64_T_FUNCTIONS
+    #define DEFINE_INT_FUNCTIONS
+
+  #endif
+#endif
+
+  /** Reference object class for integer GRT values (32 or 64bit, depending on compiler architecture).
    * 
    * aka IntegerRef
    * 
@@ -570,7 +573,7 @@ namespace grt {
    *   InegerRef(1234);
    * @endcode
    * 
-   * An implicit constructor for long is available, so you can assign
+   * An implicit constructor for long/int/size_t is available, so you can assign
    * to a IntegerRef as:
    * @code 
    *   IntegerRef i= 1234;
@@ -615,15 +618,31 @@ namespace grt {
     {
     }
 
-    Ref(long value)
+#ifdef DEFINE_INT_FUNCTIONS
+    Ref(int value)
        : ValueRef(internal::Integer::get(value))
     {
     }
+#endif
 
-    Ref(int value)
+#ifndef DEFINE_INT_FUNCTIONS
+    Ref(long int value)
        : ValueRef(internal::Integer::get(value))
     {
     }
+#endif
+
+#ifdef DEFINE_SSIZE_T_FUNCTIONS
+    Ref(ssize_t value)
+      : ValueRef(internal::Integer::get(value))
+    {
+    }
+#endif
+
+    Ref(size_t value)
+      : ValueRef(internal::Integer::get(value))
+    {
+    }
 
     inline operator storage_type () const { return *content(); }
     inline storage_type operator *() const { return *content(); }
@@ -634,31 +653,52 @@ namespace grt {
     {
       return _value == o._value || (_value && o._value && *content() == *o);
     }
-     
-    inline bool operator==(long v) const
-    {
-      return _value && (*content() == v);
-    }
-
+#ifdef DEFINE_INT_FUNCTIONS
     inline bool operator==(int v) const
     {
       return _value && (*content() == v);
     }
+#endif
+
+#ifndef DEFINE_INT_FUNCTIONS
+    inline bool operator==(long int v) const
+    {
+      return _value && (*content() == v);
+    }
+#endif
+
+#ifdef DEFINE_SSIZE_T_FUNCTIONS
+    inline bool operator==(ssize_t v) const
+    {
+      return _value && (*content() == v);
+    }
+#endif
 
     inline bool operator!=(const IntegerRef &o) const
     {
       return !(operator ==(o));
     }
-
-    inline bool operator!=(long v) const
-    {
-      return _value && (*content() != v);
-    }
-
+#ifdef DEFINE_INT_FUNCTIONS
     inline bool operator!=(int v) const
     {
       return _value && (*content() != v);
     }
+#endif
+
+#ifndef DEFINE_INT_FUNCTIONS
+    inline bool operator!=(long int v) const
+    {
+      return _value && (*content() != v);
+    }
+#endif
+
+
+#ifdef DEFINE_SSIZE_T_FUNCTIONS
+    inline bool operator!=(ssize_t v) const
+    {
+      return _value && (*content() != v);
+    }
+#endif
 
   protected:
     explicit Ref(const ValueRef &ivalue)
@@ -957,6 +997,11 @@ namespace grt {
       ++iter;
       return temp;
     }
+
+    inline size_t operator - (const TypedListConstIterator &other) const
+    {
+      return iter - other.iter;
+    }
   };
 
   template<class C>
@@ -1100,17 +1145,7 @@ namespace grt {
     {
       return content().raw_end();
     }
-/*
-    inline raw_const_reverse_iterator rbegin() const
-    {
-      return content().raw_rbegin();
-    }
-    
-    inline raw_const_reverse_iterator rend() const
-    {
-      return content().raw_rend();
-    }
-  */
+
     template<typename TPred>
     bool foreach(TPred pred) const
     {
@@ -1698,6 +1733,8 @@ namespace grt {
   */ 
     inline size_t count() const { return content().count(); }
     
+    inline std::vector<std::string> keys() const { return content().keys(); }
+
     inline void remove(const std::string &k)
     {
       content().remove(k);
@@ -1870,7 +1907,6 @@ namespace grt {
         throw type_error(DictType, content().content_type(), ListType);
     }
   };
-#endif
 
 //--------------------------------------------------------------------------------------------------
 
@@ -1961,7 +1997,7 @@ namespace grt {
    * the list of members and methods with information about types and
    * other metadata. This information is used for generating C++ classes
    * that actually implement these classes and also for exposing the
-   * interface to these classes to the scripting interface (such as Lua).
+   * interface to these classes to the scripting interface (such as Python).
    * 
    * @ingroup GRT
    */
@@ -2044,7 +2080,7 @@ namespace grt {
     /** Calls slot iterating through all members of the metaclass.
     * 
     * The slot will be called for each member of the metaclass,
-    * including inherited ones. If a member is overriden,
+    * including inherited ones. If a member is overridden,
     * it will be called only once, for the "topmost" one.
     * The slot must return true as long as the iteration is
     * to be continued. Returning false will stop it.
@@ -2055,8 +2091,8 @@ namespace grt {
     template<typename TPred>
     bool foreach_member(TPred pred)
     {
-      // set of already seen members (only overriden ones)
-      string_hash_set seen;
+      // set of already seen members (only overridden ones)
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2080,7 +2116,7 @@ namespace grt {
     /** Calls slot iterating through all methods of the metaclass.
     * 
     * The slot will be called for each method of the metaclass,
-    * including inherited ones. If a method is overriden,
+    * including inherited ones. If a method is overridden,
     * it will be called only once, for the "topmost" one.
     * The slot must return true as long as the iteration is
     * to be continued. Returning false will stop it.
@@ -2092,8 +2128,8 @@ namespace grt {
     template<typename TPred>
     bool foreach_method(TPred pred)
     {
-      // set of already seen methods (only overriden ones)
-      string_hash_set seen;
+      // set of already seen methods (only overridden ones)
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2127,8 +2163,8 @@ namespace grt {
     template<typename TPred>
     bool foreach_signal(TPred pred)
     {
-      // set of already seen methods (only overriden ones)
-      string_hash_set seen;
+      // set of already seen methods (only overridden ones)
+      std::set<std::string> seen;
       MetaClass *mc= this;
 
       do
@@ -2293,10 +2329,8 @@ namespace grt {
    * 
    * @ingroup GRT
    */ 
-  class LuaModuleLoader;
   class MYSQLGRT_PUBLIC Module//
   {
-  friend class LuaModuleLoader;
   public:
     struct Function
     {
@@ -2778,7 +2812,7 @@ namespace grt {
     
     std::vector<std::pair<float,float> > _progress_step_stack;
 
-    GStaticRecMutex _message_mutex;
+    base::RecMutex _message_mutex;
 
     std::list<ModuleLoader*> _loaders;
     std::vector<Module*> _modules;
