@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "grtpp_module_python.h"
 #include "grtpp_util.h"
 #include "base/file_utilities.h"
@@ -28,8 +26,6 @@
 
 DEFAULT_LOG_DOMAIN("ModulePython");
 
-#ifdef ENABLE_PYTHON_MODULES
-
 using namespace grt;
 using namespace base;
 
@@ -322,7 +318,7 @@ Module *PythonModuleLoader::init_module(const std::string &path)
     Py_DECREF(old_path);
     if (mod == NULL)
     {
-      PyErr_Print();
+      PythonContext::log_python_error(base::strfmt("Error importing Python module %s\n", path.c_str()).c_str());
       return 0;
     }
   }
@@ -522,5 +518,4 @@ bool PythonModuleLoader::check_file_extension(const std::string &path)
 }
 
 
-#endif // ENABLE_PYTHON_MODULES
 
