@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -116,14 +116,15 @@ def connect(connection, password):
             grt.send_info("Connection to %s apparently lost, reconnecting..." % connection.hostIdentifier)
             raise NotConnectedError("Connection error")
     except NotConnectedError, exc:
-        grt.send_info("Connecting to %s..." % connection.hostIdentifier)
+        host_identifier = connection.hostIdentifier
+        version = ""
+        grt.send_info("Connecting to %s..." % host_identifier)        
         import pyodbc
         try:
             con = db_driver.connect(connection, password)
             
             # Adds data type conversion functions for pyodbc
             if connection.driver.driverLibraryName == 'pyodbc':
-                cursor = con.cursor()
                 version = con.execute("SELECT CAST(SERVERPROPERTY('ProductVersion') AS VARCHAR)").fetchone()[0]
                 majorVersion = int(version.split('.', 1)[0])
                 if majorVersion >= 9:
@@ -141,7 +142,7 @@ def connect(connection, password):
         if not con:
             grt.send_error('Connection failed', str(exc))
             raise
-        grt.send_info("Connected")
+        grt.send_info("Connected to %s, %s" % (host_identifier, version))
         
         _connections[connection.__id__] = {"connection" : con }
         _connections[connection.__id__]["version"] = getServerVersion(connection)
@@ -181,7 +182,7 @@ def getCatalogNames(connection):
 
     [NOTE] From MSDN: [A catalog] is equivalent to a databases in SQL Server.
     """
-    query = 'exec sp_databases'
+    query = 'exec sys.sp_databases'
     return [ row[0] for row in execute_query(connection, query) ]
 
 
@@ -643,6 +644,12 @@ ORDER BY ORDINAL_POSITION"""
         if datatype is not None:
             column.simpleType = mssql_rdbms_instance.simpleDatatypes[datatype]
 
+        # When information about the columns are retrieved from  sys.columns (server version is 9 or greater) - 
+        # field CHARACTER_MAXIMUM_LENGTH return length in bytes, so length in characters for type NCHAR and NVARCHAR
+        # sholud be divided by two. 
+        if serverVersion.majorNumber >= 9 and column.simpleType is not None and column.simpleType.name in ['NCHAR', 'NVARCHAR']:
+            column.length = column.length/2
+
         default_value = row_values['COLUMN_DEFAULT']
         
         if default_value is not None:
@@ -1151,4 +1158,13 @@ def getTDSProtocolVersionChoices():
 
     return result 
 
+@ModuleInfo.export(grt.INT, grt.classes.db_mgmt_Connection)
+def resetProgressFlags(connection):
+    _connections[connection.__id__]['_rev_eng_progress_flags'] = []
+    return 0
 
+@ModuleInfo.export(grt.INT)
+def cleanup():
+    global _connections
+    _connections = {}
+    return 0
