@@ -22,7 +22,7 @@ from wb import DefineModule
 import grt
 
 from db_generic_migration_grt import GenericMigration
-from workbench.utils import find_object_with_name
+from workbench.utils import find_object_with_name, Version
 
 ModuleInfo = DefineModule(name= "DbMssqlMigration", author= "Oracle Corp.", version="1.0")
 
@@ -271,7 +271,7 @@ class MSSQLMigration(GenericMigration):
                 target_column.precision = source_column.simpleType.numericPrecision
                 target_column.scale = source_column.simpleType.numericScale
             # binary datatypes:
-            elif source_datatype in ['IMAGE', 'BINARY', 'VARBINARY']:
+            elif source_datatype == 'IMAGE':
                 if source_column.length == -1:  # VARBINARY(MAX)
                    target_datatype = 'LONGBLOB'  #TODO: Give the user the choice for this target datatype
                 elif 0 <= source_column.length < 256:
@@ -283,10 +283,16 @@ class MSSQLMigration(GenericMigration):
                     target_datatype = 'MEDIUMBLOB'
                 else:
                     target_datatype = 'LONGBLOB'
+            elif source_datatype == 'VARBINARY':
+                if source_column.length == -1:  # VARBINARY(MAX)
+                   target_datatype = 'LONGBLOB'
             # datetime datatypes:
             elif source_datatype in ['DATETIME', 'SMALLDATETIME', 'DATETIME2', 'DATETIMEOFFSET']:
                 target_datatype = 'DATETIME'
                 target_column.precision = -1
+                target_version = Version.fromgrt(targetCatalog.version)
+                if target_version.is_supported_mysql_version_at_least(5,6,4) and source_datatype != 'SMALLDATETIME':
+                    target_column.precision = source_column.precision if source_column.precision < 7 else 6
             # timestamp datatypes
             # In MS SQL Server a nonnullable timestamp column is semantically equivalent to a binary(8) column, 
             # and a nullable timestamp column is semantically equivalent to a varbinary(8) column.
@@ -298,6 +304,9 @@ class MSSQLMigration(GenericMigration):
             elif source_datatype == 'TIME':
                 target_datatype = 'TIME'
                 target_column.precision = -1
+                target_version = Version.fromgrt(targetCatalog.version)
+                if target_version.is_supported_mysql_version_at_least(5,6,4):
+                    target_column.precision = source_column.precision if source_column.precision < 7 else 6
             elif source_datatype == 'BIT':
                 target_datatype = 'TINYINT'
                 target_column.length = 1
@@ -383,8 +392,8 @@ class MSSQLMigration(GenericMigration):
                             type_cast_expression = "CAST(? as VARCHAR(max))"
                         elif source_datatype == 'SQL_VARIANT':
                             type_cast_expression = "CAST(? as NVARCHAR(max))"
-                        elif source_datatype in ['TIMESTAMP', 'ROWVERSION']:
-                            type_cast_expression = 'CAST(? as %s)' % ('BINARY' if source_column.isNotNull else 'VARBINARY')
+                        elif source_datatype in ['BINARY', 'VARBINARY', 'TIMESTAMP', 'ROWVERSION']:
+                            type_cast_expression = 'CONVERT(VARBINARY(MAX), ?, 0)'
 
                         if type_cast_expression:
                             target_column.owner.customData["columnTypeCastExpression:%s" % target_column.name] = "%s as ?" % type_cast_expression
