@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,17 +17,14 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <map>
+#include <memory>
+
 #include "grtpp_module_cpp.h"
 #include "cppdbc.h"
 
 #include "grts/structs.db.mgmt.h"
 
-#include <cppdbc.h>
-#include <memory>
-
 #define DOC_DbMySQLQueryImpl \
 "Query execution and utility routines for  MySQL servers.\n"\
 "\n"\
@@ -42,14 +39,10 @@ public:
   DbMySQLQueryImpl(grt::CPPModuleLoader *loader) 
   : grt::ModuleImplBase(loader),
   _last_error_code(0), _connection_id(0), _resultset_id(0), _tunnel_id(0)
-  {
-    _mutex = g_mutex_new();
-  }
+  {}
 
   virtual ~DbMySQLQueryImpl()
-  {
-    g_mutex_free(_mutex);
-  }
+  {}
 
   DEFINE_INIT_MODULE_DOC("1.0", "Oracle", DOC_DbMySQLQueryImpl, grt::ModuleImplBase,
                   DECLARE_MODULE_FUNCTION_DOC(DbMySQLQueryImpl::openConnection,
@@ -176,7 +169,12 @@ public:
   
   int lastConnectionErrorCode(int conn);
   std::string lastConnectionError(int conn);
-  long lastUpdateCount(int conn);
+//  #ifdef DEFINE_INT_FUNCTIONS
+  size_t lastUpdateCount(int conn);
+//  #else
+//  uint64_t lastUpdateCount(int conn);
+//  #endif
+  
 
   // returns 1/0 for ok, -1 for error
   int execute(int conn, const std::string &query);
@@ -184,7 +182,7 @@ public:
   // returns result-id or -1 for error
   int executeQuery(int conn, const std::string &query);
 
-  long resultNumRows(int result);
+  size_t resultNumRows(int result);
   int resultNumFields(int result);
   std::string resultFieldType(int result, int field);
   std::string resultFieldName(int result, int field);
@@ -238,10 +236,16 @@ private:
     sql::ConnectionWrapper conn;
     std::string last_error;
     int last_error_code;
-    long long last_update_count;
+//    #ifdef DEFINE_INT_FUNCTIONS
+	// size_t only for the time being, will be changed to uint64_t once we have big integer support
+	// for 32bit also.
+    size_t last_update_count;
+//    #else
+//    uint64_t last_update_count;
+//    #endif
   };
 
-  GMutex *_mutex;
+  base::Mutex _mutex;
   std::map<int, ConnectionInfo::Ref> _connections;
   std::map<int, sql::ResultSet*> _resultsets;
   std::map<int, boost::shared_ptr<sql::TunnelConnection> > _tunnels;
@@ -259,24 +263,6 @@ GRT_MODULE_ENTRY_POINT(DbMySQLQueryImpl);
 
 #define CLEAR_ERROR() do { _last_error.clear(); _last_error_code = 0; } while (0)
 
-
-struct Lock
-{
-  GMutex *mutex;
-
-  Lock(GMutex *mtx)
-  {
-    mutex = mtx;
-    g_mutex_lock(mutex);
-  }
-
-  ~Lock()
-  {
-    g_mutex_unlock(mutex);
-  }
-};
-
-
 int DbMySQLQueryImpl::openConnection(const db_mgmt_ConnectionRef &info)
 {
   return openConnectionP(info, grt::StringRef());
@@ -296,7 +282,7 @@ int DbMySQLQueryImpl::openConnectionP(const db_mgmt_ConnectionRef &info, const g
   try
   {
     {
-      Lock lock(_mutex);
+      base::MutexLock lock(_mutex);
       new_connection_id = ++_connection_id;
     }
     sql::ConnectionWrapper conn;
@@ -310,14 +296,14 @@ int DbMySQLQueryImpl::openConnectionP(const db_mgmt_ConnectionRef &info, const g
     else
         conn = dm->getConnection(info);
 
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     _connections[new_connection_id] = ConnectionInfo::Ref(new ConnectionInfo(conn));
   }
   catch (sql::SQLException &exc)
   {
     _last_error = exc.what();
     _last_error_code = exc.getErrorCode();
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(new_connection_id) != _connections.end())
       _connections.erase(new_connection_id);
     return -1;    
@@ -340,7 +326,7 @@ int DbMySQLQueryImpl::closeConnection(int conn)
 {
   CLEAR_ERROR();
 
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_connections.find(conn) == _connections.end())
     throw std::invalid_argument("Invalid connection");
   _connections.erase(conn);
@@ -368,7 +354,7 @@ int DbMySQLQueryImpl::execute(int conn, const std::string &query)
   ConnectionInfo::Ref cinfo;
   sql::Connection *con= 0;
   {
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(conn) == _connections.end())
       throw std::invalid_argument("Invalid connection");
     cinfo = _connections[conn];
@@ -379,7 +365,7 @@ int DbMySQLQueryImpl::execute(int conn, const std::string &query)
   {
     std::auto_ptr<sql::Statement> pstmt(con->createStatement());
     int r = pstmt->execute(query) ? 1 : 0;
-    cinfo->last_update_count = pstmt->getUpdateCount();
+    cinfo->last_update_count = (size_t)pstmt->getUpdateCount();
     return r;
   }
   catch (sql::SQLException &exc)
@@ -408,7 +394,7 @@ int DbMySQLQueryImpl::executeQuery(int conn, const std::string &query)
   ConnectionInfo::Ref cinfo;
   sql::Connection *con= 0;
   {
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(conn) == _connections.end())
       throw std::invalid_argument("Invalid connection");
     cinfo = _connections[conn];
@@ -422,7 +408,7 @@ int DbMySQLQueryImpl::executeQuery(int conn, const std::string &query)
 
     ++_resultset_id;
 
-    cinfo->last_update_count = pstmt->getUpdateCount();
+    cinfo->last_update_count = (size_t)pstmt->getUpdateCount();
     _resultsets[_resultset_id] = res;
   }
   catch (sql::SQLException &exc)
@@ -444,18 +430,28 @@ int DbMySQLQueryImpl::executeQuery(int conn, const std::string &query)
 }
 
 
-long DbMySQLQueryImpl::lastUpdateCount(int conn)
+//#ifdef DEFINE_INT_FUNCTIONS
+size_t DbMySQLQueryImpl::lastUpdateCount(int conn)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_connections.find(conn) == _connections.end())
     throw std::invalid_argument("Invalid connection");
-  return (long)_connections[conn]->last_update_count;
+  return _connections[conn]->last_update_count;
 }
+/*#else
+uint64_t DbMySQLQueryImpl::lastUpdateCount(int conn)
+{
+  base::MutexLock lock(_mutex);
+  if (_connections.find(conn) == _connections.end())
+    throw std::invalid_argument("Invalid connection");
+  return _connections[conn]->last_update_count;
+}
+#endif*/
 
 
 int DbMySQLQueryImpl::lastConnectionErrorCode(int conn)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_connections.find(conn) == _connections.end())
     throw std::invalid_argument("Invalid connection");
   return _connections[conn]->last_error_code;
@@ -464,16 +460,16 @@ int DbMySQLQueryImpl::lastConnectionErrorCode(int conn)
 
 std::string DbMySQLQueryImpl::lastConnectionError(int conn)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_connections.find(conn) == _connections.end())
     throw std::invalid_argument("Invalid connection");
   return _connections[conn]->last_error;
 }
 
 
-long DbMySQLQueryImpl::resultNumRows(int result)
+size_t DbMySQLQueryImpl::resultNumRows(int result)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -484,7 +480,7 @@ long DbMySQLQueryImpl::resultNumRows(int result)
 
 int DbMySQLQueryImpl::resultNumFields(int result)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -495,7 +491,7 @@ int DbMySQLQueryImpl::resultNumFields(int result)
 
 std::string DbMySQLQueryImpl::resultFieldType(int result, int field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -506,7 +502,7 @@ std::string DbMySQLQueryImpl::resultFieldType(int result, int field)
 
 std::string DbMySQLQueryImpl::resultFieldName(int result, int field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -517,7 +513,7 @@ std::string DbMySQLQueryImpl::resultFieldName(int result, int field)
 
 int DbMySQLQueryImpl::resultFirstRow(int result)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];  
@@ -527,7 +523,7 @@ int DbMySQLQueryImpl::resultFirstRow(int result)
 
 int DbMySQLQueryImpl::resultNextRow(int result)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];  
@@ -537,7 +533,7 @@ int DbMySQLQueryImpl::resultNextRow(int result)
 
 grt::IntegerRef DbMySQLQueryImpl::resultFieldIntValue(int result, int field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -551,18 +547,18 @@ grt::IntegerRef DbMySQLQueryImpl::resultFieldIntValue(int result, int field)
 
 double DbMySQLQueryImpl::resultFieldDoubleValue(int result, int field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
   
-  return res->getDouble(field);
+  return (double)res->getDouble(field);
 }
 
 
 grt::StringRef DbMySQLQueryImpl::resultFieldStringValue(int result, int field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -576,7 +572,7 @@ grt::StringRef DbMySQLQueryImpl::resultFieldStringValue(int result, int field)
 
 grt::IntegerRef  DbMySQLQueryImpl::resultFieldIntValueByName(int result, const std::string &field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -590,18 +586,18 @@ grt::IntegerRef  DbMySQLQueryImpl::resultFieldIntValueByName(int result, const s
 
 double DbMySQLQueryImpl::resultFieldDoubleValueByName(int result, const std::string &field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
   
-  return res->getDouble(field);
+  return (double)res->getDouble(field);
 }
 
 
 grt::StringRef DbMySQLQueryImpl::resultFieldStringValueByName(int result, const std::string &field)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     throw std::invalid_argument("Invalid resultset");
   sql::ResultSet *res = _resultsets[result];
@@ -615,7 +611,7 @@ grt::StringRef DbMySQLQueryImpl::resultFieldStringValueByName(int result, const
 
 int DbMySQLQueryImpl::closeResult(int result)
 {
-  Lock lock(_mutex);
+  base::MutexLock lock(_mutex);
   if (_resultsets.find(result) == _resultsets.end())
     return -1;
   sql::ResultSet *res = _resultsets[result];
@@ -632,7 +628,7 @@ int DbMySQLQueryImpl::loadSchemata(int conn, grt::StringListRef schemata)
   ConnectionInfo::Ref cinfo;
   sql::Connection *con = 0;
   {
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(conn) == _connections.end())
       throw std::invalid_argument("Invalid connection");
     cinfo = _connections[conn];
@@ -678,7 +674,7 @@ grt::DictRef DbMySQLQueryImpl::getServerVariables(int conn)
   ConnectionInfo::Ref cinfo;
   sql::Connection *con = 0;
   {
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(conn) == _connections.end())
       throw std::invalid_argument("Invalid connection");
     cinfo = _connections[conn];
@@ -730,7 +726,7 @@ int DbMySQLQueryImpl::loadSchemaObjects(int conn, grt::StringRef schema, grt::St
   ConnectionInfo::Ref cinfo;
   sql::Connection *con = 0;
   {
-    Lock lock(_mutex);
+    base::MutexLock lock(_mutex);
     if (_connections.find(conn) == _connections.end())
       throw std::invalid_argument("Invalid connection");
     cinfo = _connections[conn];
