@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,8 +28,18 @@
 #include "grts/structs.db.mysql.h"
 #include "grtpp_module_cpp.h"
 
+#include "mysql-parser.h"
 
-#define MysqlSqlFacade_VERSION "1.0"
+#ifdef _WIN32
+#include <unordered_set>
+#else
+#include <tr1/unordered_set>
+namespace std {
+  using tr1::unordered_set;
+};
+#endif
+
+#define MysqlSqlFacade_VERSION "2.0"
  
 #define DOC_MysqlSqlFacadeImpl \
 "MySQL parser routines. Allows parsing SQL scripts into GRT objects, checking syntax etc."
@@ -71,27 +81,30 @@ public:
                                 "filename the SQL script file to be parsed\n"
                                 "options a dictionary containing various options for the parser routine"),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::parseInserts),
-    DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::parseTriggers,
-                                "Parses triggers from the SQL script and adds them to the given table object.",
-                                "table an instantiated table object where the triggers should be added\n"
+
+    DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::parseTrigger,
+                                "Parses a trigger from the SQL script and applies it to the given view object.",
+                                "trigger an instantiated trigger object to fill\n"
                                 "sql the SQL script to be parsed"),
+
     DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::parseRoutine,
                                 "Parses a stored procedure or function from the SQL script and applies it to the given routine object.",
-                                "routine an instantiated routine object where to be initialized\n"
+                                "routine an instantiated routine object to fill\n"
                                 "sql the SQL script to be parsed"),
     DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::parseRoutines,
                                 "Parses a set of stored procedure or function from the SQL script and adds them to the given routine group object.",
-                                "routineGroup an instantiated routine group object to be filled\n"
+                                "routineGroup an instantiated routine group object to fill\n"
                                 "sql the SQL script to be parsed"),
     DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::parseView,
                                 "Parses a view from the SQL script and applies it to the given view object.",
-                                "view an instantiated view object where to be initialized\n"
+                                "view an instantiated view object to fill\n"
                                 "sql the SQL script to be parsed"),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::checkSqlSyntax),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::checkTriggerSyntax),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::checkViewSyntax),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::checkRoutineSyntax),
     DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::renameSchemaReferences),
+	  DECLARE_MODULE_FUNCTION(MysqlSqlFacadeImpl::parseStatement),
     DECLARE_MODULE_FUNCTION_DOC(MysqlSqlFacadeImpl::splitSqlStatements,
                                 "Splits the given SQL script into separate statements, returning a list of strings. For large scripts, getSqlStatementRanges() is preferred, as it will not create a copy of each statement.",
                                 "sql a SQL script, with one or more statements, separated by ;"),
@@ -112,7 +125,7 @@ public:
 
   int splitSqlScript(const std::string &sql, std::list<std::string> &statements);
   int splitSqlScript(const char *sql, size_t length, const std::string &initial_delimiter,
-    std::vector<std::pair<size_t, size_t> > &ranges, const char *line_break = "\n");
+    std::vector<std::pair<size_t, size_t> > &ranges, const std::string &line_break = "\n");
   
   Sql_parser::Ref sqlParser();
   int parseSqlScriptString(db_CatalogRef catalog, const std::string sql);
@@ -121,17 +134,17 @@ public:
   int parseSqlScriptFileEx(db_CatalogRef catalog, const std::string filename, const grt::DictRef options);
 
   Invalid_sql_parser::Ref invalidSqlParser();
-  int parseInserts(db_TableRef table, const std::string sql);
-  int parseTriggers(db_TableRef table, const std::string sql);
-  int parseRoutine(db_RoutineRef routine, const std::string sql);
-  int parseRoutines(db_RoutineGroupRef routineGroup, const std::string sql);
-  int parseView(db_ViewRef view, const std::string sql);
+  int parseInserts(db_TableRef table, const std::string &sql);
+  int parseTrigger(db_TriggerRef trigger, const std::string &sql);
+  int parseRoutine(db_RoutineRef routine, const std::string &sql);
+  int parseRoutines(db_RoutineGroupRef routineGroup, const std::string &sql);
+  int parseView(db_ViewRef view, const std::string &sql);
 
   Sql_syntax_check::Ref sqlSyntaxCheck();
-  int checkSqlSyntax(const std::string sql);
-  int checkTriggerSyntax(const std::string sql);
-  int checkViewSyntax(const std::string sql);
-  int checkRoutineSyntax(const std::string sql);
+  int checkSqlSyntax(const std::string &sql);
+  int checkTriggerSyntax(const std::string &sql);
+  int checkViewSyntax(const std::string &sql);
+  int checkRoutineSyntax(const std::string &sql);
 
   Sql_semantic_check::Ref sqlSemanticCheck();
 
@@ -154,7 +167,7 @@ public:
   // message will be there instead of the AST
   grt::BaseListRef parseAstFromSqlScript(const std::string &sql); 
 
-  Sql_statement_decomposer::Ref sqlStatementDecomposer();
+  Sql_statement_decomposer::Ref sqlStatementDecomposer(grt::DictRef db_opts = grt::DictRef());
 
   grt::BaseListRef getItemFromPath(const std::string& path, const grt::BaseListRef source);
   std::string getTypeDescription(grt::BaseListRef type_node, std::vector<std::string> *additional_type_data_paths = NULL);
@@ -162,9 +175,17 @@ public:
   virtual bool parseRoutineDetails(const std::string &sql, std::string &type, std::string &name, String_tuple_list &parameters, std::string& return_value, std::string& comments);
   virtual bool parseDropStatement(const std::string &sql, std::string &object_type, std::vector<std::pair<std::string, std::string> > &object_names);
 
+  grt::DictRef parseStatement(const std::string &sql_statement, int server_version, int utf, const std::string &sql_mode);
+
   void stop_processing();
 private:
   bool _stop;
+
+
+  grt::DictRef parseUserDefinition(MySQLRecognizerTreeWalker &walker);
+  grt::StringListRef createList(MySQLRecognizerTreeWalker &walker, int separator_symbol, std::unordered_set<int> &stop_symbols);
+  grt::StringRef concatenateTokens(MySQLRecognizerTreeWalker &walker, std::unordered_set<int> &stop_symbols, const std::string &separator = " ");
+  grt::DictRef parseGrantStatement(MySQLRecognizer &recognizer);
 };
 
 
