@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,14 +17,12 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include "db_mysql_diffsqlgen.h"
 
-#include "diffchange.h"
-#include "grtdiff.h"
-#include "changeobjects.h"
-#include "changelistobjects.h"
+#include "diff/diffchange.h"
+#include "diff/grtdiff.h"
+#include "diff/changeobjects.h"
+#include "diff/changelistobjects.h"
 
 #include "db_mysql_diffsqlgen_grant.h"
 
@@ -98,7 +96,7 @@ void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table,
     part_count_set= false, part_defs_set= false;
   std::string part_type, part_expr, subpart_type, subpart_expr;
   grt::ListRef<db_mysql_PartitionDefinition> part_defs(table->get_grt());
-  int part_count= 0;
+  ssize_t part_count = 0;
 
   // gather all relevant attributes from change object
   const grt::ChangeSet *cs= table_diffchange->subchanges();
@@ -195,7 +193,7 @@ void DiffSQLGeneratorBE::generate_set_partitioning(db_mysql_TableRef table,
   if(!part_defs_set)
     part_defs= table->partitionDefinitions();
 
-  callback->alter_table_generate_partitioning(table, part_type, part_expr, part_count, 
+  callback->alter_table_generate_partitioning(table, part_type, part_expr, (int)part_count,
                         subpart_type, subpart_expr, part_defs);
 }
 
@@ -204,7 +202,7 @@ void DiffSQLGeneratorBE::generate_create_partitioning(db_mysql_TableRef table)
   callback->alter_table_generate_partitioning(table, 
     std::string(table->partitionType().is_valid() ? table->partitionType().c_str() : ""),
     std::string(table->partitionExpression().is_valid() ? table->partitionExpression().c_str() : ""),
-    table->partitionCount(),
+    (int)table->partitionCount(),
     std::string(table->subpartitionType().is_valid() ? table->subpartitionType().c_str() : ""),
     std::string(table->subpartitionExpression().is_valid() ? table->subpartitionExpression().c_str() : ""),
     table->partitionDefinitions()
@@ -263,8 +261,8 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
         callback->create_table_index(index, false);
         continue;      
       }
-      
-      if (_skip_fk_indexes && (stricmp(index->indexType().c_str(), "FOREIGN") == 0
+
+      if (_skip_fk_indexes && (strcasecmp(index->indexType().c_str(), "FOREIGN") == 0
                                  || fk_indexes.find(index) != fk_indexes.end()))
         continue;
 
@@ -313,7 +311,8 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(table->checksum() != 0)
       callback->create_table_checksum(table->checksum());
 
-    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str()))
+    if(table->defaultCollationName().is_valid() && strlen(table->defaultCollationName().c_str())&&
+        (get_collation_cs(table->defaultCollationName()) == (table->defaultCharacterSetName().c_str())) )
       callback->create_table_collate(table->defaultCollationName());
 
     if(strlen(table->comment().c_str()))
@@ -350,6 +349,9 @@ void DiffSQLGeneratorBE::generate_create_stmt(db_mysql_TableRef table)
     if(strlen(table->rowFormat().c_str()))
       callback->create_table_row_format(table->rowFormat());
 
+    if(strlen(table->keyBlockSize().c_str()))
+      callback->create_table_key_block_size(table->keyBlockSize());
+
     if(strlen(table->mergeUnion().c_str()))
       callback->create_table_merge_union(table->mergeUnion());
 
@@ -705,7 +707,6 @@ void DiffSQLGeneratorBE::generate_alter(grt::ListRef<db_mysql_Index> indices, co
     case grt::ListItemModified: 
       {
         const grt::ListItemModifiedChange *modified_change= static_cast<const grt::ListItemModifiedChange *>(index_change.get());
-
         callback->alter_table_drop_index(db_mysql_IndexRef::cast_from(modified_change->get_new_value()));
         callback->alter_table_add_index(db_mysql_IndexRef::cast_from(modified_change->get_new_value()));
       }
@@ -803,6 +804,9 @@ void DiffSQLGeneratorBE::generate_alter(
     case grt::ListItemOrderChanged: // DROP/ADD FK
       {
         const grt::ListItemOrderChange *order_change= static_cast<const grt::ListItemOrderChange *>(fk_change.get());
+        //Only position changed nothing to do
+        if (!order_change->get_subchange())
+          break;
         fk1= db_mysql_ForeignKeyRef::cast_from(order_change->get_old_value());
         fk2= db_mysql_ForeignKeyRef::cast_from(order_change->get_new_value());
       }
@@ -839,6 +843,9 @@ void DiffSQLGeneratorBE::generate_alter(
     case grt::ListItemOrderChanged: // DROP/ADD FK
       {
         const grt::ListItemOrderChange *order_change= static_cast<const grt::ListItemOrderChange *>(fk_change.get());
+        //Only position changed nothing to do
+        if (!order_change->get_subchange())
+          break;
         callback->alter_table_drop_fk(db_mysql_ForeignKeyRef::cast_from(order_change->get_old_value()));
         callback->alter_table_add_fk(db_mysql_ForeignKeyRef::cast_from(order_change->get_new_value()));
       }
@@ -897,79 +904,121 @@ void DiffSQLGeneratorBE::generate_alter_stmt_drops(db_mysql_TableRef table, cons
 
 }
 
-void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt::DiffChange *diffchange)
+void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt::DiffChange *diffchange, AlterTableFlags alter_table_flags)
 {
   if (table->isStub())
     return;
 
-  bool process_table= true;
-
-  std::string table_name_for_filter(get_old_object_name_for_key(table, _case_sensitive));
-  if(_use_filtered_lists)
-    if(_filtered_tables.find(table_name_for_filter) == _filtered_tables.end())
-      process_table= false;
-
-  // process table triggers
   const grt::ChangeSet *cs= diffchange->subchanges();
 
-  for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); it != e; it++)
+  if ((alter_table_flags & EverythingButForeignKeys))
   {
-    const grt::ObjectAttrModifiedChange *attr_change= 
-      static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
-
-    if(attr_change->get_attr_name().compare("triggers") == 0)
+    // process table triggers
+    for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); it != e; it++)
     {
-      const grt::MultiChange *list_change= 
-        static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
-      
-      const grt::ChangeSet *triggers_cs= list_change->subchanges();
-      
-      for(grt::ChangeSet::const_iterator e2= triggers_cs->end(), jt= triggers_cs->begin(); jt != e2; jt++)
+      const grt::ObjectAttrModifiedChange *attr_change= 
+        static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
+
+      if(attr_change->get_attr_name().compare("triggers") == 0)
       {
-        const grt::DiffChange *trigger_change= jt->get();
-
-        switch(trigger_change->get_change_type())
+        const grt::MultiChange *list_change= 
+          static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
+        
+        const grt::ChangeSet *triggers_cs= list_change->subchanges();
+        
+        for(grt::ChangeSet::const_iterator e2= triggers_cs->end(), jt= triggers_cs->begin(); jt != e2; jt++)
         {
-        case grt::ListItemAdded:
-          {
-            db_mysql_TriggerRef trigger(db_mysql_TriggerRef::cast_from(
-              static_cast<const grt::ListItemAddedChange *>(trigger_change)->get_value()));
-            generate_create_stmt(trigger, false);
-          }
-          break;
-        case grt::ListItemRemoved:
-          {
-            db_mysql_TriggerRef trigger(db_mysql_TriggerRef::cast_from(
-                static_cast<const grt::ListItemRemovedChange *>(trigger_change)->get_value()));
-            generate_drop_stmt(trigger, false);
-          }
-          break;
-        case grt::ListItemModified:
-          {
-            db_mysql_TriggerRef old_trigger= db_mysql_TriggerRef::cast_from(
-              grt::ValueRef(static_cast<const grt::ListItemModifiedChange *>
-              (trigger_change)->get_old_value()));
-            db_mysql_TriggerRef new_trigger= db_mysql_TriggerRef::cast_from(
-              grt::ValueRef(static_cast<const grt::ListItemModifiedChange *>
-              (trigger_change)->get_new_value()));
+          const grt::DiffChange *trigger_change= jt->get();
 
-            generate_drop_stmt(old_trigger);
-            generate_create_stmt(new_trigger);
-          }
-          break;
-        // list item position change is not relevant        
+          switch(trigger_change->get_change_type())
+          {
+          case grt::ListItemAdded:
+            {
+              db_mysql_TriggerRef trigger(db_mysql_TriggerRef::cast_from(
+                static_cast<const grt::ListItemAddedChange *>(trigger_change)->get_value()));
+              generate_create_stmt(trigger, true);
+            }
+            break;
+          case grt::ListItemRemoved:
+            {
+              db_mysql_TriggerRef trigger(db_mysql_TriggerRef::cast_from(
+                  static_cast<const grt::ListItemRemovedChange *>(trigger_change)->get_value()));
+              generate_drop_stmt(trigger, true);
+            }
+            break;
+          case grt::ListItemModified:
+            {
+              db_mysql_TriggerRef old_trigger= db_mysql_TriggerRef::cast_from(
+                grt::ValueRef(static_cast<const grt::ListItemModifiedChange *>
+                (trigger_change)->get_old_value()));
+              db_mysql_TriggerRef new_trigger= db_mysql_TriggerRef::cast_from(
+                grt::ValueRef(static_cast<const grt::ListItemModifiedChange *>
+                (trigger_change)->get_new_value()));
+
+              generate_drop_stmt(old_trigger, true);
+              generate_create_stmt(new_trigger, true);
+            }
+            break;
+          case grt::ListItemOrderChanged:
+            {
+              const grt::ListItemOrderChange *order_change = dynamic_cast<const grt::ListItemOrderChange*>(trigger_change);
+
+              if (order_change)// && order_change->get_subchange())
+              {
+//                const grt::ListItemModifiedChange *change = dynamic_cast<const grt::ListItemModifiedChange *>(order_change->get_subchange().get());
+                
+//                if (change)
+                {
+                  generate_drop_stmt(db_mysql_TriggerRef::cast_from(order_change->get_old_value()), true);
+                  generate_create_stmt(db_mysql_TriggerRef::cast_from(order_change->get_new_value()), true);
+                }
+              }
+            }
           default: break;
+          }
         }
       }
     }
   }
 
-  if(!process_table)
-    return;
+  const std::string table_name_for_filter(get_old_object_name_for_key(table, _case_sensitive));
+  if(_use_filtered_lists && (_filtered_tables.find(table_name_for_filter) == _filtered_tables.end()))
+      return;
 
   bool partitions_processed= false;
   cs= diffchange->subchanges();
 
+  // check whether all changes detected for this table are dummy changes (index reorders or fk reorders)
+  bool dummy_changes_only = true;
+  for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); it != e; it++)
+  {
+    const grt::ObjectAttrModifiedChange *attr_change=
+    dynamic_cast<const grt::ObjectAttrModifiedChange *>(it->get());
+
+    if (attr_change->get_attr_name().compare("indices") == 0
+        || attr_change->get_attr_name().compare("foreignKeys") == 0)
+    {
+      const grt::MultiChange *list_change=
+      dynamic_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
+      // ignore changes if they're only reorderings (with no subchanges)
+      if (list_change && list_change->subchanges())
+      {
+        const grt::ListItemOrderChange *order_change = NULL;
+        if (list_change->subchanges()->changes.size() == 1)
+          order_change = dynamic_cast<const grt::ListItemOrderChange *>(list_change->subchanges()->begin()->get());
+        if (!order_change || order_change->get_subchange())
+        {
+          dummy_changes_only = false;
+          break;
+        }
+      }
+    }
+    else
+      dummy_changes_only = false;
+  }
+  if (dummy_changes_only)
+    return;
+
   callback->alter_table_props_begin(table);
 
   for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); it != e; it++)
@@ -977,6 +1026,11 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     const grt::ObjectAttrModifiedChange *attr_change= 
         static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
 
+    if (!(alter_table_flags & OnlyForeignKeys) && attr_change->get_attr_name().compare("foreignKeys") == 0)
+      continue;
+    else if (!(alter_table_flags & EverythingButForeignKeys) && attr_change->get_attr_name().compare("foreignKeys") != 0)
+      continue;
+
     if(attr_change->get_attr_name().compare("name") == 0)
     {
       const grt::SimpleValueChange *change= 
@@ -994,18 +1048,37 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
     {
       const grt::MultiChange *list_change= 
         static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
-      callback->alter_table_indexes_begin(table);
-      generate_alter(table->indices(), list_change);
-      callback->alter_table_indexes_end(table);
+      // ignore changes if they're only reorderings
+      if (list_change && list_change->subchanges())
+      {
+        const grt::ListItemOrderChange *order_change = NULL;
+        if (list_change->subchanges()->changes.size() == 1)
+          order_change = dynamic_cast<const grt::ListItemOrderChange *>(list_change->subchanges()->begin()->get());
+        if (!order_change || order_change->get_subchange())
+        {
+          callback->alter_table_indexes_begin(table);
+          generate_alter(table->indices(), list_change);
+          callback->alter_table_indexes_end(table);
+        }
+      }
     }
     else if(attr_change->get_attr_name().compare("foreignKeys") == 0)
     {
       const grt::MultiChange *list_change= 
         static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
-      
-      callback->alter_table_fks_begin(table);
-      generate_alter(table->foreignKeys(), list_change);
-      callback->alter_table_fks_end(table);
+      // ignore changes if they're only reorderings
+      if (list_change && list_change->subchanges())
+      {
+        const grt::ListItemOrderChange *order_change = NULL;
+        if (list_change->subchanges()->changes.size() == 1)
+          order_change = dynamic_cast<const grt::ListItemOrderChange *>(list_change->subchanges()->begin()->get());
+        if (!order_change || order_change->get_subchange())
+        {
+          callback->alter_table_fks_begin(table);
+          generate_alter(table->foreignKeys(), list_change);
+          callback->alter_table_fks_end(table);
+        }
+      }
     }
     else if(attr_change->get_attr_name().compare("tableEngine") == 0)
     {
@@ -1106,6 +1179,12 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_row_format(table, str);
     }
+    else if(attr_change->get_attr_name().compare("keyBlockSize") == 0)
+    {
+      const grt::SimpleValueChange *change= static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
+      grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
+      callback->alter_table_key_block_size(table, str);
+    }
     else if(attr_change->get_attr_name().compare("avgRowLength") == 0)
     {
       const grt::SimpleValueChange *change= 
@@ -1140,102 +1219,88 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_TableRef table, const grt:
       grt::StringRef str= grt::StringRef::cast_from(change->get_new_value());
       callback->alter_table_connection_string(table, str);
     }
-    else if((attr_change->get_attr_name().compare("partitionType") == 0) 
+    else if(!partitions_processed &&
+       ( (attr_change->get_attr_name().compare("partitionType") == 0) 
        || (attr_change->get_attr_name().compare("partitionExpression") == 0)
        || (attr_change->get_attr_name().compare("subpartitionType") == 0)
        || (attr_change->get_attr_name().compare("subpartitionExpression") == 0)
-       || (attr_change->get_attr_name().compare("subpartitionCount") == 0))
+       || (attr_change->get_attr_name().compare("subpartitionCount") == 0)))
     {
-      if(partitions_processed)
-        continue;
-
-      // TODO: is this get call still needed (no side effects) or can we remove it?
-/*unused      const grt::SimpleValueChange *change= */
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
       generate_set_partitioning(table, diffchange);
-
-      partitions_processed= true;
-    }
-    else
-    {
-      continue; // avoid setting comma, see below
+      partitions_processed = true;
     }
   }
 
-  bool is_range= (strcmp(table->partitionType().c_str(), "RANGE") == 0);
-
-  // partitioning options that dont require PARTITION BY clause
-  for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); 
-    (it != e) && !partitions_processed; it++)
+  if (alter_table_flags & EverythingButForeignKeys)
   {
-    const grt::ObjectAttrModifiedChange *attr_change= 
-      static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
+    bool is_range= (strcmp(table->partitionType().c_str(), "RANGE") == 0);
 
-    if(attr_change->get_attr_name().compare("partitionCount") == 0)
+    // partitioning options that dont require PARTITION BY clause
+    for(grt::ChangeSet::const_iterator e= cs->end(), it= cs->begin(); 
+      (it != e) && !partitions_processed; it++)
     {
-      const grt::SimpleValueChange *change= 
-        static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
-      grt::IntegerRef old_count= grt::IntegerRef::cast_from(change->get_old_value());
-      
-      //std::string part_count_sql(generate_change_partition_count(table, new_count));
+      const grt::ObjectAttrModifiedChange *attr_change= 
+        static_cast<const grt::ObjectAttrModifiedChange *>(it->get());
 
-      //// partition count alone can be changed only for HASH/KEY partitions
-      //// generate_change_partition_count() will return empty string otherwise
-      //// for RANGE/LIST we ignore change of this attribute and rely solely on
-      //// partition definitions change
-      //if(!part_count_sql.empty())
-      //{
-      //  alter_sql.append(comma).append(part_count_sql);
-      //  partitions_processed= true;
-      //}
-
-      std::string part_type(table->partitionType().c_str());
-
-      if((part_type.find("HASH") != std::string::npos)
-          || (part_type.find("KEY") != std::string::npos))
+      if(attr_change->get_attr_name().compare("partitionCount") == 0)
       {
-        callback->alter_table_partition_count(table, old_count);
-        partitions_processed= true;
-      }
-    }
-    else if(attr_change->get_attr_name().compare("partitionDefinitions") == 0)
-    {
-      const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
-      const grt::ChangeSet *part_cs= list_change->subchanges();
-      for(grt::ChangeSet::const_iterator e2= part_cs->end(), jt= part_cs->begin(); jt != e2; jt++)
-      {
-        const grt::DiffChange *part_change= jt->get();
-        switch(part_change->get_change_type())
+        const grt::SimpleValueChange *change= 
+          static_cast<const grt::SimpleValueChange *>(attr_change->get_subchange().get());
+        grt::IntegerRef old_count= grt::IntegerRef::cast_from(change->get_old_value());
+        
+        //std::string part_count_sql(generate_change_partition_count(table, new_count));
+
+        //// partition count alone can be changed only for HASH/KEY partitions
+        //// generate_change_partition_count() will return empty string otherwise
+        //// for RANGE/LIST we ignore change of this attribute and rely solely on
+        //// partition definitions change
+        std::string part_type(table->partitionType().c_str());
+        if((part_type.find("HASH") != std::string::npos)
+            || (part_type.find("KEY") != std::string::npos))
         {
-        case grt::ListItemAdded:
-          {
-            db_mysql_PartitionDefinitionRef part(db_mysql_PartitionDefinitionRef::cast_from(
-              static_cast<const grt::ListItemAddedChange *>(part_change)->get_value()));
-            callback->alter_table_add_partition(part, is_range);
-          }
-          break;
-        case grt::ListItemRemoved:
-          {
-            db_mysql_PartitionDefinitionRef part= db_mysql_PartitionDefinitionRef::cast_from(
-              static_cast<const grt::ListItemRemovedChange *>(part_change)->get_value());
-
-            callback->alter_table_drop_partition(part->name().c_str());
-          }
-          break;
-        case grt::ListItemModified:
-          {
-            db_mysql_PartitionDefinitionRef old_part(db_mysql_PartitionDefinitionRef::cast_from(
-              static_cast<const grt::ListItemModifiedChange *>(part_change)->get_old_value()));
-            db_mysql_PartitionDefinitionRef new_part(db_mysql_PartitionDefinitionRef::cast_from(
-              static_cast<const grt::ListItemModifiedChange *>(part_change)->get_new_value()));
-
-            callback->alter_table_reorganize_partition(old_part, new_part, is_range);
-          }
-          break;
-          default: break;
+          callback->alter_table_partition_count(table, old_count);
+          partitions_processed= true;
         }
+      }
+      else if(attr_change->get_attr_name().compare("partitionDefinitions") == 0)
+      {
+        const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
+        const grt::ChangeSet *part_cs= list_change->subchanges();
+        for(grt::ChangeSet::const_iterator e2= part_cs->end(), jt= part_cs->begin(); jt != e2; jt++)
+        {
+          const grt::DiffChange *part_change= jt->get();
+          switch(part_change->get_change_type())
+          {
+          case grt::ListItemAdded:
+            {
+              db_mysql_PartitionDefinitionRef part(db_mysql_PartitionDefinitionRef::cast_from(
+                static_cast<const grt::ListItemAddedChange *>(part_change)->get_value()));
+              callback->alter_table_add_partition(part, is_range);
+            }
+            break;
+          case grt::ListItemRemoved:
+            {
+              db_mysql_PartitionDefinitionRef part= db_mysql_PartitionDefinitionRef::cast_from(
+                static_cast<const grt::ListItemRemovedChange *>(part_change)->get_value());
 
-        partitions_processed= true;
+              callback->alter_table_drop_partition(part->name().c_str());
+            }
+            break;
+          case grt::ListItemModified:
+            {
+              db_mysql_PartitionDefinitionRef old_part(db_mysql_PartitionDefinitionRef::cast_from(
+                static_cast<const grt::ListItemModifiedChange *>(part_change)->get_old_value()));
+              db_mysql_PartitionDefinitionRef new_part(db_mysql_PartitionDefinitionRef::cast_from(
+                static_cast<const grt::ListItemModifiedChange *>(part_change)->get_new_value()));
+
+              callback->alter_table_reorganize_partition(old_part, new_part, is_range);
+            }
+            break;
+            default: break;
+          }
+
+          partitions_processed= true;
+        }
       }
     }
   }
@@ -1326,7 +1391,8 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
           generate_alter_stmt_drops(
             db_mysql_TableRef::cast_from(static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()),
             static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get());
-        }else if (table_change->get_change_type() == grt::ListItemOrderChanged)
+        }
+        else if (table_change->get_change_type() == grt::ListItemOrderChanged)
         {
             const grt::ListItemOrderChange *oc= 
               static_cast<const grt::ListItemOrderChange *>(table_change);
@@ -1345,6 +1411,8 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
     {
       const grt::MultiChange *list_change= static_cast<const grt::MultiChange *>(attr_change->get_subchange().get());
       const grt::ChangeSet *tables_cs= list_change->subchanges();
+
+      // 1st pass, do everything except FKs
       for(grt::ChangeSet::const_iterator e2= tables_cs->end(), jt= tables_cs->begin(); jt != e2; jt++)
       {
         const grt::DiffChange *table_change= jt->get();
@@ -1360,21 +1428,53 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
           break;
         case grt::ListItemModified:
           generate_alter_stmt(db_mysql_TableRef::cast_from(
-            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()), 
-            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get());
+            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()),
+            static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get(),
+                              _separate_foreign_keys ? EverythingButForeignKeys : Everything); // everything but FK 1st
           break;
         case grt::ListItemOrderChanged:
           {
             const grt::ListItemOrderChange *oc= 
               static_cast<const grt::ListItemOrderChange *>(table_change);
             if(oc->get_subchange())
-                generate_alter_stmt(db_mysql_TableRef::cast_from(oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get());
+              generate_alter_stmt(db_mysql_TableRef::cast_from(oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get(), _separate_foreign_keys ? EverythingButForeignKeys : Everything);
           }
           break;
           default: 
             break;
         }
       }
+
+      if (_separate_foreign_keys)
+      {
+        // 2nd pass, do FKs only
+        for(grt::ChangeSet::const_iterator e2= tables_cs->end(), jt= tables_cs->begin(); jt != e2; jt++)
+        {
+          const grt::DiffChange *table_change= jt->get();
+          switch(table_change->get_change_type())
+          {
+            case grt::ListItemAdded:
+            case grt::ListItemRemoved:
+              break;
+            case grt::ListItemModified:
+              generate_alter_stmt(db_mysql_TableRef::cast_from(
+                                  static_cast<const grt::ListItemModifiedChange *>(table_change)->get_new_value()),
+                                  static_cast<const grt::ListItemModifiedChange *>(table_change)->get_subchange().get(),
+                                  OnlyForeignKeys); // FK only
+              break;
+            case grt::ListItemOrderChanged:
+            {
+              const grt::ListItemOrderChange *oc=
+              static_cast<const grt::ListItemOrderChange *>(table_change);
+              if(oc->get_subchange())
+                generate_alter_stmt(db_mysql_TableRef::cast_from(oc->get_subchange()->get_new_value()), oc->get_subchange()->get_subchange().get(), OnlyForeignKeys);
+            }
+              break;
+            default: 
+              break;
+          }
+        }
+      }
     }
     else if(attr_change->get_attr_name().compare("views") == 0)
     {
@@ -1406,7 +1506,19 @@ void DiffSQLGeneratorBE::generate_alter_stmt(db_mysql_SchemaRef schema, const gr
             generate_alter_stmt(old_view, new_view, view_change);
           }
           break;
-        // list item position change is not relevant        
+          case grt::ListItemOrderChanged:
+            // list item position change is not relevant but may hold diff inside
+          {
+            if (!static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange())
+              break;
+            db_mysql_ViewRef old_view= db_mysql_ViewRef::cast_from(
+                             grt::ValueRef(static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange()->get_old_value()));
+            db_mysql_ViewRef new_view= db_mysql_ViewRef::cast_from(
+                             grt::ValueRef(static_cast<const grt::ListItemOrderChange *>(view_change)->get_subchange()->get_new_value()));
+
+            generate_alter_stmt(old_view, new_view, view_change);
+            break;
+          }
           default: break;
         }
       }
@@ -1525,102 +1637,29 @@ static void fill_set_from_list(grt::StringListRef string_list, std::set<std::str
     string_set.insert(std::string(string_list.get(i).c_str()));
 }
 
-DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, DiffSQLGeneratorBEActionInterface *cb)
-  : callback(cb), _gen_create_index(false), _use_filtered_lists(true), 
-   _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false)
+DiffSQLGeneratorBE::DiffSQLGeneratorBE(grt::DictRef options, grt::DictRef dbtraits, DiffSQLGeneratorBEActionInterface *cb)
+    : callback(cb), _gen_create_index(false), _use_filtered_lists(true),
+    _skip_foreign_keys(false), _skip_fk_indexes(false), _case_sensitive(false), _use_oid_as_dict_key(false), _separate_foreign_keys(true)
 {
-  if (options.is_valid())
-  {
-    if (options.has_key("UseOIDAsResultDictKey"))
-      _use_oid_as_dict_key = options.get_int("UseOIDAsResultDictKey", 0) != 0;
+    if (!options.is_valid())
+        return;
+    _case_sensitive = (dbtraits.get_int("CaseSensitive", _case_sensitive) != 0);
 
-    if(options.has_key("SkipForeignKeys"))
-    {
-      grt::IntegerRef SkipForeignKeys= grt::IntegerRef::cast_from(options.get("SkipForeignKeys"));
-      if(SkipForeignKeys.is_valid())
-        _skip_foreign_keys = (SkipForeignKeys.operator long() != 0);
-    }if(options.has_key("SkipFKIndexes"))
-    {
-      grt::IntegerRef skip_fk_indexes= grt::IntegerRef::cast_from(options.get("SkipFKIndexes"));
-      if(skip_fk_indexes.is_valid())
-        _skip_fk_indexes = (skip_fk_indexes.operator long() != 0);
-    }
-    if(options.has_key("UseFilteredLists"))
-    {
-      grt::IntegerRef use_filtered_lists= grt::IntegerRef::cast_from(options.get("UseFilteredLists"));
-      if(use_filtered_lists.is_valid())
-        _use_filtered_lists= (use_filtered_lists.operator long() != 0);
-    }
-    cb->set_short_names(false);
-    if(options.has_key("UseShortNames"))
-    {
-      grt::IntegerRef use_short_names = grt::IntegerRef::cast_from(options.get("UseShortNames"));
-      if(use_short_names.is_valid())
-        cb->set_short_names(use_short_names.operator long() != 0);
-    }
-    if(options.has_key("GenerateUse"))
-    {
-      grt::IntegerRef gen_use = grt::IntegerRef::cast_from(options.get("GenerateUse"));
-      if(gen_use.is_valid())
-        cb->set_gen_use(gen_use.operator long() != 0);
-    }
-    if(options.has_key("GenerateCreateIndex"))
-    {
-      grt::IntegerRef gen_index_value= grt::IntegerRef::cast_from(options.get("GenerateCreateIndex"));
-      if(gen_index_value.is_valid())
-        _gen_create_index= (gen_index_value.operator long() != 0);
-    }
-    if(options.has_key("UserFilterList"))
-    {
-      grt::StringListRef user_list= grt::StringListRef::cast_from(options.get("UserFilterList"));
-      if(user_list.is_valid())
-        fill_set_from_list(user_list, _filtered_users);
-    }
-
-    if(options.has_key("SchemaFilterList"))
-    {
-      grt::StringListRef schemata_list= grt::StringListRef::cast_from(options.get("SchemaFilterList"));
-      if(schemata_list.is_valid())
-        fill_set_from_list(schemata_list, _filtered_schemata);
-    }
-
-    if(options.has_key("TableFilterList"))
-    {
-      grt::StringListRef tables_list= grt::StringListRef::cast_from(options.get("TableFilterList"));
-      if(tables_list.is_valid())
-        fill_set_from_list(tables_list, _filtered_tables);
-    }
-
-    if(options.has_key("ViewFilterList"))
-    {
-      grt::StringListRef views_list= grt::StringListRef::cast_from(options.get("ViewFilterList"));
-      if(views_list.is_valid())
-        fill_set_from_list(views_list, _filtered_views);
-    }
-
-    if(options.has_key("RoutineFilterList"))
-    {
-      grt::StringListRef routines_list= grt::StringListRef::cast_from(options.get("RoutineFilterList"));
-      if(routines_list.is_valid())
-        fill_set_from_list(routines_list, _filtered_routines);
-    }
-
-    if(options.has_key("TriggerFilterList"))
-    {
-      grt::StringListRef triggers_list= grt::StringListRef::cast_from(options.get("TriggerFilterList"));
-      if(triggers_list.is_valid())
-        fill_set_from_list(triggers_list, _filtered_triggers);
-    }
-    
-    grt::DictRef db_settings = grt::DictRef::cast_from(options.get("DBSettings"));
-    if(db_settings.is_valid() && db_settings.has_key("CaseSensitive"))
-    {
-      grt::IntegerRef case_sensitive = grt::IntegerRef::cast_from(db_settings.get("CaseSensitive"));
-      if(case_sensitive.is_valid())
-        _case_sensitive = (case_sensitive.operator long() != 0);
-    }
-
-  }
+    grt::StringListRef empty_list(options.get_grt());
+    _use_oid_as_dict_key = options.get_int("UseOIDAsResultDictKey", _use_oid_as_dict_key) != 0;
+    _skip_foreign_keys = options.get_int("SkipForeignKeys", _skip_foreign_keys) != 0;
+    _skip_fk_indexes = options.get_int("SkipFKIndexes", _skip_fk_indexes) != 0;
+    _gen_create_index = (options.get_int("GenerateCreateIndex", _gen_create_index) != 0);
+    _use_filtered_lists = options.get_int("UseFilteredLists", _use_filtered_lists)!= 0;
+    _separate_foreign_keys = options.get_int("SeparateForeignKeys", _separate_foreign_keys)!= 0;
+    cb->set_short_names(options.get_int("UseShortNames", 0) != 0);
+    cb->set_gen_use(options.get_int("GenerateUse", 0) != 0);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("UserFilterList", empty_list)), _filtered_users);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("SchemaFilterList", empty_list)), _filtered_schemata);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("TableFilterList", empty_list)), _filtered_tables);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("ViewFilterList", empty_list)), _filtered_views);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("RoutineFilterList", empty_list)), _filtered_routines);
+    fill_set_from_list(grt::StringListRef::cast_from(options.get("TriggerFilterList", empty_list)), _filtered_triggers);
 }
 
 void DiffSQLGeneratorBE::process_diff_change(grt::ValueRef org_object, grt::DiffChange *diff, grt::DictRef map)
