@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "wb_config.h"
-
 #include <grtpp.h>
 #include "wb_model.h"
 #include "grts/structs.workbench.physical.h"
@@ -27,9 +25,6 @@
 #include <grtpp_undo_manager.h>
 #include "base/string_utilities.h"
 #include "base/wb_iterators.h"
-#include <math.h>
-#include <algorithm>
-#include <stdlib.h>
 
 #ifndef M_PI
 # define M_PI   3.14159265358979323846
@@ -319,9 +314,9 @@ class Layouter
 
     bool shuffle();
     double calc_energy();
-    double calc_node_energy(const int i, const Node& n);
-    long distance_to_node(const int n1, const int n2, bool *is_horiz = 0);
-    double calc_node_pair(const int i1, const int i2);
+    double calc_node_energy(const size_t i, const Node& n);
+    long distance_to_node(const size_t n1, const size_t n2, bool *is_horiz = NULL);
+    double calc_node_pair(const size_t i1, const size_t i2);
     void prepare_layout_stages();
 
     const double _w;
@@ -332,7 +327,7 @@ class Layouter
       Node(const model_FigureRef& figure);
       void move_by(const long dx, const long dy);
       void move(const long x, const long y);
-      bool is_linked_to(const int node) const;
+      bool is_linked_to(const ssize_t node) const;
 
       long                  w;
       long                  h;
@@ -341,7 +336,7 @@ class Layouter
       long                  x2;
       long                  y2;
       model_FigureRef       fig;
-      std::vector<int>      linked;
+      std::vector<ssize_t>   linked;
     };
     typedef std::vector<Node> NodesList;
     NodesList       _all_figures;
@@ -383,11 +378,11 @@ void Layouter::Node::move_by(const long dx, const long dy)
 }
 
 //------------------------------------------------------------------------------
-bool Layouter::Node::is_linked_to(const int node) const
+bool Layouter::Node::is_linked_to(const ssize_t node) const
 {
   bool found = false;
 
-  for (int i = linked.size() - 1; i >= 0; --i)
+  for (ssize_t i = linked.size() - 1; i >= 0; --i)
   {
     if (linked[i] == node)
     {
@@ -396,7 +391,7 @@ bool Layouter::Node::is_linked_to(const int node) const
     }
   }
 
-  return false;
+  return found;
 }
 
 //------------------------------------------------------------------------------
@@ -409,9 +404,8 @@ Layouter::Layouter(const model_LayerRef &layer)
          , _layer(layer)
 {
   const ListRef<model_Figure> figures = layer->figures();
-  const int size = figures->count();
 
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < figures->count(); ++i)
     _all_figures.push_back(figures[i]);
 }
 
@@ -419,8 +413,7 @@ Layouter::Layouter(const model_LayerRef &layer)
 //------------------------------------------------------------------------------
 void Layouter::add_figure_to_layout(const model_FigureRef& figure)
 {
-  const int size = _all_figures.size();
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _all_figures.size(); ++i)
   {
     if (_all_figures[i].fig == figure)
     {
@@ -432,11 +425,10 @@ void Layouter::add_figure_to_layout(const model_FigureRef& figure)
 //------------------------------------------------------------------------------
 void Layouter::connect(const model_FigureRef& f1, const model_FigureRef& f2)
 {
-  int n1 = -1;
-  int n2 = -1;
+  ssize_t n1 = -1;
+  ssize_t n2 = -1;
 
-  const int size = _figures.size();
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _figures.size(); ++i)
   {
     Node& n = _figures[i];
     if (n1 == -1 && n.fig == f1)
@@ -457,7 +449,7 @@ void Layouter::connect(const model_FigureRef& f1, const model_FigureRef& f2)
 }
 
 //------------------------------------------------------------------------------
-long Layouter::distance_to_node(const int i1, const int i2, bool* is_horiz)
+long Layouter::distance_to_node(const size_t i1, const size_t i2, bool* is_horiz)
 {
   const Node& n1 = _figures[i1];
   const Node& n2 = _figures[i2];
@@ -535,7 +527,7 @@ inline double line_len2(long x1, long y1, long x2, long y2)
 }
 
 //------------------------------------------------------------------------------
-double Layouter::calc_node_pair(const int i1, const int i2)
+double Layouter::calc_node_pair(const size_t i1, const size_t i2)
 {
   const Node* n1 = &(_figures[i1]);
   const Node* n2 = &(_figures[i2]);
@@ -624,16 +616,16 @@ double Layouter::calc_node_pair(const int i1, const int i2)
 //------------------------------------------------------------------------------
 double Layouter::calc_energy()
 {
-  const int size = _figures.size();
   double e = 0.0;
 
-  for (int i = 0; i < size; ++i)
+  size_t size = _figures.size();
+  for (size_t i = 0; i < size; ++i)
   {
     const Node& node = _figures[i];
     if ((node.x1 < 0) || (node.y1 < 0) || (node.x2 + 20 > _w) || (node.y2 + 20 > _h))
       e += 1000000000000.0;
 
-    for (int j = i + 1; j < size; ++j)
+    for (size_t j = i + 1; j < size; ++j)
     {
       if (j >= size)
         break;
@@ -645,15 +637,14 @@ double Layouter::calc_energy()
 }
 
 //------------------------------------------------------------------------------
-double Layouter::calc_node_energy(const int node_i, const Node& node)
+double Layouter::calc_node_energy(const size_t node_i, const Node& node)
 {
-  const int size = _figures.size();
   double e = 0.0;
 
   if ((node.x1 < 0) || (node.y1 < 0) || (node.x2 + 20 > _w) || (node.y2 + 20 > _h))
     e += 1000000000000.0;
 
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _figures.size(); ++i)
   {
     if (node_i != i)
       e += calc_node_pair(node_i, i);
@@ -668,8 +659,7 @@ bool Layouter::shuffle()
   bool found_smaller_energy = false;
   const int step = (rand() % 5) + 1;
 
-  const int size = _figures.size();
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _figures.size(); ++i)
   {
     Node& n = _figures[i];
     const int wstep = _cell_w * step;
@@ -710,10 +700,9 @@ void Layouter::prepare_layout_stages()
   double total_w = 0;
   double total_h = 0;
   std::sort(_figures.begin(), _figures.end(), compare_node_links);
-  const int size = _figures.size();
 
   // reset layout
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _figures.size(); ++i)
   {
     Node& n = _figures[i];
     // place all tables in some initial position
@@ -759,8 +748,7 @@ int Layouter::do_layout()
   }
 
   // update actual figures with new coords
-  const int size = _figures.size();
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < _figures.size(); ++i)
   {
     Node& n = _figures[i];
     model_FigureRef& f = n.fig;
@@ -777,8 +765,7 @@ int WbModelImpl::do_autolayout(const model_LayerRef &layer, ListRef<model_Object
   Layouter  layout(layer);
   if (selection.count() > 0)
   {
-    const int size = selection->count();
-    for (int i = 0; i < size; ++i)
+    for (size_t i = 0; i < selection->count(); ++i)
     {
       const model_ObjectRef figure = selection[i];
       if (workbench_physical_TableFigureRef::can_wrap(figure) || workbench_physical_ViewFigureRef::can_wrap(figure))
@@ -788,8 +775,7 @@ int WbModelImpl::do_autolayout(const model_LayerRef &layer, ListRef<model_Object
   else
   {
     const ListRef<model_Figure> figures = layer->figures();
-    const int size = figures->count();
-    for (int i = 0; i < size; ++i)
+    for (size_t i = 0; i < figures->count(); ++i)
     {
       const model_ObjectRef figure = figures[i];
       if (workbench_physical_TableFigureRef::can_wrap(figure) || workbench_physical_ViewFigureRef::can_wrap(figure))
@@ -798,8 +784,7 @@ int WbModelImpl::do_autolayout(const model_LayerRef &layer, ListRef<model_Object
   }
 
   ListRef<model_Connection> connections = layer->owner()->connections();
-  const int size = connections->count();
-  for (int i = 0; i < size; ++i)
+  for (size_t i = 0; i < connections->count(); ++i)
   {
     const model_ConnectionRef conn = connections[i];
     layout.connect(conn->startFigure(), conn->endFigure());
@@ -1032,12 +1017,12 @@ static workbench_physical_DiagramRef create_view_for_object_count(workbench_phys
 {
   int xpages= 2, ypages= 1;
   int pages;
-  // guesstimate about 20 objects per page
-  pages= object_count / 20;
+  // guesstimate about 15 objects per page
+  pages= (int)ceil(object_count / 15.0);
   ypages= (int)sqrt((float)pages);
   if (ypages < 1) 
     ypages= 1;
-  xpages= pages / ypages;
+  xpages= (int)ceil((float)pages / ypages);
   if (xpages < 1) 
     xpages= 1;
 
@@ -1052,12 +1037,12 @@ static workbench_physical_DiagramRef create_view_for_object_count(workbench_phys
 
 int WbModelImpl::createDiagramWithObjects(workbench_physical_ModelRef model, grt::ListRef<GrtObject> objects)
 {
-  int object_count= objects.count();
+  size_t object_count= objects.count();
 
   if (object_count > 0)
   {
     begin_undo_group();
-    workbench_physical_DiagramRef view= create_view_for_object_count(model, object_count);
+    workbench_physical_DiagramRef view = create_view_for_object_count(model, (int)object_count);
 
     do_autoplace_any_list(view, objects);
     ListRef<db_Table> tables(get_grt());
@@ -1072,9 +1057,9 @@ int WbModelImpl::createDiagramWithObjects(workbench_physical_ModelRef model, grt
     }
     autoplace_relations(view, tables);
 
-    autolayout(view);
-
     end_undo_group(_("Create Diagram with Objects"));
+
+    bec::GRTManager::get_instance_for(model.get_grt())->run_once_when_idle(boost::bind(&WbModelImpl::autolayout, this, view));
   }
 
   return 0;
@@ -1083,14 +1068,14 @@ int WbModelImpl::createDiagramWithObjects(workbench_physical_ModelRef model, grt
 
 int WbModelImpl::createDiagramWithCatalog(workbench_physical_ModelRef model, db_CatalogRef catalog)
 {
-  int object_count= 0;
-  ListRef<db_Schema> schemata= catalog->schemata();
-  for (size_t n= 0, count= schemata.count(); n < count; ++n)
+  size_t object_count = 0;
+  ListRef<db_Schema> schemata = catalog->schemata();
+  for (size_t n = 0, count = schemata.count(); n < count; ++n)
   {
-    db_SchemaRef schema= schemata[n];
-    object_count+= schema->tables().count();
-    object_count+= schema->views().count()/4; // views and routine groups
-    object_count+= schema->routineGroups().count()/3; // take less space
+    db_SchemaRef schema = schemata[n];
+    object_count += schema->tables().count();
+    object_count += schema->views().count()/3; // views and routine groups
+    object_count += schema->routineGroups().count()/2; // take less space
   }
   if (object_count > 250)
     throw logic_error("Cannot create diagram: too many objects to place.\nTry dividing your model into several sub-diagrams with less than 200 objects each.");
@@ -1098,7 +1083,7 @@ int WbModelImpl::createDiagramWithCatalog(workbench_physical_ModelRef model, db_
   DictRef wb_options= DictRef::cast_from(get_grt()->get("/wb/options/options"));
   
   begin_undo_group();
-  workbench_physical_DiagramRef diagram= create_view_for_object_count(model, object_count);
+  workbench_physical_DiagramRef diagram = create_view_for_object_count(model, (int)object_count);
 
   for (size_t n= 0, count= schemata.count(); n < count; ++n)
   {
@@ -1310,18 +1295,17 @@ int WbModelImpl::center(model_DiagramRef view)
   double view_height= rootLayer->height();
   double xmin= view_width, ymin= view_height, xmax= 0, ymax= 0;
 
-  /*KKK
   // find out model bounds
-  for (size_t c= rootLayer.subLayers().count(), i= 0; i < c; i++)
+  for (size_t c= rootLayer->subLayers().count(), i= 0; i < c; i++)
   {
-    model_LayerRef layer(rootLayer.subLayers().get(i));
+    model_LayerRef layer(rootLayer->subLayers().get(i));
 
-    xmin= std::min(xmin, *layer.left());
-    ymin= std::min(ymin, *layer.top());
+    xmin= min(xmin, *layer->left());
+    ymin= min(ymin, *layer->top());
 
-    xmax= std::max(xmax, *layer.left() + *layer.width());
-    ymax= std::max(ymax, *layer.top() + *layer.height());
-  }*/
+    xmax= max(xmax, *layer->left() + *layer->width());
+    ymax= max(ymax, *layer->top() + *layer->height());
+  }
 
   for (size_t c= rootLayer->figures().count(), i= 0; i < c; i++)
   {
@@ -1345,23 +1329,19 @@ int WbModelImpl::center(model_DiagramRef view)
     double xoffs, yoffs;
 
     // determine offset to move everything so it gets centered
-    xoffs= (view_width - model_bounds.width)/2 - model_bounds.left - model_bounds.width/2;
-    yoffs= (view_height - model_bounds.height)/2 - model_bounds.top - model_bounds.height/2;
+    xoffs= (view_width - model_bounds.width)/2 - model_bounds.left;
+    yoffs= (view_height - model_bounds.height)/2 - model_bounds.top;
 
     begin_undo_group();
 
-    /*
     // center layers that are on rootLayer
-    for (size_t c= rootLayer.subLayers().count(), i= 0; i < c; i++)
+    for (size_t c= rootLayer->subLayers().count(), i= 0; i < c; i++)
     {
-      model_LayerRef layer(rootLayer.subLayers().get(i));
-
-      do_transactable_reposition(_undo_man,
-                                layer,
-                                *layer.left() + xoffs,
-                                *layer.top() + yoffs);
-    }KKK*/
+      model_LayerRef layer(rootLayer->subLayers().get(i));
 
+      layer->left(*layer->left() + xoffs);
+      layer->top(*layer->top() + yoffs);
+    }
 
     for (size_t c= rootLayer->figures().count(), i= 0; i < c; i++)
     {
@@ -1385,7 +1365,7 @@ int WbModelImpl::fitObjectsToContents(const grt::ListRef<model_Object> &selectio
     if (selection[i].is_instance<model_Figure>())
     {
       model_FigureRef figure(model_FigureRef::cast_from(selection[i]));
-      int o= *figure->manualSizing();
+      ssize_t o = *figure->manualSizing();
       if (o != 0)
       {
         figure->manualSizing(0);
