@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2004, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #ifndef _WIN32
 #include <cctype>
 #include <algorithm>
@@ -33,6 +31,7 @@
 #include "grt/grt_manager.h"
 #include <grtpp_util.h>
 #include "tinyxml.h"
+#include "base/log.h"
 #include "grtdb/db_object_helpers.h"
 #include "stream_conv.h"
 #include "grtsqlparser/module_utils.h"
@@ -40,6 +39,8 @@
 #include "grts/structs.workbench.h"
 #include "base/string_utilities.h"
 
+DEFAULT_LOG_DOMAIN("dbd4import");
+
 using namespace base;
 
 const float XSCALE_FACTOR= 1.0F;
@@ -74,7 +75,7 @@ static std::string dbd_string_to_utf8(const char *str)
     return "";
 
   std::string res;
-  gchar *tmp= (gchar*)g_malloc(strlen(str)*4+1);
+  gchar *tmp= (gchar*)g_malloc((int)strlen(str)*4+1);
   const char *ptr;
   gchar *tmpstr;
 
@@ -132,7 +133,7 @@ static std::string dbd_string_to_utf8(const char *str)
   }
   *tmpstr= 0;
 
-  tmpstr= g_convert(tmp, strlen(tmp), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+  tmpstr= g_convert(tmp, (gssize)strlen(tmp), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
   g_free(tmp);
   if (!tmpstr)
     return str;
@@ -319,7 +320,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
   _grt= model.get_grt();
   _catalog= db_mysql_CatalogRef::cast_from(model->catalog());
 
-  _grt->send_info("Started import DBD4 model.");
+  log_info("Started import DBD4 model.\n");
 
   bec::GRTManager *grtm= bec::GRTManager::get_instance_for(_grt);
 
@@ -344,7 +345,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       double viewWidth;
       double viewHeight;
 
-      view= workbench_physical_DiagramRef::cast_from(model->addNewDiagram(0));
+      view= workbench_physical_DiagramRef::cast_from(model->addNewDiagram(1));
 
       calculate_view_size(wb->doc()->pageSettings(), viewWidth, viewHeight);
 
@@ -452,13 +453,13 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
           if (table_prefix_el)
             table_prefix_el= table_prefix_el->NextSiblingElement();
 
-          _grt->send_info("Schemata:");
+          log_info("Schemata:\n");
           while (table_prefix_el)
           {
             std::string schema_name= dbd_string_to_utf8(table_prefix_el->Attribute("Name"));
-            _grt->send_info(std::string("...").append(schema_name));
+            log_info("...%s\n", schema_name.c_str());
 
-            ensure_schema_created(_schemata.size(), schema_name.c_str());
+            ensure_schema_created((int)_schemata.size(), schema_name.c_str());
 
             table_prefix_el= table_prefix_el->NextSiblingElement();
           }
@@ -496,13 +497,13 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       {
         if (regions_el)
         {
-          _grt->send_info("Layers:");
+          log_info("Layers:\n");
 
           const TiXmlElement *region_el= regions_el->FirstChildElement("REGION");
           while (region_el)
           {
             std::string layer_name= dbd_string_to_utf8(region_el->Attribute("RegionName"));
-            _grt->send_info(std::string("...").append(layer_name));
+            log_info("...%s\n", layer_name.c_str());
 
             model_LayerRef layer= workbench_physical_LayerRef(_grt);
             layer->owner(view);
@@ -535,7 +536,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       {
         if (tables_el)
         {
-          _grt->send_info("Tables:");
+          log_info("Tables:\n");
 
           TableInsertsLoader table_inserts_loader(grtm);
 
@@ -543,7 +544,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
           while (table_el)
           {
             std::string table_name= dbd_string_to_utf8(table_el->Attribute("Tablename"));
-            _grt->send_info(std::string("...").append(table_name));
+            log_info("...%s\n", table_name.c_str());
 
             // table
             table_el->QueryIntAttribute("TablePrefix", &id);
@@ -743,13 +744,13 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       {
         if (relations_el)
         {
-          _grt->send_info("Connections:");
+          log_info("Connections:\n");
 
           const TiXmlElement *relation_el= relations_el->FirstChildElement("RELATION");
           while (relation_el)
           {
             std::string relation_name= dbd_string_to_utf8(relation_el->Attribute("RelationName"));
-            _grt->send_info(std::string("...").append(relation_name));
+            log_info("...%s\n", relation_name.c_str());
 
             relation_el->QueryIntAttribute("DestTable", &id);
             db_mysql_TableRef srcTable= _tables[id];
@@ -923,13 +924,13 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       {
         if (notes_el)
         {
-          _grt->send_info("Notes:");
+          log_info("Notes:\n");
 
           const TiXmlElement *note_el= notes_el->FirstChildElement("NOTE");
           while (note_el)
           {
             std::string note_name= dbd_string_to_utf8(note_el->Attribute("NoteName"));
-            _grt->send_info(std::string("...").append(note_name));
+            log_info("...%s\n", note_name.c_str());
 
             workbench_model_NoteFigureRef note_figure(_grt);
             note_figure->owner(view);
@@ -955,7 +956,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
       {
         if (images_el)
         {
-          _grt->send_info("Images:");
+          log_info("Images:\n");
 
           const std::string tmp_dir= grtm->get_unique_tmp_subdir();
           // crete temp dir
@@ -972,7 +973,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
             if ((img_format.compare("PNG") == 0) && (id == 0))
             {
               std::string image_name= dbd_string_to_utf8(image_el->Attribute("ImageName"));
-              _grt->send_info(std::string("...").append(image_name));
+              log_info("...%s\n", image_name.c_str());
 
               std::string filename;
               filename
@@ -1021,7 +1022,7 @@ int Wb_mysql_import_DBD4::import_DBD4(workbench_physical_ModelRef model, const c
 
   remove_unused_schemata();
 
-  _grt->send_info("Finished import DBD4 model.");
+  log_info("Finished import DBD4 model.\n");
 
   return 1; // success
 }
