@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,12 +17,8 @@
  * 02110-1301  USA
  */
 
-#ifndef _STDAFX_H_
-#define _STDAFX_H_
+#pragma once
 
-#include <boost/signals2.hpp>
-
-#ifdef _WIN32
 #define NOMINMAX
 #define WIN32_LEAN_AND_MEAN 
 #include <windows.h>
@@ -34,6 +30,10 @@
 #include <map>
 #include <iostream>
 
-#endif // _WIN32
+#include <boost/signals2.hpp>
 
-#endif // _STDAFX_H_
+#include "grts/structs.model.h"
+#include "grts/structs.db.query.h"
+#include "grts/structs.workbench.physical.h"
+#include "grts/structs.workbench.logical.h"
+#include "grts/structs.db.migration.h"
