@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014 Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -20,9 +20,8 @@ import sys
 
 import grt
 import mforms
-from workbench import db_utils
 
-from workbench.utils import replace_string_parameters
+from workbench.log import log_debug
 
 class NotSupportedError(Exception):
     pass
@@ -191,9 +190,9 @@ class MigrationSource(object):
             raise NotSupportedError('MySQL connections through SSH are not supported in this version '
                                     'of the MySQL Workbench Migration Wizard.')
         self._rev_eng_module.connect(self.connection, self.password or "")
-        if str(self.getDriverDBMSName()).upper() == 'ACCESS':
-            raise NotSupportedError('Microsoft Access is not supported in this version of the MySQL Workbench '
-                                    'Migration Wizard.')
+        #if str(self.getDriverDBMSName()).upper() == 'ACCESS':
+        #    raise NotSupportedError('Microsoft Access is not supported in this version of the MySQL Workbench '
+        #                            'Migration Wizard.')
         return True
 
     def disconnect(self):
@@ -236,7 +235,7 @@ class MigrationSource(object):
         
         grt.send_progress(0.0, "Checking connection...")
         self.connect()
-        if self.rdbms.doesSupportCatalogs:
+        if self.rdbms.doesSupportCatalogs > 0:
             grt.send_progress(0.1, "Fetching catalog names...")
             self.state.sourceSchemataNames.remove_all()
             catalog_names = self.getCatalogNames()
@@ -247,27 +246,31 @@ class MigrationSource(object):
                 catalog_names = list(set(only_these_catalogs).difference(inexistent_catalogs)) or self.getCatalogNames()
             self._catalog_names = catalog_names
             grt.send_progress(0.1, "Fetching schema names...")
-            i = 0.0
             accumulated_progress = 0.1
             step_progress_share = 1.0 / (len(catalog_names) + 1e-10)
             for catalog in catalog_names:
                 grt.send_progress(accumulated_progress, 'Fetching schema names from %s...' % catalog)
                 schema_names = self.getSchemaNames(catalog)
                 for schema in schema_names:
-                    self.state.sourceSchemataNames.append("%s.%s" % (catalog, schema))
+                    self.state.sourceSchemataNames.append("%s.%s" % (self._db_module.quoteIdentifier(catalog), self._db_module.quoteIdentifier(schema)))
                 accumulated_progress += 0.9 * step_progress_share
-        else:  # The rdbms doesn't support catalogs
+        elif self.rdbms.doesSupportCatalogs == 0:  # The rdbms doesn't support catalogs
             grt.send_progress(0.1, "Fetching schema names...")
             schema_names = self.getSchemaNames('')
             if only_these_catalogs:  # Here only_these_catalogs would rather mean only these schemata
                 inexistent_schemata = set(only_these_catalogs).difference(schema_names)
                 if inexistent_schemata:
-                    grt.send_warning('The following schemata where not found: ' + ', '.join(list(inexistent_schemata)))
+                    grt.send_warning('The following schemas where not found: ' + ', '.join(list(inexistent_schemata)))
                 schema_names = list(set(only_these_catalogs).difference(inexistent_schemata))  or self.getSchemaNames('')
             self._catalog_names = []
             self.state.sourceSchemataNames.remove_all()
             for schema in schema_names:
-                self.state.sourceSchemataNames.append('def.%s' % schema)
+                self.state.sourceSchemataNames.append('%s.%s' % (self._db_module.quoteIdentifier('def'), self._db_module.quoteIdentifier(schema)))
+        else: # no schema either
+            self._catalog_names = []
+            self.state.sourceSchemataNames.remove_all()
+            for schema in self.getSchemaNames(''):
+                self.state.sourceSchemataNames.append(self._db_module.quoteIdentifier(schema))
         grt.send_progress(1.0, "Finished")
 
 
@@ -352,6 +355,14 @@ class MigrationSource(object):
         grt.send_info("Reverse engineering %s from %s" % (", ".join(self.selectedSchemataNames), self.selectedCatalogName))
         self.state.sourceCatalog = self._rev_eng_module.reverseEngineer(self.connection, self.selectedCatalogName, self.selectedSchemataNames, self.state.applicationData)
 
+    def resetProgressFlags(self):
+        if hasattr(self._rev_eng_module, 'resetProgressFlags'):
+          self._rev_eng_module.resetProgressFlags(self.connection)
+
+    def cleanup(self):
+        if hasattr(self._rev_eng_module, "cleanup"):
+            self._rev_eng_module.cleanup()
+
 
 
 class MigrationPlan(object):
@@ -391,6 +402,8 @@ class MigrationPlan(object):
             self.wbcopytables_path = self.wbcopytables_path.encode("UTF-8")
 
     def close(self):
+        if self.migrationSource:
+            self.migrationSource.cleanup()
         self.state.owner = None
         grt.root.wb.migration = None
         self.state = None
@@ -486,4 +499,9 @@ class MigrationPlan(object):
         self.state.creationLog.remove_all()
         return grt.modules.DbMySQLFE.createCatalogObjects(self.migrationTarget.connection, self.migrationTarget.catalog, self.state.objectCreationParams, self.state.creationLog)
 
+    def migrationUpdate(self):
+        if hasattr(self.migrationSource.migration, 'migrateUpdateForChanges'):
+            self.migrationTarget.catalog = self.migrationSource.migration.migrateUpdateForChanges(self.state, self.migrationTarget.catalog)
+        else:
+            log_debug('migrateUpdateForChanges method call was skipped for module %s\n' % self.migrationSource.migration.__name__)
 
