@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -34,11 +34,11 @@ import stat
 import time
 import traceback
 
-from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users, sanitize_sudo_output
+from wb_common import PermissionDeniedError, InvalidPasswordError, OperationCancelledError, Users
 from workbench.utils import server_version_str2tuple
+from wb_common import CmdOptions, CmdOutput
 
-from grt import log_info, log_error, log_warning, log_debug, log_debug2, log_debug3
-_this_file = os.path.basename(__file__)
+from workbench.log import log_info, log_warning, log_error, log_debug, log_debug2, log_debug3
 
 try:
     import paramiko
@@ -52,7 +52,7 @@ except:
 
 def normalize_windows_path_for_ftp(path):
     idx = -1
-    if sys.platform != 'win32':
+    if platform.system().lower() != 'windows':
         idx = path.find(':')
 
     # Gets rid of the unit from the path
@@ -67,9 +67,9 @@ def normalize_windows_path_for_ftp(path):
 
 if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
     from paramiko.message import Message
-    from paramiko.common import *
+    from paramiko.common import MSG_CHANNEL_OPEN
     from paramiko.channel import Channel
-    from paramiko.ssh_exception import SSHException, BadAuthenticationType, ChannelException
+    from paramiko.ssh_exception import SSHException 
     import threading
     OPEN_CHANNEL_TIMEOUT = 15
 
@@ -133,7 +133,7 @@ if paramiko and server_version_str2tuple(paramiko.__version__) >= (1, 7, 4):
     paramiko.Transport.open_channel = wba_open_channel
 else:
     print "Warning! Can't use connect with timeout in paramiko", paramiko and paramiko.__version__
-    log_warning(_this_file, 'Cannot use connect with timeout in paramiko version %s\n' % paramiko.__version__)
+    log_warning('Cannot use connect with timeout in paramiko version %s\n' % paramiko.__version__)
 
 #===============================================================================
 class ConnectionError(Exception):
@@ -212,7 +212,7 @@ class WbAdminSFTP(object):
             try:
                 self.sftp.chdir(path)
                 ret = True
-            except IOError, e:
+            except IOError:
                 ret = False
 
         return ret
@@ -231,7 +231,7 @@ class WbAdminSSH(object):
         assert hasattr(password_delegate, "get_password_for")
         assert hasattr(password_delegate, "reset_password_for")
 
-        log_debug2(_this_file, "%s: starting connect\n" % self.__class__.__name__)
+        log_debug2("%s: starting connect\n" % self.__class__.__name__)
         self.client = None
 
         #loginInfo = settings.loginInfo
@@ -242,7 +242,7 @@ class WbAdminSSH(object):
         pwd = None # It is ok to keep pwd set to None even if we have it in server settings
                    # it will be retrived later
         port = settings.ssh_port#loginInfo['ssh.port']
-
+        self.keepalive = settings.ssh_keepalive
         if usekey == 1:
             # We need to check if keyfile needs password. For some reason paramiko does not always
             # throw exception to request password
@@ -252,7 +252,7 @@ class WbAdminSSH(object):
             f = None
             try:
                 f = open(key_filename, 'r')
-            except IOError, e:
+            except IOError:
                 f = None # set file handle to None indicating that open failed
 
             keycont = None # Will hold contents of the keyfile
@@ -263,8 +263,6 @@ class WbAdminSSH(object):
                 usekey = 0 # Reset usekey to 0 so paramiko will not use non-existent key file
                 key_filename = None
 
-            accepted = None
-
             if usekey == 0:
                 # We need password for password ssh auth as keyfile is missing. Retrieve password or ask for it
                 pwd = password_delegate.get_password_for("ssh")
@@ -293,7 +291,7 @@ class WbAdminSSH(object):
                 else:
                     password_delegate.reset_password_for("ssh")
                 raise exc
-            except paramiko.PasswordRequiredException, e:
+            except paramiko.PasswordRequiredException:
                 pwd = password_delegate.get_password_for("sshkey")
                 self.connect(host, port, settings.ssh_username, pwd, usekey, key_filename)
                 # Retry key pwd
@@ -310,7 +308,7 @@ class WbAdminSSH(object):
                 raise exc
             #else:
             #  raise OperationCancelledError("Cancelled login")
-        log_debug2(_this_file, "%s: Leave\n" % self.__class__.__name__)
+        log_debug2("%s: Leave\n" % self.__class__.__name__)
 
     def connect(self, host, port, user, pwd, usekey = 0, key = None):
         if port == None or port == 0:
@@ -330,6 +328,7 @@ class WbAdminSSH(object):
         #client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
         client.set_missing_host_key_policy(paramiko.WarningPolicy()) # This is a temp workaround, later we need to have UI with buttons accept
 
+           
         try:
             if 'timeout' in paramiko.SSHClient.connect.func_code.co_varnames:
                 client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
@@ -337,10 +336,19 @@ class WbAdminSSH(object):
             else:
                 client.connect(hostname = host, port = int(port), username = user, password = pwd, pkey = None,
                                     key_filename = key, look_for_keys=False, allow_agent=bool(usekey) )
-            log_info(_this_file, "%s: Connected via ssh to %s\n" % (self.__class__.__name__, host) )
+            log_info("%s: Connected via ssh to %s\n" % (self.__class__.__name__, host) )
+            if self.keepalive != 0:
+                client.get_transport().set_keepalive(self.keepalive)
+
             self.client = client
         except socket.error, exc:
-            raise SSHDownException()
+            import traceback
+            log_error("Error opening SSH connection to %s: %s\n" % (host, traceback.format_exc()))
+
+            if exc.args[0] == 8: # [Errno 8] nodename nor servname provided, or not known
+                raise ConnectionError("Unable to resolve host name '%s'" % host)
+            else:
+                raise SSHDownException(str(exc))
             #if exc.args[0] == 61: # connection refused
             #  raise ConnectionError("Could not establish SSH connection: %r.\nMake sure the SSH daemon is running and is accessible." % exc)
             #else:
@@ -351,15 +359,23 @@ class WbAdminSSH(object):
             else:
                 raise exc
         except paramiko.BadAuthenticationType, exc:
+            import traceback
+            log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
             if 'keyboard-interactive' in exc.allowed_types and pwd is not None:
                 # wrong password
                 raise InvalidPasswordError("Authentication failed for SSH user %s" % user)
             raise ConnectionError("Could not establish SSH connection: %s." % exc)
         except paramiko.AuthenticationException, exc:
+            import traceback
+            log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
             raise InvalidPasswordError("Invalid password for SSH user %s" % user)
         except paramiko.SSHException, exc:
+            import traceback
+            log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
             raise ConnectionError("Could not establish SSH connection: %s." % exc)
         except Exception, exc:
+            import traceback
+            log_error("Error opening SSH connection: %s\n" % traceback.format_exc())
             raise ConnectionError("Could not establish SSH connection. %s." % exc)
 
     def is_connected(self):
@@ -372,6 +388,8 @@ class WbAdminSSH(object):
             sftp.mkdir(path)
             sftp.close()
         except:
+            import traceback
+            log_error("Error creating remote dir via ssh: %s\n" % traceback.format_exc())
             sftp.close()
             raise
 
@@ -381,6 +399,8 @@ class WbAdminSSH(object):
             sftp.rmdir(path)
             sftp.close()
         except:
+            import traceback
+            log_error("Error removing remote dir via ssh: %s\n" % traceback.format_exc())
             sftp.close()
             raise
 
@@ -391,6 +411,8 @@ class WbAdminSSH(object):
             sftp.remove(path)
             sftp.close()
         except:
+            import traceback
+            log_error("Error removing remote file via ssh: %s\n" % traceback.format_exc())
             sftp.close()
             raise
 
@@ -411,6 +433,8 @@ class WbAdminSSH(object):
             if e.errno != errno.ENOENT:
               raise
         except:
+            import traceback
+            log_error("Error checking remote file via ssh: %s\n" % traceback.format_exc())
             sftp.close()
             raise
 
@@ -423,10 +447,12 @@ class WbAdminSSH(object):
         try:
             ret = sftp.stat(path)
             sftp.close()
-        except IOError, e:
+        except IOError:
             ret = None
             sftp.close()
         except:
+            import traceback
+            log_error("Error stating remote file via ssh: %s\n" % traceback.format_exc())
             ret = None
             sftp.close()
 
@@ -445,7 +471,7 @@ class WbAdminSSH(object):
             ret = True
             sftp.close()
         except IOError, e:
-            log_error(_this_file, '%s: Retreival of file "%s" failed: %s\n' % (self.__class__.__name__, source, str(e)) )
+            log_error('%s: Retrieval of file "%s" failed: %s\n' % (self.__class__.__name__, source, str(e)) )
             raise
 
         return ret
@@ -476,12 +502,12 @@ class WbAdminSSH(object):
         sftp.close()
         return ret
 
-    def set_contents(self, path, data): # raises IOError
+    def set_contents(self, path, data, mode = "w"): # raises IOError
         path = normalize_windows_path_for_ftp(path)
         
         sftp = self.client.open_sftp()
         try:
-            f = sftp.file(path, "w")
+            f = sftp.file(path, mode)
             f.chmod(stat.S_IREAD | stat.S_IWRITE)
         except IOError, exc:
             sftp.close()
@@ -501,15 +527,136 @@ class WbAdminSSH(object):
             ret = True
             sftp.close()
         except Exception, e:
-            log_error(_this_file, '%s: Sending of file "%s" to the server failed: %s\n' % (self.__class__.__name__, source, str(e)) )
+            log_error('%s: Sending of file "%s" to the server failed: %s\n' % (self.__class__.__name__, source, str(e)) )
             raise
 
         return ret
     
-    
-    def exec_cmd(self, cmd, as_user = Users.CURRENT, user_password = None, output_handler = None, read_size = 128, get_channel_cb = None):
+    def _read_streams(self, chan, stdout, stderr, read_size, end_text = None, timeout=0, spawn_process = False, wait_output = CmdOutput.WAIT_ALWAYS):
+        """
+          Will read data from the channel streams with different exit criteria:
+          - end_text is specified and found on the read data.
+          - timeout is specified and it has completed.
+          - If the server indicated the command completed and there's nothing left to read
+          
+          Rturns a tuple containing:
+          - Command return code if completed on the server else None
+          - A boolean value indicating if the end_text was found
+          - The data read from the stdout stream
+          - The data read from the stderr stream
+          
+          Expected Output
+          - Used to determine whether output should be read or not from the executed
+            command, possible values include:
+            - YES : Will wait for output all the time
+            - NO  : Will not wait for output all the time
+            - ON_OK : Will only wait for output if the command succeeds
+            - ON_ERROR : Will only wait for output if the command fails
+          """
+        t = time.time()
+        all_data = ''
+        all_error= ''
+        
+        srt_timeout = 2
+        status_ready_time = None
+        
+        
+        
+        # Initializes the exit criteria
+        cmd_ret = None
+        found_text = False
+        read_done = False
+        time_complete = False
+        
+        
+        #while not found_text and cmd_ret is None and not time_complete:
+        while not found_text and not read_done and not time_complete:
+            data = ''
+            error = ''
+            
+            # Reads from the stdout if available
+            while chan.recv_ready() and len(data) < read_size:
+                data += stdout.read(1)
+            if data:
+                log_debug2("ssh session stdout: %s\n" % data)
+
+            # Reads from the stderr if available
+            while chan.recv_stderr_ready() and len(error) < read_size:
+                error += stderr.read(1)
+            if error:
+                log_debug2("ssh session stderr: %s\n" % error)
+
+            # Appends any read data on stdout and stderr
+            if data or error:
+                all_data += data
+                all_error += error
+            
+            # If nothing was read, checks if the command completed and
+            # Reads the exist status
+            elif chan.exit_status_ready():
+                if cmd_ret is None:
+                    cmd_ret = chan.recv_exit_status()
+                
+                # No need to read output...
+                if all_data or all_error or wait_output == CmdOutput.WAIT_NEVER or\
+                   (wait_output == CmdOutput.WAIT_IF_OK and cmd_ret != 0) or\
+                   (wait_output == CmdOutput.WAIT_IF_FAIL and cmd_ret == 0):
+                   read_done = True
+                
+                # Needs to read output : YES, ON_OK and RC is 0 and ON_ERROR and RC != 0
+                elif status_ready_time:
+                    new_time = time.time()
+                    diff = new_time - status_ready_time
+                    read_done = diff >= srt_timeout
+                else:
+                    status_ready_time = time.time()
+                      
+
+                
+            # If end_text was specified, checks if it was found on the
+            # read data to exit properly
+            if end_text:
+                if all_data and all_data.find(end_text) != -1:
+                    found_text = True
+                
+                # Same check on the stderr data if needed
+                if spawn_process and all_error and all_error.find(end_text) != -1:
+                    found_text = True
+            
+            # If timeout was specified checks wether it has completed
+            if timeout:
+                time_complete = time.time() - t >= timeout
+        
+        return (cmd_ret, found_text, all_data, all_error)
+
+
+    def _prepare_channel(self, transport):
+          channel = None
+          stdin = None
+          stdout = None
+          stderr = None
+
+          channel = transport.open_session()
+
+          if channel:
+              channel.setblocking(True)
+              channel.settimeout(10)
+              channel.get_pty()
+              
+              bufsize = -1
+              stdin = channel.makefile('wb', bufsize)
+              stdout = channel.makefile('rb', bufsize)
+              stderr = channel.makefile_stderr('rb', bufsize)
+
+          return channel, stdin, stdout, stderr
+
+
+
+    def exec_cmd(self, cmd, as_user = Users.CURRENT, user_password = None, output_handler = None, read_size = 128, get_channel_cb = None, options = None):
         out = ""
-        ret = None
+        error = ""
+        ret_code = None
+        
 
         # For some reason we have bool in pwd
         if type(user_password) is unicode:
@@ -519,94 +666,71 @@ class WbAdminSSH(object):
             user_password = None
             
         expect_sudo_failure = False
-                
-        def read_nonblocking_until_nl_or(chan, fd, text, timeout=0):
-            import select
-            data = ""
-            t = time.time()
-            process_is_done = False
-            
-            while time.time() - t < timeout:
-                if not chan.recv_ready():
-                    if chan.exit_status_ready():
-                        break
-                    continue
-                
-                data += fd.read(1)
-                
-                if data.endswith(text) or "\n" in data:
-                    ndata = sanitize_sudo_output(data)
-                    if ndata != data:
-                        data = ndata
-                        continue
-                    break
-            
-            return data
-        
+
+        read_timeout = 10
 
         if self.client is not None:
             transport = self.client.get_transport()
-            chan = None
             try:
-                chan = transport.open_session() # There should be timeout someday. The patch was sent.
+                spawn_process = False
+                if 'nohup' in cmd:
+                    spawn_process = True
+
+                chan, stdin, stdout, stderr = self._prepare_channel(transport)
+                wait_output = CmdOutput.WAIT_ALWAYS
+                if options and options.has_key(CmdOptions.CMD_WAIT_OUTPUT):
+                    wait_output = options[CmdOptions.CMD_WAIT_OUTPUT]
+
                 if chan:
-                    chan.setblocking(True)
-                    
-                    # Updating the timeout to an hour
-                    chan.settimeout(10)
-                    
+                    ret_code = None
                     initial_data = ""
+                    error = ""
+                    more_data = True
+
+                    log_debug2("About to execute command through ssh session: %s\n" % cmd)
+                    chan.exec_command(cmd)
+                    pass_prompt_count = 0
+                  
                     if (as_user != Users.CURRENT and user_password is not None):
-                        #initial_data = self.handle_command_with_sudo(cmd, chan, user_password)
-                        bufsize = -1
-                        # Request a pseudo terminal on the server
-                        # needed for an interactive session
-                        chan.get_pty()
+                      
+                        ret_code, prompted, initial_data, error = self._read_streams(chan, stdout, stderr, 1, 'EnterPasswordHere', read_timeout+5, spawn_process, wait_output)
+                        log_debug2("%s.exec_cmd initial read for command [%s]:\nRetCode : [%s]\nPrompted : [%s]\nData : [%s]\nError : [%s]\n" % (self.__class__.__name__, cmd, ret_code, prompted, initial_data, error) )
                         
-                        # Request the interactive session
-                        chan.exec_command(cmd)
-                        
-                        stdin = chan.makefile('wb', bufsize)
-                        stdout = chan.makefile('rb', bufsize)
-                        stderr = chan.makefile_stderr('rb', bufsize)
-                    
-                        initial_data = read_nonblocking_until_nl_or(chan, stdout, 'EnterPasswordHere', 15)
-                        
-                        if initial_data.endswith("EnterPasswordHere"):
+                        if prompted:
                             initial_data = ''
                             
                             # feed the password
-                            log_debug2(_this_file, "exec_cmd: sending password to child...\n")
+                            log_debug2("exec_cmd: sending password to child...\n")
                             
                             try:
                                 stdin.write(user_password+"\n")
                                 stdin.flush()
+                                stdin.close()
                             except socket.error, e:
                                 es = str(e)
                                 if not('ocket' in es and 'closed' in es):
                                     raise
+                            
                             expect_sudo_failure = True # we could get a Sorry or the password prompt again
                         else:
-                            log_debug(_this_file, "exec_cmd: was expecting sudo password prompt, but it never came\n")
-                            if output_handler:
-                                output_handler(initial_data)
+                            if ret_code:
+                                log_error("exec_cmd: error on sudo operation : %s\n" % error)
+                                more_data = False
                             else:
-                                out = initial_data
+                                log_debug("exec_cmd: was expecting sudo password prompt, but it never came\n")
+            
+                                if initial_data:
+                                    wait_output = CmdOutput.WAIT_NEVER
 
-                    else:
-                        # Even the usage of get_pty is suppossed to be unneeded when executing
-                        # commands with exec_command. It just doesn't work, it is needed to
-                        # retrieve the output.
-
-                        # When running a command with nohup, there's a different behavior in linux
-                        # and mac. If get_pty is called in linux, it will not spawn the process
-                        # that's why this hack was added.
-                        if '_linux_spawn_' in cmd:
-                            cmd = cmd.replace('_linux_spawn_', '')
+                        # Outputs the initial read
+                        if output_handler:
+                            output_handler(initial_data)
                         else:
-                            chan.get_pty()
+                            if prompted:
+                                pass_prompt_count=1
+
+                            out = initial_data
 
-                        chan.exec_command(cmd)
 
                     # TODO: Is this really needed????
                     #       The call with channel get_channel_cb is only done in two places:
@@ -614,48 +738,39 @@ class WbAdminSSH(object):
                     #         is returned. (So the channel is not used)
                     #       - On wba_monitor_be.WinRemoteStats where it is only stored and not used (pretty much the same as the Test class)
                     if get_channel_cb is not None:
-                        log_debug3(_this_file, "%s.exec_cmd: Getting channel via passed cb (%s)\n" % (self.__class__.__name__, get_channel_cb.__name__) )
+                        log_debug3("%s.exec_cmd: Getting channel via passed cb (%s)\n" % (self.__class__.__name__, get_channel_cb.__name__) )
                         get_channel_cb(chan)
 
                     # Now reads the command output and sends it to the handler if needed.
                     # Note that is is needed to enter this loop even if there's no output 
                     # handler to prevent ending the channel before the command is completed.
-                    more_data = True
                     while more_data:
-                        try:
-                            chunk = chan.recv(read_size)
-                            if chunk is not None and chunk != "":
-                                if expect_sudo_failure and (chunk.find("EnterPasswordHere") >= 0 or chunk.find("Sorry, try again") >= 0):
-                                    raise InvalidPasswordError("Incorrect password for sudo")
-                                elif output_handler:
-                                    output_handler(chunk)
-                                else:
-                                    out += chunk
-                            elif not chan.recv_ready() and chan.exit_status_ready():
-                                more_data = False
-                        except socket.timeout:
-                            break
+                        ret_code, prompted, chunk, error = self._read_streams(chan, stdout, stderr, read_size, None, read_timeout, False, wait_output)
+                        
+                        if chunk or error:
+                            out += chunk
+                            # Detects an additional password prompt which could come in 2 formats
+                            # EnterPasswordHere is checked against pass_prompt_count as out could may or may not contain
+                            # the initial password prompt, the Sorry.... 
+                            if expect_sudo_failure and (out.count("EnterPasswordHere") > pass_prompt_count or out.count("Sorry, try again") > 0):
+                                raise InvalidPasswordError("Incorrect password for sudo")
+                            elif output_handler:
+                                output_handler(chunk)
+                                
+                            if error:
+                                log_warning("%s.exec_cmd error : %s\n" % (self.__class__.__name__, error))
 
-                    # At this point there's no data and the exist status should normally be ready
-                    # However, the paramiko docs indicate that there are cases where the command 
-                    # doesn't return an exit status so we wait a little bit before deciding this
-                    # is one of those cases.
-                    
-                    # Another case this could happen is if i.e. we executed a sudo operation and
-                    # The password prompt was not what we expected.
-                    t = time.time()
-                    while not chan.exit_status_ready() and time.time() - t < 2:
-                        time.sleep(0.25)
+                        # Quits reading when the exit code is read
+                        if ret_code != None:
+                            more_data = False
 
-                    if chan.exit_status_ready():
-                        ret = chan.recv_exit_status()
-                    else:
-                        log_warning(_this_file, "%s: Read from the peer is done, but status code is not available\n" % self.__class__.__name__)
+                    if ret_code is None:
+                        log_warning("%s: Read from the peer is done, but status code is not available\n" % self.__class__.__name__)
 
             except paramiko.SSHException, e:
                 if chan and chan.recv_ready():
                     out = chan.recv(128)
-                log_error(_this_file, "SSHException in SSH: %s\n" % str(e))
+                log_error("SSHException in SSH: %s\n" % str(e))
                 traceback.print_exc()
             except InvalidPasswordError, e:
                 raise
@@ -665,10 +780,9 @@ class WbAdminSSH(object):
                 # A "socket closed" exception happens sometimes when sending the password, for some reason.
                 # Not sure, but it could be that the command was accepted without a password (maye sudo doing internal
                 # password caching for some time). Until that is investigated, exceptions shouldn't bubble up
-                log_error(_this_file, "Exception in SSH: %s\n" % str(e))
-                traceback.print_exc()
+                log_error("Exception in SSH: %s\n%s\n" % (str(e), traceback.format_exc()))
             except:
-                log_error(_this_file, "Unknown exception in ssh\n")
+                log_error("Unknown exception in ssh\n")
 
             if chan is not None:
                 chan.close()
@@ -679,9 +793,9 @@ class WbAdminSSH(object):
         if out is not None and user_password is not None and not initial_data:
             out = out.replace(user_password, "", 1)
 
-        log_debug3(_this_file, "%s.exec_cmd(cmd=%s, output=%s. Retcode = %s)\n" % (self.__class__.__name__.encode('utf8'), cmd.encode('utf8'), out.decode('utf8'), ret) )
-
-        return (out, ret)
+        log_debug3("%s.exec_cmd(cmd=%s, output=%s. Retcode = %s)\n" % (self.__class__.__name__.encode('utf8'), cmd.encode('utf8'), out.decode('utf8'), ret_code) )
+        
+        return (out, ret_code)
 
     def getftp(self):
         self.sftp = WbAdminSFTP(self.client.open_sftp())
@@ -692,10 +806,15 @@ class WbAdminSSH(object):
             self.client.close()
 
 
+
+
+
+            
+
+
 # === Unit tests ===
 if __name__ == "__main__":
     #import threading
-    import time
 
     class Settings:
         def __init__(self):
