@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -25,7 +25,7 @@ export_options = {
 #    },
     "Inserts":{
     "extended-insert":["Use multiple-row INSERT syntax that include several VALUES lists.","TRUE"],
-    "delayed-insert":["Write INSERT DELAYED statements rather than INSERT statements.","FALSE"],
+    "delayed-insert":["Write INSERT DELAYED statements rather than INSERT statements.","FALSE", ("5.0.0", "5.7.0")],
     "add-locks":["Surround each table dump with LOCK TABLES and UNLOCK TABLES statements.","TRUE"],
     "replace":["Write REPLACE statements rather than INSERT statements.","FALSE"],
     "insert-ignore":["Write INSERT IGNORE statements rather than INSERT statements.","FALSE"],
@@ -51,6 +51,7 @@ export_options = {
     #"no-data":["Do not dump table contents.","FALSE"],
     "order-by-primary":["Dump each table's rows sorted by its primary key, or by its first unique index.","FALSE"],
     "dump-date":["Include dump date as \"Dump completed on\" comment if --comments is given.","TRUE"],
+    "$internal$show-internal-schemas":["Show internal MySQL schemas (mysql, information_schema, performance_schema) in the export schema list.","FALSE"],
     "tz-utc":["Add SET TIME_ZONE='+00:00' to the dump file.","TRUE"]
 #    "xml":["Produce XML output.","FALSE"]
     }
