@@ -96,6 +96,8 @@ class AdministratorContext:
         self.server_profile = None
         self.admin_pages = {}
         self.page_instances = {}
+        self.page_titles = {}
+        self.disabled_pages = {}
         self.admin_tab = None
         self.error_box = None
         self.ctrl_be = None
@@ -289,10 +291,22 @@ class AdministratorContext:
 
     def _sidebar_entry_clicked(self, entry_id):
         if entry_id == "configure":
-            grt.modules.Workbench.showInstanceManagerFor(self.editor.connection)
+            openConnectionSettings(self.editor)
         else:
+            if entry_id in self.disabled_pages:
+                Utilities.show_error(self.page_titles[entry_id],
+                                     self.disabled_pages[entry_id],
+                                     "OK", "", "")
+                return
             self.open_into_section(entry_id)
 
+    def refresh_admin_links(self):
+        for ident in self.disabled_pages.keys():
+            if (not self.server_profile or (self.server_profile and not self.server_profile.is_local and not self.server_profile.remote_admin_enabled)):
+                pass
+            else:
+                del self.disabled_pages[ident]
+                self.sidebar.set_section_entry_enabled(ident, True)
 
     def open_into_section(self, entry_id, select_item=False):
         page_class, needs_remote_access = self.admin_pages.get(entry_id, (None, None))
@@ -332,6 +346,8 @@ class AdministratorContext:
         if not section_id:
             section_id = "wba_management" # the default
 
+        self.page_titles[page_class.identifier()] = title
+
         self.admin_pages[page_class.identifier()] = (page_class, needs_remote_access)
         icon_path = page_class.identifier()+".png"
         for sname, stitle, sitems in self.sidebar_sections:
@@ -347,6 +363,7 @@ class AdministratorContext:
                 server_version = None
             self.shown_in_sidebar = True
             first = True
+            self.disabled_pages = {}
             for sname, stitle, sitems in self.sidebar_sections:
                 flags = mforms.TaskSectionShowConfigButton if sname == "wba_instance" else mforms.TaskSectionPlain
                 if first:
@@ -354,14 +371,16 @@ class AdministratorContext:
                     first = False
                 self.sidebar.add_section(sname, stitle, flags)
                 for ident, ititle, icon_path in sitems:
-                    self.sidebar.add_section_entry(sname, ident, ititle, icon_path, mforms.TaskEntryLink)
+                    self.sidebar.add_section_entry(sname, ident, ititle, icon_path, mforms.TaskEntryAlwaysActiveLink)
                     mod, requires_remote_access = self.admin_pages.get(ident, (None, True))
                     enabled = True
                     if requires_remote_access and (not self.server_profile or (self.server_profile and not self.server_profile.is_local and not self.server_profile.remote_admin_enabled)):
                         enabled = False
+                        self.disabled_pages[ident] = "Feature requires remote host access.\nClick the wrench icon to configure a remote administration method for this connection."
                     elif getattr(mod, "min_server_version", None):
                         if server_version and not server_version.is_supported_mysql_version_at_least(*mod.min_server_version):
                             enabled = False
+                            self.disabled_pages[ident] = "This feature requires MySQL version %s or newer" % ".".join([str(x) for x in mod.min_server_version])
 
                     self.sidebar.set_section_entry_enabled(ident, enabled)
 
@@ -425,12 +444,12 @@ def autoDetectLocalInstance(connection):
                 return []
             path += "/mysql.profiles"
         version = Version.fromstr(version or "5.6")
-        files = os.listdir(path)
+        files = [f for f in os.listdir(path) if f.endswith(".xml")]
         profiles = []
         matched_profiles = []
         for f in files:
             data = grt.unserialize(os.path.join(path, f))
-            if data.has_key("sys.system") and data["sys.system"] == system:
+            if data and data.has_key("sys.system") and data["sys.system"] == system:
                 profiles.append(data)
                 profile_version = Version.fromstr(data.get("serverVersion"))
                 if version.majorNumber == profile_version.majorNumber or version.minorNumber == profile_version.minorNumber:
@@ -841,6 +860,9 @@ except ImportError:
 @ModuleInfo.export(grt.INT, grt.classes.db_query_Editor)
 def openConnectionSettings(editor):
     grt.modules.Workbench.showInstanceManagerFor(editor.connection)
+    context = grt.fromgrt(editor.customData["adminContext"])
+    if context:
+        context.refresh_admin_links()
 
 
 @ModuleInfo.plugin("wb.admin.reset_password_cache", type="standalone", input=[wbinputs.currentSQLEditor()])
