@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -20,6 +20,9 @@ import time
 import getpass
 import threading
 import functools
+import random
+import string
+import os
 
 import mforms
 
@@ -27,6 +30,7 @@ from mforms import newBox, newLabel, newButton, newTextEntry, newTreeNodeView, n
 from wb_admin_utils import not_running_warning_label, make_panel_header
 from wb_admin_security_be import AdminSecurity, PrivilegeInfo, PrivilegeReverseDict, SecurityAdminRoles, WBSecurityValidationError
 from wb_common import PermissionDeniedError
+from workbench.log import log_error
 
 
 SCHEMA_OBJECT_RIGHTS = [
@@ -46,6 +50,7 @@ SCHEMA_DDL_RIGHTS = [
 "Create_view_priv",
 "Create_routine_priv",
 "Alter_routine_priv",
+"Event_priv",
 "Drop_priv",
 "Trigger_priv"
 ]
@@ -145,6 +150,9 @@ class ThreadedInputValidator(object):
         if not self.timer or not self.timer.is_alive():
             self.timer = threading.Timer(self.delay, self.validate)
             self.timer.start()
+    def shutdown(self):
+        if self.timer:
+          self.timer.cancel()
 
     def validate(self):
         def compute_password_strength(passwd):
@@ -197,6 +205,7 @@ class ThreadedInputValidator(object):
                 mforms.Utilities.perform_from_main_thread(functools.partial(self.update_validation_label, estimate=estimate),
                                                           False)
             time.sleep(0.2)
+        mforms.Utilities.driver_shutdown()
 
 
 class AddSchemaPrivilegeForm(mforms.Form):
@@ -245,8 +254,12 @@ class AddSchemaPrivilegeForm(mforms.Form):
         self.schema3 = newRadioButton(self.schema1.group_id())
         self.schema3.add_clicked_callback(self.schema_radio_changed)
         self.schema3.set_text("Selected schema:")
-        table.add(self.schema3, 0, 1, 2, 3, mforms.HFillFlag)
-        table.add(dLabel("Select a specific schema name for the rule to apply to."), 2, 3, 2, 3, mforms.HFillFlag)
+        alignbox = mforms.newBox(False)
+        alignbox.add(self.schema3, False, False)
+        table.add(alignbox, 0, 1, 2, 3, mforms.HFillFlag)
+        label = dLabel("Select a specific schema name for the rule to apply to.")
+        label.set_text_align(mforms.TopLeft)
+        table.add(label, 2, 3, 2, 3, mforms.HFillFlag)
 
         self.schema3sel = newListBox(False)
         table.add(self.schema3sel, 1, 2, 2, 3, mforms.HFillFlag|mforms.HExpandFlag|mforms.VFillFlag|mforms.VExpandFlag)
@@ -341,15 +354,7 @@ class SecuritySchemaPrivileges(mforms.Box):
         panel.set_title("Object Rights")
         box = newBox(False)
         box.set_padding(8)
-        for name in SCHEMA_OBJECT_RIGHTS:
-            cb = newCheckBox()
-            label, desc = PrivilegeInfo.get(name, ("", None))
-            cb.set_text(label)
-            if desc:
-                cb.set_tooltip(desc)
-            cb.add_clicked_callback(self.schema_priv_checked)
-            box.add(cb, False, False)
-            self.schema_rights_checks[name] = cb
+        self.object_rights_box = box
         panel.add(box)
         hbox.add(panel, True, True)
 
@@ -357,15 +362,7 @@ class SecuritySchemaPrivileges(mforms.Box):
         panel.set_title("DDL Rights")
         box = newBox(False)
         box.set_padding(8)
-        for name in SCHEMA_DDL_RIGHTS:
-            cb = newCheckBox()
-            label, desc = PrivilegeInfo.get(name, ("", None))
-            cb.set_text(label)
-            if desc:
-                cb.set_tooltip(desc)
-            cb.add_clicked_callback(self.schema_priv_checked)
-            box.add(cb, False, False)
-            self.schema_rights_checks[name] = cb
+        self.ddl_rights_box = box
         panel.add(box)
         hbox.add(panel, True, True)
 
@@ -373,15 +370,7 @@ class SecuritySchemaPrivileges(mforms.Box):
         panel.set_title("Other Rights")
         box = newBox(False)
         box.set_padding(8)
-        for name in SCHEMA_OTHER_RIGHTS:
-            cb = newCheckBox()
-            label, desc = PrivilegeInfo.get(name, ("", None))
-            cb.set_text(label)
-            if desc:
-                cb.set_tooltip(desc)
-            cb.add_clicked_callback(self.schema_priv_checked)
-            box.add(cb, False, False)
-            self.schema_rights_checks[name] = cb
+        self.other_rights_box = box
         panel.add(box)
         hbox.add(panel, False, True)
         self.add(hbox, False, True)
@@ -419,6 +408,48 @@ class SecuritySchemaPrivileges(mforms.Box):
 
     ####
 
+    def update(self):
+        if self.schema_rights_checks:
+            return
+
+        global_privilege_names = self._owner.owner.secman.global_privilege_names
+        for name in SCHEMA_OBJECT_RIGHTS:
+            if name not in global_privilege_names:
+                continue
+            cb = newCheckBox()
+            label, desc = PrivilegeInfo.get(name, ("", None))
+            cb.set_text(label)
+            if desc:
+                cb.set_tooltip(desc)
+            cb.add_clicked_callback(self.schema_priv_checked)
+            self.object_rights_box.add(cb, False, False)
+            self.schema_rights_checks[name] = cb
+
+        for name in SCHEMA_DDL_RIGHTS:
+            if name not in global_privilege_names:
+                continue
+            cb = newCheckBox()
+            label, desc = PrivilegeInfo.get(name, ("", None))
+            cb.set_text(label)
+            if desc:
+                cb.set_tooltip(desc)
+            cb.add_clicked_callback(self.schema_priv_checked)
+            self.ddl_rights_box.add(cb, False, False)
+            self.schema_rights_checks[name] = cb
+
+        for name in SCHEMA_OTHER_RIGHTS:
+            if name not in global_privilege_names:
+                continue
+            cb = newCheckBox()
+            label, desc = PrivilegeInfo.get(name, ("", None))
+            cb.set_text(label)
+            if desc:
+                cb.set_tooltip(desc)
+            cb.add_clicked_callback(self.schema_priv_checked)
+            self.other_rights_box.add(cb, False, False)
+            self.schema_rights_checks[name] = cb
+
+
     def schema_priv_checked(self):
         privs = []
         for name, cb in self.schema_rights_checks.iteritems():
@@ -458,6 +489,10 @@ class SecuritySchemaPrivileges(mforms.Box):
         sel.remove_from_parent()
         self.schema_priv_selected()
         self._owner.set_dirty()
+        
+        self.add_entry_button.set_enabled(True)
+        self.del_entry_button.set_enabled(True)
+        self.refresh_button.set_enabled(True)
 
 
     def schema_priv_selected(self):
@@ -580,20 +615,20 @@ class SecurityAccount(mforms.Box):
         bottom_box.add(self.add_button, False, True)
         self.add_button.add_clicked_callback(self.add_account)
 
-        self.dup_button = newButton()
-        self.dup_button.set_text("Duplicate")
+        #self.dup_button = newButton()
+        #self.dup_button.set_text("Duplicate")
         #bottom_box.add(self.dup_button, False, True)
-        self.dup_button.add_clicked_callback(self.dup_account)
+        #self.dup_button.add_clicked_callback(self.dup_account)
 
         self.del_button = newButton()
-        self.del_button.set_text("Drop")
+        self.del_button.set_text("Delete")
         bottom_box.add(self.del_button, False, True)
         self.del_button.add_clicked_callback(self.del_account)
 
         self.refresh_button = newButton()
         self.refresh_button.set_text("Refresh")
         self.refresh_button.add_clicked_callback(self.owner.refresh)
-        bottom_box.add_end(self.refresh_button, False, True)
+        bottom_box.add(self.refresh_button, False, True)
 
         self.save_button = newButton()
         self.save_button.set_text("Apply")
@@ -605,7 +640,7 @@ class SecurityAccount(mforms.Box):
         bottom_box.add_end(self.revert_button, False, True)
         self.revert_button.add_clicked_callback(self.revert)
 
-        if self.owner.ctrl_be.server_version >= (5, 6, 7):
+        if self.owner.ctrl_be.target_version and self.owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 6, 7):
             self.expire_button = newButton()
             self.expire_button.set_text("Expire Password")
             self.expire_button.set_tooltip("Force user to change password after next login. The user will be unable to issue any command other than SET PASSWORD.")
@@ -666,20 +701,21 @@ class SecurityAccount(mforms.Box):
         table.set_column_spacing(8)
 
         self.username = newTextEntry()
-        self.username.set_size(150, -1)
+        self.username.set_size(180, -1)
         self.username.add_changed_callback(self.set_dirty)
         self.username.set_max_length(16) # max username length for mysql
         self.password = newTextEntry(mforms.PasswordEntry)
-        self.password.set_size(150, -1)
-        self.password_advice = 'Consider using a password with 8 or more characters with mixed case letters, numbers and punctuation marks.'
+        self.password.set_size(180, -1)
+        self.password_advice = 'Consider using a password with 8 or more characters with\nmixed case letters, numbers and punctuation marks.'
         self.password_label = dLabel(self.password_advice)
         self.password_validator = ThreadedInputValidator(self, self.password, self.password_label, self.owner.ctrl_be)
         self.password.add_changed_callback(self.password_validator.callback)
         self.confirm = newTextEntry(mforms.PasswordEntry)
-        self.confirm.set_size(150, -1)
+        self.confirm.set_size(180, -1)
         self.confirm.add_changed_callback(self.set_dirty)
 
         self.hostlimithost = newTextEntry()
+        self.hostlimithost.set_size(180, -1)
         self.hostlimithost.add_changed_callback(self.hostlimithost_changed)
 
         table.add(rLabel("Login Name:"), 0, 1, 0, 1, mforms.HFillFlag)
@@ -692,7 +728,8 @@ class SecurityAccount(mforms.Box):
             self.auth_type_label = rLabel("Authentication Type:")
             table.add(self.auth_type_label, 0, 1, 1, 2, mforms.HFillFlag)
             self.auth_type_sel = newSelector()
-            table.add(self.auth_type_sel, 1, 2, 1, 2, mforms.HFillFlag|mforms.HExpandFlag)
+            self.auth_type_sel.set_size(180, -1)
+            table.add(self.auth_type_sel, 1, 2, 1, 2, mforms.HFillFlag)
             table.add(dLabel("\nFor the standard password and/or host based authentication,\nselect 'Standard'.\n"), 2, 3, 1, 2, mforms.HFillFlag)
             self.auth_type_sel.add_changed_callback(self.auth_type_changed)
             self.auth_type_list = []
@@ -706,7 +743,7 @@ class SecurityAccount(mforms.Box):
             self.auth_type_list = None
             self.auth_string_param = None
 
-        self.hostlimithost_caption = rLabel("Limit Connectivity to Hosts Matching:")
+        self.hostlimithost_caption = rLabel("Limit to Hosts Matching:")
         table.add(self.hostlimithost_caption, 0, 1, 2, 3, mforms.HFillFlag)
         table.add(self.hostlimithost, 1, 2, 2, 3, mforms.HFillFlag)
         self.hostlimit_box = newBox(True)
@@ -721,7 +758,7 @@ class SecurityAccount(mforms.Box):
         table.add(self.password, 1, 2, 3, 4, mforms.HFillFlag)
         self.password_caption2 = dLabel("Type a password to reset it.")
         table.add(self.password_caption2, 2, 3, 3, 4, mforms.HFillFlag|mforms.HExpandFlag)
-        table.add(self.password_label, 1, 3, 4, 5, mforms.HFillFlag|mforms.HExpandFlag)
+        table.add(self.password_label, 1, 3, 4, 5, mforms.HFillFlag)
         self.confirm_caption = rLabel("Confirm Password:")
         table.add(self.confirm_caption, 0, 1, 5, 6, mforms.HFillFlag)
         table.add(self.confirm, 1, 2, 5, 6, mforms.HFillFlag)
@@ -736,10 +773,26 @@ class SecurityAccount(mforms.Box):
             self.auth_string_label = rLabel("Authentication String:")
             self.auth_string_desc = dLabel("Authentication plugin specific parameters.")
             table.add(self.auth_string_label, 0, 1, 7, 8, mforms.HFillFlag)
-            table.add(self.auth_string_param, 1, 2, 7, 8, mforms.HFillFlag|mforms.HExpandFlag)
-            table.add(self.auth_string_desc, 2, 3, 7, 8, mforms.HFillFlag)
+            table.add(self.auth_string_param, 1, 2, 7, 8, mforms.HFillFlag)
+            table.add(self.auth_string_desc, 2, 3, 7, 8, mforms.HFillFlag|mforms.HExpandFlag)
             self.auth_string_help = newLabel("")
             vbox.add(self.auth_string_help, False, True)
+            
+        self.bottom_message_hbox = newBox(True)
+        self.bottom_message_hbox.set_spacing(8)        
+        
+        self.bottom_message_caption = newLabel('')
+        self.bottom_message_caption.set_color('#aa3333')
+        
+        self.upgrade_account_button = newButton()
+        self.upgrade_account_button.set_text("Upgrade")
+        self.upgrade_account_button.set_tooltip("Upgrade the authentication type of the account, from the obsolete format to the newer, safer one. You must set the password for the account before clicking this button.")
+        self.upgrade_account_button.add_clicked_callback(self.upgrade_account)
+        
+        self.bottom_message_hbox.add(self.bottom_message_caption, True, True)
+        self.bottom_message_hbox.add(self.upgrade_account_button, False, True)        
+        vbox.add_end(self.bottom_message_hbox, False)
+        self.bottom_message_hbox.show(False)
 
         tabView.add_page(vbox, "Login")
 
@@ -823,6 +876,9 @@ class SecurityAccount(mforms.Box):
 
         self.user_selected()
 
+    def shutdown(self):
+       self.password_validator.shutdown() 
+
     @property
     def active_plugins(self):
         return [name for name, ptype in self.owner.ctrl_be.server_active_plugins if ptype == "AUTHENTICATION"]
@@ -947,7 +1003,7 @@ class SecurityAccount(mforms.Box):
         self._selected_user_original = None
         self.show_user(None)
 
-        self.dup_button.set_enabled(False)
+        #self.dup_button.set_enabled(False)
         self.del_button.set_enabled(False)
 
         if sel:
@@ -959,7 +1015,7 @@ class SecurityAccount(mforms.Box):
             else:
                 self.owner.secman.async_get_account(self.show_user, user, host)
         else:
-            self.account_label.set_text("Select an account to edit or click Add Account to create a new one")
+            self.account_label.set_text("Select an account to edit or click [Add Account] to create a new one")
 
 
     def show_zombie_user(self, user, host):
@@ -967,7 +1023,7 @@ class SecurityAccount(mforms.Box):
         self.inner_tabview.show(False)
         if self.expire_button:
             self.expire_button.set_enabled(False)
-        self.dup_button.set_enabled(False)
+        #self.dup_button.set_enabled(False)
         self.del_button.set_enabled(True)
 
         privs = self.owner.secman.get_zombie_privs(user, host)
@@ -992,10 +1048,10 @@ class SecurityAccount(mforms.Box):
         self._selected_user_original = user and user.snapshot_for_revert()
 
         if user:
-            self.dup_button.set_enabled(True)
+            #self.dup_button.set_enabled(True)
             self.del_button.set_enabled(True)
             if self.expire_button:
-                self.expire_button.set_enabled(not user.password_expired)
+                self.expire_button.set_enabled(user.is_commited and not user.password_expired)
 
             self.username.set_value(user.username)
             self.password.set_value(user.password or "")
@@ -1051,12 +1107,10 @@ class SecurityAccount(mforms.Box):
 
             self.schema_privs.show_user(None)
 
-        if user and user.password_expired:
-            self.password_label.set_text("Password is expired. User must change password to use the account.")
-            self.password_label.set_color('#aa3333')
-        else:
-            self.password_label.set_text(self.password_advice)
-            self.password_label.set_color('#000000')
+        self.password_label.set_text(self.password_advice)
+        self.password_label.set_color('#000000')
+        
+        self.setup_bottom_message_box(user)           
 
 
     def _find_user_position(self, user, host):
@@ -1076,6 +1130,48 @@ class SecurityAccount(mforms.Box):
             self.user_list.select_node(self.user_list.node_at_row(pos))
         self.user_selected()
         self.set_dirty()
+        self.add_button.set_enabled(False)
+        self.del_button.set_enabled(False)
+        self.refresh_button.set_enabled(False)
+        
+    def upgrade_account(self):
+        def generate_password(length = 8):    
+            chars = string.ascii_letters + string.digits + '!@#$%^&*()'
+            random.seed = (os.urandom(1024))
+            return ''.join(random.choice(chars) for i in range(length))
+        
+        def change_password(password, confirm_password):
+            self._selected_user.password = password
+            self._selected_user.confirm_password = confirm_password
+            try:
+                self._selected_user.upgrade_password_format()
+            except WBSecurityValidationError, exc:
+                Utilities.show_error("Upgrade", str(exc), "OK", "", "")
+                return
+            except Exception, exc:
+                import traceback
+                log_error("Exception while upgrading account auth type: %s\n" % traceback.format_exc())
+                Utilities.show_error("Error Upgrading Authentication Method", str(exc), "OK", "", "")
+                return
+
+        if self.password.get_string_value() == self._selected_user._orig_password:
+            upgrade_account_msg = ["A password must be provided for the account (either the original or a new one) before it can be upgraded."]
+            reset_to_expire_caption = ""
+            if self.owner.ctrl_be.target_version and self.owner.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 6, 0):
+                upgrade_account_msg.append("Click [Reset to Expired] to set a random password and expire it, so the user will have to reset it next time they login.")
+                reset_to_expire_caption = "Reset to Expired"
+            result = Utilities.show_warning("Upgrade Authentication Method", "\n".join(upgrade_account_msg), "OK", "", reset_to_expire_caption)
+            if result == mforms.ResultOther:
+                gen_password = generate_password()                
+                change_password(gen_password, gen_password)
+                Utilities.show_message("Random Generated Password", 'Password changed to: %s' % gen_password, "OK", "", "")
+                self.expire()
+            else:
+                return
+        else:
+            change_password(self.password.get_string_value(), self.confirm.get_string_value())
+        
+        self.reload_user(False)
 
 
     def dup_account(self):
@@ -1111,6 +1207,10 @@ class SecurityAccount(mforms.Box):
                 self.owner.secman.wipe_zombie(username, host)
                 user.remove_from_parent()
 
+        self.add_button.set_enabled(True)
+        self.del_button.set_enabled(True)
+        self.refresh_button.set_enabled(True)
+
 
     def refresh(self):
         selected_row = None
@@ -1139,12 +1239,10 @@ class SecurityAccount(mforms.Box):
             self.user_list.select_node(selected_row)
             self.user_selected()
 
-        if su and su.password_expired:
-            self.password_label.set_text("Password is expired. User must change password to use the account.")
-            self.password_label.set_color('#aa3333')
-        else:
-            self.password_label.set_text(self.password_advice)
-            self.password_label.set_color('#000000')
+        self.password_label.set_text(self.password_advice)
+        self.password_label.set_color('#000000')
+        
+        self.setup_bottom_message_box(su) 
 
 
     def refresh_priv_list(self):
@@ -1181,7 +1279,7 @@ class SecurityAccount(mforms.Box):
                                  all( int(item) <= 255 for item in subnet_mask.split('.') ) )
                               )
 
-        if self.valid_name:
+        if self.valid_name or ':' in host: # give up and allow anything that looks like ipv6... this validation is silly anyway
             #self.set_dirty()
             self.hostlimithost_valid_icon.show(False)
         else:
@@ -1206,7 +1304,11 @@ class SecurityAccount(mforms.Box):
   
     def expire(self):
         if self._selected_user:
-            self._selected_user.expire_password()
+            try:
+                self._selected_user.expire_password()
+            except Exception, e:
+                    title, message = e.args[:2] if len(e.args) > 1 else ('Error:', str(e))
+                    Utilities.show_error(title, message, 'OK', '', '')
         self.refresh()
 
   
@@ -1233,6 +1335,9 @@ class SecurityAccount(mforms.Box):
         if not self.user_list.get_selected_node():
             self.user_selected()
 
+        self.add_button.set_enabled(True)
+        self.del_button.set_enabled(True)
+        self.refresh_button.set_enabled(True)
 
     def commit(self):
         if self._selected_user:
@@ -1255,6 +1360,13 @@ class SecurityAccount(mforms.Box):
                         "It is a security hazard to create an account with no password.\nPlease confirm creation of '%s'@'%s' with no password."%(username, host),
                         "Create", "Cancel", "") != mforms.ResultOk:
                     return
+            
+            lcase_host = host.lower()
+            if lcase_host != host:
+                if Utilities.show_message_and_remember("Save Account Changes",
+                                          "MySQL only allows lowercase characters for hostnames, the account host will be updated accordingly.",
+                                          "Ok", "", "","wb.admin.warn_ucase_hostnames", "Don't show this message again"):
+                    self.hostlimithost.set_value(lcase_host)
 
             self._selected_user.username = self.username.get_string_value()
             self._selected_user.password = self.password.get_string_value()
@@ -1298,6 +1410,8 @@ class SecurityAccount(mforms.Box):
                         'Cannot convert "%s" to a valid non-negative integer.\nPlease correct this value and try again.' % self.max_uconnections.get_string_value(),
                         'OK', '', '')
                 return
+            
+            
 
             if is_new_user and self.has_extra_plugins:
                 self._selected_user.auth_plugin = self.selected_plugin_type()
@@ -1318,21 +1432,55 @@ class SecurityAccount(mforms.Box):
                 return
             except Exception, exc:
                 import traceback
-                traceback.print_exc()
+                log_error("Exception while saving account: %s\n" % traceback.format_exc())
                 Utilities.show_error("Error Saving Account",
                       str(exc), "OK", "", "")
                 return
 
-            the_name = self._selected_user.formatted_name()
+            self.reload_user(is_new_user)
+
+        self.add_button.set_enabled(True)
+        self.del_button.set_enabled(True)
+        self.refresh_button.set_enabled(True)
+
+
+    def update(self):
+        self.schema_privs.update()
+
+    def setup_bottom_message_box(self, user):
+        if not user:
+            return               
+
+        caption = ''
+        if user.old_authentication:
+            caption = '''This account is using the pre-mysql-4.1.1 password hashing type.
+The user will not be able to login if the secure_auth option is enabled.
+Please click [Upgrade Account] to fix that.
+Either the account password must be provided to reset it
+or a new password must be supplied.'''
+        elif user.password_expired:
+            caption = 'Password is expired. User must change password to use the account.'
+        elif not user.username:
+            caption = 'This is an anonymous account. It is usually advisable to delete this account.'
+        elif user.blank_password :
+            caption = 'No password is set for this account.'
+        
+        self.bottom_message_caption.set_text(caption)
+        self.bottom_message_hbox.show(user.old_authentication or user.password_expired or user.blank_password or not user.username)
+        self.upgrade_account_button.show(user.old_authentication) 
+
+
+    def reload_user(self, is_new_user):
+        the_name = self._selected_user.formatted_name()
+        self._selected_user.load(self._selected_user.username, self._selected_user.host)
+        self.unset_dirty()
+        self.owner.refresh()
+        if is_new_user:
+            mforms.App.get().set_status_text("Created account '%s'" % the_name)
+        else:
+            mforms.App.get().set_status_text("Updated account '%s'" % the_name)
 
-            self._selected_user.load(self._selected_user.username, self._selected_user.host)
-            self.unset_dirty()
-            self.owner.refresh()
 
-            if is_new_user:
-                mforms.App.get().set_status_text("Created account '%s'" % the_name)
-            else:
-                mforms.App.get().set_status_text("Updated account '%s'" % the_name)
 
 
 #############################
@@ -1350,6 +1498,9 @@ class WbAdminSecurity(mforms.Box):
     def identifier(cls):
         return "admin_manage_privs"
 
+    def shutdown(self):
+        self.account_tab.shutdown()
+
     def __init__(self, ctrl_be, server_profile, main_view):
         mforms.Box.__init__(self, False)
         self.set_managed()
@@ -1440,9 +1591,11 @@ class WbAdminSecurity(mforms.Box):
                 self.secman.async_refresh(self.do_refresh)
             except PermissionDeniedError:
                 self.show_no_permission()
+            mforms.Utilities.driver_shutdown()
 
 
 
     def do_refresh(self):
         if not self.account_tab.dirty:
             self.account_tab.refresh()
+            self.account_tab.update()
