@@ -31,6 +31,7 @@ from workbench.notifications import NotificationCenter
 
 from sql_reformatter import formatter_for_statement_ast
 from text_output import TextOutputTab
+from run_script import RunScriptForm
 from sqlide_catalogman_ext import show_schema_manager
 from sqlide_tableman_ext import show_table_inspector
 
@@ -38,6 +39,7 @@ from sqlide_resultset_ext import handleResultsetContextMenu
 import sqlide_catalogman_ext
 import sqlide_tableman_ext
 import sqlide_schematree_ext
+import sqlide_import_spatial
 
 # define this Python module as a GRT module
 ModuleInfo = DefineModule(name= "SQLIDEUtils", author= "Oracle Corp.", version="1.1")
@@ -54,6 +56,7 @@ def initialize0():
     nc.add_observer(sqlide_schematree_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow") # must be 1st
     nc.add_observer(sqlide_catalogman_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow")
     nc.add_observer(sqlide_tableman_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow")
+    nc.add_observer(sqlide_import_spatial.handleContextMenu, name = "GRNLiveDBObjectMenuWillShow")
 
 
 
@@ -103,6 +106,7 @@ def importRecordsetDataFromFile(resultset):
                             resultset.setFloatFieldValue(column, value)
                         else:
                             resultset.setFieldNull(column)
+                resultset.addNewRow() # needed in Windows to refresh display for last row
     return 0
 
 
@@ -183,28 +187,30 @@ def verticalOutput(editor):
     statement = editor.currentStatement
     if statement:
         rsets = editor.owner.executeScript(statement)
-        rset = rsets and rsets[0]
-        if rset:
+        output = [ '> %s\n' % statement ]
+        for idx, rset in enumerate(rsets):
+            if len(rsets) > 1:
+                output.append('Result set %i' % (idx+1))
             column_name_length = max(len(col.name) for col in rset.columns)
-            output = [ '> %s\n' % rset.sql ]
             ok = rset.goToFirstRow()
             while ok:
                 output.append('******************** %s. row *********************' % (rset.currentRow + 1))
-                for column in rset.columns:
-                    col_name, col_value = column.name.rjust(column_name_length), rset.stringFieldValueByName(column.name)
+                for i, column in enumerate(rset.columns):
+                    col_name, col_value = column.name.rjust(column_name_length), rset.stringFieldValue(i)
                     output.append('%s: %s' % (col_name, col_value if col_value is not None else 'NULL'))
                 ok = rset.nextRow()
             output.append('%d rows in set' % (rset.currentRow + 1))
+            rset.reset_references()            
+            if len(rsets) > 1:
+              output.append('')
+        view = TextOutputTab('\n'.join(output) + '\n')
+        
+        dock = mforms.fromgrt(editor.resultDockingPoint)
+        dock.dock_view(view, '', 0)
+        dock.select_view(view)
+        dock.set_view_title(view, 'Vertical Output')
+
 
-            view = TextOutputTab('\n'.join(output) + '\n')
-            
-            dock = mforms.fromgrt(editor.resultDockingPoint)
-            dock.dock_view(view, '', 0)
-            dock.select_view(view)
-            dock.set_view_title(view, 'Vertical Output')
-          
-            rset.reset_references()
-            
     return 0
 
 
@@ -218,7 +224,7 @@ def doReformatSQLStatement(text, return_none_if_unsupported):
 
     ast = ast_list[0]
     
-    def trim_ast_fix_bq(text, node):
+    def trim_ast_fix_bq(text, node, add_rollup):
         s = node[0]
         v = node[1]
         c = node[2]
@@ -229,13 +235,18 @@ def doReformatSQLStatement(text, return_none_if_unsupported):
             if begin > 0 and text[begin-1] == '`' and text[end] == '`':
                 v = "`%s`" % v.replace("`", "``")
         l = []
-        for i in c:
-            l.append(trim_ast_fix_bq(text, i))
+        for i, nc in enumerate(c):
+            l.append(trim_ast_fix_bq(text, nc, add_rollup))
+            if add_rollup and nc[0] == "olap_opt" and nc[1].upper() == "WITH" and (i == len(c)-1 or c[i+1][1].upper() != "ROLLUP"):
+                l.append(("olap_opt", "ROLLUP", []))
         return (s, v, l)
 
     formatter = formatter_for_statement_ast(ast)
     if formatter:
-        p = formatter(trim_ast_fix_bq(text, ast))
+        # workaround a bug in parser where WITH ROLLUP is turned into WITH
+        add_rollup = "WITH ROLLUP" in text.upper()
+
+        p = formatter(trim_ast_fix_bq(text, ast, add_rollup))
         return p.run()
     else:
         if return_none_if_unsupported:
@@ -310,6 +321,8 @@ def enbeautificate(editor):
         try:
             result = doReformatSQLStatement(statement, True)
         except:
+            import traceback
+            log_error("Error reformating SQL: %s\n%s\n" % (statement, traceback.format_exc()))
             result = None
         if result:
             ok_count += 1
@@ -342,7 +355,7 @@ def enbeautificate(editor):
 
 def apply_to_keywords(editor, callable):
     from grt.modules import MysqlSqlFacade
-    non_keywords = ["ident", "TEXT_STRING", "text_string", "TEXT_STRING_filesystem", "TEXT_STRING_literal", "TEXT_STRING_sys",
+    non_keywords = ["ident", "ident_or_text", "TEXT_STRING", "text_string", "TEXT_STRING_filesystem", "TEXT_STRING_literal", "TEXT_STRING_sys",
                     "part_name"]
 
     text = editor.selectedText
@@ -577,7 +590,7 @@ def showInspector(editor, selection):
     for s in selection:
         if s.type == "db.Schema":
             schema_insp.append(s.schemaName)
-        elif s.type == "db.Table":
+        elif (s.type == "db.Table") or (s.type == "db.View"):
             table_insp.append((s.schemaName, s.name))
         elif s.type == "db.Index":
             table_insp_idx.append((s.schemaName, s.owner.name))
@@ -597,3 +610,11 @@ def showInspector(editor, selection):
   #def refactorRenameSchema(editor):
 #    pass
 
+
+@ModuleInfo.plugin("wb.sqlide.runScript", caption = "Run SQL Script", input=[wbinputs.currentSQLEditor()])
+@ModuleInfo.export(grt.INT, grt.classes.db_query_Editor)
+def runSQLScript(editor):
+    form = RunScriptForm(editor)
+    form.run()
+    return 0
+
